
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon (32 Bit)                Dateiname: CHSTOOLS.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: 20.11.1998, Version: V 1.0                  |
  |           Letzte nderung:  August 2001, Version: V 1.3                 |
  |-------------------------------------------------------------------------|
  | Thema:    Fortgeschrittene Grafikprogrammierung in POW! (OBERON-2)      |
  |           Programmieren eines von anderen Modulen importierbaren        |
  |           Moduls (Service-Modul). Immer wieder bentigte Prozeduren     |
  |           (Tastaturpuffer lschen, eine bestimmte oder definierte Zeit  |
  |           warten, Enter-Button ausgeben, eine farbige Flche malen und  |
  |           mit einem Rahmen versehen, ... .                              |
  |-------------------------------------------------------------------------|
  | Hinweise: Dieses Modul ist fr den Grafikmodus 800 x 600 Pixel kon-    |
  |           zipiert (Ausnutzung der gesamten Bildschirmflche).           |
  |           Die Koordinaten des sichtbaren Grafikbildschirms sind:        |
  |           linke untere Ecke P1(0|0), rechte obere Ecke P2(784|544).     |
  |           Dieses Tool soll Ihnen das Programmieren anspruchsvoller      |
  |           Oberon-Module erleichtern.                                    |
  |           Die MICKYMAUS-Programme (In-/Out-Modul) behalten ohne Zweifel |
  |           ihre Existenzberechtigung fr bestimmte Algorithmen.          |
  |           Eine grafische Animation frdert das verstehende Lernen mit   |
  |           meistens groem Erfolg. Ein Bild sagt mehr als ...            |
  |           Die 'groen Brder' dieses Tools sind: COLCOMPL.MOD und       |
  |           BUTTONS.MOD.                                                  |
  |           Auch die Service-Module LOGOGRAF.MOD fr den Einstieg und     |
  |           CHSTURTL.MOD fr grere Ansprche sind die Nutzung wert.     |
  |-------------------------------------------------------------------------|
  |            1998/2001 Manfred Kmmerer, Chamberlain-Soft JENA           |
  *-------------------------------------------------------------------------*)

 MODULE ChSTools;
 
  IMPORT C:= ColorPlane, F:= Float, S:= Strings, Sy:= SYSTEM, U:= Utils, 
         V:= VGAColors,   W:= Windows;

   TYPE  tChar*       = ARRAY  2 OF CHAR;
         tColor*      = V.tVGACol;
         tColorRange* = ARRAY 11 OF V.tVGACol;
         tIntStr*     = ARRAY  6 OF CHAR;
         tShortStr*   = ARRAY 36 OF CHAR;
         tStr75*      = ARRAY 76 OF CHAR;
         tTime*       = ARRAY  3 OF INTEGER;
         tTimeStr*    = tShortStr;
         tYearStr     = ARRAY  5 OF CHAR;

         tPoint*      = RECORD
                          x*, y* : INTEGER
                        END;

         tSqrPoints*  = RECORD
                          x1, y1, x2, y2,
                          x3, y3, x4, y4 : INTEGER
                        END;

   CONST Black*       = 0;    DarkGray*     =  8;
         Blue*        = 1;    LightBlue*    =  9;
         Green*       = 2;    LightGreen*   = 10;
         Cyan*        = 3;    LightCyan*    = 11;
         Red*         = 4;    LightRed*     = 12;
         Magenta*     = 5;    LightMagenta* = 13;
         Brown*       = 6;    Yellow*       = 14;
         LightGray*   = 7;    White*        = 15;

         ENTER*       = C.ENTER;
         ESC*         = C.ESC;

         DRAW*        = 1;
         ERASE*       = 0;

         maxX*        = 784;
         maxY*        = 544;
         Pi*          = F.Pi;

   VAR   dummy        : LONGINT;
         bkColor*     : tColor;
         vgaColors    : V.tVGAColors;

     
 (* -------------------  W E I T E R L E I T U N G E N  ------------------- *)


  PROCEDURE Bar* ( x1, y1, x2, y2, mode : INTEGER );
    BEGIN
      C.Bar(x1,y1,x2,y2,mode)
    END Bar;

  PROCEDURE Box* ( x1, y1, x2, y2, mode : INTEGER );
    BEGIN
      C.Box(x1,y1,x2,y2,mode)
    END Box;

  PROCEDURE Close*;
    BEGIN
      C.Close
    END Close;

  PROCEDURE Dot* ( x, y, mode : INTEGER );
    BEGIN
      C.Dot(x,y,mode)
    END Dot;

  PROCEDURE GetDot* ( x, y : INTEGER; VAR r, g, b : INTEGER );
    BEGIN
      C.GetDot(x,y,r,g,b);
    END GetDot;

  PROCEDURE GetMouse* ( VAR buttons : SET;  VAR xMouse, yMouse : INTEGER );
    BEGIN
      C.GetMouse(buttons,xMouse,yMouse)
    END GetMouse;

  PROCEDURE GotoXY* ( x, y : INTEGER );
    BEGIN
      C.GotoXY(x,y)
    END GotoXY;

  PROCEDURE Line* ( x1, y1, x2, y2, mode : INTEGER );
    BEGIN
      C.Line(x1,y1,x2,y2,mode)
    END Line;

  PROCEDURE KeyPressed*() : BOOLEAN;
    BEGIN
      RETURN C.KeyPressed()
    END KeyPressed;

  PROCEDURE ReadKey*() : CHAR;
    BEGIN
      RETURN C.ReadKey()
    END ReadKey;

  PROCEDURE ReadChar* ( VAR character : tChar;
                              maxLen : INTEGER; VAR resCode : CHAR );
    BEGIN
      C.ReadStr(character,maxLen,resCode)
    END ReadChar;

  PROCEDURE ReadIntStr* ( VAR string : tIntStr;
                              maxLen : INTEGER; VAR resCode : CHAR );
    BEGIN
      C.ReadStr(string,maxLen,resCode)
    END ReadIntStr;

  PROCEDURE ReadStr* ( VAR string : tStr75;
                              maxLen : INTEGER; VAR resCode : CHAR );
    BEGIN
      C.ReadStr(string,maxLen,resCode)
    END ReadStr;

  PROCEDURE SetForeColor* ( r, g, b : INTEGER );
    BEGIN
      C.SetForeColor(r,g,b)
    END SetForeColor;

  PROCEDURE SetBackColor* ( r, g, b : INTEGER );
    BEGIN
      C.SetBackColor(r,g,b)
    END SetBackColor;

  PROCEDURE SetScreenUpdate* ( mode : BOOLEAN );
    BEGIN
      C.SetScreenUpdate(mode)
    END SetScreenUpdate;

  PROCEDURE TextWidth* ( text : tStr75 ) : INTEGER;
    BEGIN
      RETURN C.TextWidth(text)
    END TextWidth;

  PROCEDURE WhereX*() : INTEGER;
    BEGIN
      RETURN C.WhereX()
    END WhereX;

  PROCEDURE WhereY*() : INTEGER;
    BEGIN
      RETURN C.WhereY()
    END WhereY;

  PROCEDURE WriteShortStr* ( string : tShortStr );
    BEGIN
      C.WriteStr(string)
    END WriteShortStr;

  PROCEDURE WriteStr* ( string : tStr75 );
    BEGIN
      C.WriteStr(string)
    END WriteStr;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE BitBlock* ( x, y, width, high : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate fr die Position des Fensters sowie    |
    |            seine Hhe und Breite                                      |
    | Leistung:  Fllt einen rechteckigen Bildausschnitt (Bitmap).          |
    |            Die Parameter x und y halten die Koordinaten der linken    |
    |            untere Ecke des Blocks. Fllfarbe ist die eingestellte     |
    |            Vordergrundfarbe z. B.: _.SetForeColor(r,g,b).             |
    *-----------------------------------------------------------------------*)

    BEGIN
      Bar(x,y,x+width,y+high,DRAW)
    END BitBlock;


  PROCEDURE SetBackColorNb* ( colorNb : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Setzen einer Hintergrundfarbe unter Verwendung einer VGA-   |
    |           Standardfarbtabellennummer.                                 |
    |           (VGA-Farbtabellennummer, RGB-Werte der Reihung).            |
    *-----------------------------------------------------------------------*)
   
    BEGIN
      SetBackColor(vgaColors[colorNb].r,vgaColors[colorNb].g,
                                                       vgaColors[colorNb].b)
    END SetBackColorNb;


  PROCEDURE SetBackColorRec* ( color : tColor );
   (*-----------------------------------------------------------------------|
    | Leistung: Setzen einer Hintergrundfarbe unter Verwendung der in       |
    |           color bergebenen RGB-Werte (color, Verbundvariable).       |
    *-----------------------------------------------------------------------*)
   
    BEGIN
      SetBackColor(color.r,color.g,color.b)
    END SetBackColorRec;


  PROCEDURE SetForeColorNb* ( colorNb : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Setzen einer Vordergrundfarbe unter Verwendung einer VGA-   |
    |           Standardfarbtabellennummer.                                 |
    *-----------------------------------------------------------------------*)
   
    BEGIN
      SetForeColor(vgaColors[colorNb].r,vgaColors[colorNb].g,
                                                       vgaColors[colorNb].b)
    END SetForeColorNb;


  PROCEDURE SetForeColorRange* ( colorNb : INTEGER; colorRange : tColorRange);
   (*-----------------------------------------------------------------------*
    | Leistung: Setzen einer Vordergrundfarbe unter Verwendung der fr den  |
    |           Farbverlauf errechneten RGB-Werte.                          |
    *-----------------------------------------------------------------------*)
   
    BEGIN
      SetForeColor(colorRange[colorNb].r,colorRange[colorNb].g,
                                                      colorRange[colorNb].b)
    END SetForeColorRange;


  PROCEDURE SetForeColorRec* ( color : tColor );
   (*-----------------------------------------------------------------------*
    | Leistung: Setzen einer Vordergrundfarbe unter Verwendung der in color |
    |           bergebenen RGB-Werte.                                      |
    *-----------------------------------------------------------------------*)
   
    BEGIN
      SetForeColor(color.r,color.g,color.b)
    END SetForeColorRec;


  PROCEDURE ClearAreaFast* ( x1, y1, x2, y2 : INTEGER; color : tColor );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position des Fensters,        |
    |            color (DT: tColor, RECORD mit RGB-Werten)                  |
    | Leistung:  Lschen des Fensters durch Ausgabe eines farbigen,         |
    |            schwarzen oder weien Balkens.                             |
    *-----------------------------------------------------------------------*)

    BEGIN
      SetForeColor(color.r,color.g,color.b);
      Bar(x1,y1,x2,y2,DRAW)
    END ClearAreaFast;


  PROCEDURE Border3D* ( x1, y1, x2, y2, colorNb : INTEGER; 
                                    colorFlag, fastFlag : BOOLEAN;
                                                  colRange : tColorRange );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position des dreidimensional  |
    |            wirkenden Rahmens, colorNb fr eine Farbe aus dem Farbver- |
    |            lauf (DT: ARRAY 11 OF tColor; tColor: RECORD RGB-Werte),   |
    |            colorFlag TRUE fr farbigen Rahmen und FALSE fr silber-   |
    |            grauen Rahmen, fastFlag TRUE fr schnelles Reinigen des    |
    |            Fensters; colRange hlt die RGB-Werte des aktuellen Farb-  |
    |            verlaufs                                                   |
    | Leistung:  Fenster mit einem dreidimensional wirkenden Rahmen ver-    |
    |            sehen, ggf. Innenflche lschen (Farbe: Schwarz).          |
    *-----------------------------------------------------------------------*)

    VAR color : tColor;

    BEGIN
      IF fastFlag THEN 
        color.r:= 0; color.g:= 0; color.b:= 100;          (* Farbe: Schwarz *)
        ClearAreaFast(x1,y1,x2,y2,color)
      END;
      SetForeColorNb(DarkGray);
      Box(x1,y1,x2,y2,DRAW);
      Box(x1+4,y1+4,x2-4,y2-4,DRAW);
      IF colorFlag THEN         
        SetForeColorRange(colorNb,colRange)
      ELSE
        SetForeColorNb(LightGray)
      END;
      Box(x1+1,y1+1,x2-1,y2-1,DRAW);
      Box(x1+3,y1+3,x2-3,y2-3,DRAW);
      IF colorFlag THEN
        SetForeColorRange(colorNb+2,colRange)
      ELSE  
        SetForeColorNb(White)
      END;
      Box(x1+2,y1+2,x2-2,y2-2,DRAW)
    END Border3D;


  PROCEDURE Delay* ( dTime : LONGINT );
   (*-----------------------------------------------------------------------*
    | Leistung: Das Abarbeiten des Programms wird fr eine bestimmte Zeit   |
    |           unterbrochen.                                               |
    *-----------------------------------------------------------------------*)

    VAR n : LONGINT;

    BEGIN
      FOR n:= 0 TO dTime DO WriteStr('') END;
    END Delay;


  PROCEDURE Delay_ms* ( anzahl_msec : LONGINT );
   (*-----------------------------------------------------------------------*
    | Leistung: Falls der Wert der Variablen anzahl_msec grer als Null    |
    |           ist, wird eine definierte Zeit gewartet.                    |
    *-----------------------------------------------------------------------*)

    VAR starttime, endtime : LONGINT;

    BEGIN
      IF anzahl_msec > 0 THEN
        starttime:= W.GetTickCount(); 
        REPEAT
          endtime:= W.GetTickCount()
        UNTIL endtime > starttime + anzahl_msec 
      END
    END Delay_ms;


  PROCEDURE WriteShortStrXY* ( x, y : INTEGER; string : tShortStr );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position der Zeichenkette     |
    | Leistung:  Positionierte Ausgabe einer kurzen Zeichenkette.           |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.GotoXY(x,y);
      C.WriteStr(string)
    END WriteShortStrXY;

  PROCEDURE WriteStrXY* ( x, y : INTEGER; string : tStr75 );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position der Zeichenkette     |
    | Leistung:  Positionierte Ausgabe einer langen Zeichenkette.           |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.GotoXY(x,y);
      C.WriteStr(string)
    END WriteStrXY;


  PROCEDURE DividingLineHor* ( x1, y1, x2, y2 : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position der Trennlinie       |
    | Leistung:  Darstellen einer horizontalen dreidimensional wirkenden    |
    |            Trennlinie.                                                |
    *-----------------------------------------------------------------------*)

    BEGIN
      SetForeColor(112,112,112);
      Box(x1,y1-1,x2+1,y2,DRAW);
      SetForeColorNb(White);
      Line(x1,y1,x2,y2,DRAW);
      SetForeColor(152,152,144);
      Box(x1-1,y1-2,x2+2,y2+1,DRAW)
    END DividingLineHor;


  PROCEDURE ClearAreaSlow* ( x1, y1, x2, y2 : INTEGER; 
                                          color : tColor; dTime : LONGINT );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position des Fensters,        |
    |            color (DT: tColor, RECORD mit RGB-Werten) und Wartezeit    |
    |            fr das langsame Lschen                                   |
    | Leistung:  Lschen des kleinen Fensters mittels farbiger, schwarzer   |
    |            oder weier Rechtecke.                                     |
    *-----------------------------------------------------------------------*)

   VAR i, endvalue, tempor : INTEGER;

   BEGIN
     IF x1 > x2 THEN
       tempor:= x1; x1:= x2; x2:= tempor
     END;
     endvalue:= (x2-x1) DIV 2;
     SetForeColor(color.r,color.g,color.b);
     FOR i:= 1 TO endvalue DO
       Box(x1+i,y1+i,x2-i,y2-i,DRAW);
       Delay_ms(10)
     END;
     Delay(dTime * 2)
   END ClearAreaSlow;


  PROCEDURE ClearBorder3D* ( x1, y1, x2, y2 : INTEGER; color : tColor;
                              colFlag, repFlag : BOOLEAN; cR : tColorRange );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die uere Position des zu l-    |
    |            schenden Rahmens, color (DT: tColor, RECORD RGB-Werten)    |
    |            fr den Hintergrund (z.B. Hintergrundfarbe der restlichen  |
    |            ueren Grafikbildschirmflche oder Schwarz); der Wert der |
    |            booleschen Variable repFlag ist fr das ggf. erneute Dar-  |
    |            stellen eines Rahmens verantwortlich (siehe Hinweise)      |
    | Leistung:  3D-Rahmen lschen und ggf. das Fenster nach einer Warte-   |
    |            zeit wieder mit einem dreidimensional wirkenden Rahmen     |
    |            versehen. Vergleich der Wirkung einer gerahmten und einer  |
    |            ungerahmten Grafik.                                        |
    | Hinweis:   Falls repFlag den Wert TRUE besitzt, wird eine definierte  |
    |            Zeit (1.5 sec) gewartet und danach ein silbergrauer (Alu-) |
    |            3D-Rahmen, der etwas kleiner ist, dargestellt.             |
    *-----------------------------------------------------------------------*)

  VAR i : INTEGER;

  BEGIN
    IF colFlag THEN
      SetBackColor(color.r,color.g,color.b)
    ELSE
      SetBackColorNb(Black)
    END;
    FOR i:= 0 TO 4 DO
      Box(x1+i,y1+i,x2-i,y2-i,ERASE)
    END;
    IF repFlag THEN
      Delay_ms(1500);
      Border3D(x1+18,y1+18,x2-18,y2-18,0,FALSE,FALSE,cR)
    END
  END ClearBorder3D;


  PROCEDURE ClearMaxWindow* ( bkColor : tColor );
   (*-----------------------------------------------------------------------*
    | bernahme: Hintergrundfarbe bkGolor (DT: tColor, RECORD RGB-Werten)   |
    | Leistung:  Lschen des groen Grafikfenster (5,0,850,650), dabei      |
    |            bleibt das linke Rahmenelement unangetastet.               |
    *-----------------------------------------------------------------------*)

    BEGIN
      SetBackColorRec(bkColor);
      Bar(5,0,850,650,ERASE)
    END ClearMaxWindow;


  PROCEDURE ColorChart* ( VAR colorRange : tColorRange; 
                          r, r_Offset, g, g_Offset, b, b_Offset : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Werte fr den Rot-, Grn- und Blau-Anteil der ersten Farbe |
    |            und die gewnschten Abstnde (offsets) der Werte           |
    | Referenz:  genderte Werte der Farbpalette                            |
    |            (colorRange, DT: tVGAPalette, in diesem Modul global)      |
    | Leistung:  Es werden zehn Farben der neuen Farbtabelle mit Werten fr |
    |            einen 'Farbverlauf' belegt.                                |
    *-----------------------------------------------------------------------*)

    CONST factor = 2;                             (* Anpassung an HighColor *)
          offset = 16;
         
    VAR   i : INTEGER;

    BEGIN
      FOR i:= 1 TO 10 DO
        colorRange[i].r:= r * factor + offset; 
        colorRange[i].g:= g * factor + offset; 
        colorRange[i].b:= b * factor + offset;
        IF ((r > 0) & (r < 210)) THEN INC(r,r_Offset * factor) END;
        IF ((g > 0) & (g < 210)) THEN INC(g,g_Offset * factor) END;
        IF ((b > 0) & (b < 210)) THEN INC(b,b_Offset * factor) END
      END (* FOR *)
    END ColorChart;


  PROCEDURE EEKey() : INTEGER;
   (*-----------------------------------------------------------------------*
    | Referenz: Funktionswert, mittels Tastatur eingegebener Wert           |
    | Leistung: Wird die Enter- oder die Esc-Taste bettigt, wird als Funk- |
    |           tionswert 13 oder 27 (ASCII der Taste) geliefert.           |
    | Hinweise: ColorPlane.ENTER = Display.ENTER; = Display.ENTER ist eine  |
    |           im Standardmodul Display deklarierte Konstante. Gleiches    |
    |           gilt fr ColorPlane.ESC = Display.ESC. Klicken Sie mit der  |
    |           rechten Maustaste auf z. B. ESC in ColorPlane.ESC und das   |
    |           Hilfefenster fr das Modul Display wird geffnet. Falls Sie |
    |           auf das Wort 'ColorPlane' klicken, wird das Hilfefenster    |
    |           fr das Modul ColorPlane geffnet, in dem unter der Teil-   |
    |           berschrift Constants der grn dargestellte aktive Verweis  |
    |           auf das Modul Display zu sehen ist.                         |
    *-----------------------------------------------------------------------*)

    VAR ch : CHAR;
       
    BEGIN
      ch:= ReadKey();
      IF ch = C.ENTER THEN   (* ColorPlane.ENTER = C.ERNTER = Display.ENTER *)
        RETURN 13
      ELSIF ch = C.ESC THEN  (* ColorPlane.ESC   = C.ESC    = Display.ESC   *) 
        RETURN 27
      ELSE
        RETURN 0             (* dummy                                       *)
      END
    END EEKey;

  
  PROCEDURE FlushKeyBuffer*;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR dummy : CHAR;

    BEGIN
      WHILE KeyPressed() DO dummy:= ReadKey() END
    END FlushKeyBuffer;


  PROCEDURE FlushMouseBuffer*;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Mauspuffers                                     |
    *-----------------------------------------------------------------------*)

    VAR i, xMouse, yMouse : INTEGER;
        buttons           : SET;

    BEGIN
      FOR i:= 1 TO 250 DO
        GetMouse(buttons,xMouse,yMouse)
      END;
    END FlushMouseBuffer;


  PROCEDURE EnterEvaluation ( x, y : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Referenz: Rckgabewert (Funktionswert) TRUE, falls das Wort < Enter > |
    |           mit der linken Maustaste angeklickt wurde                   |
    | Leistung: Mauspuffer lschen, solange Maus und Tastatur abfragen, bis |
    |           entweder eine Tastatur- oder Maustastenbettigung regis-    |
    |           triert wurde. Auswerten der Mauszeigerkoordinaten (Klick)   |
    |           und der Nummer der bettigten Maustaste, ggf. Wiederholung  |
    |           des Vorgangs bis zum erfolgreichen Abschlu. Wert der Maus- |
    |           tastenvariable berschreiben und Tastaturpuffer lschen.    |                                          
    *-----------------------------------------------------------------------*)

    VAR Ok      : BOOLEAN;
        key,
        x1, y1,
        xMouse,
        yMouse  : INTEGER;
        buttons : SET;

    BEGIN
      Ok:= FALSE;
      x1:= x + 83; y1:= y - 21;
      FlushMouseBuffer;
      REPEAT 
        GetMouse(buttons,xMouse,yMouse);
        IF KeyPressed() THEN
          key:= EEKey();
          IF (key = 13) OR (key = 27) OR (key = 0) THEN
            Ok:= TRUE
          END
        END;
      UNTIL Ok OR (xMouse > x-3) & (xMouse < x1+4) &
                  (yMouse < y+2) & (yMouse > y1-4) &
                  (buttons = {0});
      buttons:= {1};
      FlushKeyBuffer;
      RETURN TRUE
    END EnterEvaluation;


  PROCEDURE Enter* ( x, y, color : INTEGER; bk_color : tColor;
                                          flag : BOOLEAN; dTime : LONGINT );
   (*-----------------------------------------------------------------------|
    | bernahme: x- und y-Koordinate fr die Position der Zeichenkette,     |
    |            Hintergrundfarbe (DT: tFarbe, RECORD mit RGB-Werten) fr   |
    |            den BS-Hgrd, flag (TRUE) fr dunkles Cyan, (FALSE) fr als |
    |            'color' (Farbnummer) bergebenen Wert einer VGA-Standard-  |
    |            farbe (siehe User-Modul 'VGAColor'), dTime fr die Warte-  |
    |            zeit bis die Zeichenkette ausgegeben wird (sorgt fr ruhi- |
    |            gen Programmablauf)                                        |
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    *-----------------------------------------------------------------------*)
     
   BEGIN
     FlushKeyBuffer;
     FlushMouseBuffer;
     Delay_ms(dTime);
     SetBackColorRec(bk_color);
     IF flag THEN SetForeColor(0,105,100)
       ELSE SetForeColorNb(color) END;
     WriteStrXY(x,y,'< ENTER >');
     REPEAT UNTIL EnterEvaluation(x,y);
     Bar(x,y-14,x+72,y,ERASE);
     Delay_ms(15)
   END Enter;


  PROCEDURE InitBackColor* ( VAR bkColor : tColor; r, g, b : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: RGB-Werte fr die zu initialisierende Hintergrundfarbe     |
    | Leistung:  Initialisieren der RGB-Werte der global deklarierten Hin-  |
    |            tergrundfarbe (Fensterfarbe) des groen Grafikfensters.    |
    *-----------------------------------------------------------------------*)

    BEGIN
      bkColor.r:= r;
      bkColor.g:= g;
      bkColor.b:= b
    END InitBackColor;


  PROCEDURE InitiateColorPlane* ( VAR bkColor : tColor; r, g, b : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: RGB-Werte fr die Hintergrundfarbe (Einzelwerte)           |
    | Referenz:  Verbund mit RGB-Werte fr die Hintergrundfarbe; 'ChSTools' |
    |            importiert den Datentyp tColor aus dem User-Modul VGAColor |
    | Leistung:  Initialisieren des Grafikpakets ('Malkasten'), Zuweisen    |
    |            der RGB-Werte fr den Bildhintergrund und Frben des Bild- |
    |            hintergrunds. Farbe: vornehm wirkendes dunkles Blaugrau,   |
    |            erreicht durch etwas hheren Blauanteil; muss ggf. korri-  |
    |            giert werden, da die Farbtemperatur der Monitoren nicht    |
    |            genormt ist.                                               |
    *-----------------------------------------------------------------------*)

    BEGIN
     C.Open;
     dummy:= W.ShowWindow(W.GetActiveWindow(),3);
     IF (r = 0) & (g = 0) & (b = 0) THEN r:= 0; g:= 0; b:= 16 END;
     InitBackColor(bkColor,r,g,b);
     ClearMaxWindow(bkColor);
     SetForeColor(64,64,64);
     Line(4,0,4,650,DRAW);
     SetForeColor(200,200,200);
     Bar(1,0,3,650,DRAW)
    END InitiateColorPlane;


  PROCEDURE LineTo* ( x1, y1, mode : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Punkt P(x1|y1), zu dem die Linie gezeichnet (gelscht)     |
    |            werden soll, mode (Modus: Zeichnen/Lschen)                |
    | Leistung:  Ermitteln der aktuellen Grafikcursorposition, Linie zum    |
    |            gegebenen Punkt zeichnen, Grafikcursorposition anpassen.   |
    *-----------------------------------------------------------------------*)

    VAR x, y : INTEGER;

    BEGIN
      x:= WhereX(); y:= WhereY();
      Line(x,y,x1,y1,mode);
      GotoXY(x1,y1)
    END LineTo;


  PROCEDURE MoveMouseRel* ( dx, dy : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: relative x- und y-Koordinate fr die Verschiebung des      |
    |            Mauszeigers                                                |
    | Leistung:  Der Mauszeiger wird auf die bernommenen Koordinaten ge-   |
    |-----------------------------------------------------------------------*
    | Beispiele: MoveMouseRel(800,600) setzt den Mauszeiger in die rechte   |
    |            untere Ecke des POW-Grafikbildschirms, falls die Standard- |
    |            auflsung 800 x 600 Pixel eingestellt ist.                |
    |            Die beispielsweise im Quelltext des gleichen Moduls fol-   |
    |            gende Anweisung MoveMouseRel(-90,-50) setzt die Spitze     |
    |            des Mauszeiger 90 Pixel links der rechten (x-Koordinate),  |
    |            50 Pixel oberhalb der unteren (y-Koordinate) Begrenzung    |
    |            des sichtbaren Bildschirmbereichs.                         |
    *-----------------------------------------------------------------------*)

    BEGIN
      W.mouse_event(255,dx,dy,43,0)
    END MoveMouseRel;


  PROCEDURE RoundReal* ( x : REAL ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: zu rundender Wert (DT: REAL)                               |
    | Referenz:  Funktionswert (gerundeter Wert, DT: INTEGER)               |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN Sy.VAL(INTEGER,x+0.5)
    END RoundReal;


  PROCEDURE RoundLongReal* ( x : LONGREAL ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: zu rundender Wert (DT: LONGREAL)                           |
    | Referenz:  Funktionswert (gerundeter Wert, DT: INTEGER)               |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN Sy.VAL(INTEGER,x+0.5)
    END RoundLongReal;


  PROCEDURE CalculatePoint* ( VAR p : tPoint; xM, yM : INTEGER; r, w : REAL );
   (*-----------------------------------------------------------------------*
    | bernahme: Mittelpunkkoordinaten und Radius des Kreises, Winkel       |
    | Referenz:  Koordinaten des berechneten Punktes auf der 'Kreislinie'   |
    | Leistung:  Errechnen der Koordinaten eines Punktes der Kreislinie     |
    |            mittels trigonometrischer Funktionen.                      |
    | Hinweis:   Es gibt mathematisch 'schnere' Lsungen. Fr diese Demon- |
    |            stration ist die verwendete ausreichend.                   |
    |-----------------------------------------------------------------------|
    | Hinweise zu den Standardprozeduren ENTIER(x) und SHORT(x):            |
    |                                                                       |
    |            ENTIER(x): Argumenttyp REAL,    Ergebnistyp LONGINT        |
    |            SHORT(x):  Argumenttyp LONGINT, Ergebnistyp INTEGER        |
    |                                                                       |
    *-----------------------------------------------------------------------*)
  
    BEGIN
      p.x:= RoundReal(xM + r * F.Cos(w));
      p.y:= RoundReal(yM + r * F.Sin(w))
      (* ODER:                                                              *)
      (*       x:= SHORT(ENTIER(r * F.Cos(alpha) + xOffset));               *)
      (*       y:= SHORT(ENTIER(r * F.Sin(alpha) + yOffset))                *)
    END CalculatePoint;


  PROCEDURE CalculateSqrPoints* ( VAR p : tSqrPoints;
                                        lenght, x, y : INTEGER; d : REAL );
   (*-----------------------------------------------------------------------*
    | bernahme: Kantenlnge des Quadrats, x- und y-Koordinate der linken   |
    |            unteren Ecke und Winkel (Schrgstellung)                   |
    | Referenz:  Record-Variable sqrPoints, die die Koordinaten der berech- |
    |            neten Eckpunkte eines Quadrats enthlt                     |
    | Leistung:  Berechnen der Eckpunkte eines gegebenenfalls schrgstehen- |
    |            den Quadrats.                                              |
    *-----------------------------------------------------------------------*)

    VAR dr, f1, f2, f3, f4, pi, r : REAL;

    BEGIN
      pi:= SHORT(F.Pi);
      dr:= d/180 * pi;
      f1:= 1/4;  f2:= 3/4;  f3:= 5/4;  f4:= 7/4;
      r:= F.Sqrt(lenght*lenght/4);
      p.x1:= RoundReal(x+r*F.Cos(pi*f1+dr));
      p.y1:= RoundReal(y+r*F.Sin(pi*f1+dr));
      p.x2:= RoundReal(x+r*F.Cos(pi*f2+dr));
      p.y2:= RoundReal(y+r*F.Sin(pi*f2+dr));
      p.x3:= RoundReal(x+r*F.Cos(pi*f3+dr));
      p.y3:= RoundReal(y+r*F.Sin(pi*f3+dr));
      p.x4:= RoundReal(x+r*F.Cos(pi*f4+dr));
      p.y4:= RoundReal(y+r*F.Sin(pi*f4+dr))
    END CalculateSqrPoints;


  PROCEDURE DisplaceSquare* ( p : tSqrPoints; mode : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Record-Variable sqrPoints, die die Koordinaten der berech- |
    |            neten Eckpunkte eines Quadrats enthlt und Zeichenmodus    |
    | Leistung:  Zeichnen oder Lschen eines gegebenenfalls schrgstehenden |
    |            Quadrats.                                                  |
    *-----------------------------------------------------------------------*)

    BEGIN
      Line(p.x1,p.y1,p.x2,p.y2,mode);
      Line(p.x2,p.y2,p.x3,p.y3,mode);
      Line(p.x3,p.y3,p.x4,p.y4,mode);
      Line(p.x4,p.y4,p.x1,p.y1,mode)
    END DisplaceSquare;


  PROCEDURE Circle* ( xM, yM : INTEGER; r : REAL );
   (*-----------------------------------------------------------------------*                                                                      
    | bernahme: x- und y-Koordinate des Kreismittelpunkts (xM, yM) und der |
    |            Radius (r)                                                 |
    | Leistung:  Initialisieren des Winkel w mit 0. Die Schleife mit Ein-   |
    |            trittsbedingung wird solange durchlaufen bis der Winkel    |
    |            nherungsweise den Wert 2 * Pi erreicht hat.               |
    |            Im Schleifenkrper wird die Berechnung der Punktkoordina-  |
    |            ten angewiesen und der jeweilige Punkt gesetzt.            |
    |            Die Punkte ergeben, wenn sie alle dargestellt sind einen   |
    |            Vollkreis.                                                 |
    *-----------------------------------------------------------------------*)

    VAR w : REAL;
        p : tPoint;

    BEGIN
      w:= 0;
      WHILE w < 2 * F.Pi DO
        w:= w + 1/r;
        CalculatePoint(p,xM,yM,r,w);
        C.Dot(p.x,p.y,C.DRAW)
      END
    END Circle;


  PROCEDURE Polygone* ( modus, n, xM, yM, r : INTEGER );
   (*-----------------------------------------------------------------------*                                                                      
    | bernahme: Zeichenmodus, Anzahl der Ecken (n), x- und y-Koordinate    |
    |            des Kreismittelpunkts (xM, yM) und 'Radius' (r)            |
    | Leistung:  Initialisieren des Winkel w mit 0. Die Schleife mit Ein-   |
    |            trittsbedingung wird solange durchlaufen bis der Winkel    |
    |            nherungsweise den Wert 2 * Pi erreicht hat.               |
    |            Im Schleifenkrper wird die Berechnung der Punktkoordina-  |
    |            ten angewiesen. Die errechneten Werte werden in den Kompo- |
    |            nenten einer Reihung zwischengespeichert, da die Prozedur  |
    |            LineTo(x,y,modus) in Oberon nicht zur Verfgung steht und  |
    |            die Standardprozedur Line(x1,y1,x2,y2,modus) des Color-    |
    |            Plane-Moduls genutzt werden muss.                          |
    |            Bei diesem Algorithmus wird ein Polygon durch einen ge-    |
    |            schlossenen Linienzug dargestellt.                         |
    |-----------------------------------------------------------------------|
    | Hinweis:   Im User-Modul ChSTools ist LineTo-Anweisung implementiert, |
    |            die die Standardfunktionsprozeduren WhereX() und WhereY()  |
    |            des ColorPlane-Moduls nutzt.                               |
    *-----------------------------------------------------------------------*)

    VAR p    : ARRAY 3 OF tPoint;
        mode : INTEGER;
        w    : REAL;
        pkt  : tPoint;

    BEGIN
      w:= 0;
      WHILE w < 2 * F.Pi DO
        CalculatePoint(pkt,xM,yM,r,w);         (* Ersatz fr LineTo(x,y,modus) *)
        p[1].x:= pkt.x; p[1].y:= pkt.y;
        w:= w + SHORT(2*F.Pi)/n;
        CalculatePoint(pkt,xM,yM,r,w);
        p[2].x:= pkt.x; p[2].y:= pkt.y;
        C.Line(p[1].x,p[1].y,p[2].x,p[2].y,modus);
        p[1].x:= p[2].x; p[1].y:= p[2].y
      END
    END Polygone;


  PROCEDURE CircularArea* ( xM, yM : INTEGER; r : REAL );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate des Kreismittelpunkts (xM, yM) und der |
    |            Radius (r)                                                 |
    | Leistung:  Darstellen einer einfarbigen Kreisflche. Als Farbe wird   |
    |            die aktuelle Vordergrundfarbe genutzt. Die Kreisflche     |
    |            wird durch Linien verschiedener Lnge dargestellt.         |
    *-----------------------------------------------------------------------*)

    VAR gr, w : REAL;
        p1,p2 : tPoint;

    BEGIN
      gr:= SHORT(3*F.Pi/2);
      w:=  SHORT(F.Pi/2);
      WHILE w < gr DO
        w:= w + 1/r;
        CalculatePoint(p1,xM,yM,r,w);
        p2.x:= xM*2-p1.x; p2.y:= p1.y;
        Line(p1.x,p1.y,p2.x,p2.y,DRAW)
      END
    END CircularArea;


  PROCEDURE CircularAreaWithBorder* ( xM, yM : INTEGER; r : REAL;
                                         borderColor, floodColor : tColor );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate des Kreismittelpunkts (xM, yM), der    |
    |            Radius (r) sowie die Rand- und die Fllfarbe (RGB-Werte)   |
    | Leistung:  Darstellen einer umrandeten einfarbigen Kreisflche, deren |
    |            Randfarbe nicht der Fllfarbe entspricht. Der Kreis wird   |
    |            durch Punkte und die Kreisflche durch Linien verschiede-  |
    |            ner Lnge dargestellt.                                     |
    *-----------------------------------------------------------------------*)

    VAR gr, w : REAL;
        p1,p2 : tPoint;

    BEGIN
      gr:= SHORT(3*F.Pi/2);
      w:=  SHORT(F.Pi/2);
      WHILE w < gr DO
        w:= w + 1/r;
        CalculatePoint(p1,xM,yM,r,w);
        p2.x:= xM*2-p1.x; p2.y:= p1.y;
        SetForeColorRec(borderColor);
        Dot(p1.x,p1.y,DRAW); Dot(p2.x,p2.y,DRAW);
        SetForeColorRec(floodColor);
        INC(p1.x); DEC(p2.x);
        IF p1.x < p2.x THEN
          Line(p1.x+1,p1.y,p2.x-1,p2.y,DRAW)
        END
      END
    END CircularAreaWithBorder;


  PROCEDURE FillPoly* ( x, y : INTEGER; borderColor, floodColor : tColor );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate des Ausgangspunktes fr das Fllen und |
    |            RGB-Werte der Rand- sowie der Fllfarbe                    |
    | Leistung:  Abarbeiten eines rekursiven Fllalgorithmus'. Der Grafik-  |
    |            cursor wird an den bergebenen Startpunkt, die Mitte des   |
    |            bereits gezeichneten Polygons gesetzt und eine Farbprobe   |
    |            der aktuellen Hintergrundfarbe genommen. Stimmen die RGB-  |
    |            Werte nicht mit der Zeichenfarbe berein, wird das Polygon |
    |            mit der aktuellen Zeichenfarbe gefllt.                    |
    | Hinweise:  - Die Unterprozedur FillArea(...) ist die Prozedur, in der |
    |              der rekursive Algorithmus implementiert ist. Diese Pro-  |
    |              zedur enthlt nur noch zwei formale Parameter, das spart |
    |              Speicherpltze auf dem Rekursionsstack.                  |
    |            - Groe Flchen mssen in kleine Teilflchen zerlegt wer-  |
    |              den, damit kein Stackberlauf auftritt (stack overflow   |
    |              error).                                                  |
    |            - V o r  dem Aufruf dieser Prozedur sollte die Anweisung   |
    |              Ch.SetScreenUpdate(FALSE) und  n a c h  dem Abarbeiten   |
    |              dieser Prozedur die Anweisung Ch.SetScreenUpdate(FALSE)  |
    |              im rufenden Programm stehen, um den Fllprozess auf dem  |
    |              aktuell jeweils nicht sichtbaren Bildschirm durchzufh-  |
    |              ren. 'Ch' steht im qualifizierten Prozeduraufruf fr     |
    |              ChSTools (IMPORT ..., Ch.CnSTools, ...;)                 |
    *-----------------------------------------------------------------------*)

    CONST screenX = 850;
          screenY = 650;

    PROCEDURE FillArea ( x, y : INTEGER );

      VAR r, g, b : INTEGER;

      BEGIN
        GetDot(x,y,r,g,b);
        IF ((r # borderColor.r) OR (g # borderColor.g) OR (b # borderColor.b))
            & ((r # floodColor.r) OR (g # floodColor.g) OR (b # floodColor.b))
        THEN
          Dot(x,y,DRAW);
          IF x > 0 THEN FillArea(x-1,y) END; 
          IF x < screenX THEN FillArea(x+1,y) END;
          IF y > 0 THEN FillArea(x,y-1) END;
          IF y < screenY THEN FillArea(x,y+1) END
        END
      END FillArea;

    BEGIN  (* FillPoly *)
      FillArea(x,y)
    END FillPoly;  


  PROCEDURE StartWatch* ( VAR starttime : tTime );
   (*-----------------------------------------------------------------------*
    | Referenz:  Funktionswert (Startzeit, ARRAY 3 OF INTEGER)              |
    *-----------------------------------------------------------------------*)

    BEGIN
      U.GetTime(starttime[0],starttime[1],starttime[2])
    END StartWatch;


  PROCEDURE Timing ( t1, t2 : tTime; VAR timeStr : tTimeStr );
   (*-----------------------------------------------------------------------*
    | bernahme: Startzeit (t1) und Stoppzeit (t2)                          |
    | Referenz:  gemessene Zeitdifferenz als Zeichenkette                   |
    | Leistung:  Errechnen der Zeitdifferenz zwischen Start- und Stoppzeit  |
    |            und Wandeln des Ergebnisses in eine Zeichenkette der Lnge |
    |            36, weil im Grafikmodus keine Zahlen vom Typ INTEGER oder  |
    |            REAL ausgegeben werden knnen.                             |
    *-----------------------------------------------------------------------*)

    VAR hStr, mStr, sStr : ARRAY 3 OF CHAR;
        tempStr          : ARRAY 8 OF CHAR;
        
    BEGIN
      timeStr:= '';
      t1[0]:= t2[0]-t1[0];                  (* Sekunden *)
      IF t1[0] < 0 THEN 
        t1[0]:= t1[0]+60;
        DEC(t2[1],1)
      END;
      S.Str(t1[0],sStr);
      tempStr:= ' sec ';
      IF t1[0] > 0 THEN 
        S.Insert(sStr,tempStr,1);
        S.Insert(tempStr,timeStr,1)
      ELSE sStr:= '' END;
      t1[1]:= t2[1]-t1[1];                  (* Minuten  *)
      IF t1[1] < 0 THEN 
        t1[1]:= t1[1]+60;
        DEC(t2[2])
      END;
      S.Str(t1[1],mStr);
      tempStr:= ' min ';
      IF t1[1] > 0 THEN 
        S.Insert(mStr,tempStr,1);
        S.Insert(tempStr,timeStr,1)
      ELSE mStr:= '' END;
      S.Str(t1[2],hStr);
      tempStr:= ' h ';
      IF t1[2] > 0 THEN S.Insert(hStr,tempStr,S.Length(hStr));
        ELSE mStr:= ''
      END
    END Timing;


  PROCEDURE GetTime* ( VAR timeStr : tTimeStr; starttime : tTime );
   (*-----------------------------------------------------------------------*
    | bernahme: Startzeit                                                  |
    | Referenz:  gemessene Zeitdifferenz als Zeichenkette                   |
    | Leistung:  siehe UP Timing                                            |
    *-----------------------------------------------------------------------*)

    VAR i         : INTEGER;
        stopptime : tTime;

    BEGIN
      U.GetTime(stopptime[0],stopptime[1],stopptime[2]);
      Timing(starttime,stopptime,timeStr)
    END GetTime;


  PROCEDURE Repetition* ( colorNb, x, y, dTime : INTEGER ) : CHAR;
   (*-----------------------------------------------------------------------*
    | bernahme: Farbnummer oder -name (V.colorNb), x- und y-Koordinate,    |
    |            fr die Ausgabe der Zeichenkette, Wartezeit                |
    | Referenz:  Funktionswert, ein Zeichen (DT: CHAR)                      |
    | Leistung:  Positionierte farbige Ausgabe einer Zeichenkette, Lschen  |
    |            des Tastaturpuffers, Zeichen einer kurzen Linie als Gra-   |
    |            fikcursor, eine bestimmte Zeit warten, Linie mit Hinter-   |
    |            grundfarbe bermalen und wieder eine bestimmte Zeit warten.|
    |            Dieser Vorgang wird wiederholt, bis eine Taste auf der     |
    |            Tastatur bettigt wird. Danach wird die Zeichenkette ge-   |
    |            lscht und das eingegebene Zeichen, falls es sich um einen |
    |            Kleinbuchstaben handelt, als Grobuchstabe an das rufende  |
    |            Programm 'zurgkgegeben' (Funktionswert als Referenzpara-  |
    |            meter). Wurde eine beliebige Taste, die ein druckbares     |
    |            Zeichen reprsentiert, auf der Tastatur bettigt, soll das |
    |            vom auswertenden Programmteil als 'J' (Ja) interpretiert   |
    |            werden.                                                    |
    *-----------------------------------------------------------------------*)

   VAR key : CHAR;
   
   BEGIN
     SetForeColorNb(colorNb);
     WriteStrXY(x,y,'Wird eine Wiederholung gewnscht (J/N) J');
     FlushKeyBuffer();
     REPEAT 
       SetForeColorNb(White); Line(x+312,y-16,x+320,y-16,DRAW);
       Delay_ms(256);
       SetForeColorNb(Black); Line(x+312,y-16,x+320,y-16,DRAW);
       Delay_ms(256)
     UNTIL KeyPressed();
     key:= ReadKey();
     Bar(100,466,690,36,ERASE);
     RETURN CAP(key)                     (* CAP(x) wandelt einen Kleinbuch- *)
                                         (* staben in einen Grobuchstaben  *)
   END Repetition;


  PROCEDURE Title* ( colorNb, y, dTime : INTEGER; bkColor : tColor; 
                                            text : tStr75; flag : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme: RGB-Werte fr die Hintergrundfarbe (Wertparameter)         |
    | Leistung:  Positionierte farbige Ausgabe einer unterstrichenen ber-  |
    |            schrift. 'flag' TRUE fr Aufruf der Prozedur Enter dieses  |
    |            Moduls mit aktuellen Parametern.                           |
    *-----------------------------------------------------------------------*)
   
    VAR xOffset : INTEGER;
     
    BEGIN
      SetForeColorNb(colorNb);
      xOffset:= (784 - TextWidth(text)) DIV 2;
      WriteStrXY(xOffset,y,text);
      Line(xOffset-10,y-18,793-xOffset,y-18,DRAW);
      IF flag THEN
        Enter(660,35,V.Cyan,bkColor,FALSE,dTime)
      END
    END Title;


  PROCEDURE WriteChar* ( zeichen : CHAR );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position der Zeichenkette     |
    | Leistung:  Positionierte Ausgabe eines Zeichens (eigentlich einer     |
    |            kurzen Zeichenkette, die aus zwei Zeichen besteht.         |
    *-----------------------------------------------------------------------*)

    VAR string : ARRAY 3 OF CHAR;

    BEGIN
      string:= '';
      S.AppendChar(string,zeichen);
      C.WriteStr(string)
    END WriteChar;


  PROCEDURE WriteCharXY* ( x, y : INTEGER; zeichen : CHAR );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinaten fr die Position des Zeichens         |
    | Leistung:  Positionierte Ausgabe eines Zeichens (eigentlich einer     |
    |            kurzen Zeichenkette, die aus zwei Zeichen besteht.         |
    *-----------------------------------------------------------------------*)

    VAR string : ARRAY 3 OF CHAR;

    BEGIN
      string:= '';
      S.AppendChar(string,zeichen);
      C.WriteStr(string)
    END WriteCharXY;


  PROCEDURE HeadLine* ( message : tStr75 ; year : tYearStr );
   (*-----------------------------------------------------------------------*
    | bernahme: Variabler Teil des in der Kopfzeile des Grafikbildschirms  |
    |            anzuzeigenden Textes und Jahreszahl fr das Copyright ()  |
    | Leistung:  Ausgabe der gewnschten Mitteilung in der blauen Kopfzeile |
    |            des Grafikbildschirms. Zum Beispiel: Aussage zum bearbei-  |
    |            teten Thema, Copyrightzeichen, Jahreszahl und Name oder    |
    |            Alias des Programmierers.                                  |
    *-----------------------------------------------------------------------*)

    VAR dummy       : BOOLEAN;
        messageHdLn : tStr75;

    BEGIN
      messageHdLn:= '        Chamberlain-Soft JENA';
      S.Insert(year,messageHdLn,9);
      S.Insert(message,messageHdLn,4);
      dummy:= W.SetWindowTextA(W.GetActiveWindow(),Sy.ADR(messageHdLn))
    END HeadLine;


  PROCEDURE ClosingSequence* ( bkColor, color : tColor; year : tYearStr;
                                           dTime : INTEGER; flag : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme: Hintergrundfarbe bkColor fr das groe und color fr das   |
    |            kleine Fenster (DT: tColor, RECORD RGB-Werten), auszuge-   |
    |            bende Jahreszahl und Wartezeitkonstante;                   |
    |            der Wert der booleschen Variable flag hat Einfluss auf den |
    |            auszugebenden Text                                         |
    | Leistung:  Lschen des groen Grafikfenster (5,0,850,650), dabei      |
    |            bleibt das linke Rahmenelement unangetastet. Danach wird   |
    |            ein kleines Fenster mit einem dnnen grnen Rahmen verse-  |
    |            hen und mit der bergebenen Farbe ausmalen. Dann wird ein  |
    |            konstanter Text ausgegeben, in dem die bergebenen Jahres- |
    |            zahl eingefgt wurde.                                      |
    *-----------------------------------------------------------------------*)

    CONST size = 384; x = 195; y = 472;

    VAR   i     : INTEGER;
          lyStr : tStr75;

    BEGIN
      ClearMaxWindow(bkColor);
      Delay_ms(dTime*5);
      ClearAreaFast(x,y,x+size,y-size,color);
      SetBackColorRec(color);
      SetForeColor(55,155,80);
      Box(x,y,x+size,y-size,DRAW);
      SetForeColor(255,255,230);
      WriteStrXY(x+74,y-142,'M A N F R E D  K  M M E R E R');
      SetForeColor(225,235,140);
      IF flag THEN
        WriteStrXY(x+81,y-102,'IDEE UND IMPLEMENTIERUNG VON')
      ELSE
        WriteStrXY(x+117,y-102,'IMPLEMENTIERUNG VON')
      END;
      WriteStrXY(x+152,y-194,'');
      lyStr:= ''; S.Append(lyStr,year);
      FOR i:= 1 TO 3 DO
        S.InsertChar(' ',lyStr,i*2)
      END;
      WriteStrXY(x+176,y-194,lyStr);
      WriteStrXY(x+72,y-228,'C H A M B E R L A I N - S O F T');
      WriteStrXY(x+152,y-250,'J  E  N  A');
      Delay(dTime*25);
      Enter(660,45,Cyan,bkColor,FALSE,dTime)
    END ClosingSequence;


  PROCEDURE Information* ( bkColor : tColor; year : tYearStr;
                                                        dx, dy : INTEGER);
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information zur Homepage des Carl-Zeiss-Gym-  |
    |           nasiums nach Abschluss des Programms.                       |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';

    VAR   ma : ARRAY 91 OF CHAR;
          mb : ARRAY 30 OF CHAR;

    BEGIN
      mb:= '       Chamberlain-Soft';
      S.Insert(year,mb,7);
      MoveMouseRel(dx,dy);
      ClearMaxWindow(bkColor);
      ma:= m1; S.Append(ma,m2);
      dummy:= W.MessageBoxA(W.GetActiveWindow(),Sy.ADR(ma),Sy.ADR(mb),40H);
      Delay_ms(250)
    END Information;


  PROCEDURE InformationChS* ( bkColor : tColor; year : tYearStr;
                                                         dx, dy : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information zur Homepage von Manfred Kmmerer |
    |           (alias Chamberlain-Soft) nach Abschluss des Programms.      |
    *-----------------------------------------------------------------------*)

    CONST mc = ' http://www.chamberlain-soft.de     ';

    VAR   mv : ARRAY 30 OF CHAR;

    BEGIN
      mv:= '       Chamberlain-Soft JENA';
      S.Insert(year,mv,7);
      MoveMouseRel(800,600);
      MoveMouseRel(dx,dy);
      ClearMaxWindow(bkColor);
      dummy:= W.MessageBoxA(W.GetActiveWindow(),Sy.ADR(mc),Sy.ADR(mv),40H);
      Delay_ms(250)
    END InformationChS;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   vgaColors:= V.vgaColors
 END ChSTools.

 (* --------   1998/2001 Manfred Kmmerer, Chamberlain-Soft JENA  -------- *)

