 
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32-Bit)         Dateiname: COLORVAL.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: Dezember 2000 Version: V 1.2                |
  |-------------------------------------------------------------------------|
  | Thema:    Grafikprogrammierung in POW! (OBERON-2)                       |
  |           T e s t p r o g r a m m  zum MODUL VGAColor                   |
  |-------------------------------------------------------------------------|
  | Leistung: Das Programm zeigt die Belegung der Farbtabelle und die       |
  |           Farbauflsung des eingestellten Grafikmodus an. Farbmodi      |
  |           unter 32768 Farben (Highcolor, 15 Bit) sind fr POW! nicht    |
  |           geeignet!                                                     |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Programm enthlt eine Arbeitsanweisung, die einen Eintrag |
  |           im Programm VGAColor, Dateiname VGACOLOR.MOD, betrifft.       |
  |           Dieses Programm ist fr den Grafikmodus 800 x 600 Pixel      |
  |           konzipiert (Ausnutzung der gesamten Bildschirmflche).        |
  |-------------------------------------------------------------------------|
  |            2000/01  Manfred Kmmerer, Chamberlain-Soft JENA            |
  *-------------------------------------------------------------------------*)

 MODULE ColorVal;

  IMPORT Ch:= ChSTools, S:= Strings, V:= VGAColors, W:= Windows;

  TYPE   tStrg4 = ARRAY 4 OF CHAR;

  CONST  dTime     = 125;

         message   = 'VGA-Standardfarbtabelle';
         year      = '2001';


  VAR    dummy  : LONGINT;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Farbbalken ( VAR gruen : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: G-Wert (RGB-Werte) der Farbe Grn. Er wird zur Bestimmung   |
    |           der Farbauflsung genutzt.                                  |
    | Leistung: 16 farbige Balken (je 8 Punkte hoch, Standard-VGA-Farben)   |
    |           am Bildschirm untereinander ausgeben. Dabei wird geprft,   |
    |           ob 'i' die maximale Anzahl vorhandener Farben berschreitet.|
    *-----------------------------------------------------------------------*)

   VAR i, k, dummy : INTEGER;
     
   BEGIN
     FOR i:= 0 TO 15 DO
       IF i IN V.colorNumbers THEN
         Ch.SetForeColorNb(i)    (* Zur Sicherheit bei anderen Anwendungen: *)
       ELSE                      (* wenn 'i' grer als 15 ist, was dann?   *)
         Ch.SetForeColor(0,96,96) (*  --->  V.ColorNumbers, Bereich {0..15 } *)
       END; (* If *)
       Ch.Bar(302,396-i*16,418,407-i*16,Ch.DRAW);
     END; (* For i .. *)  
     Ch.SetForeColor(V.vgaColors[V.LightGray].r,V.vgaColors[V.LightGray].g,
                                                  V.vgaColors[V.LightGray].b);
     Ch.Box(302,395,418,407,Ch.DRAW);
     Ch.GetDot(310,406-8*16,dummy,gruen,dummy)
   END Farbbalken;


  PROCEDURE Formatieren ( Wert : INTEGER; VAR Strg : Ch.tShortStr );
   (*-----------------------------------------------------------------------*
    | bernahme: Zu formatierender Wert und zu belegender Referenzparameter |
    | Referenz:  Wert als formatierte Zeichenkette (DT: ARRAY 4 OF CHAR)    |
    | Leistung:  Der bergebene Wert (positive ganze Zahl) wird in eine     |
    |            Zeichenkette gewandelt. Danach werden falls erforderlich   |
    |            Leerzeichen vorangestellt (Dezimalformatierung).           |
    *-----------------------------------------------------------------------*)

   BEGIN
     S.Str(Wert,Strg);
     IF Wert < 10 THEN 
       S.Insert('  ',Strg,0) 
     ELSIF Wert < 100 THEN 
       S.Insert(' ',Strg,0) 
     END
   END Formatieren;


  PROCEDURE Ermitteln_und_Beschriften;
   (*-----------------------------------------------------------------------*
    | Leistung: Positionierte farbige Ausgabe von drei Zeichenketten, Aus-  |
    |           gabe der Farbnummer und der RGB-Werte vor bzw. hinter dem   |
    |           Farbbalken (VGA-Standard-Farben).                           |
    *-----------------------------------------------------------------------*)

   CONST y = 410;
     
   VAR   Strg       : Ch.tShortStr;
         i, B, G, R : INTEGER;

   BEGIN
     Ch.SetForeColorNb(Ch.LightRed);
     Ch.GotoXY(448,y+20); Ch.WriteStr('R(ed)');
     Ch.SetForeColorNb(Ch.LightGreen);
     Ch.GotoXY(507,y+20); Ch.WriteStr('G(reen)');
     Ch.SetForeColorNb(Ch.LightBlue);
     Ch.GotoXY(581,y+20); Ch.WriteStr('B(lue)');
     Ch.SetForeColorNb(Ch.White);
     FOR i:= 0 TO 15 DO
       S.Str(i,Strg);
       Formatieren(i,Strg);
       Ch.WriteShortStrXY(147,y-i*16,Strg);
       Strg:= '';
       S.Append(Strg,V.colorName[i]);
       Ch.WriteShortStrXY(184,y-i*16,Strg);
       Ch.GetDot(310,y-10-i*16,R,G,B);
       Formatieren(R,Strg);
       Ch.WriteShortStrXY(450,y-i*16,Strg);
       Formatieren(G,Strg);
       Ch.WriteShortStrXY(520,y-i*16,Strg);
       Formatieren(B,Strg);
       Ch.WriteShortStrXY(590,y-i*16,Strg)
     END
   END Ermitteln_und_Beschriften;


  PROCEDURE Aufloesung ( Gruen : INTEGER; BkColor : V.tVGACol );
   (*-----------------------------------------------------------------------*
    | bernahme: G-Wert der Farbe Grn, RGB-Werte der Hintergrundfarbe      |
    | Leistung:  Vergleich des G-Wertes mit dem von MS-WINDOWS ermittelten |
    |            Wert der naheliegendsten Farbe (nearest color) der inter-  |
    |            nen Farbtabelle und Ausgabe der entsprechenden Meldung und |
    |            Anweisung am Bildschirm.                                   |
    |            Sind weniger als 15 Bit Farbtiefe eingestellt, wird eine   |
    |            Warnung ausgegeben.                                        |
    | Hinweise:  Geditterte Farben sehen schlimm aus. Der Hintergrund von   |
    |            Zeichenketten wird ungedittert in einer den Standard-VGA-  |
    |            Farben entsprechenden Farbe ausgegeben!                    |
    |            Natrlich kann die Abfrage  v i e l  professioneller er-   |
    |            folgen. Dafr mssten 'WINDOWS-Interna' abgefragt werden.  |
    |            Wer kennt sich da aus? Lohnt sich der ntige Aufwand?      |
    *-----------------------------------------------------------------------*) 

   PROCEDURE Eintrag ( ziffer : Ch.tShortStr );
     BEGIN
       Ch.SetForeColorNb(Ch.White);
       Ch.WriteShortStrXY(478,62,ziffer)
     END Eintrag;


   BEGIN
     Ch.SetForeColorNb(Ch.Cyan);
     IF (Gruen = 80) OR (Gruen = 84) OR (Gruen = 85) THEN
       Ch.WriteStrXY(144,80,'Tragen Sie bitte im Programm VGAColor, ');
       Ch.WriteStr('Dateiname VGACOLOR.MOD,');
       Ch.WriteStrXY(262,62,'unter CONST modus den Wert   ein!');
     ELSE
       Ch.SetForeColorNb(Ch.Yellow);
       Ch.WriteStrXY(148,100,
                "Dieser Farbmodus ist fr den 'POW!-Graphikmodus' ungeeignet")
     END;
     Ch.SetForeColorNb(Ch.Yellow);
     IF Gruen = 80 THEN
       Ch.WriteStrXY(222,110,'H I G H C O L O R  (32768 Farben, 15 Bit)');
       Eintrag('3')
     ELSIF Gruen = 84 THEN
       Ch.WriteStrXY(222,110,'H I G H C O L O R  (65536 Farben, 16 Bit)');
       Eintrag('2')
     ELSIF Gruen = 85 THEN
       Ch.WriteStrXY(158,110,
                 'T R U E C O L O R  (16777216 Farben, 24 Bit, ggf. 32 Bit)');
       Eintrag('1')
     END;
     Ch.MoveMouseRel(-78,-50);
     Ch.Enter(660,45,Ch.Cyan,Ch.bkColor,FALSE,dTime)
   END Aufloesung;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)
 
 PROCEDURE ProgMain*();

   VAR gruen     : INTEGER;
       backColor : V.tVGACol;
     
   BEGIN             
     Ch.MoveMouseRel(800,600);
     Ch.Title(Ch.White,480,dTime,Ch.bkColor,
                     'Ermitteln der RGB-Werte der z. Z. aktiven VGA-Palette',FALSE);
     (* Ch.Line(170,462,614,462,Ch.DRAW); *)
     Farbbalken(gruen);
     IF gruen # 128 THEN Ermitteln_und_Beschriften END;
     Aufloesung(gruen,backColor);
     Ch.Close
   END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   Ch.InitiateColorPlane(Ch.bkColor,0,0,32);   (* Malkasten ffnen und gro- *)
                                               (* es Fenster einstellen    *)
   Ch.HeadLine(message,year);                  (* Eintrag in die Bildleiste *)
   Ch.Delay_ms(500)
 END ColorVal.

 (* --------   2000/01  Manfred Kmmerer, Chamberlain-Soft JENA  --------- *)

