
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)          Dateiname: EINSSTR.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: Mai 2001 Version: V 1.0                     |
  |-------------------------------------------------------------------------|
  | Thema:    Eingabestream, Ausgabe  (In- und Out-Modul)                   |
  |           Vereinbarung eines Datentyps                                  |
  |-------------------------------------------------------------------------|
  | Importe:  Das Modul EinsStr importiert die von POW! bereitgestellten    |
  |           Module In und Out (Oakwood Guidelines). Damit werden Proze-   |
  |           duren fr Ein- und Ausgabeanweisungen bereitgestellt.         |
  | Leistung: Einlesen von Zahlen und einer Zeichenkette, die vom Anwender  |
  |           mittels Tastatur eingegebenen werden. Verarbeitung der Zahlen |
  |           (Addition) und Ausgabe des Ergebnisses am Bildschirm.         |
  |           Speichern der Eingaben (inputstream) in einer Textdatei.      |
  |           Einlesen der gespeicherten Daten, erneute Verarbeitung der    |
  |           beiden Zahlen (Subtraktion) und Ausgabe des Ergebnisses am    |
  |           Bildschirm. Die Zeichenkette wird ebenfalls ausgegeben.       |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE EinsStr;
 
  IMPORT In, Out;                    (* Imort der Standardmodule In und Out *)

  TYPE   tStr25 = ARRAY 26 OF CHAR;  (* Vereinbarung eines Datentyps,       *)
                                     (* mit tStr25 ist fr dieses Modul ein *)
                                     (* Datentyp vereinbart, dessen Variab- *)
                                     (* len 25 druckbare Zeichen aufnehmen  *)
                                     (* knnen.                             *)


 (* ---------------------  U N T E R P R O G R A M M ---------------------- *)


  PROCEDURE Eingabe ( VAR a, b : INTEGER; VAR zeichenkette : tStr25 );
   (*-----------------------------------------------------------------------*
    | Referenz: INTEGER-Variablen a und b sowie die Variable zeichenkette   |
    | Leistung: Ausgabe einer Arbeitsanweisung, Einlesen von zwei ganzen    |
    |           Zahlen und einer Zeichenkette aus maximal 25 Zeichen, die   |
    |           mittels Tastatur eingegeben wurden.                         |
    *-----------------------------------------------------------------------*)

    BEGIN
      Out.Ln;
      Out.String('  Bitte zwei ganze Zahlen und eine Zeichenkette (max. 25');
      Out.String(' Zeichen) jeweils'); Out.Ln;
      Out.String('  durch ein Leerzeichen getrennt eingeben und die Eingabe');
      Out.String(' durch Bettigen'); Out.Ln;
      Out.String('  der Enter-Taste abschlieen!'); Out.Ln;
      Out.Ln;
      Out.String('  Das Leerzeichen ist Trennzeichen und muss zwischen den');
      Out.String(' Eingaben stehen!'); Out.Ln;
      In.Prompt('Eingaben');            (* Beschriftung, die links neben    *)
                                        (* dem Eingabefenster erscheint     *)
      In.Int(a);
      In.Int(b);
      In.String(zeichenkette)
    END Eingabe;


  PROCEDURE BerechnenUndAusgeben ( a, b : INTEGER; zeichenkette : tStr25 );
   (*-----------------------------------------------------------------------*
    | bernahme: Werte der INTEGER-Variablen a und b sowie der Variablen    |
    |            zeichenkette als Kopie der Originalwerte                   |
    | Leistung:  Berechnen der Summe (a+b) und Ausgabe dieser, in eine Zei- |
    |            chenkette gekleidet, am Monitor.                           |
    |            Ausgabe der bernommenen Zeichenkette am Monitor.          |
    *-----------------------------------------------------------------------*)

    VAR  ergebnis : INTEGER;

    BEGIN
      ergebnis:= a + b;
      Out.Ln;
      Out.String('    Berechnen:     a + b = '); Out.Int(ergebnis,1); Out.Ln;
      Out.Ln;
      Out.String('    Zeichenkette: '); Out.String(zeichenkette); Out.Ln
    END BerechnenUndAusgeben;


  PROCEDURE Linie;
   (*-----------------------------------------------------------------------*
    | Leistung: Darstellen einer waagerechten Volllinie. Das Darstellen     |
    |           einer gepunktete Linie ist vorbereitet.                     |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      Out.Ln; Out.Char(' ');
      FOR i:= 1 TO 56 DO Out.Char('') END;
            (* '' = <Alt> + <0><1><7><5> auf dem Nummernblock der Tastatur *)
            (* '' = <Alt> + <0><1><6><8> auf dem Nummernblock der Tastatur *)
      Out.Ln
    END Linie;


  PROCEDURE Speichern;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Arbeitsanleitung, Warten auf die geforderte   |
    |           Eingabe mittels Tastatur, Aufruf des Unterprogramms Linie   |
    |           und Anzeige einer Besttigungsmeldung am Monitor.           |
    *-----------------------------------------------------------------------*)

    VAR dummy : CHAR;
    
    BEGIN
      Linie;
      Out.String('  Arbeitsanleitung zum Speichern des Eingabestroms:');
      Out.Ln;  Out.Ln;
      Out.String('    Men:   Pane                     (Mausklick)'); Out.Ln;
      Out.String('    Option: save input to file...    (Mausklick)'); Out.Ln;
      Out.String("    Datei:  'eins.txt'               ( <Enter> )"); Out.Ln;
      Out.Ln;
      Out.String('  Nach dem Speichern bitte ein Leerzeichen eingeben ');
      Out.Ln;
      Out.String('  und die Enter-Taste bettigen!'); Out.Ln;
      In.Prompt('Ohne Eingabe!');
      In.Char(dummy);
      Linie;
      Out.String('  Die Textdatei wurde angelegt und die Daten geschrieben.');
      Out.Ln
    END Speichern;


  PROCEDURE LadenVerarbeitenAusgeben;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Frage am Monitor, Einlesen einer alternativen |
    |           Antwort, die mittels Tastatur eingegeben wurde, ggf. Wand-  |
    |           lung des eingelesenen Buchstaben in einen Grobuchstaben.   |
    |           Falls das eingelesene Zeichen nach der Konvertierung der    |
    |           Grobuchstabe J ist, wird das Unterprogramm Laden aufgeru-  |
    |           fen, die Differenz der beiden aus der Textdatei eingelese-  |
    |           nen ganzen Zahlen gebildet, diese, in eine Zeichenkette ge- |
    |           setzt und ausgegeben. Anschlieend wird noch die ebenfalls  |
    |           aus der Textdatei eingelesene Zeichenkette am Monitor ange- |
    |           zeigt.                                                      |
    *-----------------------------------------------------------------------*)

    VAR zeichenkette : tStr25;
        zeichen      : CHAR;
        a, b, ergebn : INTEGER;

    PROCEDURE Laden;
     (**
      | Leistung: Aufruf des Unterprogramms Linie, Ausgabe einer Arbeitsan- |
      |           leitung, Einlesen von zwei INTEGER-Werten aus der Textda- |
      |           tei, Einlesen einer Zeichenkette, die maximal 25 Zeichen  |
      |           enthlt ebenfalls aus dieser Textdatei, erneuter Aufruf   |
      |           des UP Linie und Anzeige einer Besttigungsmeldung am Mo- |
      |           nitor.                                                    |
      **)

      BEGIN
        Linie;
        In.Prompt('Ohne Eingabe!');
        Out.String('  Arbeitsamleitung zum Lesen der Daten aus einer Datei:');
        Out.Ln;  Out.Ln;
        Out.String('    Men:   Pane                     (Mausklick)');
        Out.Ln;
        Out.String('    Option: read input from file...  (Mausklick)');
        Out.Ln;
        Out.String('    Pfad:   ...\quelltext            (Doppelklick)');
        Out.Ln;
        Out.String('    Datei:  eins.txt ffnen          (Doppelklick)');
        Out.Ln;
        In.Int(a);
        In.Int(b);
        In.String(zeichenkette);
        Linie;
        Out.String('  Der Inhalt der Textdatei wurde gelesen und den');
        Out.Ln;
        Out.String('  Variablen a, b und zeichenkette zugewiesen.'); Out.Ln;
      END Laden;

    BEGIN  (* Dritte *)
      Out.String('  Sollen die Daten wieder eingelesen werden?'); Out.Ln;
      In.Prompt('Daten lesen (J/N)');
      In.Char(zeichen);
      zeichen:= CAP(zeichen);  (* Kleinbuchstaben in Grobuchstaben wandeln *)
      IF zeichen # 'N' THEN    (* # = relationale Ungleichheit              *)
        Laden;
        ergebn:= a - b;
        Out.Ln;
        Out.String('    Berechnen:     a - b = '); Out.Int(ergebn,1); Out.Ln;
        Out.Ln;
        Out.String('    Zeichenkette: '); Out.String(zeichenkette); Out.Ln;
        Out.Ln
      END
    END LadenVerarbeitenAusgeben;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR a, b, ergebnis : INTEGER;
        zeichenkette   : tStr25;

    BEGIN
      Eingabe(a,b,zeichenkette);
      BerechnenUndAusgeben(a,b,zeichenkette);
      Speichern;
      LadenVerarbeitenAusgeben
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;                   (* Die Initialisierung des In- und Out-Moduls *)
   Out.Open                   (* kann weggelassen werden!                   *)
 END EinsStr.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

