
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)             Dateiname: VIER.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: Mai 2001 Version: V 1.0                     |
  |-------------------------------------------------------------------------|
  | Thema:    Eingabe, Ausgabe, grafische Grundelemente (ColorPlane-Modul)  |
  |-------------------------------------------------------------------------|
  | Importe:  Das Modul Vier importiert das von POW! bereitgestellte Modul  |
  |           ColorPlane. Es werden Prozeduren fr Ein- und Ausgabeanwei-   |
  |           sungen und Werkzeuge fr Grafik bereitgestellt.               |
  | Leistung: Einlesen von Zeichenketten (Text), die vom Anwender mittels   |
  |           Tastatur eingegeben werden.                                   |
  |           Farbige Ausgabe von Zeichenketten.                            |
  |           Z. T. Wandlung des Textes in Zahlen (Koordinaten), die fr    |
  |           grafische Grundelemente bentigt werden.                      |
  |-------------------------------------------------------------------------|
  | Hinweise: Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |-------------------------------------------------------------------------|
  |            2001  Manfred Kmmerer, Chamberlain-Soft JENA               |
  *-------------------------------------------------------------------------*)

 MODULE Vier;
 
  IMPORT C:= ColorPlane,       (* Import der Standardmodule ColorPlane und  *)
         F:= Float,            (* Standardmodul fr die Arbeit mit reelle   *)
                               (* Zahlen                                    *)
         R:= Randomnumbers,    (* USER-Mod. Bereitstellen von Zufallszahlen *)
         S:= Strings,          (* Strings (Verarbeitung von Zeichenketten)  *)
         SYSTEM,               (* Standardmodul fr 'low level'-Anweisungen *)
         W:= Windows;          (* Definitionsdatei mit vielen Konstanten,   *)
                               (* Variablen und Prozeduren fr das WIN-API  *)

  TYPE   tStr     = ARRAY 76 OF CHAR;  (* (Daten)Typvereinbarungen fr Zei- *)
         tZahlStr = ARRAY  7 OF CHAR;  (* chenketten (strings) mit maximal  *)
                                       (* 75 bzw. 6 Zeichen                 *)
         tFarbe   = RECORD
                      r, g, b : INTEGER   (* Verbund fr ein RGB-Tripel     *)
                    END;

         tPunkt   = RECORD
                      x, y : INTEGER   (* Verbund fr die x- und y-Koordi-  *)
                    END;               (* naten eines Punktes               *)

  CONST  dTime    = 125;        (* global eingestellte Wartezeit            *)

  VAR    bc_Red,
         bc_Green,
         bc_Blue  : INTEGER;    (* HG-Farben des groen Grafikfensters      *)
                                (* globale Variable fr die Initialisierung *)
                                (* des groen Grafikfensters auf dem BS     *)
         dummy    : LONGINT;


 (* ---------------------  U N T E R P R O G R A M M ---------------------- *)


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE WriteStrXY ( x, y : INTEGER; text : tStr );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate und der auszugebender Text (string)    |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette im Grafikmodus.   |
    |            x = Pixelspalte; x = 0 entspricht der linken Begrenzung    |
    |                                   des aktuellen Grafikfensters        |
    |            y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung   |
    |                                   des aktuellen Grafikfensters        |
    *-----------------------------------------------------------------------*)

   BEGIN
     C.GotoXY(x,y); C.WriteStr(text)
   END WriteStrXY;


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR dummy : CHAR;

    BEGIN
      WHILE C.KeyPressed() DO         (* entspricht: Display.FlushKeyBuffer *)
        dummy:= C.ReadKey()
      END;
    END FlushKeyBuffer;


  PROCEDURE Enter;
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten, die Textfarbe definieren, die   |
    |           Zeichenkette '< ENTER >' an der Position x = 600, y = 48    |
    |           ausgeben, den Tastaturpuffer lschen, auf eine Tastenbet-  |
    |           tigung (Tastatur) warten, eine bestimmte Zeit warten und    |
    |           die Zeichenkette und den Tastaturpuffer lschen (sozusagen  |
    |           das Enter-Zeichen abholen).                                 |
    *-----------------------------------------------------------------------*)

    BEGIN
      Delay_ms(dTime);
      C.SetForeColor(160,160,160);
      WriteStrXY(660,48,'< ENTER >');
      FlushKeyBuffer;
      REPEAT UNTIL C.KeyPressed();
      Delay_ms(dTime);
      C.Bar(660,48,740,30,C.ERASE);
      FlushKeyBuffer
    END Enter;


  PROCEDURE RoundReal ( x : REAL ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: zu rundende reelle Zahl                                    |
    | Referenz:  Funktionswert (gerundete Zahl; DT: INTEGER)                |
    | Leistung:  Runden einer reellen Zahl nach der Rundungsregel unter     |
    |            Verwendung der systemnahen Standardprozedur VAL(x).        |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN SYSTEM.VAL(INTEGER,x+0.5)
    END RoundReal;


  PROCEDURE BerechnePkt ( VAR p : tPunkt; xM, yM : INTEGER; r, w : REAL );
   (*-----------------------------------------------------------------------*
    | bernahme: Mittelpunkkoordinaten und Radius des Kreises, Winkel       |
    | Referenz:  Koordinaten des berechneten Punktes auf der 'Kreislinie'   |
    | Leistung:  Errechnen der Koordinaten eines Punktes der Kreislinie     |
    |            mittels trigonometrischer Funktionen.                      |
    | Hinweis:   Es gibt mathematisch 'schnere' Lsungen. Fr diese Demon- |
    |            stration ist die verwendete ausreichend.                   |
    *-----------------------------------------------------------------------*)
  
    BEGIN
      p.x:= RoundReal(xM + r * F.Cos(w));
      p.y:= RoundReal(yM + r * F.Sin(w))
    END BerechnePkt;


  PROCEDURE WandelnStrInt ( nStr : tZahlStr ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: Zeichenkette, die in eine ganze Zahl zu wandeln (konver-   |
    |            tieren) ist                                                |
    | Referenz:  Funktionswert (konvertierte Zeichenkette; DT: INTEGER)     |
    | Leistung:  Unter Nutzung der Standardfunktionsprozedur Val(text) aus  |
    |            dem Modul Strings wird eine Zeichenkette in eine ganze     |
    |            Zahl gewandelt. Da der Resultatsdatentyp LONGINT ist, wird |
    |            durch Anwenden der Standardprozedur SHORT(x) dieser Daten- |
    |            typ in den Datentyt INTEGER gewandelt.                     |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN SHORT(S.Val(nStr))
    END WandelnStrInt;


  PROCEDURE WandelnIntStr ( ergebn : INTEGER; VAR ergebnStr : tZahlStr );
   (*-----------------------------------------------------------------------*
    | bernahme: ganze Zahl, die in eine Zeichenkette zu konvertieren ist   |
    | Referenz:  konvertierte Zahl, ergebnStr, DT: Zeichenkette (tZahlStr)  |
    | Leistung:  Eine ganze Zahl wird unter Nutzung der Standardprozedur    |
    |            Str(x,text) in eine Zeichenkette konvertiert.              |
    *-----------------------------------------------------------------------*)

    BEGIN
      S.Str(ergebn,ergebnStr)
    END WandelnIntStr;


  PROCEDURE Ueberschrift1;
    BEGIN
      C.SetForeColor(120,240,192);
      WriteStrXY(256,486,'Ein- und Ausgaben im ColoPlane-Modul');
      C.Line(248,468,552,468,C.DRAW);
      WriteStrXY(270,458,'Farbgrafikmodus, 800 x 600 Pixel');
      C.SetScreenUpdate(TRUE);            (* aktives Fenster im Vordergrund *)
      Delay_ms(dTime*5)
    END Ueberschrift1;


  PROCEDURE Ueberschrift2;
    BEGIN
      C.SetForeColor(120,240,192);
      WriteStrXY(232,486,'Pixel- und Liniengrafik im ColoPlane-Modul');
      C.Line(224,468,576,468,C.DRAW);
      WriteStrXY(270,458,'Farbgrafikmodus, 800 x 600 Pixel');
      Delay_ms(dTime*5)
    END Ueberschrift2;


  PROCEDURE Erste;
   (*-----------------------------------------------------------------------*
    | Leistung: Zwei Zeichenketten einlesen, jeweils in eine ganze Zahl     |
    |           konvertieren, beide Zahlen addieren, das Ergebnis in eine   |
    |           Zeichenkette umwandeln und kommentiert auf dem Monitor an-  |
    |           zeigen. Beim Einlesen der Summanden und vor der Ausgabe des |
    |           Ergebnisses werden Zulssigkeitsprfungen durchgefhrt.     |
    *-----------------------------------------------------------------------*)

    CONST xText  = 208;
          offset =  70;

    VAR   resCode   : CHAR;
          a, b      : INTEGER;
          ergebn    : LONGINT;
          ergebnStr : tZahlStr;

    PROCEDURE EinlesenZahlStr ( VAR zahl : INTEGER; spalte, zeile : INTEGER);
     (**
      | bernahme: Koordinaten fr die Pixelspalte (x) und die Pixelzeile   |
      |            (y) an der die Eingabe beginnen soll                     |
      | Referenz:  ganze Zahl, aus der Wertemenge des Datentyps INTEGER     |
      | Leistung:  Einlesen einer Zeichenkette, die eine maximale Lnge von |
      |            fnf Zahlzeichen plus ein Minuszeichen besitzt. Es wird  |
      |            eine Gltigkeitsprfung durchgefhrt.                    |
      **)

      VAR erstesZeichen  : ARRAY 2 OF CHAR;
          ok, vorzeichen : BOOLEAN;
          firstChar      : CHAR;
          anzahl, n      : INTEGER;
          eingabeStr     : tZahlStr;

      PROCEDURE Test ( eingabeStr : tZahlStr;
                                      startwert, anzahl : INTEGER ) : BOOLEAN;
       (*                                  *
        | bernahme: Zeichenkette, die die Zahlzeichen (Ziffern) und ggf.   |
        |            das Minuszeichen enthlt. Startwert und Anzahl Zeichen |
        |            dienen der Analyse der Zeichenkette. Enthlt die Zei-  |
        |            chenkette ein Minuszeichen, wird als Startwert 1 sonst |
        |            0 bergeben.                                           |
        | Referenz:  Funktionswert (DT: BOOLEAN), er ist TRUE, wenn die     |
        |            Zeichenkette nur gltige Zeichen enthlt, sonst FALSE. |
        | Leistung:  Die Zeichenkette wird auf das ausschlieliche Vorhan-  |
        |            densein von Ziffern geprft.                           |
        *                                  *)

        VAR ok   : BOOLEAN;
            i, n : INTEGER;

        BEGIN
          ok:= TRUE;
          FOR i:= startwert TO anzahl DO
            n:= ORD(eingabeStr[i]);
            IF ~((n > 47) & (n < 58)) THEN ok:= FALSE END
          END;
          RETURN ok
        END Test;

      PROCEDURE EinlesenErstesZeichen () : CHAR;
       (*                                  *
        | Referenz: Funktionswert (DT: CHAR), es kann sich um das Minuszei- |
        |           chen oder um eine Ziffer handeln.                       |
        | Leistung: Die Schleife mit Eintrittsbedingung dient dem Darstel-  |
        |           len eines blinkenden Cursors. Wird eine Tastenbetti-   |
        |           gung der Tastatur registriert, bleibt der Cursor un-    |
        |           sichtbar und das Zeichen der bettigten Taste wird aus  |
        |           dem Tastaturpuffer abgeholt. Da im Grafikmodus nur Zei- |
        |           chenketten ausgegeben werden knnen, muss das Zeichen,  |
        |           damit es am Monitor dargestellt werden kann, in eine    |
        |           leere Zeichenkette kopiert und danach positioniert aus- |
        |           gegeben werden (selbst gestaltetes Bildschirmecho).     |
        *                                  *)

        BEGIN
          WHILE ~C.KeyPressed() DO
            C.SetForeColor(96,220,220);
            C.GotoXY(spalte,zeile); C.WriteStr('_');
            Delay_ms(dTime*2);
            C.SetForeColor(bc_Red,bc_Green,bc_Blue);
            C.GotoXY(spalte,zeile); C.WriteStr('_');
            Delay_ms(dTime*2)
          END;
          C.SetForeColor(96,220,220);
          firstChar:= C.ReadKey();   (* das sieht umstndlich aus,  m u s s *)
                                     (* aber sein, da C.ReadKey()  o h n e  *)
                                     (* Cursor und  o h n e  Bildschirmecho *)
                                     (* arbeitet                            *)
          erstesZeichen:= '';
          S.InsertChar(firstChar,erstesZeichen,0);
          C.GotoXY(spalte,zeile); C.WriteStr(erstesZeichen);
          RETURN firstChar
        END EinlesenErstesZeichen;

      BEGIN  (*  EinlesenZahlStr *)
        REPEAT
          ok:= TRUE;
          vorzeichen:= FALSE;
          WriteStrXY(spalte,zeile,'      ');
          firstChar:= EinlesenErstesZeichen();
          n:= ORD(firstChar);
          IF ~((n = 45) OR (n > 47) & (n < 58)) THEN
            ok:= FALSE
          ELSIF n = 45 THEN          (* erstes Zeichen ist das Minuszeichen *)
            vorzeichen:= TRUE;
            C.ReadStr(eingabeStr,5,resCode)
          ELSE
            C.ReadStr(eingabeStr,4,resCode)
          END;
          IF ok THEN
            S.InsertChar(firstChar,eingabeStr,0);
            anzahl:= SHORT(S.Length(eingabeStr));
            IF vorzeichen & (anzahl > 1) THEN
              ok:= Test(eingabeStr,1,anzahl-1)
            ELSIF ~vorzeichen & (anzahl > 0) THEN
              ok:= Test(eingabeStr,0,anzahl-1)
            END
          END;
          IF ok THEN
            zahl:= WandelnStrInt(eingabeStr)
          END
        UNTIL ok & (zahl <= MAX(INTEGER)) & (zahl >= MIN(INTEGER))
      END EinlesenZahlStr;

    BEGIN  (* Erste *)
      C.SetForeColor(96,220,220);
      WriteStrXY(xText,400,'Bitte zwei ganze Zahlen eingeben und jede Eingabe');
      WriteStrXY(xText,382,'durch Bettigen der Enter-Taste abschlieen!');
      WriteStrXY(xText+offset,350,'1. ganze Zahl ( a ) = ');
      EinlesenZahlStr(a,xText+offset+176,350);
      WriteStrXY(xText+offset,332,'2. ganze Zahl ( b ) = ');
      EinlesenZahlStr(b,xText+offset+176,332);
      ergebn:= a + b;
      IF (ergebn <= MAX(INTEGER)) & (ergebn >= MIN(INTEGER)) THEN
        WandelnIntStr(SHORT(ergebn),ergebnStr);
        WriteStrXY(xText+offset,300,'Berechnen:    a + b = ');
        C.WriteStr(ergebnStr)
      ELSE
        C.SetForeColor(255,96,96);
        WriteStrXY(xText-32,300,'Das Ergebnis liegt auerhalb des INTEGER-Zahlenbereichs')
      END;
      Enter
    END Erste;


  PROCEDURE Zweite;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Arbeitsanweisung (Aufforderung zur Interakti- |
    |           vitt), Einlesen einer maximal 25 Zeichen langen Zeichen-   |
    |           kette, Lschen der Eingabezeile (Bildschirm) und Kontroll-  |
    |           ausgabe der eingegebenen Zeichenkette.                      |
    *-----------------------------------------------------------------------*)

    CONST xText  = 208;
          offset =  70;

    VAR   zeichenkette : ARRAY 26 OF CHAR;
                                     (* kann 25 druckbare Zeichen aufnehmen *)
          resCode      : CHAR;
          i            : INTEGER;

    BEGIN
      C.SetForeColor(96,220,220);
      WriteStrXY(xText,238,'Bitte eine Zeichenkette eingeben und durch');
      WriteStrXY(xText,220,'Bettigen der Enter-Taste abschlieen!');
      WriteStrXY(xText,186,'Zeichenkette (max. 25 Zeichen): ');
      C.ReadStr(zeichenkette,25,resCode);
      C.GotoXY(xText+256,186);
      FOR i:= 1 TO 13 DO C.WriteStr('  ') END;
      C.SetForeColor(0,200,128);
      WriteStrXY(xText+24,154,'Kontrollausgabe: ');
      C.SetForeColor(96,220,220);
      C.WriteStr(zeichenkette);
      Enter
    END Zweite;


  PROCEDURE Linien;
   (*-----------------------------------------------------------------------*
    | Leistung: Positionierte farbige Ausgabe einer Zeichenkette. Danach    |
    |           werden fnf farbiger Linien gezeichnet. Nach jeder Linien-  |
    |           darstellung wird eine bestimmte Zeit gewartet, damit der    |
    |           Vorgang gut am Monitor verfolgbar ist.                      |
    *-----------------------------------------------------------------------*)

   CONST xOffset = 270;  yOffset = 390;

    VAR  i, x, y : INTEGER;

    BEGIN
      C.SetForeColor(96,220,220);
      WriteStrXY(xOffset+140,yOffset-15,'Linien');
      C.SetForeColor(255,128,255);
      C.Line(xOffset,yOffset,xOffset+75,yOffset-25,C.DRAW);
      Delay_ms(dTime);
      C.SetForeColor(255,0,0);
      C.Line(xOffset+15,yOffset,xOffset+100,yOffset,C.DRAW);
      Delay_ms(dTime);
      C.SetForeColor(255,255,96);
      C.Line(xOffset+5,yOffset-35,xOffset+80,yOffset-10,C.DRAW);
      Delay_ms(dTime);
      C.SetForeColor(0,255,128);
      C.Line(xOffset+45,yOffset,xOffset+95,yOffset-30,C.DRAW);
      Delay_ms(dTime);
      C.SetForeColor(255,255,255);
      C.Line(xOffset,yOffset-40,xOffset+90,yOffset-45,C.DRAW);
      Delay_ms(dTime);
      Enter
    END Linien;


  PROCEDURE ZeichneRechteckMitPixeln;
   (*-----------------------------------------------------------------------*
    | Leistung: Positionierte farbige Ausgabe einer Zeichenkette. Anschlie- |
    |           end wird ein Rechteck gezeichnet und das Unterprogramm     |
    |           Enter aufgerufen. Wurde vom Anwender eine Taste der Tasta-  |
    |           tur bettigt, wird der Zufallszahlengenerator mit einem     |
    |           Wert (Timertick, DT: LONGINT) initialisiert. Danach wird    |
    |           eine zufllig verteilte Punktmenge im Inneren des Rechtecks |
    |           dargestellt und die anfangs ausgegebene Zeichenkette wird   |
    |           vervollstndigt.                                            |
    *-----------------------------------------------------------------------*)

   CONST xOffset = 270;  yOffset = 300;

    VAR  i, x, y : INTEGER;

    BEGIN
      C.SetForeColor(96,220,220);
      WriteStrXY(xOffset+140,yOffset-50,'Rechteck');
      C.SetForeColor(220,220,220);
      C.Box(xOffset-2,yOffset+2,xOffset+102,yOffset-102,C.DRAW);
      Enter;
      R.Randomize;
      C.SetForeColor(255,255,128);
      FOR i:= 1 TO 250 DO
        x:= RoundReal(R.Uniform() * 100) + xOffset;
        y:= RoundReal(R.Uniform() * 100) + yOffset - 100;
        C.Dot(x,y,C.DRAW);
        Delay_ms(dTime DIV 50);
      END;
      C.SetForeColor(96,220,220);
      WriteStrXY(xOffset+212,yOffset-50,'mit Pnktchen');
      Enter
    END ZeichneRechteckMitPixeln;


  PROCEDURE Kreise;
   (*-----------------------------------------------------------------------*
    | Leistung: Positionierte farbige Ausgabe einer Zeichenkette. Danach    |
    |           werden die RGB-Werte fr die fnf darzustellenden Kreise    |
    |           und der Zufallsgenerator initialisiert. Anschlieend werden |
    |           im Anweisungsteil einer Zhlschleife fr die Radien und die |
    |           Mittelpunkskoordinaten der Kreise Zufallszahlen innerhalb   |
    |           eines bestimmten Wertebereichs ermittelt und die Kreise     |
    |           punktweise gezeichnet.                                      |
    *-----------------------------------------------------------------------*)

   CONST xOffset = 270;  yOffset = 120;

    VAR  farben   : ARRAY 6 OF tFarbe;
         i, x, xM,
         y, yM    : INTEGER;
         r, w     : REAL;
         p        : tPunkt;

    PROCEDURE InitialisiereFarben;
     (**
      | Leistung: Wertzuweisungen an die RGB-Farbwerte, die als Komponenten |
      |           einer eindimensionalen Reihung definiert sind. Jede Kom-  |
      |           ponente enthlt einen Verbund, der die drei Farbwerte     |
      |           aufnimmt.                                                 |
      **)

      BEGIN
        farben[1].r:=  96; farben[1].g:= 220; farben[1].b:= 220;
        farben[2].r:= 255; farben[2].g:= 128; farben[2].b:=   0;
        farben[3].r:= 255; farben[3].g:= 128; farben[3].b:= 255;
        farben[4].r:=   0; farben[4].g:= 255; farben[4].b:= 128;
        farben[5].r:= 220; farben[5].g:= 255; farben[5].b:=  96;
      END InitialisiereFarben;

    PROCEDURE ZeichneKreis;
     (**
      | Leistung: Einstellen der Vordergrundfarbe mit den zum jeweiligen    |
      |           Aufruf gehrenden RGB-Farbwerten. Anschlieend wird der   |
      |           Winkel w mit 0 initialisiert. Die Schleife mit Eintritts- |
      |           bedingung wird solange durchlaufen bis der Winkel nhe-   |
      |           rungsweise den Wert 2 * Pi erreicht hat. Im Schleifen-    |
      |           krper wird die Berechnung der Punktkoordinaten angewie-  |
      |           sen und der jeweilige Punkt gesetzt. Die Punkte ergeben,  |
      |           wenn sie alle dargestellt sind den Vollkreis.             |
      **)

      BEGIN
        C.SetForeColor(farben[i].r,farben[i].g,farben[i].b);
        w:= 0;
        WHILE w < 2 * F.Pi DO
          w:= w + 1/r;
          BerechnePkt(p,xM,yM,r,w);
          C.Dot(p.x,p.y,C.DRAW)
        END
      END ZeichneKreis;

    BEGIN  (* Kreise *)
      C.SetForeColor(96,220,220);
      WriteStrXY(xOffset+140,yOffset-15,'Kreise (selbstdefiniert)');
      InitialisiereFarben;
      R.InitSeed(314159);
      FOR i:= 1 TO 5 DO
        r := RoundReal(R.Uniform() * 120);
        xM:= RoundReal(R.Uniform() *  50) + xOffset + 20;
        yM:= RoundReal(R.Uniform() *  40) + yOffset - 20;
        ZeichneKreis;
        Delay_ms(dTime)
      END;
      Enter  
    END Kreise;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();
    BEGIN
      Ueberschrift1;
      Erste;
      Zweite;
      C.Bar(5,0,850,650,C.ERASE);               (* groes Fenster lschen   *)
      Ueberschrift2;
      Linien;
      ZeichneRechteckMitPixeln;
      Kreise
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   C.SetScreenUpdate(FALSE);       (* aktives Fenster im Hintergrund        *)
   dummy:= W.ShowWindow(W.GetActiveWindow(),3); (* groes Grafikfenster     *)
   bc_Red:= 8; bc_Green:= 0; bc_Blue:= 32;    (* HG-Farbe groes Fenster  *)
   C.SetBackColor(bc_Red,bc_Green,bc_Blue);     (* HG-Farbe setzen          *)
   C.Clear;                                     (* groes Fenster mit der   *)
                                                (* HG-Farbe 'bermalen'     *)
   C.SetForeColor(200,200,200);    (* Am linken Bildschirmrand wird ein     *)
   C.Bar(0,0,3,650,C.DRAW);        (* grauer Streifen mit Schatten erzeugt, *)
   C.SetForeColor(96,96,96);       (* da das Grafikfenster sonst linkssei-  *)
   C.Line(4,0,4,650,C.DRAW);       (* tig nicht abgeschlossen wre.         *)
   C.SetForeColor(96,220,220);     (* Da die Vordergrundfarbe Schwarz auto- *)
                                   (* matisch eingestellt wird,  m u s s    *)
                                   (* die Textfarbe bereits hier initiali-  *)
                                   (* siert werden. Sonst sieht man die     *)
                                   (* Eingabe bei der Anweisung C.ReadStr() *)
                                   (* auf dem dunklen Hintergrund nicht!    *)
 END Vier.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

