
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon (32 Bit)               Dateiname: VGACOLORS.MOD |
  |-------------------------------------------------------------------------|
  | Autor: Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                      |
  |        Erstellungsdatum: Juli 1998,  Version: 1.0                       |
  |        Letze nderung:   08.06.2001, Version: 1.5                       |
  |-------------------------------------------------------------------------|
  | Thema:    Grafikprogrammierung in POW! (OBERON-2)                       |
  |           Erzeugen einer VGA-Farbtabelle, deren Werte dem VGA-Standard  |
  |           entsprechen. (16-teilige VGA-Farbtabelle)                     |
  |           Programmieren eines Moduls, dessen Prozeduren von anderen     |
  |           Modulen importiert werden knnen (Service-Moduls)             |
  |-------------------------------------------------------------------------|
  | Hinweise: Siehe ...\GRUNDLAGEN\ALLGEMEIN\QUELLTEXT\VGACOLORS.MOD        |
  |-------------------------------------------------------------------------|
  |         1998/2001  Manfred Kmmerer, Chamberlain-Soft JENA             |
  *-------------------------------------------------------------------------*)


 MODULE VGAColors;
 
   IMPORT C:= ColorPlane;
   
   TYPE   tVGACol*       = RECORD
                             r*, g*, b* : INTEGER
                           END;
                      
          tVGAColors*    = ARRAY 16 OF tVGACol;
          tVGAColornames = ARRAY 16, 13 OF CHAR;         

   VAR    colorNumbers*  : SET;
          colorName*     : tVGAColornames;
          vgaC,
          vgaColors*     : tVGAColors;
          
   CONST  modus      =  1;  (* Hier ist der Wert, der durch das Programm    *)
                            (* ColorVal, Dateiname COLORVAL.MOD, ermittelt  *)
                            (* wurde einzutragen                            *)

          Black*     =  0;   DarkGray*     =  8;
          Blue*      =  1;   LightBlue*    =  9;
          Green*     =  2;   LightGreen*   = 10;
          Cyan*      =  3;   LightCyan*    = 11;
          Red*       =  4;   LightRed*     = 12;
          Magenta*   =  5;   LightMagenta* = 13;
          Brown*     =  6;   Yellow*       = 14;
          LightGray* =  7;   White*        = 15;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)

  PROCEDURE InitiateColorname;
    BEGIN
      colorName[0]  := 'Black';      colorName[8]  := 'DarkGray';
      colorName[1]  := 'Blue';       colorName[9]  := 'LightBlue';
      colorName[2]  := 'Green';      colorName[10] := 'LightGreen';
      colorName[3]  := 'Cyan';       colorName[11] := 'LightCyan';
      colorName[4]  := 'Red';        colorName[12] := 'LightRed';
      colorName[5]  := 'Magenta';    colorName[13] := 'LightMagenta';
      colorName[6]  := 'Brown';      colorName[14] := 'Yellow';
      colorName[7]  := 'LightGray';  colorName[15] := 'White'
    END InitiateColorname;


  PROCEDURE InitiateVGAColors ( mode : INTEGER );
    BEGIN
      colorNumbers:= {0 .. 15};
      CASE mode OF
       1: (*--------------------------------------------------------------------*
           | FARBMODUS: T R U E C O L O R  (16777216 Farben, 24 Bit)            |                        | 
           *--------------------------------------------------------------------*)
         vgaC[0].r:=    0; vgaC[0].g:=    0; vgaC[0].b:=    0; (*  Black        *)
         vgaC[1].r:=    0; vgaC[1].g:=    0; vgaC[1].b:=  169; (*  Blue         *)
         vgaC[2].r:=    0; vgaC[2].g:=  169; vgaC[2].b:=    0; (*  Green        *)
         vgaC[3].r:=    0; vgaC[3].g:=  169; vgaC[3].b:=  169; (*  Cyan         *)
         vgaC[4].r:=  169; vgaC[4].g:=    0; vgaC[4].b:=    0; (*  Red          *)
         vgaC[5].r:=  169; vgaC[5].g:=    0; vgaC[5].b:=  169; (*  Magenta      *)
         vgaC[6].r:=  169; vgaC[6].g:=   85; vgaC[6].b:=    0; (*  Brown        *)
         vgaC[7].r:=  169; vgaC[7].g:=  169; vgaC[7].b:=  169; (*  LightGray    *)
         vgaC[8].r:=   85; vgaC[8].g:=   85; vgaC[8].b:=   85; (*  DarkGray     *)
         vgaC[9].r:=   85; vgaC[9].g:=   85; vgaC[9].b:=  255; (*  LightBlue    *)
         vgaC[10].r:=  85; vgaC[10].g:= 255; vgaC[10].b:=  85; (*  LightGreen   *)
         vgaC[11].r:=  85; vgaC[11].g:= 255; vgaC[11].b:= 255; (*  LightCyan    *)
         vgaC[12].r:= 255; vgaC[12].g:=  85; vgaC[12].b:=  85; (*  LightRed     *)
         vgaC[13].r:= 255; vgaC[13].g:=  85; vgaC[13].b:= 255; (*  LightMagenta *)
         vgaC[14].r:= 255; vgaC[14].g:= 255; vgaC[14].b:=  85; (*  Yellow       *)
         vgaC[15].r:= 255; vgaC[15].g:= 255; vgaC[15].b:= 255  (*  White        *)
   
     | 2: (*--------------------------------------------------------------------*
           | FARBMODUS: H I G H C O L O R  (32768 Farben, 15 Bit)               |
           |            Windows korrigiert die Tabelle auf folgende Werte:      |
           |            (nchstliegende Werte, engl, nearst color)              |
           *--------------------------------------------------------------------*)
         vgaC[0].r:=    0; vgaC[0].g:=    0; vgaC[0].b:=    0; (*  Black        *)
         vgaC[1].r:=    0; vgaC[1].g:=    0; vgaC[1].b:=  168; (*  Blue         *)
         vgaC[2].r:=    0; vgaC[2].g:=  168; vgaC[2].b:=    0; (*  Green        *)
         vgaC[3].r:=    0; vgaC[3].g:=  168; vgaC[3].b:=  168; (*  Cyan         *)
         vgaC[4].r:=  168; vgaC[4].g:=    0; vgaC[4].b:=    0; (*  Red          *)
         vgaC[5].r:=  168; vgaC[5].g:=    0; vgaC[5].b:=  168; (*  Magenta      *)
         vgaC[6].r:=  168; vgaC[6].g:=   80; vgaC[6].b:=    0; (*  Brown        *)
         vgaC[7].r:=  168; vgaC[7].g:=  168; vgaC[7].b:=  168; (*  LightGray    *)
         vgaC[8].r:=   80; vgaC[8].g:=   80; vgaC[8].b:=   80; (*  DarkGray     *)
         vgaC[9].r:=   80; vgaC[9].g:=   80; vgaC[9].b:=  248; (*  LightBlue    *)
         vgaC[10].r:=  80; vgaC[10].g:= 248; vgaC[10].b:=  80; (*  LightGreen   *)
         vgaC[11].r:=  80; vgaC[11].g:= 248; vgaC[11].b:= 248; (*  LightCyan    *)
         vgaC[12].r:= 248; vgaC[12].g:=  80; vgaC[12].b:=  80; (*  LightRed     *)
         vgaC[13].r:= 248; vgaC[13].g:=  80; vgaC[13].b:= 248; (*  LightMagenta *)
         vgaC[14].r:= 248; vgaC[14].g:= 248; vgaC[14].b:=  80; (*  Yellow       *)
         vgaC[15].r:= 248; vgaC[15].g:= 248; vgaC[15].b:= 248  (*  White        *)

     | 3: (*--------------------------------------------------------------------*
           | FARBMODUS: H I G H C O L O R  (65536 Farben, 16 Bit)               |
           |            Windows korrigiert die Tabelle auf folgende Werte:      |
           |            (nchstliegende Werte, engl, nearst color)              |
           *--------------------------------------------------------------------*)
         vgaC[0].r:=    0; vgaC[0].g:=    0; vgaC[0].b:=    0; (*  Black        *)
         vgaC[1].r:=    0; vgaC[1].g:=    0; vgaC[1].b:=  168; (*  Blue         *)
         vgaC[2].r:=    0; vgaC[2].g:=  168; vgaC[2].b:=    0; (*  Green        *)
         vgaC[3].r:=    0; vgaC[3].g:=  168; vgaC[3].b:=  168; (*  Cyan         *)
         vgaC[4].r:=  168; vgaC[4].g:=    0; vgaC[4].b:=    0; (*  Red          *)
         vgaC[5].r:=  168; vgaC[5].g:=    0; vgaC[5].b:=  168; (*  Magenta      *)
         vgaC[6].r:=  168; vgaC[6].g:=   84; vgaC[6].b:=    0; (*  Brown        *)
         vgaC[7].r:=  168; vgaC[7].g:=  168; vgaC[7].b:=  168; (*  LightGray    *)
         vgaC[8].r:=   80; vgaC[8].g:=   84; vgaC[8].b:=   80; (*  DarkGray     *)
         vgaC[9].r:=   80; vgaC[9].g:=   84; vgaC[9].b:=  248; (*  LightBlue    *)
         vgaC[10].r:=  80; vgaC[10].g:= 252; vgaC[10].b:=  80; (*  LightGreen   *)
         vgaC[11].r:=  80; vgaC[11].g:= 252; vgaC[11].b:= 248; (*  LightCyan    *)
         vgaC[12].r:= 248; vgaC[12].g:=  84; vgaC[12].b:=  80; (*  LightRed     *)
         vgaC[13].r:= 248; vgaC[13].g:=  84; vgaC[13].b:= 248; (*  LightMagenta *)
         vgaC[14].r:= 248; vgaC[14].g:= 252; vgaC[14].b:=  80; (*  Yellow       *)
         vgaC[15].r:= 248; vgaC[15].g:= 252; vgaC[15].b:= 248  (*  White        *)
      ELSE
      END;
      vgaColors:= vgaC
    END InitiateVGAColors;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)   

 BEGIN
   InitiateVGAColors(modus);
   InitiateColorname
 END VGAColors.

 (* --------   1998/2001  Manfred Kmmerer, Chamberlain-Soft JENA  --------*)
