
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon (POW!-32)                   Dateiname: ZWEI.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: Ostern 2001 Version: V 1.0                  |
  |-------------------------------------------------------------------------|
  | Thema:    A b s t r a k t e r   D a t e n t y p   L i s t e             |
  |-------------------------------------------------------------------------|
  | Leistung: Das Modul importiert u. a. das Service-Modul ADTLISTE.        |
  |           Erzeugen eines Stapels (Keller, stack), Textausgabe;          |
  |           einen INTEGER-Wert stapeln und wieder abholen, Textausgabe;   |
  |           Mitteilung, dass der Stapel leer ist, Textausgabe.            |
  |-------------------------------------------------------------------------|
  | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.    |
  |           (Oberon-2 V4)                                                 |
  |-------------------------------------------------------------------------|
  | Hinweise: - Eine Version fr den Display-Modus befindet sich unter:     |
  |               ...\adt\demo\display\adtliste.mod                         |
  |                                                                         |
  |           - Grafikmodus 800 x 600 Pixel, Farbmodus mindestens 15 Bit   |
  |             siehe:                                            |
  |               ...\grundlagen\allgemein\doc\grafik.doc  oder             |
  |               ...\grundlagen\allgemein\txt\grafik.txt                   |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE zwei;

  IMPORT ADTL:= adtliste, C:= ColorPlane, S:= Strings, Sy:= SYSTEM,
         W:= Windows;

  CONST  delayTime  =   2;     (* 2 ist der Wert fr eine CPU i80486 66 MHz *)
                               (* Bitte an Ihren Prozessor anpassen!        *)
         breite     = 300;
         hoehe      = 480;
      (* maxX       = 784; *)  (* sichtbaren Flche bei einer Auflsung von *)
      (* maxY       = 544; *)  (* 800 x 600 Pixel                          *)
         obergrenze =  16;     (* maximal 16 Elemente pro Stapel            *)
         xOffset    =  50;
         xText      = 440;
         xStapel1   =  82;
         xStapel2   = 210;
         yOffset    =  35;
         yStapel    =  80;
         yText      = 420;

  TYPE   tZiffer    = ARRAY  2 OF CHAR;
         tDatenStr  = ARRAY  7 OF CHAR;
         tShStr     = ARRAY 36 OF CHAR;

  VAR    eins, zwei : ADTL.Liste; (* zwei Stapel (Listen) aus dem ADTL-Modul*)

         st1, st2   : BOOLEAN;
         
         bc_Red, bc_Green, bc_Blue,
         bc_red, bc_green, bc_blue,
         elementNr1, elementNr2    : INTEGER;
         
         dummy      : LONGINT;
         
 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Inhalts des Tastaturpuffers                     |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;

    BEGIN
      WHILE C.KeyPressed() DO
        Dummy:= C.ReadKey()
      END
    END FlushKeyBuffer;


  PROCEDURE Delay ( dTfaktor : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Das Abarbeiten des Programms wird fr eine bestimmte Zeit   |
    |           unterbrochen. Die Wartezeit ist vom Prozessortyp abhngig!  |
    *-----------------------------------------------------------------------*)

    VAR blue, green, red : INTEGER;
        i, endwert       : LONGINT;

   BEGIN
     endwert:= delayTime * dTfaktor;
     FOR i:= 1 TO endwert DO C.GetDot(0,0,red,green,blue) END 
   END Delay;


  PROCEDURE Delay_ms ( msec : LONGINT );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine definierte Zeit warten.                                |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE LoescheBereich ( y, zeilen : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Nummer der Pixelzeile (y) und Hhe des zu lschenden Bild- |
    |            schirmbereichs in Textzeilen (je 20 Pixel)                 |
    | Leistung:  Lschen des gewnschten Bereichs. Die linke Begrenzung ist |
    |            durch die globale Konstante xText (Pixelspalte) festgelegt.|
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.Bar(xText-16,y,xText+308,y-zeilen*20,C.ERASE)
    END LoescheBereich;


  PROCEDURE LoeschenStapelelement ( elementNr, nr : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Nummer des zu lschenden Stapelelements (DT: INTEGER) und  |
    |            Ziffer des Stapelbezeichners                               |
    | Leistung:  Lschen eines kompletten Stapelelements (Inhalt und Rahmen)|
    |            im Stapelanzeigefenster.                                   |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      IF nr = 1 THEN
        C.Bar(xStapel1,yStapel+elementNr*24,
                xStapel1+110,yStapel+elementNr*24-22,C.ERASE)
      ELSE
        C.Bar(xStapel2,yStapel+elementNr*24,
                xStapel2+110,yStapel+elementNr*24-22,C.ERASE)
      END
    END LoeschenStapelelement;


  PROCEDURE WriteCharXY ( x, y : INTEGER; character : CHAR );

   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate sowie das auszugebende Zeichen         |
    | Leistung:  Positionierte Ausgabe eines Zeichens.                      |
    *-----------------------------------------------------------------------*)

    VAR twoChar : ARRAY 3 OF CHAR;

    BEGIN
      twoChar:= ''; S.AppendChar(twoChar,character);
      C.GotoXY(x,y);
      C.WriteStr(twoChar)
    END WriteCharXY;


  PROCEDURE WriteStrXY ( x, y : INTEGER; Text : tShStr );

   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate sowie der auszugebende Text            |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette.                  |
    *-----------------------------------------------------------------------*)

   BEGIN
     C.GotoXY(x,y); C.WriteStr(Text)
   END WriteStrXY;


  PROCEDURE Enter ( x, y : INTEGER );
   (*-----------------------------------------------------------------------|
    | bernahme: x- und y-Koordinate fr die Position der Zeichenkette      |
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    |            Die Wartezeit sorgt fr einen ruhigeren Programmlauf.      |
    *-----------------------------------------------------------------------*)

    VAR Taste : CHAR;

    BEGIN
      Delay(delayTime*500);
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(32,160,160);
      C.GotoXY(x,y); C.WriteStr('< ENTER >');
      FlushKeyBuffer;
      REPEAT
        Taste:= C.ReadKey()
      UNTIL (Taste = C.ENTER) OR (Taste = C.ESC);
      LoescheBereich(y,1)
    END Enter;


  PROCEDURE ReadUpCaseKey () : CHAR;

   (*-----------------------------------------------------------------------*
    | Leistung:  Positionierte Ausgabe einer Zeichenkette.                  |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN CAP(C.ReadKey())
    END ReadUpCaseKey;



  PROCEDURE ReadStrXY ( x, y, laenge : INTEGER; flag : BOOLEAN;
                           VAR zeichenkette : tDatenStr; VAR resCode : CHAR );
   (*-----------------------------------------------------------------------*
    | bernahme: Position (x = Pixelspalte, y = Pixelzeile) des blinkenden  |
    |            Cursors, Anzahl der einlesbaren Zeichen (laenge) und ein   |
    |            Flag (boolesche Variable)                                  |
    | Referenz:  zeichenkette, DT: ARRAY 7 OF CHAR,damit die Prozedur       |
    |                          ReadStrXY() fr alle Eingaben einsetzbar ist |
    |            resCode,      DT: CHAR, zum Auswerten der vom System be-   |
    |                          reitgestellten Konstanten C.ENTER und C.ESC  |
    | Leistung:  Einlesen einer Zeichenkette im Grafikmodus an einer durch  |
    |            die Werte der Variablen x (Pixelspalte) und y (Pixelzeile) |
    |            bestimmten Stelle des Bildschirms.                         |
    |            Besitzt die boolesche Variable flag den Wert TRUE, wird    |
    |            ein Hinweis zur Programmsteuerung am Bildschirm angezeigt. |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF flag THEN
        C.SetForeColor(96,143,143);
        WriteStrXY(xText+65,88,'Zurck zum Menue:');
        WriteStrXY(xText,68,'Abschluss der Eingabe mit < ESC >')
      END;
      C.SetForeColor(96,220,220);
      C.GotoXY(x,y); C.ReadStr(zeichenkette,laenge,resCode);
      IF flag THEN LoescheBereich(88,2) END
    END ReadStrXY;


  PROCEDURE DarstellenStapelboden ( nr : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Nummer des Stapels                                         |
    | Leistung:  Ausgabe des beschrifteten Symbols eines Stapelbodens im    |
    |            Stapelausgabefenster (fr nr = 1 im linken Bereich)        |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetBackColor(128,0,96);
      C.SetForeColor(96,255,255);
      IF nr = 1 THEN
        C.Bar(xStapel1-5,yStapel-1,xStapel1+115,yStapel-22,C.ERASE);
        WriteStrXY(xStapel1+24,yStapel-4,'Stapel 1')
      ELSE
        C.Bar(xStapel2-5,yStapel-1,xStapel2+115,yStapel-22,C.ERASE);
        WriteStrXY(xStapel2+24,yStapel-4,'Stapel 2')
      END;
      C.SetBackColor(bc_Red,bc_Green,bc_Blue)
    END DarstellenStapelboden;


  PROCEDURE AusgabeObergrenze;
   (*-----------------------------------------------------------------------*
    | Leistung: Anzeige der 'Obergrenze' des am Bildschirm darstellbaren    |
    |           Stapels.                                                    |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetForeColor(80,160,128);
      WriteStrXY(xOffset+42,490,'Obere Begrenzung der Stapel');
      C.Line(xOffset+20,470,xOffset+breite-20,470,C.DRAW);
    END AusgabeObergrenze;


  PROCEDURE FuehrendeLeerzeichen ( VAR datenStr : tDatenStr );
   (*-----------------------------------------------------------------------*
    | Referenz: Zeichenkette, die die Daten eines Stapelelements enthlt    |
    | Leistung: Solange die Zeichenkette weniger als sechs zugelassene Zei- |
    |           chen (Vorzeichen und Ziffern) enthlt, werden fhrende Leer-|
    |           zeichen eingefgt.                                          |
    | Hinweis:  Eine Dezimalformatierung ist in OBERON-POW! bei Import des  |
    |           Float-Moduls mglich (Float.StrF) ist jedoch recht aufwen-  |
    |           dig (siehe kontextsensitive Hilfe, < Strg > + < F1 >).      |
    *-----------------------------------------------------------------------*)

    VAR i :INTEGER;

    BEGIN
      WHILE S.Length(datenStr) < 6 DO
        S.Insert(' ',datenStr,1)
      END;
    END FuehrendeLeerzeichen;


  PROCEDURE DarstellenStapelelement ( elementNr, stapelNr : INTEGER;
                                                          daten : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Nummer des Elements, Stapelnummer (1 oder 2) und Daten     |
    | Leistung:  Darstellen eines Stapelelements (Rahmen und Daten) im Sta- |
    |            pelfenster an der geforderten Position.                    |
    *-----------------------------------------------------------------------*)

    VAR datenStr : tDatenStr;

    BEGIN
      S.Str(daten,datenStr);
      FuehrendeLeerzeichen(datenStr);
      C.SetBackColor(bc_red,bc_green,bc_blue);
      IF elementNr = obergrenze -2 THEN AusgabeObergrenze END;
      C.SetForeColor(128,0,96);
      IF stapelNr = 1 THEN
        C.Box(xStapel1,yStapel+elementNr*24,
              xStapel1+110,yStapel+elementNr*24-22,C.DRAW);
        C.SetForeColor(96,255,255);
        C.GotoXY(xStapel1+31,yStapel+elementNr*24-4); C.WriteStr(datenStr)
      ELSE
        C.Box(xStapel2,yStapel+elementNr*24,
              xStapel2+110,yStapel+elementNr*24-22,C.DRAW);
        C.SetForeColor(96,255,255);
        C.GotoXY(xStapel2+31,yStapel+elementNr*24-4); C.WriteStr(datenStr)
      END;
      C.SetBackColor(bc_Red,bc_Green,bc_Blue)
    END DarstellenStapelelement;


  PROCEDURE Wiederholung () : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Referenz: Funktionswert vom Datentyp BOOLEAN (TRUE fr Wiederholung,  |
    |           FALSE fr ohne Wiederholung)                                |
    | Leistung: Ausgabe der Frage mit unterstrichenem 'J' als Hinweis, dass |
    |           eine beliebige Tastenbettigung fr 'JA' (Wiederholung ge-  |
    |           wnscht) und < N > (n-Taste) fr 'NEIN' (Wiederholung nicht |
    |           gewnscht) steht.                                           |
    *-----------------------------------------------------------------------*)

    VAR taste, resCode : CHAR;

    BEGIN
      C.SetForeColor(127,160,160);
      WriteStrXY(xText+32,68,'Wiederholung (J/N)?');
      C.Line(xText+144,54,xText+152,54,C.DRAW);
      C.Line(xText+144,53,xText+152,53,C.DRAW);
      taste:= ReadUpCaseKey();
      IF taste = 'N' THEN
        WriteCharXY(xText+196,68,taste)
      ELSE
        WriteCharXY(xText+196,68,'J')
      END;
      Delay_ms(500);
      C.SetForeColor(96,220,220);
      LoescheBereich(68,1);
      IF taste = 'N' THEN
        RETURN FALSE
      ELSE
        RETURN TRUE
      END
    END Wiederholung;


  PROCEDURE AusgebenStapelnummer ( nr : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Nummer des Stapels                                         |
    | Leistung:  Ausgabe der in einen Erluterungstext eingefgten Nummer   |
    |            des erzeugten Stapels.                                     |
    *-----------------------------------------------------------------------*)

    VAR nummerStr : tZiffer;

    BEGIN
      C.SetForeColor(0,228,200);
      WriteStrXY(xText,yText-100,'Es wurde ein leerer Stapel erzeugt.');
      WriteStrXY(xText,yText-140,'Der Stapel besitzt die Nummer ');
      S.Str(nr,nummerStr); C.WriteStr(nummerStr);
      C.WriteStr('.')
    END AusgebenStapelnummer;


  PROCEDURE AusgebenAbgeholtesElement ( daten, nr : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Inhalt (daten) des Stapelelements und Nummer (nr) des Sta- |
    |            pels von dem das Element geholt ('entstapelt') wurde       |
    | Leistung:  Ausgabe einer zusammengesetzten (Zahlen und Text) Zeichen- |
    |            kette am Bildschirm                                        |
    *-----------------------------------------------------------------------*)

    VAR datenStr : tDatenStr;
        nummerStr : tZiffer;

    BEGIN
      C.SetForeColor(0,228,200);
      WriteStrXY(xText,yText-100,'Vom Stapel ');
      S.Str(nr,nummerStr); C.WriteStr(nummerStr);
      C.WriteStr(' geholte Zahl = '); 
      S.Str(daten,datenStr);
      C.WriteStr(datenStr)
    END AusgebenAbgeholtesElement;


  PROCEDURE AusgebenStapelzustand ( flag : BOOLEAN; nr : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Farbflag (boolesche Variable flag) und Stapelnummer (nr)   |
    | Leistung:  Vordergrundfarbe in Abhngigkeit des Wertes der booleschen |
    |            Variable setzen (TRUE = Blaugrn, FALSE = Hellrot).        |
    |            Ausgabe einer Zeichenkette, deren Inhalt in Abhngigkeit   |
    |            des Werts der booleschen Variable zusammengestellt wird.   |
    *-----------------------------------------------------------------------*)

    VAR nummerStr : tZiffer;

    BEGIN
      IF flag THEN
        C.SetForeColor(0,228,200);
      ELSE
        C.SetForeColor(255,96,0)
      END;
      WriteStrXY(xText,yText-100,'Der Stapel ');
      S.Str(nr,nummerStr); C.WriteStr(nummerStr);
      IF flag THEN C.WriteStr(' ist nicht leer!')
        ELSE C.WriteStr(' ist leer!')
      END
   END AusgebenStapelzustand;


  PROCEDURE AusgebenStapelnErfolgt ( offset : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Pixelzeilenabstand zum bereits ausgegebenen Text (offset)  |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetForeColor(96,220,220);
      WriteStrXY(xText,yText-140-offset,'Es wurde ein Element gestapelt.')
    END AusgebenStapelnErfolgt;


  PROCEDURE AusgebenStapelExistiertNicht;                 (* ohne Kommentar *)
    BEGIN
      C.SetForeColor(255,96,0);
      WriteStrXY(xText,yText-100,'Der Stapel existiert nicht!')
    END AusgebenStapelExistiertNicht;


  PROCEDURE AusgebenStapelueberlauf ( offset : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Pixelzeilenabstand zum bereits ausgegebenen Text (offset)  |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetForeColor(255,96,0);
      WriteStrXY(xText,yText-100,'Der Stapel ist bereits voll!');
      WriteStrXY(xText,yText-120-offset,'Es wurde kein Element gestapelt.')
    END AusgebenStapelueberlauf;


  PROCEDURE EinlesenStapelnummer ( VAR resCode : CHAR; VAR nr : INTEGER;
                                                           zeile : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Position der Textausgabe (zeile)                           |
    | Referenz:  Stapelnummer (nr)                                          |
    | Leistung:  Lschen eines begrenzten Bildschirmbereichs und sicheres   |
    |            Einlesen einer gltigen Stapelnummer.                      |
    *-----------------------------------------------------------------------*)

    VAR x, y   : INTEGER;
        nummer : tDatenStr;

    BEGIN
      REPEAT
        LoescheBereich(yText-100,3);
        C.SetForeColor(96,220,220);
        x:= xText + 160;
        y:= yText - zeile * 20;
        C.GotoXY(x,y); C.WriteStr(' ');
        WriteStrXY(xText,y,'Stapel (1 oder 2) = ');
        REPEAT
          ReadStrXY(x,y,1,TRUE,nummer,resCode)
        UNTIL (nummer = '1') OR (nummer = '2');
         (* Der Aufwand ist ntig, weil die Funktionsprozedur S.Val(nummer) *)
         (* fr den Wert '' (leere Zeichenkette) den Laufzeitfehler: value  *)
         (* out of range' liefert, der die Fortsetzung der Programmabarbei- *)
         (* tung verhindert.                                                *)
        nr:= SHORT(S.Val(nummer));
      UNTIL (nr = 1) OR (nr = 2)
    END EinlesenStapelnummer;


  PROCEDURE ErzeugenStapel;
   (*-----------------------------------------------------------------------*
    | Leistung: Einlesen der vom Anwender gewnschten Stapelnummer,         |
    |           existiert dieser Stapel bereits, Ausgabe einer Meldung,     |
    |           sonst Erzeugen des Stapels (der Liste, Modul adtliste),     |
    |           Darstellen des Stapelbodens im Stapelanzeigefenster und     |
    |           Ausgabe der Erfolgsmeldung am Bildschirm.                   |
    |           Wurde bisher nur ein Stapel erzeugt, wird die Frage nach    |
    |           Wiederholung des Vorgangs gestellt. Wurden bereits beide    |
    |           Stapel erzeugt, wird < ENTER > am Bildschirm ausgegeben     |
    |           auf eine Tastenbettigung gewartet.                         |
    *-----------------------------------------------------------------------*)

    VAR resCode : CHAR;
        nr      : INTEGER;

    BEGIN
      C.SetForeColor(96,255,236);
      WriteStrXY(xText-16,yText-20,'STAPEL ERZEUGEN:');
      REPEAT
        EinlesenStapelnummer(resCode,nr,3);
        IF resCode # C.ESC THEN
          IF (nr = 1) & st1 OR (nr = 2) & st2 THEN
            C.SetForeColor(255,96,0);
            WriteStrXY(xText,yText-100,'Dieser Stapel existiert bereits!')
          ELSE
            IF nr = 1 THEN
              ADTL.Erzeugen(eins);
              DarstellenStapelboden(1);
              st1:= TRUE
            ELSE
              ADTL.Erzeugen(zwei);
              DarstellenStapelboden(2);
              st2:= TRUE
            END;
            AusgebenStapelnummer(nr)
          END;
          IF st1 & st2 THEN Enter(650,32) END
        END (* if resCode (Stapelnummer) *)
      UNTIL (resCode = C.ESC) OR st1 & st2 OR ~Wiederholung();
        (* '(resCode = C.ESC) OR st1 & st2'  m  s s e n  unbedingt vor dem *)
        (* Aufruf der Funktionsprozedur Wiederholung() stehen, da letztere  *)
        (* sonst  v o r  der Prfung der anderen booleschen Werte aufgeru-  *)
        (* fen wrde, was, falls die Variable 'resCode' den Code fr C.ESC  *)
        (* besitzt oder beide Stapel bereits existieren (st1 = st2 = TRUE), *)
        (* nicht den erwarteten Rcksprung zum Men zur Folge htte.        *)
      LoescheBereich(yText-20,7)
    END ErzeugenStapel;


  PROCEDURE StapelnElement;
   (*-----------------------------------------------------------------------*
    | Leistung:  Abgesichertes Einlesen eines Stapelelements (berwachung   |
    |            des Gltigkeitsbereichs der Eingabe hinsichtlich Datentyp).|
    |            Einlesen der Stapelnummer. Sollte der Stapel noch nicht    |
    |            existieren, wird er angelegt und eine entsprechende Mel-   |
    |            dung ausgegeben. Das Element wird gestapelt (in die Liste  |
    |            eingetragen), die Elementnummer des entsprechenden Stapels |
    |            inkrementiert. Das Element wird symbolisch im Stapelanzei- |
    |            gefenster auf dem Stapelboden bzw. auf dem bereits exis-   |
    |            tierenden noch aufnahmefhigen Stapel abgelegt und der Vor-|
    |            gang mit einer Erfolgsmeldung abgeschlossen.               |
    |            Falls der Stapel bereits die volle Kapazitt erreicht hat, |
    |            wird eine Meldung ausgegeben und das Einfgen des Elements |
    |            unterbunden. Ist noch mindestens ein Stapel aufnahmefhig  |
    |            wird die Frage nach der Wiederholung des Vorgangs gestellt.|
    |            Wurde nach einer Eingabe die Esc-Taste bettigt, die Wie-  |
    |            derholungsanfrage mit 'NEIN' beantwortet oder sollten be-  |
    |            reits beide Stapel die volle Kapazitt erreicht haben,     |
    |            kehrt das System zum Men zurck. Die Punkt 1 bis 3 werden |
    |            automatisch gesperrt.                                      |
    *-----------------------------------------------------------------------*)

    VAR resCode   : CHAR;
        daten, nr,
        offset    : INTEGER;
        tempDat   : LONGINT;
        datenStr  : tDatenStr;

    BEGIN
      C.SetForeColor(96,255,236);
      WriteStrXY(xText-16,yText-20,'ELEMENT STAPELN:');
      C.SetForeColor(96,220,220);
      WriteStrXY(xText,yText-60,'Eintrag (DT: INTEGER) =');
      REPEAT
        LoescheBereich(yText-100,5);
        C.SetForeColor(96,220,220);
        REPEAT
          C.GotoXY(xText+192,yText-3*20); C.WriteStr('      ');
          ReadStrXY(xText+192,yText-60,6,TRUE,datenStr,resCode);
          tempDat:= S.Val(datenStr);    (* vermeiden eines Laufzeitfehlers, *)
          tempDat:= SHORT(tempDat)      (* falls eine sechsstellige Zahl,   *)
                                        (* statt einer fnfstelligen plus   *)
                                        (* Vorzeichen eingegeben wird       *)
        UNTIL (resCode = C.ESC) OR 
              (tempDat >= MIN(INTEGER)) & (tempDat <= MAX(INTEGER));
                                        (* Zahl ist im Gltigkeitsbereich   *)
        IF resCode # C.ESC THEN
          daten:= SHORT(tempDat);
          EinlesenStapelnummer(resCode,nr,5);
          IF resCode # C.ESC THEN
            IF (nr = 1) & ~st1 THEN
              ADTL.Erzeugen(eins);
              DarstellenStapelboden(nr);
              st1:= TRUE;
              offset:= 40;
              AusgebenStapelnummer(nr)
            ELSIF (nr = 2) & ~st2 THEN
              ADTL.Erzeugen(zwei);
              DarstellenStapelboden(nr);
              st2:= TRUE;
              offset:= 40;
              AusgebenStapelnummer(nr)
            ELSE
              offset:= 0
            END;
            CASE nr OF
               1: IF elementNr1 < obergrenze THEN
                    INC(elementNr1);
                    ADTL.EinfuegenElement(eins,daten);
                    AusgebenStapelnErfolgt(offset);
                    DarstellenStapelelement(elementNr1,nr,daten)
                  ELSE
                    AusgebenStapelueberlauf(offset)
                  END
             | 2: IF elementNr2 < obergrenze THEN
                    INC(elementNr2);
                    ADTL.EinfuegenElement(zwei,daten);
                    AusgebenStapelnErfolgt(offset);
                    DarstellenStapelelement(elementNr2,nr,daten)
                  ELSE
                    AusgebenStapelueberlauf(offset)
                  END
            ELSE                        (* ELSE-Zweig bleibt absichtlich leer *)
            END (* case *)
          END (* if resCode ... (Inhalt eines Stapelelements) *)
        END (* if resCode (Stapelnummer) *)
      UNTIL (resCode = C.ESC) OR (elementNr1 = obergrenze)
                               & (elementNr2 = obergrenze) OR ~Wiederholung();
      LoescheBereich(yText-20,9)
    END StapelnElement;


  PROCEDURE EntstapelnElement;
   (*-----------------------------------------------------------------------*
    | Leistung:  Einlesen der Stapelnummer (nr) des Stapels, von dem das    |
    |            oberste Element entnommen werden soll. Falls der Stapel    |
    |            leer ist, wird eine entsprechende Mitteilung ausgegeben,   |
    |            sonst wird der Inhalt des Elements in einen Satz eingefgt |
    |            und zur Kontrolle am Bildschirm angezeigt. Das Element     |
    |            wird aus der Liste (Modul adtliste) entfernt und im Stapel-|
    |            ausgabefenster symbolisch vom Stapel 'weggenommen'.        |
    |            Die Elementnummer des entsprechenden Stapels wird dekre-   |
    |            mentiert und die Frage nach Wiederholung des Vorgangs ge-  |
    |            stellt.                                                    |
    *-----------------------------------------------------------------------*)

    VAR resCode   : CHAR;
        daten, nr : INTEGER;

    BEGIN
      C.SetForeColor(96,255,236);
      WriteStrXY(xText-16,yText-20,'ELEMENT VOM STAPEL HOLEN:');
      REPEAT
        EinlesenStapelnummer(resCode,nr,3);
        IF resCode # C.ESC THEN
          CASE nr OF
            1: IF ADTL.ListeLeer(eins) & st1 THEN
                 AusgebenStapelzustand(FALSE,nr)
               ELSIF st1 THEN
                 ADTL.HoleEintrag(eins,daten);
                 AusgebenAbgeholtesElement(daten,nr);
                 ADTL.LoeschenElement(eins);
                 LoeschenStapelelement(elementNr1,nr);
                 DEC(elementNr1)
               ELSE
                 AusgebenStapelExistiertNicht
               END
          | 2: IF ADTL.ListeLeer(zwei) & st2 THEN
                 AusgebenStapelzustand(FALSE,nr)
               ELSIF st2 THEN
                 ADTL.HoleEintrag(zwei,daten);
                 AusgebenAbgeholtesElement(daten,nr);
                 ADTL.LoeschenElement(zwei);
                 LoeschenStapelelement(elementNr2,nr);
                 DEC(elementNr2)
               ELSE
                 AusgebenStapelExistiertNicht
               END
          ELSE                          (* ELSE-Zweig bleibt absichtlich leer *)
          END; (* case *)
        END (* if resCode (Stapelnummer) *)  
      UNTIL (resCode = C.ESC) OR ~Wiederholung();
      LoescheBereich(yText-20,9)
    END EntstapelnElement;


  PROCEDURE StapelInspizieren;
   (*-----------------------------------------------------------------------*
    | Leistung:  Einlesen der Stapelnummer (nr) des Stapels, der inspiziert |
    |            werden soll. Ist der Stapel leer, enthlt er mindestens    |
    |            ein Element oder existiert dieser Stapel nicht, wird eine  |
    |            entsprechende Meldung ausgegeben. Danach wird die Frage    |
    |            nach Wiederholung des Vorgangs gestellt.                   |
    *-----------------------------------------------------------------------*)

    VAR leer    : BOOLEAN;
        resCode : CHAR;
        nr      : INTEGER;

    BEGIN
      REPEAT
        C.SetForeColor(96,255,236);
        WriteStrXY(xText-16,yText-20,'STAPEL INSPIZIEREN:');
        EinlesenStapelnummer(resCode,nr,3);
        IF resCode # C.ESC THEN
          CASE nr OF
            1: IF ADTL.ListeLeer(eins) & st1 THEN
                 AusgebenStapelzustand(FALSE,nr)
               ELSIF st1 THEN
                 AusgebenStapelzustand(TRUE,nr)
               ELSE
                 AusgebenStapelExistiertNicht
               END
           |2: IF ADTL.ListeLeer(zwei) & st2 THEN
                 AusgebenStapelzustand(FALSE,nr)
               ELSIF st2 THEN
                 AusgebenStapelzustand(TRUE,nr)
               ELSE
                 AusgebenStapelExistiertNicht
               END
          END (* case *)
        END (* if resCode (Stapelnummer) *)
      UNTIL (resCode = C.ESC) OR ~Wiederholung();
      LoescheBereich(yText-20,9)
    END StapelInspizieren;


  PROCEDURE Ueberschrift;

    VAR i : INTEGER;

    BEGIN
      C.SetForeColor(96,255,255);
      C.GotoXY(390,480);
      C.WriteStr('V E R W A L T E N   Z W E I E R   S T A P E L');
      C.SetForeColor(92,220,220);
      C.GotoXY(376,460);
      FOR i:= 1 TO 6 DO C.WriteStr('') END
                           (* '' = <Alt>+<0><1><6><8> auf dem Nummernblock *)
    END Ueberschrift;


  PROCEDURE Stapelfenster;
   (*-----------------------------------------------------------------------*
    | Leistung: Kleines gerahmtes Fenster mit dunklem Hintergrund bereit-   |
    |           stellen, in dem die beiden Stapel auf- und abgebaut werden. |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(xOffset,yOffset,xOffset+breite,yOffset+hoehe,C.ERASE);
      C.SetForeColor(96,160,160);
      C.Box(xOffset-1,yOffset-1, xOffset+breite+1,yOffset+hoehe+1,C.DRAW)
    END Stapelfenster;


  PROCEDURE EinrichtenTextfarbe ( flag : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme: Boolesche Variable (s. Leistung)                           |
    | Leistung:  Setzen der beiden Farben fr die Meneintrge.             |
    |            flag = TRUE:  Lindgrn (whlbarer Meneintrag)             |
    |            flag = FALSE: dunkles Trkis (Meneintrag deaktiviert)     |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF flag THEN
        C.SetForeColor(96,255,220)
      ELSE  
        C.SetForeColor(96,127,127)
      END;
    END EinrichtenTextfarbe;


  PROCEDURE Menue ( VAR kennzahl : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: gltiger Wert der Variablen kennzahl                        |
    | Leistung: Farbige Ausgabe der Wahlpunkte des Mens samt Kennziffern.  |
    |           Bedeutung der Farben siehe Prozedur EinrichtenTextfarbe().  |
    |           Einlesen der vom Anwender eingegebenen Kennziffer.          |
    |           Abweisen der Kennziffer, falls sie zur Zeit nicht zum akti- |
    |           ven Kennziffernvorrat gehrt und Auffordern zur Wiederho-   |
    |           lung der Eingabe (blinkender Cursor).                       |
    *-----------------------------------------------------------------------*)

    CONST x  = 460;

    VAR   ok         : BOOLEAN;
          resCode    : CHAR;
          y          : INTEGER;
          kennziffer : tDatenStr;

    BEGIN
      ok:= FALSE;
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      IF ~st1 OR ~st2 THEN
        EinrichtenTextfarbe(TRUE)
      ELSE
        EinrichtenTextfarbe(FALSE)
      END;
      WriteStrXY(x,yText,'(1) Leeren Stapel erzeugen');
      IF (elementNr1 < obergrenze) OR (elementNr2 < obergrenze) THEN
        EinrichtenTextfarbe(TRUE)
      ELSE
        EinrichtenTextfarbe(FALSE)
      END;
      WriteStrXY(x,yText-20,'(2) Element stapeln');
      IF (elementNr1 > 0) OR (elementNr2 > 0) THEN
        EinrichtenTextfarbe(TRUE)
      ELSE
        EinrichtenTextfarbe(FALSE)
      END;
      WriteStrXY(x,yText-40,'(3) Element entnehmen');
      EinrichtenTextfarbe(TRUE);
      WriteStrXY(x,yText-60,'(4) Stapel inspizieren');
      WriteStrXY(x,yText-88,'(0) Programm beenden');
      y:= yText - 120;
      REPEAT
        C.SetForeColor(96,220,220);
        WriteStrXY(x+32,y,'Kennziffer =');
        C.GotoXY(x+136,y); C.WriteStr(' ');
        REPEAT
          ReadStrXY(x+136,y,1,FALSE,kennziffer,resCode)
        UNTIL (kennziffer # '') & ((kennziffer >= '0') & (kennziffer < '5'))
             & ((resCode = C.ENTER) OR (resCode = C.ESC));
        kennzahl:= SHORT(S.Val(kennziffer));
        IF kennzahl = 0 THEN kennzahl:= 5 END;
        IF (kennzahl = 1) & (~st1 OR ~st2) THEN
          ok:= TRUE
        ELSIF (kennzahl = 2)
          & ((elementNr1 < obergrenze) OR (elementNr2 < obergrenze)) THEN
          ok:= TRUE
        ELSIF (kennzahl = 3) & ((elementNr1 > 0) OR (elementNr2 > 0)) THEN
          ok:= TRUE
        ELSIF (kennzahl = 4) OR (kennzahl = 5) THEN
          ok:= TRUE
        END
      UNTIL ok
    END Menue;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m : ARRAY 91 OF CHAR;

    BEGIN
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBoxA(W.GetActiveWindow(),Sy.ADR(m),Sy.ADR(m3),40H);
      Delay(delayTime)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR ProgEnde : BOOLEAN;
        kennzahl : INTEGER;

    BEGIN
      ProgEnde:= FALSE;
      Ueberschrift;
      Stapelfenster;
      REPEAT
        Menue(kennzahl);
        C.SetBackColor(bc_Red,bc_Green,bc_Blue);
        LoescheBereich(yText,7);
        CASE kennzahl OF
          1: ErzeugenStapel
        | 2: StapelnElement
        | 3: EntstapelnElement
        | 4: StapelInspizieren
        ELSE
          ProgEnde:= TRUE
        END (* case *)
      UNTIL ProgEnde;
      (* DISPOSE(pointer); Ist nicht notwendig, wenn im Menue 'Options'     *)
      (*                   Untermenue 'Linkler...' die Option 'Garbage Col- *)
      (*                   lection' angekreuzt ist.                         *)
      (*                   garbage collection = Speicher, der mittels NEW() *)
      (*                   angefordert (allociert) und nicht mehr bentigt  *)
      (*                   wird, kann der automatischen Freigabe durch das  *)
      (*                   System unterzogen werden                         *)
      (*         Achtung!  Die Performance groer Programme kann deutlich   *)
      (*                   geringer werden.                                 *)
      Information
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3); (* Groes Fenster ffnen    *)
   bc_Red:=  24; bc_Green:=  16; bc_Blue:=  48; (* HG-Farbe groes Fenster  *)
   bc_red:=   8; bc_green:=  12; bc_blue:=  16; (* HG-Farbe kleines Fenster *)
   C.SetBackColor(bc_Red,bc_Green,bc_Blue);
   C.Bar(5,0,850,650,C.ERASE);
   C.SetForeColor(200,200,200);
   C.Bar(1,0,3,650,C.DRAW);
   C.SetForeColor(96,96,96);
   C.Line(4,0,4,650,C.DRAW);
   C.SetForeColor(96,220,220);
   (* Da die Vordergrundfarbe Schwarz automatisch eingestellt ist,  m u s s *)
   (* die Textfarbe bereits hier initialisiert werden. Sonst sieht man die  *)
   (* Eingabe bei der Anweisung 'C.ReadStr()' nicht!                        *)
   st1:= FALSE;
   st2:= FALSE;
   elementNr1:= 0;
   elementNr2:= 0
 END zwei.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

