
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)      Dateiname: TAGESANZAHL.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    D a t u m  (Tagesanzahl eines Monats und eines Jahres)        |
  |-------------------------------------------------------------------------|
  | Leistung: Ermitteln der Tagesanzahl eines Monats und eines Jahres       |
  |           unter Bercksichtigung des Schaltjahrs                        |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst, das 'Lcken' im Quell- |
  |           text enthlt, befindet sich in:                               |
  |            ...\datum1\good\tagesanz.mod                                 |
  |-------------------------------------------------------------------------|
  | Aufgabenstellung: ...\datum1\txt\datum.txt u. ...\datum1\doc\datum.doc  |
  *-------------------------------------------------------------------------*)
 
 MODULE Tagesanzahl;

  IMPORT In, Out;

  TYPE   tName = ARRAY 10 OF CHAR;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Ueberschrift;

    VAR i : INTEGER;

    BEGIN
      Out.Ln; Out.Ln;
      Out.String('   Tagesanzahl eines Monats und eines Jahres'); Out.Ln;
      Out.String('  '); FOR i:= 1 TO 43 DO Out.Char('') END;
      Out.Ln
    END Ueberschrift;


  PROCEDURE Schaltjahr ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres (DT: INTEGER)           |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            Schaltjahr handelt.                                        |
    |            TRUE = Schaltjahr, FALSE = kein Schaltjahr                 |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    VAR c, j : INTEGER;

    BEGIN
      c:= jahreszahl DIV 100; 
      j:= jahreszahl MOD 100;
      
      IF j = 0 THEN 
        RETURN c MOD 4 = 0
      ELSE
        RETURN j MOD 4 = 0 
      END;
    END Schaltjahr;


  PROCEDURE Monatsname ( monatsnummer : INTEGER; VAR monatsname : tName );
   (*-----------------------------------------------------------------------*
    | bernahme: gltige Monatsnummer (monatsnummer, DT: INTEGER)           |
    | Referenz:  Monatsname (monatsname, DT: ARRAY 10 OF CHAR)              |
    | Leistung:  Die Monatsnummer (case lable) wird innerhalb einer mehr-   |
    |            seitigen Auswahl (case selection) genutzt, um der Variable |
    |            monatsname einen realen Monatsmanem (string) zuzuweisen.   |
    *-----------------------------------------------------------------------*)

    BEGIN
      CASE monatsnummer OF
         1: monatsname:= 'Januar'
       | 2: monatsname:= 'Februar'
       | 3: monatsname:= 'Mrz'
       | 4: monatsname:= 'April'
       | 5: monatsname:= 'Mai'
       | 6: monatsname:= 'Juni'
       | 7: monatsname:= 'Juli'
       | 8: monatsname:= 'August'
       | 9: monatsname:= 'September'
       |10: monatsname:= 'Oktober'
       |11: monatsname:= 'November'
       |12: monatsname:= 'Dezember'
      END;
    END Monatsname;


  PROCEDURE TagesanzahlMonat ( schaltjahr : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme: Wahrheitswert (schaltjahr, DT: BOOLEAN), um die Tagesan-   |
    |            zahl des Monats Februar exakt ermitteln zu knnen.         |
    | Leistung:  Ermitteln und Ausgabe der Tagesanzahl eines Monats unter   |
    |            Bercksichtigung des Sonderfalls Schaltjahr.               |
    |            Wird vom Anwender eine unkorrekte Monatsnummer eingegeben, |
    |            erscheint am Bildschirm eine Fehlermeldung.                |
    *-----------------------------------------------------------------------*)

    VAR anzahl,
        monatsnummer : INTEGER;
        monatsname   : tName;

    BEGIN
      In.Prompt('Monatsnummer');
      In.Int(monatsnummer);
      CASE monatsnummer OF
       |  2: IF schaltjahr THEN anzahl:= 29
               ELSE anzahl:= 28
             END
       |  1, 3, 5, 7, 8, 10, 12: anzahl:= 31
       |  4, 6, 9, 11: anzahl:= 30
      ELSE
        Out.Ln; Out.String('   Fehlerhafte Monatsnummer!'); Out.Ln;
        anzahl:= 0
      END;
      IF anzahl > 0 THEN
        Out.Ln; Out.String('   Der Monat ');
        Monatsname(monatsnummer,monatsname); Out.String(monatsname);
        Out.String(' besitzt ');
        IF schaltjahr & (monatsnummer = 2) THEN
          Out.String('in diesem Jahr ')
        END;
        Out.Int(anzahl,1);
        Out.String(' Tage.'); Out.Ln
      END
    END TagesanzahlMonat;


  PROCEDURE TagesanzahlJahr ( VAR korrekt, schaltjahr : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | Referenz: Wahrheitswerte (korrekt und schaltjahr, DT: BOOLEAN)        |
    |           korrekt    = TRUE, die eingelesene Jahreszahl ist gltig    |
    |           schaltjahr = TRUE, das zur eingegebenen Jahreszahl gehren- |
    |                              de Kalenderjahr ist (wird) ein Schalt-   |
    |                              jahr (sein)                              |
    | Leistung: Ermitteln und Ausgabe der Anzahl der Tage eines Kalender-   |
    |           jahrs unter Bercksichtigung des Sonderfalls Schaltjahr.    |
    |           Wird vom Anwender eine unkorrekte Jahreszahl eingegeben,    |
    |           erscheint am Bildschirm eine Fehlermeldung.                 |
    *-----------------------------------------------------------------------*)

    VAR jahreszahl : INTEGER;

    BEGIN
      In.Prompt('Jahreszahl  ( jjjj )');
      In.Int(jahreszahl);
      korrekt:= TRUE;
      IF (jahreszahl < 1583) OR (jahreszahl > 3000) THEN
        Out.Ln;
        Out.String('  Nur Jahreszahlen von 1583 bis 3000 eingeben!');
        Out.Ln; Out.Ln;
        korrekt:= FALSE
      END;
      IF korrekt THEN
        Out.Ln; Out.String('   Das Jahr'); Out.Int(jahreszahl,5);
        Out.String(' besitzt');
        IF Schaltjahr(jahreszahl) THEN
          Out.Int(366,4);
          schaltjahr:= TRUE
        ELSE  
          Out.Int(365,4);
          schaltjahr:= FALSE
        END;
        Out.String(' Tage.'); Out.Ln
      END
    END TagesanzahlJahr;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR korrekt,
        schaltjahr : BOOLEAN;

    BEGIN
      Ueberschrift;
      TagesanzahlJahr(korrekt,schaltjahr);
      IF korrekt THEN
        TagesanzahlMonat(schaltjahr)
      END
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END Tagesanzahl.

 (* ----------------------------------------------------------------------- *)

