
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)         Dateiname: EXISTENZ.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    D a t u m  (korrektes Datum)                                  |
  |-------------------------------------------------------------------------|
  | Leistung: Feststellen, ob drei eingelesene ganze Zahlen ein korrektes   |
  |           Datum reprsentieren.                                         |
  |           Zu beachten ist die Reihenfolge: Jahr Monat Tag!              |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst, das eine 'Lcke' im    |
  |           Quelltext enthlt, befindet sich in:                          |
  |            ...\datum1\good\existenz.mod                                 |
  |-------------------------------------------------------------------------|
  | Aufgabenstellung: ...\datum1\txt\datum.txt und                          |
  |                   ...\datum1\doc\datum.doc                              |
  *-------------------------------------------------------------------------*)

 MODULE Existenz;

  IMPORT In, Out;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Ueberschrift;

    VAR i : INTEGER;

    BEGIN
      Out.Ln; Out.Ln;
      Out.String('   Existenz eines Tages (gregorianischer Kalender)'); Out.Ln;
      Out.String('  '); FOR i:= 1 TO 49 DO Out.Char('') END;
      Out.Ln
    END Ueberschrift;


  PROCEDURE Beispieldaten;
    BEGIN
      Out.Ln;
      Out.String('   Beispieldaten:  ');
      Out.String('2120  5 17  3001 15  7  2100 15 47  2002  1 32'); Out.Ln;
      Out.String('                   ');
      Out.String('2003  4 31  2002  2 29  2000  2 29  2001  6 28'); Out.Ln;
      Out.Ln
    END Beispieldaten;


  PROCEDURE Schaltjahr ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres                         |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            Schaltjahr handelt.                                        |
    |            TRUE = Schaltjahr, FALSE = kein Schaltjahr                 |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    VAR c, j : INTEGER;

    BEGIN
      c:= jahreszahl DIV 100; 
      j:= jahreszahl MOD 100;
      IF j = 0 THEN 
        RETURN c MOD 4 = 0
      ELSE
        RETURN j MOD 4 = 0 
      END;
    END Schaltjahr;


  PROCEDURE JahreszahlKorrekt ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres                         |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Liegt der Wert der Variablen jahreszahl nicht im Gltig-   |
    |            keitsbereich, wird eine Fehlermeldung ausgegeben.          |
    |            Jahreszahl korrekt:   Funktionswert = TRUE                 |
    |            Jahreszahl unkorrekt: Funktionswert = FALSE                |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF (jahreszahl < 1583) OR (jahreszahl > 3000) THEN
        Out.Ln;
        Out.String('   Nur Jahreszahlen von 1583 bis 3000 eingeben!');
        Out.Ln; Out.Ln;
        RETURN FALSE
      ELSE
        RETURN  TRUE
      END
    END JahreszahlKorrekt;


  PROCEDURE MonatsnummerKorrekt ( monatsnummer : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Monatsnummer des zu testenden Monats                       |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Liegt die bergebene Monatsnummer nicht im gltigen Werte- |
    |            bereich, wird eine Fehlermeldung ausgegeben.               |
    |            Monatsnummer korrekt:   Funktionswert = TRUE               |
    |            Monatsnummer unkorrekt: Funktionswert = FALSE              |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF (monatsnummer < 1) OR (monatsnummer > 12) THEN
        Out.Ln;
        Out.String('   Die Monate haben die Nummern von 1 bis 12!');
        Out.Ln; Out.Ln;
        RETURN FALSE
      ELSE
        RETURN  TRUE
      END
    END MonatsnummerKorrekt;


  PROCEDURE Tagesanzahl ( jahreszahl, monatsnummer, tagesnummer : INTEGER;
                               VAR anzahl : INTEGER; VAR korrekt : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monatsnummer und Tagesnummer des zu testenden  |
    |            Kalendermonats                                             |
    | Referenz:  Tagesanzahl dieses Monats und Wahrheitswert der booleschen |
    |            Variablen korrekt                                          |
    | Leistung:  Innerhalb der Anweisungsfolge einer mehrseitigen Auswahl   |
    |            wird, unter Bercksichtigung eines mglichen Schaltjahrs,  |
    |            der Variablen anzahl ein von der Monatsnummer abhngiger   |
    |            Wert zugewiesen. Liegt der Wert der Variablen tagesnummer  |
    |            zwischen eins und dem Wert der Variablen anzahl, bekommt   |
    |            die boolesche Variable den Wert TRUE, ansonsten FALSE zu-  |
    |            gewiesen.                                                  |
    *-----------------------------------------------------------------------*)

    BEGIN
      CASE monatsnummer OF
         1, 3, 5, 7, 8, 10, 12: anzahl:= 31 
       | 4, 6, 9, 11:           anzahl:= 30
       | 2:                     IF Schaltjahr(jahreszahl) THEN
                                  anzahl:= 29
                                ELSE
                                  anzahl:= 28
                                END
      END;
      IF tagesnummer > anzahl THEN
        Out.Ln;
        Out.String('   Die Anzahl der Tage im Monat muss beachtet werden!');
        Out.Ln; Out.Ln;
        korrekt:= FALSE
      ELSE
        korrekt:= TRUE
      END
    END Tagesanzahl;


  PROCEDURE Test;
   (*-----------------------------------------------------------------------*
    | Leistung: Bewerten eines vom Anwender eingegebenen Kalenderdatums     |
    |           (gregorianischer Kalender) auf Korrektheit.                 |
    |           Ausgabe dieses Datums im Ausgabebereich des Bildschirms und |
    |           Besttigen der Korrektheit oder Anzeige der Inkorrektheit.  |
    *-----------------------------------------------------------------------*)
    
    VAR korrekt      : BOOLEAN;
        anzahl,
        jahreszahl,
        monatsnummer,
        tagesnummer  : INTEGER;
        

    BEGIN
      REPEAT
        In.Prompt('Jahreszahl  ( jjjj )');
        In.Int(jahreszahl);
      UNTIL JahreszahlKorrekt(jahreszahl);
      REPEAT
        In.Prompt('Monatsnummer');
        In.Int(monatsnummer)
      UNTIL MonatsnummerKorrekt(monatsnummer);
      REPEAT
        In.Prompt('Tagesnummer');
        In.Int(tagesnummer);
        Tagesanzahl(jahreszahl,monatsnummer,tagesnummer,anzahl,korrekt)
      UNTIL korrekt;
      Out.Ln;
      Out.String('   Das eingegebene Datum lautet:');
      Out.Int(jahreszahl,6); Out.Int(monatsnummer,3); Out.Int(tagesnummer, 3);
      Out.Ln; Out.Ln;
      IF korrekt THEN
        Out.String('   Das Datum ist korrekt.')
      ELSE
        Out.String('   Das Datum ist nicht korrekt.')
      END
    END Test;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();
    BEGIN
      Ueberschrift;
      Beispieldaten;
      Test
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END Existenz.

 (* ----------------------------------------------------------------------- *)


