 
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)         Dateiname: RECHNUNG.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    D a t u m  (Wochentagsnamen ermitteln)                        |
  |-------------------------------------------------------------------------|
  | Leistung: Ermitteln des Wochentagsnamens eines 'beliebigen' Datums.     |
  |           Mit Hilfe der Kalenderformel nach JACOBSTHAL kann fr ein ge- |
  |           gebenes Datum (Jahr, Monat, Tag) der dazugehrige Wochentags- |
  |           name ermittelt werden.                                        |
  |           Es drfen nur korrekte Kalenderdaten eingegeben werden, da    |
  |           k e i n e  Prfung der Existenz vorgenommen wird.             |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |           (siehe ...\datum1\fine\rechnung.mod)                          |
  |-------------------------------------------------------------------------|
  | Aufgabenstellung: ...\datum1\txt\datum.txt u. ...\datum1\doc\datum.doc  |
  | Erluterungstext: ...\datum1\txt\rechnung.txt und                       |
  |                   ...\datum1\doc\rechnung.doc                           |
  *-------------------------------------------------------------------------*)

 MODULE Rechnung;

  IMPORT In, Out;


 (* ---------------------  U N T E R P R O G R A M M  --------------------- *)


  PROCEDURE Ueberschrift;

    VAR i : INTEGER;

    BEGIN
      Out.Ln; Out.Ln;
      Out.String('   Wochentagsname eines Kalenderdatums'); Out.Ln;
      Out.String('  '); FOR i:= 1 TO 37 DO Out.Char('') END;
      Out.Ln
    END Ueberschrift;


  PROCEDURE Beispieldaten;
    BEGIN
      Out.Ln;
      Out.String('   Beispieldaten:  ');
      Out.String('1600 1 1  =  Samstag    2001 4 18  =  Mittwoch'); Out.Ln;
      Out.Ln
    END Beispieldaten;


  PROCEDURE Gueltig ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres (jahreszahl)            |
    | Referenz:  Wahrheitswert als Rckgabewert Funktionswert)              |
    |            TRUE = korrekt, FALSE = nicht korrekt (siehe Hinweis)      |
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            nach dem gregorianischen Kalender gltiges Jahr handelt.   |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF (jahreszahl < 1583) OR (jahreszahl > 3000) THEN
        Out.Ln;
        Out.String('   Nur Jahreszahlen von 1583 bis 3000 eingeben!'); Out.Ln;
        Out.String('           (gregorianischer Kalender)'); Out.Ln;
        RETURN FALSE
      ELSE
        RETURN TRUE
      END;
    END Gueltig;


  PROCEDURE Schaltjahr ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres (DT: INTEGER)           |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            Schaltjahr handelt.                                        |
    |            TRUE = Schaltjahr, FALSE = kein Schaltjahr                 |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    VAR c, j : INTEGER;

    BEGIN
      c:= jahreszahl DIV 100; 
      j:= jahreszahl MOD 100;

      IF j = 0 THEN 
        RETURN c MOD 4 = 0
      ELSE
        RETURN j MOD 4 = 0 
      END;
    END Schaltjahr;


  PROCEDURE Wochentagsname;
   (*-----------------------------------------------------------------------*
    | Leistung: Ermitteln des Wochentagsnamens eines nach dem  gregoriani-  |
    |           schen Kalender gltigen Datums mit Hilfe der Kalenderformel |
    |           nach JACOBSTHAL.                                            |
    *-----------------------------------------------------------------------*)

    VAR c, d, j, k,
        jahreszahl, 
        monatsnummer,
        tagesnummer  : INTEGER;
        schaltjahr   : BOOLEAN;

    BEGIN
      REPEAT
        In.Prompt('Jahreszahl  ( jjjj )');
        In.Int(jahreszahl);
      UNTIL Gueltig(jahreszahl);
      In.Prompt('Monat  ( mm )');
      In.Int(monatsnummer);
      In.Prompt('Tag  ( tt )');
      In.Int(tagesnummer);
      Out.Ln; Out.String('   Datum:');
      Out.Int(jahreszahl,14); Out.Int(monatsnummer,3); Out.Int(tagesnummer,3);
      Out.Ln; Out.Ln;

      schaltjahr:= Schaltjahr(jahreszahl);

      CASE monatsnummer OF
          1: IF schaltjahr THEN k:= 5 ELSE k:= 6 END;
       |  2: IF schaltjahr THEN k:= 1 ELSE k:= 2 END;
       |  3: k := 2
       |  4: k := 5
       |  5: k := 0
       |  6: k := 3
       |  7: k := 5
       |  8: k := 1
       |  9: k := 4
       | 10: k := 6
       | 11: k := 2
       | 12: k := 4
      END;

      c:= jahreszahl DIV 100; 
      j:= jahreszahl MOD 100;

      d:= tagesnummer + k + j + j DIV 4 - 2 * (c MOD 4);
    
      Out.String('   Wochentagsname: ');
      
      CASE d MOD 7 OF
         0: Out.String('Sonntag')
       | 1: Out.String('Montag')
       | 2: Out.String('Dienstag')
       | 3: Out.String('Mittwoch')
       | 4: Out.String('Donnerstag')
       | 5: Out.String('Freitag')
       | 6: Out.String('Samstag')
      END;

      Out.Ln
    END Wochentagsname;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();
    BEGIN
      Ueberschrift;
      Beispieldaten;
      Wochentagsname
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END Rechnung.

 (* ----------------------------------------------------------------------- *)
 
