
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)          Dateiname: HILBERT.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    HILBERT-Kurven, David HILBERT (1891)                          |
  |           Vorwrtsdeklarationen von Prozeduern                          |
  |-------------------------------------------------------------------------|
  | Leistung: Darstellen hilbertscher Kurven 2. - 7. Ordnungen.             |
  |           Das Modul enthlt Vorwrtsdeklarationen von Prozeduern        |
  |           (Erweiterung der Sichtbarkeitsregel)                          |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |           ...\hilbert\good\hilbert.mod                                  |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\hilbert\txt\hilbert.txt                           |
  |                   ...\hilbert\doc\hilbert.doc                           |
  | Eingabehinweise:  ...\hilbert\doc\hilbtool.txt                          |
  | (Turtle/Hilbert)  ...\hilbert\doc\hilbtool.doc                          |
  *-------------------------------------------------------------------------*)

 MODULE Hilbert;

  IMPORT Cpl:= ColorPlane, In, Out, S:= Strings, T:= Turtle, W:= Windows;

  CONST  delayTime = 100;

  VAR    dummy   : LONGINT;
         h, msec : INTEGER;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Delay;
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR   i : LONGINT;

   BEGIN
     FOR i:= 1 TO delayTime * msec DO Out.String('') END 
   END Delay;


  PROCEDURE Delay_ms;
   (*-----------------------------------------------------------------------*
    | Leistung: Eine definierte Zeit warten.                                |
    | ACHTUNG!  Diese Procedure ist besonders fr sehr schnelle Prozessoren |
    |           geeignet (Prozessortakt >= 1 GHz)                           |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE Ost;
   (*-----------------------------------------------------------------------*
    | Leistung: Diese Prozedur zeichnet eine Strecke der Lnge h von der    |
    |           aktuellen Position nach rechts.                             |
    *-----------------------------------------------------------------------*)
      
    BEGIN 
      T.Aufkurs(0);
      T.Vorwaerts(h);
      Delay(*_ms*)
    END Ost;


  PROCEDURE Nord;
   (*-----------------------------------------------------------------------*
    | Leistung: Diese Prozedur zeichnet eine Strecke der Lnge h von der    |
    |           aktuellen Position nach oben.                               |
    *-----------------------------------------------------------------------*)

    BEGIN 
      T.Aufkurs(90);
      T.Vorwaerts(h);
      Delay(*_ms*)
    END Nord;


  PROCEDURE West;
   (*-----------------------------------------------------------------------*
    | Leistung: Diese Prozedur zeichnet eine Strecke der Lnge h von der    |
    |           aktuellen Position nach links.                              |
    *-----------------------------------------------------------------------*)

    BEGIN 
      T.Aufkurs(180);
      T.Vorwaerts(h);
      Delay(*_ms*)
    END West;


  PROCEDURE Sued;
   (*-----------------------------------------------------------------------*
    | Leistung: Diese Prozedur zeichnet eine Strecke der Lnge h von der    |
    |           aktuellen Position nach unten.                              |
    *-----------------------------------------------------------------------*)

    BEGIN 
      T.Aufkurs(270);
      T.Vorwaerts(h);
      Delay(*_ms*)
    END Sued;


  PROCEDURE ^B ( i : INTEGER );     (* Vorwrtsdeklarationen der Prozeduern *)
  PROCEDURE ^C ( i : INTEGER );     (* B, C, D. Damit erfolgt eine Erweite- *)
  PROCEDURE ^D ( i : INTEGER );     (* rung der Sichtbarkeitsregel          *)


  PROCEDURE A ( i : INTEGER );
    BEGIN
      IF i > 0 THEN
        D(i - 1); West;
        A(i - 1); Sued;
        A(i - 1); Ost;
        B(i - 1)
      END
    END A;


  PROCEDURE B ( i : INTEGER );
    BEGIN
      IF i > 0 THEN
        C(i - 1); Nord;
        B(i - 1); Ost;
        B(i - 1); Sued;
        A(i - 1)
      END
    END B;


  PROCEDURE C ( i : INTEGER );
    BEGIN
      IF i > 0 THEN
        B(i - 1); Ost;
        C(i - 1); Nord;
        C(i - 1); West;
        D(i - 1)
      END
    END C;


  PROCEDURE D ( i : INTEGER );
    BEGIN
      IF i > 0 THEN
        A(i - 1); Sued;
        D(i - 1); West;
        D(i - 1); Nord;
        C(i - 1)
      END
    END D;


  PROCEDURE Beschriften ( ordnung : INTEGER );

    VAR ordnungStr : ARRAY 30 OF CHAR;

    BEGIN
      S.Str(ordnung,ordnungStr);
      S.Insert('Hilbertsche Kurve ',ordnungStr,1);
      S.Append(ordnungStr,'. Ordnung');
      Cpl.GotoXY(275,20);
      Cpl.WriteStr(ordnungStr);
    END Beschriften;


  PROCEDURE Zeichne;
   (*-----------------------------------------------------------------------*
    | Leistung: Einlesen der Werte fr die Ordnung, die Startkoordinaten    |
    |           der Grafik und die Linienlnge (DT: alle INTEGER),          |
    |           Initialisieren der Turtelposition und 'Startschuss' fr das |
    |           Ausfhren des Hilbertalgorithmus'.                          |
    *-----------------------------------------------------------------------*
    | Hinweis:  Die Zahlen knnen im Eingabefenster auch als Stream (Einga- |
    |           bestrom) eingegeben werden.                                 |
    | Beispiel: '5 385 215 5' ohne Hochkommas eingeben und mit < Enter >    |
    |           abschlieen.                                                |
    *-----------------------------------------------------------------------*)

    VAR ordnung, x0, y0: INTEGER;

    BEGIN
      In.Prompt('Ordnung');
      In.Int(ordnung);
      In.Prompt('x-Koordinate');
      In.Int(x0);
      In.Prompt('y-Koordinate');
      In.Int(y0);
      In.Prompt('Linienlnge');
      In.Int(h);
      In.Prompt('Wartezeit');
      In.Int(msec);
      IF msec = 0 THEN msec:= 1 END;      
      T.BildNeu(x0,y0);
      Beschriften(ordnung); 
      A(ordnung)
    END Zeichne;


  PROCEDURE Wiederholung () : BOOLEAN;                 (* Funktionsprozedur *)

    VAR schluessel : CHAR;

    BEGIN
      In.Prompt('Wiederholung (j/n)');
      In.Char(schluessel);
      IF CAP(schluessel) = 'N' THEN  (* CAP(char) = wandelt einen Klein- in *)
        RETURN FALSE                 (*             einen Grobuchstaben    *)
      ELSE
        RETURN TRUE
      END
    END Wiederholung;


  PROCEDURE AusgabeBeispieldaten;
    BEGIN
      Out.Ln;
      Out.String('                Beispiel 1   Beispiel 2   Beispiel 3');
      Out.String('   Beispiel 4   Beispiel 5   Beispiel 6'); Out.Ln;
      Out.String('   Ordnung:          2            3            4');
      Out.String('            5            6            7'); Out.Ln;
      Out.String('   x-Koordinate:   490          490          500');
      Out.String('          510          515          515'); Out.Ln;
      Out.String('   y-Koordinate:   280          280          290');
      Out.String('          300          300          310'); Out.Ln;
      Out.String('   Linienlnge:     70           30           15');
      Out.String('            8            4            2'); Out.Ln;
      Out.String('   Wartezeit:      100           50           25');
      Out.String('           10            5            1'); Out.Ln
    END AusgabeBeispieldaten;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();
    BEGIN
      AusgabeBeispieldaten;
      REPEAT
        Zeichne;
      UNTIL ~Wiederholung()
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   dummy:= W.ShowWindow(W.GetActiveWindow(),3)
 END Hilbert.

 (* ----------------------------------------------------------------------- *)

