
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)       Datei-Name: LABYRINTH.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    S u c h e   i m   L a b y r i n t h   (Backtracking)          |
  |-------------------------------------------------------------------------|
  | Leistung: Das Programm liest nach dem Start die Textdatei bild.txt      |
  |           automatisch ein, zeigt das Labyrinth am Bildschirm an und     |
  |           sucht alle mglichen Wege durch dieses Labyrinth.             |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |            ...\laby\good\labyrint.mod                                   |
  |           Eine Demoversion des Programms, die maussentitiv ist, befin-  |
  |           det sich unter ...\laby\demo\labydemo.mod                     |
  |-------------------------------------------------------------------------|
  | Erluterungstext:  ...\laby\txt\labyrinth.txt                           |
  |                    ...\laby\doc\labyrinth.doc                           |
  *-------------------------------------------------------------------------*)

 MODULE Labyrinth;

  IMPORT C:= ColorPlane, F:= File, S:= Strings, W:= Windows;

  CONST dTime     =  25;            (* 25 ist ein gut geeigneter Mittelwert *)
        faktor    =   9;
        xOffset   =  50;
        xText     = 410;
        yOffset   =  30;

        fPfad     = 'BILD01.TXT';

  TYPE  tBild     = ARRAY 80, 80 OF CHAR;
        tKoordStr = ARRAY  3 OF CHAR;

  VAR   bc_Red,
        bc_Green,
        bc_Blue,                               (* HG-Farben groes Fenster  *)
        xMax, yMax,
        xStart, yStart,
        xZiel, yZiel   : INTEGER;
        laby           : tBild;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Einlesen;
   (*-----------------------------------------------------------------------*
    | Leistung: Falls der Pfad gltig ist, werden folgende Aktionen ausge-  |
    |           fhrt:                                                      |
    |            - Datei ffnen (fHandle = Schnittstelle fr WINDOWS)       |
    |            - Block lesen der Gre des DT: tKoordStr lesen und in     |
    |              Zahl wandeln (xMax, DT: INTEGER), Breite des Labyrinths  |
    |            - Wiederholung der Operationen fr yMax, Hhe des Labys    |
    |            - Eintritt in zwei Zhlschleifen, um die Daten fr den Bau |
    |              des Labyrinths zeichenweise einzulesen                   |
    |            - falls das eingelesene Zeichen eine 3 oder 4 ist, wurden  |
    |              die Koordinaten des Start und des Zielpunks der Wegsuche |
    |              durch das Labyrinth 'gefunden'                           |
    |            - ist die innere Schleife komplett abgearbeitet (x= xMax), |
    |              wird ein Zeichen gelesen, dem keine Zuweisung folgt, da  |
    |              hiermit nur der Zeilenwechsel berlesen wird
    *-----------------------------------------------------------------------*)

    VAR KoordStr : tKoordStr;
        CR       : BOOLEAN;
        zeichen  : CHAR;
        fHandle  : F.Handle;
        anzahl,
        resCode,
        x, y     : INTEGER;
        read     : LONGINT;

    BEGIN
      IF F.Exist(fPfad) THEN
        F.Open(fPfad,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode);
        F.ReadBlock(fHandle,KoordStr,SIZE(tKoordStr),read,resCode);
        xMax:= SHORT(S.Val(KoordStr));      (* xMax = Breite des Labyrinths *)
        F.ReadBlock(fHandle,KoordStr,SIZE(tKoordStr),read,resCode);
        yMax:= SHORT(S.Val(KoordStr));      (* yMax = Hhe des Labyrinths   *)
        FOR y:= yMax TO 0 BY -1 DO
          FOR x:= 0 TO xMax DO
            F.ReadChar(fHandle,zeichen,resCode);
            laby[x,y]:= zeichen;
            IF zeichen = '3' THEN xStart:= x; yStart:= y END;
            IF zeichen = '4' THEN xZiel:= x;  yZiel:= y END;
          END;
          F.ReadChar(fHandle,zeichen,resCode); (* Zeilenwechsels berlesen  *)
        END
      END
    END Einlesen;


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine definierte Zeit warten.                                |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE Delay ( dt : INTEGER);
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    | Hinweis:  Diese Prozedur kommt nur zum Einsatz, falls der Computer    |
    |           ber geringe Rechenleistung verfgt. Dazu sind an den Pro-  |
    |           duraufrufen Delay_ms die drei Zeichen '_ms' zu lschen      |
    |           Sollte dann die Suche zu schnell ablaufen, muss die global  |
    |           deklarierte Konstante dTime gendert werden.                |
    *-----------------------------------------------------------------------*)

    VAR i : LONGINT;

    BEGIN
      FOR i:= 0 TO dt DO C.WriteStr('') END;
    END Delay;


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR dummy : CHAR;

    BEGIN
      WHILE C.KeyPressed() DO
        dummy:= C.ReadKey()
      END
    END FlushKeyBuffer;


  PROCEDURE Enter ( x, y : INTEGER; DelayTime : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    *-----------------------------------------------------------------------*)

    VAR dummy : CHAR;

    BEGIN
      Delay_ms(dTime * 5);
      C.SetForeColor(0,100,100);
      C.GotoXY(x,y); C.WriteStr('< ENTER >');
      FlushKeyBuffer;
      dummy:= C.ReadKey();
      C.Bar(x,y-14,x+72,y,C.ERASE);
      C.SetForeColor(0,0,0)
    END Enter;


  PROCEDURE StartZielSetzen ( x, y : INTEGER; flag : BOOLEAN);
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate des Start- bzw. Zielsymbols, der Wert  |
    |            der booleschen Variablen flag hlt die ntige Information: |
    |             - TRUE  fr Startsymbol darstellen                        |
    |             - FALSE fr Zielsymbol darstellen                         |
    | Leistung:  Darstellen des farbigen Start- und Zielsymbols             |
    |             - Rot  = Startsymbol                                      |
    |             - Grn = Zielsymbol                                       |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF flag THEN
        C.SetForeColor(200,80,0)
      ELSE
        C.SetForeColor(80,200,0)
      END;
      C.Bar(faktor*x+2+xOffset,faktor*y+2+yOffset,
            faktor*x+6+xOffset,faktor*y+6+yOffset,C.DRAW);
      C.SetForeColor(0,0,0);
      C.Box(faktor*x+1+xOffset,faktor*y+1+yOffset,
            faktor*x+7+xOffset,faktor*y+7+yOffset,C.DRAW)
    END StartZielSetzen;


  PROCEDURE Bauen;
   (*-----------------------------------------------------------------------*
    | Leistung: Das gesamten Labyrinth wird zeilenweise samt Start- und     |
    |           und Zielsymbol dargestellt.                                 |
    *-----------------------------------------------------------------------*)

    VAR zeichen : CHAR;
        zei     : ARRAY 2 OF CHAR;
        x, y    : INTEGER;

    PROCEDURE MauerSetzen ( x, y : INTEGER );
      BEGIN
        C.Bar(faktor*x+xOffset,faktor*y+yOffset,
              faktor*x+8+xOffset,faktor*y+8+yOffset,C.DRAW)
      END MauerSetzen;

    BEGIN  (* Bauen *)
      FOR y:= 1 TO yMax DO
        FOR x:= 0 TO xMax-1 DO
          zeichen:= laby[x,y];
          IF (zeichen = '1') OR (zeichen = '2') THEN MauerSetzen(x,y) END
        END
      END;
      StartZielSetzen(xStart,yStart,TRUE);
      StartZielSetzen(xZiel,yZiel,FALSE)
    END Bauen;


  PROCEDURE Suchen ( VAR i : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: Anzahl gefundener Wege durch das Labyrinth                  |
    | Leistung: Schritt fr Schritt (Rekursion) wird ein Weg durch das La-  |
    |           byrinth gesucht. Dabei wird die 'Linke-Hufregel', deren     |
    |           Entwickler Minotaurus gewesen sein soll, dauerhaft angewen- |
    |           det. Der bereits zurckgelegte Weg einer Wegsuche wird mit  |
    |           kleinen Farbflchen gekennzeichnet, Prozedur SteinSetzen(). |
    |           Von Wegen die rckwrts beschritten werden mssen, Irrwege, |
    |           werden die Farbflchen wieder entfernt, Prozedur SteinEnt-  |
    |           nen().                                                      |
    *-----------------------------------------------------------------------*)

    PROCEDURE SteinSetzen ( x, y : INTEGER );
      BEGIN
        Delay_ms(dTime);
        C.SetForeColor(0,80,200);
        C.Bar(faktor*x+2+xOffset,faktor*y+2+yOffset,
              faktor*x+6+xOffset,faktor*y+6+yOffset,C.DRAW);
      END SteinSetzen;
      
    PROCEDURE SteinEntfernen ( x, y : INTEGER );
      BEGIN
        Delay_ms(dTime DIV 2);
        C.Bar(faktor*x+2+xOffset,faktor*y+2+yOffset,
              faktor*x+6+xOffset,faktor*y+6+yOffset,C.ERASE);
      END SteinEntfernen;
      
    PROCEDURE Schritt ( xAlt, yAlt : INTEGER );

      VAR zahlStr    : ARRAY 3 OF CHAR;
          richtung,
          xNeu, yNeu : INTEGER;

      BEGIN
        FOR richtung:= 1 TO 4 DO
          CASE richtung OF
              1: xNeu:= xAlt + 1; yNeu:= yAlt
            | 2: xNeu:= xAlt;     yNeu:= yAlt + 1
            | 3: xNeu:= xAlt - 1; yNeu:= yAlt
            | 4: xNeu:= xAlt;     yNeu:= yAlt - 1
           END;
           IF laby[xNeu,yNeu] = '4' THEN
             INC(i);
             S.Str(i,zahlStr);
             C.GotoXY(428,260); C.WriteStr(zahlStr); C.WriteStr('. Lsung');
             Enter(500,35,dTime*5)
           END;
           IF laby[xNeu,yNeu] = '0' THEN
             SteinSetzen(xNeu,yNeu);  
             laby[xNeu,yNeu]:= '5';  
             Schritt(xNeu,yNeu);                       (* rekursiver Aufruf *)
             laby[xNeu,yNeu]:= '0';  
             SteinEntfernen(xNeu,yNeu)
           END
        END
      END Schritt;
      
    BEGIN  (* Suchen *)
      i:= 0;
      Schritt(xStart,yStart)                           (* erster Aufruf     *)
    END Suchen;


  PROCEDURE Beschriften;
    BEGIN
      StartZielSetzen(42,30,TRUE);
      StartZielSetzen(42,28,FALSE);
      C.GotoXY(444,311); C.WriteStr('= Start');
      C.GotoXY(444,293); C.WriteStr('= Ziel');
      Enter(500,35,dTime*5)
    END Beschriften;


  PROCEDURE AnzahlLoesungen ( i : INTEGER );

    VAR zahlStr : ARRAY 3 OF CHAR;

    BEGIN
      S.Str(i,zahlStr);
      C.SetForeColor(0,80,200);
      C.GotoXY(428,260); C.WriteStr('Es existieren ');
      C.GotoXY(428,240); C.WriteStr(zahlStr); C.WriteStr(' Lsungen');
      Enter(500,35,dTime*5)
    END AnzahlLoesungen;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR i : INTEGER;

    BEGIN
      Einlesen;
      Bauen;
      Beschriften;
      i:= 0;
      Suchen(i);
      AnzahlLoesungen(i)
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open
 END Labyrinth.

 (* ----------------------------------------------------------------------- *)

