
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)         Dateiname: MONODEMO.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    Monoalpabetische Verschlsselung                              |
  |-------------------------------------------------------------------------|
  | Leistung: Verschlsseln und Entschlsseln eines Textes mittels          |
  |           monoalphabetischer Substitution.                              |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |            ...\mono\good\mono.mod                                       |
  *-------------------------------------------------------------------------*)


 MODULE MonoDemo;

  IMPORT D:= Display, F:= File, S:= Strings, W:= Windows;


  CONST  delayTime = 25;
         
         fPfad1    = 'ALPHABET.TXT';
         fPfad2    = 'GEHEIMTX.TXT';

         message   = 'Die Datei, die den Geheimtext enthlt, wurde ';

  TYPE   tAlphabet = ARRAY 26 OF CHAR;
         tText     = ARRAY 60 OF CHAR;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE Enter ( spalte, zeile : INTEGER; dTime : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;

    BEGIN
      Delay_ms(dTime * 4);
      D.WriteStrXY(spalte,zeile,'< ENTER >');
      D.FlushKeyBuffer;
      Dummy:= D.ReadKey();
      D.WriteStrXY(spalte,zeile,'         ');
    END Enter;


  PROCEDURE Pfadfehler;
    BEGIN
      D.WriteStrXY(24,23,'Die Datei wurde nicht gefunden!');
      HALT(1)
    END Pfadfehler;


  PROCEDURE ErstesAlphabet ( wahl : INTEGER; VAR alphabet : tAlphabet );
   (*-----------------------------------------------------------------------*
    | Leistung: Einlesen der Elemente der Reihung alphabet1 aus der Text-   |
    |           datei ...\ALPHABET.TXT.                                     |
    *-----------------------------------------------------------------------*)

    VAR name       : ARRAY 65 OF CHAR;
        zeichen    : CHAR;
        fHandle    : F.Handle;
        anzahl, i,
        resCode    : INTEGER;

    BEGIN
      IF F.Exist(fPfad1) THEN
        F.Open(fPfad1,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode)
      ELSE
        Pfadfehler
      END;
      FOR i:= 0 TO 82 DO
        F.ReadChar(fHandle,zeichen,resCode);
        IF ~(zeichen = CHR(13)) & ~(zeichen = CHR(10)) THEN
                    (* CHR(13) = carage return  und  CHR(10) = line feed *)
          CASE wahl OF
            1: IF i < 26 THEN alphabet[i]:= zeichen END
           |2: IF (i > 27) & (i < 54) THEN alphabet[i-28]:= zeichen END
           |3: IF (i > 55) THEN alphabet[i-56]:= zeichen END
          ELSE
          END
        END
      END;
      F.Close(fHandle);
      D.WriteStrXY(5,3,'1. Alphabet eingelesen')
    END ErstesAlphabet;


  PROCEDURE ZweitesAlphabet ( alphabet1 : tAlphabet; VAR alphabet2 : tAlphabet );
   (*-----------------------------------------------------------------------*
    | Leistung: Ermitteln der Elemente der Reihung alphabet2.               |
    *-----------------------------------------------------------------------*)

    VAR gefunden : BOOLEAN;
        zeichen  : CHAR;
        i, k     : INTEGER;

    BEGIN
      FOR i:= 0 TO 25 DO
        zeichen:= CHR(65+i);
        gefunden:= FALSE;
        k:= 0;
        WHILE ~gefunden & (k <= 25) DO
          IF alphabet1[k] = zeichen THEN
            gefunden:= TRUE 
          ELSE
            INC(k)
          END
        END;
        IF ~gefunden THEN
          HALT(200)
        END;                 
        alphabet2[i]:= CHR(97+k)
      END;
      D.WriteStrXY(5,5,'2. Alphabet ermittelt')
    END ZweitesAlphabet;
    

  PROCEDURE HoleBuchstabe ( spalte : INTEGER ) : CHAR;
   (*-----------------------------------------------------------------------*
    | Leistung: Einen Kleinbuchstaben von der Tastatur einlesen. Falls ver- |
    |           sehentlich ein Grobuchstabe eingegeben wurde, wird er in   |
    |           einen Kleinbuchstaben gewandelt.                            |
    *-----------------------------------------------------------------------*)

    CONST zeile = 8;

    VAR buchstabe : CHAR;

    BEGIN
      D.WriteCharXY(spalte,zeile,'_');
      REPEAT
        buchstabe:= D.ReadKey();
        IF (buchstabe >= 'A') & (buchstabe <= 'Z') THEN
          buchstabe:= CHR(ORD(buchstabe)+32)  (* in Kleinbuchstaben wandeln *)
        END;
      UNTIL (ORD(buchstabe) = 13) OR (buchstabe >= 'a') & (buchstabe <= 'z');
      IF ORD(buchstabe) # 13 THEN
        D.WriteCharXY(spalte,zeile,buchstabe)
      ELSE
        D.WriteCharXY(spalte,zeile,' ')
      END;
      RETURN buchstabe
    END HoleBuchstabe;


  PROCEDURE EinlesenKlartext ( VAR klartext : tText );
   (*-----------------------------------------------------------------------*
    | Referenz: Eingelesener Klartext                                       |
    | Leistung: Klartext zeichenweise einlesen, Prozedur HoleBuchstabe(),   |
    |           und in einer Reihung ablegen. Das abschlieende Enter wird  |
    |           ignoriert.                                                  |
    *-----------------------------------------------------------------------*)

    VAR klarbuchstabe : CHAR;
        spalte        : INTEGER;

    BEGIN
      D.WriteStrXY(5,3,'Bitte Klartextzeile eingeben und mit < ENTER > abschlieen!');
      D.WriteStrXY(5,5,'ACHTUNG! Der Klartext sollte nur aus Kleinbuchstaben bestehen.');
      D.WriteStrXY(7,8,'Klartext:');
      spalte:= 18;
      klartext:= '';
      REPEAT
        INC(spalte);
        klarbuchstabe:= HoleBuchstabe(spalte);
        IF klarbuchstabe # D.ENTER THEN
          S.AppendChar(klartext,klarbuchstabe)
        END
      UNTIL (klarbuchstabe = D.ENTER) OR (spalte > 75)
    END EinlesenKlartext;


  PROCEDURE Verschluesseln ( VAR alphabet : tAlphabet; klartext : tText;
                                                    VAR geheimtext : tText );
   (*-----------------------------------------------------------------------*
    | Leistung: Verschlsseln des Klartextes, das Zeilenabschlusszeichen    |
    |           wird ignoriert.                                             |
    *-----------------------------------------------------------------------*)

    CONST zeile = 10;

    VAR   geheimbuchstabe,
          klarbuchstabe   : CHAR;
          anzahlZeichen,
          i, spalte       : INTEGER;

    BEGIN
      ErstesAlphabet(1,alphabet);
      spalte:= 18;
      geheimtext:= '';
      FOR i:= 0 TO SHORT(S.Length(klartext))-1 DO
        INC(spalte);
        geheimbuchstabe:= alphabet[ORD(klartext[i])-97];
        D.WriteCharXY(spalte,zeile,geheimbuchstabe);
        S.AppendChar(geheimtext,geheimbuchstabe)
      END
    END Verschluesseln;


  PROCEDURE DateiSchreiben ( geheimtext : tText );

    CONST zeile = 16;

    VAR   fHandle         : F.Handle;
          resCode, spalte : INTEGER;

    BEGIN
      IF F.Exist(fPfad2) THEN F.Delete(fPfad2,resCode) END;
      F.Open(fPfad2,TRUE,F.DENYNONE,F.WRITEONLY,fHandle,resCode);
      F.WriteLn(fHandle,geheimtext,resCode);
      F.Close(fHandle);
      D.WriteStrXY(5,zeile,message);
      D.WriteStr('geschrieben.')
    END DateiSchreiben;


  PROCEDURE DateiLesen ( VAR geheimtext : tText );

    CONST zeile = 18;

    VAR   fHandle : F.Handle;
          resCode : INTEGER;

    BEGIN
      IF F.Exist(fPfad2) THEN
        F.Open(fPfad2,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode);
        F.ReadLn(fHandle,geheimtext,resCode);
      ELSE
        Pfadfehler
      END;
      F.WriteLn(fHandle,geheimtext,resCode);
      F.Close(fHandle);
      D.WriteStrXY(5,zeile,message);
      D.WriteStr('gelesen.');
    END DateiLesen;


  PROCEDURE Entschluesseln ( alphabet2 : tAlphabet; geheimtext : tText);
   (*-----------------------------------------------------------------------*
    | Leistung:  Entschlsseln des Geheimtextes.                            |
    *-----------------------------------------------------------------------*)

    CONST zeile = 22;

    VAR   geheimbuchstabe,
          klarbuchstabe    : CHAR;
          i                : INTEGER;

    BEGIN
      D.WriteStrXY(5,zeile-2,'Der Geheimtext wird entschlsselt.');
      D.WriteStrXY(7,zeile,'Klartext:');
      FOR i:= 0 TO SHORT(S.Length(geheimtext))-1 DO
        klarbuchstabe:= alphabet2[ORD(geheimtext[i])-65];
        D.WriteCharXY(20+i,zeile,klarbuchstabe);
        Delay_ms(delayTime);
      END
    END Entschluesseln;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR alphabet1,
        alphabet2  : tAlphabet;
        klartext,
        geheimtext : tText;

    BEGIN
      D.SetWindowTitle('Monoalphabetische Substitution');
      EinlesenKlartext(klartext);
      Verschluesseln(alphabet1,klartext,geheimtext);
      DateiSchreiben(geheimtext);
      ZweitesAlphabet(alphabet1,alphabet2);
      DateiLesen(geheimtext);
      Entschluesseln(alphabet2,geheimtext);
      Enter(65,23,delayTime)
    END ProgMain;

 END MonoDemo.

 (* ----------------------------------------------------------------------- *)
