
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)             Dateiname: POLY.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    Polyalpabetische Verschlsselung                              |
  |-------------------------------------------------------------------------|
  | Leistung: Verschlsseln und Entschlsseln eines Textes mittels          |
  |           polyalphabetischer Substitution.                              |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |            ...\poly\good\poly.mod                                       |
  |-------------------------------------------------------------------------|
  | Erluterungstexte: ...\poly\txt\poly.txt                                |
  |                    ...\poly\txt\polytool.txt                            |
  |                    ...\poly\doc\poly.doc                                |
  |                    ...\poly\doc\polytool.doc                            |
  *-------------------------------------------------------------------------*)

 MODULE Poly;

  IMPORT In, Out;

  VAR schluessel       : ARRAY 20 OF CHAR;
      schluessellaenge : INTEGER;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Erlaeuterung;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe des Erluterungstexts am Bildschirm.                |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      Out.Ln;
      Out.String('  H I N W E I S:'); Out.Ln;
      Out.Ln;
      Out.String('    J e d e  eingegebene Zeichenkette ist mit der Tilde ');
      Out.String('(~),'); Out.Ln;
      Out.String('    Tastenkombination < Alt Gr > + < + > und < Enter > ');
      Out.String('abzu-'); Out.Ln;
      Out.String('    schlieen!'); Out.Ln;
      Out.Ln; Out.String('  '); FOR i:= 1 TO 58 DO Out.Char('') END; Out.Ln
    END Erlaeuterung;


  PROCEDURE WandelnGrKl ( zeichen : CHAR ) : CHAR;
   (*-----------------------------------------------------------------------*
    | bernahme: Zu wandelnden Buchstaben                                   |
    | Referenz:  Funktionswert, ggf. gewandelter Buchstabe                  |
    | Leistung:  Wandeln eines bernommenen Gro- in einen Kleinbuchstaben. |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF (zeichen >= 'A') & (zeichen <= 'Z') THEN
        zeichen:= CHR(ORD(zeichen)+32)
      ELSIF (zeichen >= 'a') & (zeichen <= 'z') THEN
        zeichen:= zeichen
      ELSE
        zeichen:= CHR(0)                                  (* CHR(0) = dummy *)
      END;
      RETURN zeichen
    END WandelnGrKl;


  PROCEDURE WandelnKlGr ( zeichen : CHAR ) : CHAR;
   (*-----------------------------------------------------------------------*
    | bernahme: Zu wandelnden Buchstaben                                   |
    | Referenz:  Funktionswert, ggf. gewandelter Buchstabe                  |
    | Leistung:  Wandeln eines bernommenen Klein- in einen Grobuchstaben. |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF (zeichen >= 'a') & (zeichen <= 'z') THEN
        zeichen:= CHR(ORD(zeichen)-32)
      ELSIF (zeichen >= 'A') & (zeichen <= 'Z') THEN
        zeichen:= zeichen
      ELSE
        zeichen:= CHR(0)                                  (* CHR(0) = dummy *)
      END;
      RETURN zeichen
    END WandelnKlGr;


  PROCEDURE Schluesseleingabe;
   (*-----------------------------------------------------------------------*
    | Leistung: Schlsselwort zeichenweise einlesen. Abschlusszeichen ist   |
    |           die Tilde (~). '~' = < AltGR > + < + >                      |
    |           Wurde versehentlich ein Grobuchstabe eingegeben, wird er   |
    |           in einen Kleinbuchstaben gewandelt.                         |
    |           Die Schlssellnge wird ermittelt (schluessellaenge)        |
    *-----------------------------------------------------------------------*)

    VAR i       : INTEGER; 
        zeichen : CHAR;

    BEGIN
      i:= 0;
      In.Prompt('Schlssel:');
      REPEAT
        In.Char(zeichen);
        WHILE zeichen = ' ' DO
          In.Char(zeichen)
        END;
        IF zeichen # '~' THEN
          zeichen:= WandelnGrKl(zeichen);
          IF ORD(zeichen) > 0 THEN      (* zeichen = 'A'..'Z' oder 'a'..'z' *)
            schluessel[i]:= zeichen;
            INC(i)
          END
        END
      UNTIL zeichen = '~';
      schluessellaenge:= i - 1;
      Out.Ln;
      Out.String('    Schlssel eingelesen '); Out.Ln
    END Schluesseleingabe;


  PROCEDURE Verschluesseln;
   (*-----------------------------------------------------------------------*
    | Leistung: Verschlsseln des Klartextes (zeichenweise), dabei werden   |
    |           nur Gro- und Kleinbuchstaben ausgewertet, d.h. alle ande-  |
    |           ren druckbaren Zeichen werden ausgesondert. Wurde vesehent- |
    |           lich ein Grobuchstabe eingegeben, wird er in einen Klein-  |
    |           buchstaben gewandelt.                                       |
    *-----------------------------------------------------------------------*)

    VAR differenz, nummer, i : INTEGER;
        klarbuchstabe,
        schluesselbuchstabe, 
        geheimbuchstabe      : CHAR;

    BEGIN
      Out.Ln;
      In.Prompt('Klartext:');
      Out.String('    Geheimtext: ');
      i:= 0;
      REPEAT
        In.Char(klarbuchstabe);
        WHILE klarbuchstabe = ' ' DO
          In.Char(klarbuchstabe)
        END;
        IF klarbuchstabe # '~' THEN
          klarbuchstabe:= WandelnGrKl(klarbuchstabe);
          IF ORD(klarbuchstabe) > 0 THEN  (* zeichen = 'A'..'Z' o. 'a'..'z' *)
            schluesselbuchstabe:= schluessel[i];
            INC(i);
            IF i > schluessellaenge THEN
              i:= 0
            END;
            differenz:= ORD(schluesselbuchstabe) - 97;
            nummer:= ORD(klarbuchstabe) + differenz;
            IF nummer > 122 THEN
              nummer:= nummer - 26
            END;
            geheimbuchstabe:= CHR(nummer-32);
            Out.Char(geheimbuchstabe)
          END
        END  
      UNTIL klarbuchstabe = '~';
      Out.Ln; Out.Ln;
      Out.String('    Klartext eingelesen'); Out.Ln
    END Verschluesseln;


  PROCEDURE Entschluesseln;
   (*-----------------------------------------------------------------------*
    | Leistung: Entschlsseln des Geheimtextes (zeichenweise), dabei werden |
    |           nur Gro- und Kleinbuchstaben ausgewertet, d.h. alle ande-  |
    |           ren druckbaren Zeichen werden ausgesondert. Wurde vesehent- |
    |           lich ein Grobuchstabe eingegeben, wird er in einen Klein-  |
    |           buchstaben gewandelt.                                       |
    *-----------------------------------------------------------------------*)

    VAR differenz, nummer, i : INTEGER;
        klarbuchstabe,
        schluesselbuchstabe, 
        geheimbuchstabe      : CHAR;

    BEGIN
      Out.Ln;
      Out.String('    Geheimtext zeichenweise (Grobuchstaben) eingeben!   ');
      i:= 0;
      Out.Ln; Out.Ln;
      Out.String('    Klartext:   ');
      REPEAT
        In.Prompt('Geheimbuchstabe:');
        In.Char(geheimbuchstabe);
        WHILE geheimbuchstabe = ' ' DO
          In.Char(geheimbuchstabe)
        END;
        IF geheimbuchstabe # '~' THEN
          geheimbuchstabe:= WandelnKlGr(geheimbuchstabe);
          IF ORD(klarbuchstabe) > 0 THEN
            schluesselbuchstabe:= schluessel[i];
            INC(i)
          END;
          IF i > schluessellaenge THEN
            i:= 0
          END;
          nummer:=    ORD(geheimbuchstabe) + 32;
          differenz:= ORD(schluesselbuchstabe) - 97;
          nummer:=    nummer - differenz;
          IF nummer < 97 THEN
            nummer:= nummer + 26
          END;
          klarbuchstabe:= CHR(nummer);
          IF ORD(klarbuchstabe) < 123 THEN
            Out.Char(klarbuchstabe)
          END
        END
      UNTIL geheimbuchstabe = '~'
    END Entschluesseln;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();
    BEGIN
      Erlaeuterung;
      Schluesseleingabe;
      Verschluesseln;
      Entschluesseln
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END Poly.

 (* ----------------------------------------------------------------------- *)
