
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)          Dateiname: GRAFIKA.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    SORTIEREN durch Auswhlen  -  Ergnzung zur 5. Variante       |
  |-------------------------------------------------------------------------|
  | Leistung: Grafische Darstellung der Zeitkomplexitt                     |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul ist lauffhig, stellt aber nur ein Grundgerst dar. |
  |           Siehe ...\sort1\fine\auswaehl.mod                             |
  *-------------------------------------------------------------------------*)

 MODULE GrafikA;
  
  IMPORT In, Out, F:= File, Sy:= SYSTEM, XY:= XYplane,W:= Windows;

  CONST  max    = 16;       (* maximale Anzahl Messungen                    *)

  TYPE   tArt   = RECORD
                    eigen, trenn : INTEGER
                  END;

         tPunkt = RECORD
                    x, y : INTEGER
                  END;      (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tDaten = ARRAY max+1 OF tPunkt;

         tPfad  = ARRAY 14 OF CHAR;
         tShStr = ARRAY 45 OF CHAR;


  VAR    dummy  : LONGINT;
         daten  : tDaten;
         fPfad  : tPfad;


  PROCEDURE Anleitung1;
    BEGIN
      Out.Ln;
      Out.String('  - Ziehen Sie im Fenster XYPLANE den Button im ');
      Out.String('vertikalen Rollbalken'); Out.Ln;
      Out.String('    mit der Maus an den   u n t e r e n   Anschlag!');
      Out.Ln; Out.Ln;
      Out.String('  - Geben Sie den kompletten Dateinamen der auszuwerten');
      Out.String('den Messdatendatei an!'); Out.Ln;
      Out.String('    Zum Beispiel: AUSWAE01.DAT'); Out.Ln
    END Anleitung1;


  PROCEDURE Anleitung2 ( xMax, yMax : INTEGER );
    BEGIN
      Out.Ln;  Out.Ln;
      Out.String('   max. x-Ausdehnung ='); Out.Int(xMax,3); Out.Ln;
      Out.String('   max. y-Ausdehnung ='); Out.Int(yMax,3); Out.Ln;
      Out.Ln;  Out.Ln; Out.Ln
    END Anleitung2;


  PROCEDURE Punkt ( x, y, art : INTEGER );
    BEGIN
      x := x + 2;  y := y + 3;
      CASE art OF
        1: XY.Dot(x-2,y-1,XY.draw);
           XY.Dot(x-2,y,XY.draw);
           XY.Dot(x-1,y-3,XY.draw);
           XY.Dot(x-1,y-2,XY.draw);
           XY.Dot(x-1,y-1,XY.draw);
           XY.Dot(x-1,y,XY.draw);
           XY.Dot(x,y-1,XY.draw);
           XY.Dot(x,y,XY.draw)
      | 2: XY.Dot(x-2,y-2,XY.draw);
           XY.Dot(x-2,y-1,XY.draw);
           XY.Dot(x-1,y-2,XY.draw);
           XY.Dot(x-1,y-1,XY.draw);
           XY.Dot(x,y-2,XY.draw);
           XY.Dot(x,y-1,XY.draw)
      | 3: XY.Dot(x-2,y-3,XY.draw);
           XY.Dot(x-2,y-2,XY.draw);
           XY.Dot(x-1,y-3,XY.draw);
           XY.Dot(x-1,y-2,XY.draw);
           XY.Dot(x-1,y-1,XY.draw);
           XY.Dot(x-1,y,XY.draw);
           XY.Dot(x,y-3,XY.draw);
           XY.Dot(x,y-2,XY.draw)
      END
    END Punkt;


  PROCEDURE Skalieren ( anzahl  : INTEGER; VAR xS, yS : REAL );

    CONST x = 400;  y = 300;

    VAR xMax, yMax : LONGINT;
        art, i     : INTEGER;

    BEGIN
      xMax:= 0;  yMax:= 0;
      FOR i:= 1 TO anzahl DO
        IF daten[i].x > xMax THEN xMax:= daten[i].x END;
        IF daten[i].y > yMax THEN yMax:= daten[i].y END;
      END;
      xS:= x/xMax;
      yS:= y/yMax
    END Skalieren;


  PROCEDURE RoundReal ( x : REAL ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme:  zu rundender Wert (DT: REAL)                              |
    | Referenz:   gerundeter Wert (Funktionswert, DT: INTEGER)              |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN Sy.VAL(INTEGER,x+0.5)
    END RoundReal;


  PROCEDURE Zeichnen ( anzahl  : INTEGER; xS, yS : REAL );

    VAR art, i, x, y : INTEGER;

    BEGIN
      REPEAT
        In.Prompt('Art ( 1..3 )');              (* Art der Punktdarstellung *)
        In.Int(art)
      UNTIL  (art > 0) & (art < 4);
      Anleitung2(400,300);
      FOR i:= 1 TO anzahl DO
        Punkt(RoundReal(daten[i].x*xS),RoundReal(daten[i].y*yS),art)
      END
    END Zeichnen;


  PROCEDURE DateiLesen ( VAR anzahl : INTEGER );

    VAR handle  : F.Handle;
        resCode : INTEGER;
        n, read : LONGINT;
        message : tShStr;
        punkt   : tPunkt;

    BEGIN
      F.Open(fPfad,FALSE,F.DENYNONE,F.READONLY,handle,resCode);
      anzahl:= -1;
      n:= SIZE(tPunkt);
      WHILE ~F.AtEnd(handle) DO
        INC(anzahl);
        F.ReadBlock(handle,daten[anzahl],n,read,resCode);
      END;
      F.Close(handle)
    END DateiLesen;


  PROCEDURE ProgMain*();

    VAR ok     : BOOLEAN;
        anzahl : INTEGER;
        xS, yS : REAL;

    BEGIN
      Anleitung1;
      REPEAT 
        In.Prompt('Dateiname');
        In.String(fPfad);
        IF F.Exist(fPfad) THEN
          ok:= TRUE;
        ELSE 
          ok:= FALSE;
          Out.Ln;
          Out.String('    Datei steht nicht im aktuellen Pfad!'); Out.Ln;
        END
      UNTIL ok; 
      IF ok THEN DateiLesen(anzahl) END;
      Skalieren(anzahl,xS,yS);
      Zeichnen(anzahl,xS,yS)
    END ProgMain;
  

 BEGIN
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   XY.Open
 END GrafikA.
