
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)        Dateiname: QUIDEMOOS.MOD |
  |-------------------------------------------------------------------------|
  |  Autor:   Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: 03-04.01.2001 Version: V 1.0                |
  |-------------------------------------------------------------------------|
  | Thema:    SORTIEREN  (Quicksort); grafische Darstellung des             |
  |                      Sortiervorgangs nach Auswahl von Parameter         |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von INTEGER-Zahlen (Quicksort, Teile und Herrsche)  |
  |-------------------------------------------------------------------------|
  | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.    |
  |           QUICKSO1.MOD  (1. Variante, Oberon-2 V4)                      |
  |-------------------------------------------------------------------------|
  | Hinweise: Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |           siehe:                                                        |
  |                ...\GRUNDLAGEN\ALLGEMEIN\DOC\GRAFIK.DOC  oder            |
  |                ...\GRUNDLAGEN\ALLGEMEIN\TXT\GRAFIK.TXT                  |
  |-------------------------------------------------------------------------|
  |            2001  Manfred Kmmerer, Chamberlain-Soft JENA               |
  *-------------------------------------------------------------------------*)

 MODULE QuiDemoOs;                              (* Quicksortdemo ohne Sound *)

  IMPORT C:= ColorPlane, F:=  Float,  P:=  Process, R:= RandomNumbers,
         S:= Strings,    Sy:= SYSTEM, W:= Windows;

  CONST  delayTime =    2;     (* 2 ist der Wert fr eine CPU i80486 66 MHz *)
                               (* Bitte an Ihren Prozessor anpassen!        *)
         anzahl    =  359;
         bc_red    =    8;
         bc_green  =   16;
         bc_blue   =   48;
         bc_blue1  =  127;
         br        =  480;
         chrAnz    =   15;               (* Anzahl Zeichen pro Ausgabezeile *)
         default   =   25;               (* Standardverzgerungsfaktor      *)
         max_dTf   =  500;               (* maximaler Verzgerungsfaktor    *)
         strBreite =   32;
         xOffset   =   33;
         yOffset   =  530;
         zeiHoehe  =   20;

  TYPE   tZahlStr  = ARRAY  6 OF CHAR;
         tStr30    = ARRAY 31 OF CHAR;
         tReihung  = ARRAY anzahl + 1 OF INTEGER;

         tArt      = RECORD
                       eigen, trenn, dTfaktor : INTEGER
                     END;

  VAR    start     : BOOLEAN;
         dummy     : LONGINT;
         art       : tArt;
         a         : tReihung;
         zahlStr   : tStr30;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;
 
    BEGIN
      WHILE C.KeyPressed() DO Dummy:= C.ReadKey() END
    END FlushKeyBuffer;


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE IntStr ( n : INTEGER; VAR zahlFStr : tZahlStr );

    BEGIN
      F.StrF(n,3,0,zahlFStr); S.Delete(zahlFStr,S.Length(zahlFStr),1);
      S.AppendChar(zahlFStr,' ')
    END IntStr;


  PROCEDURE WriteStrXY ( x, y : INTEGER; string : tStr30 );
   (*-----------------------------------------------------------------------*
    | bernahme: x = Pixelspalte; x = 0 entspricht der linken Begrenzung    |
    |                                   des aktuellen Grafikfensters        |
    |            y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung   |
    |                                   des aktuellen Grafikfensters        |
    |            string = Zeichenkette, maximale Lnge: 30 Zeichen          |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette im Grafikmodus,   |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.GotoXY(x,y); C.WriteStr(string)
    END WriteStrXY;


  PROCEDURE WriteNumberXY ( x, y, n : INTEGER; im_Ausgabefenster : BOOLEAN);
   (*-----------------------------------------------------------------------*
    | bernahme: Pixelspalte, x, Pixelzeile, y, auszugebende Zahl, n und    |
    |            Wert einer booleschen Variable                             |
    | Leistung:  Positionierte Ausgabe der INTEGER-Zahl n als Zeichenkette  |
    |            im Grafikmodus. In Abhngigkeit vom Wert der booleschen    |
    |            Variable werden unterschiedliche Vorder- und Hintergrund-  |
    |            farben gesetzt (TRUE fr Ausgabefenster, FALSE fr groes  |
    |            Fenster).                                                  |
    *-----------------------------------------------------------------------*)

    VAR zahlFStr : tZahlStr;

    BEGIN
      IF im_Ausgabefenster THEN
        C.SetBackColor(0,0,0);
        C.SetForeColor(255,255,127)
      ELSE
        C.SetBackColor(bc_red,bc_green,bc_blue);
        C.SetForeColor(120,208,160)
      END;
      IntStr(n,zahlFStr);
      C.GotoXY(x,y); C.WriteStr(zahlFStr)
    END WriteNumberXY;


  PROCEDURE WriteNumberInversXY ( x, y, n : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Pixelspalte, x, Pixelzeile, y und auszugebende Zahl, n     |
    | Leistung:  Positionierte Ausgabe einer INTEGER-Zahl als Zeichenkette  |
    |            im Grafikmodus. Die Hintergrundfarbe ist nun Hellblau und  |
    |            Zeichenkette helles Trkis.                                |
    *-----------------------------------------------------------------------*)

    VAR zahlStr : tZahlStr;

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue1);
      C.SetForeColor(32,255,255);
      IntStr(n,zahlStr);
      C.GotoXY(x,y); C.WriteStr(zahlStr);
      C.SetBackColor(0,0,0)
    END WriteNumberInversXY;


  PROCEDURE WritePivoteelementXY ( x, y, t : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Pixelspalte, x, Pixelzeile, y und auszugebende Zahl, t     |
    | Leistung:  Positionierte Ausgabe des Trennelements als Zeichenkette   |
    |            im Grafikmodus. Die Zeichenkette wird in krftigem Rot     |
    |            dargestellt.                                               |
    *-----------------------------------------------------------------------*)

    VAR zahlStr : tZahlStr;

    BEGIN
      C.SetBackColor(0,0,0);
      C.SetForeColor(255,95,0);
      IntStr(t,zahlStr);
      C.GotoXY(x,y); C.WriteStr(zahlStr);
      Delay_ms(art.dTfaktor*5);
      C.SetBackColor(0,0,0)
    END WritePivoteelementXY;


  PROCEDURE Wiederholung() : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information, Warten auf eine Tastenbettigung |
    |           (Tastatur), Wandeln des Zeichens in einen Grobuchstaben,   |
    |           falls ein Kleinbuchstabe eingegeben wurde und Auswerten der |
    |           Eingabe (boolescher Wert).                                  |
    | Referenz: Funktionswert TRUE, falls eine Wiederholung gewnscht wird, |
    |           d.h. es wurde eine beliebige Taste auer <N> bettigt,      |
    |           sonst FALSE.                                                |
    *-----------------------------------------------------------------------*)

    VAR ch : CHAR;
      
    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(576,230,766,74,C.ERASE);
      Delay_ms(delayTime*25);
      C.SetForeColor(32,160,160);
      WriteStrXY(575,50,'Wiederholung (J/N) J');
      FlushKeyBuffer;
      ch:= C.ReadKey();
      ch:= S.UpCaseChar(ch);
      C.Bar(575,50,735,34,C.ERASE);
      RETURN ~(ch = 'N') 
    END Wiederholung;


  PROCEDURE StartStopp;

    CONST x = 664;

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      IF start THEN
        C.SetForeColor(120,208,160);
      ELSE
        C.SetForeColor(32,127,127)
      END;
      WriteStrXY(x,90,'Start');
      IF ~start THEN
        C.SetForeColor(120,208,160);
      ELSE
        C.SetForeColor(32,127,127)
      END;
      WriteStrXY(x+48,90,'Stopp')
    END StartStopp;


  PROCEDURE Erlaeuterung;

    CONST x = 576;

    BEGIN
      C.SetForeColor(32,127,127);
      WriteStrXY(x,230,'<CURUP>    fr grer'); 
      WriteStrXY(x,210,'<CURDOWN>  fr kleiner');
      WriteStrXY(x,190,'<PAGEUP>   fr  0');
      WriteStrXY(x,170,'<PAGEDOWN> fr 25');
      WriteStrXY(x,90,'<ENTER>'); WriteStrXY(x+128,90,'/');
      StartStopp
    END Erlaeuterung;


  PROCEDURE AendernVerzoegerungsfaktor;

    VAR taste   : CHAR;
        zahlStr : tStr30;

    BEGIN
      taste:= C.ReadKey();
      IF taste = CHR(0) THEN (* Da beim Bettigen einer Sondertaste zwei    *)
        taste:= C.ReadKey(); (* Tastaturcods gesendet werden, mssen beide  *)
      END;                   (* Codes abgeholt werden (Null und ASCII-Wert) *)
      IF (taste = C.CURSUP) & (art.dTfaktor < max_dTf) THEN
        INC(art.dTfaktor,5)
      ELSIF (taste = C.CURSDOWN) & (art.dTfaktor > 0) THEN
        IF art.dTfaktor > 9 THEN
          DEC(art.dTfaktor,5)
        ELSE
          DEC(art.dTfaktor)
        END;
      ELSIF (taste = C.PAGEDOWN) & (art.dTfaktor > 0) THEN
        art.dTfaktor:= 0;
      ELSIF (taste = C.PAGEUP) THEN
        art.dTfaktor:= default
      END;
      IF (taste = C.CURSUP) OR (taste = C.CURSDOWN)
            OR (taste = C.PAGEDOWN) OR (taste = C.PAGEUP) THEN
        S.Str(art.dTfaktor,zahlStr);
        C.SetBackColor(bc_red,bc_green,bc_blue);
        C.SetForeColor(120,208,160);
        WriteStrXY(728,260,'     ');
        WriteStrXY(728,260,zahlStr)
      END;
      IF (taste = C.ENTER) THEN
        IF start = TRUE THEN
          start:= FALSE;
          StartStopp;
          REPEAT UNTIL C.KeyPressed();
          FlushKeyBuffer;
          start:= TRUE;
          StartStopp
        END
      END;
      FlushKeyBuffer;
    END AendernVerzoegerungsfaktor;


  PROCEDURE Ausgabefenster;
    BEGIN
      C.SetBackColor(0,0,0);
      C.Bar(xOffset+2,yOffset-br-18,xOffset+br+6,yOffset-15,C.ERASE);
      C.SetForeColor(0,192,192);
      C.Box(xOffset-1,yOffset-br-21,xOffset+br+9,yOffset-12,C.DRAW)
    END Ausgabefenster;


  PROCEDURE Ueberschrift;
    BEGIN
      Ausgabefenster;
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.SetForeColor(32,208,208);
      WriteStrXY(590,495,'Q U I C K S O R T');
      C.Line(580,476,734,476,C.DRAW);
      C.Line(580,475,734,475,C.DRAW);
      Delay_ms(delayTime*25)
    END Ueberschrift;


  PROCEDURE AusgabeReihung;

    VAR i, spalte, zeile : INTEGER; 

    BEGIN
      spalte:= xOffset; zeile:= yOffset;
      C.SetBackColor(0,0,0);
      FOR i:= 1 TO anzahl DO
        IF i MOD chrAnz = 1 THEN
          DEC(zeile,zeiHoehe);
          spalte:= xOffset
        END;
        F.StrF(a[i],3,0,zahlStr); S.Delete(zahlStr,S.Length(zahlStr),1);
        WriteStrXY(spalte,zeile,zahlStr);
        INC(spalte,strBreite)
      END
    END AusgabeReihung;


  PROCEDURE ReihungAnlegenAusgeben;
   (*-----------------------------------------------------------------------*
    | Leistung: In Abhngigkeit des Wertes der Variablen art, werden die    |
    |           Komponenten der Reihung mit aufsteigenden, zuflligen oder  |
    |           absteigenden Werten der Menge {1..n} belegt und im Ausgabe- |
    |           fenster als violetter Punkt (C.Dot(x,y,C.DRAW) dargestellt. |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      C.SetForeColor(255,127,255);
      IF art.eigen = 1 THEN
        FOR i:= 1 TO anzahl DO
          a[i]:= i
        END
      ELSIF art.eigen = 2 THEN
        FOR i:= 1 TO anzahl DO
          a[i]:= 1 + SHORT(ENTIER(R.Uniform()*anzahl))
        END
      ELSE
        FOR i:= 1 TO anzahl DO
          a[i]:= 1 + anzahl - i
        END
      END;
      C.SetForeColor(0,127,127);
      AusgabeReihung
    END ReihungAnlegenAusgeben;


  PROCEDURE Eigenschaften;

    CONST x1 = 560;  x2 = 576;  x3 = 592;

    VAR   resCode : CHAR;
          zahlStr : tZahlStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(x1,445,'Die Zahlen sind ');
      WriteStrXY(x1,420,'(1) bereits aufsteigend');
      WriteStrXY(x3,400,'sortiert,');
      WriteStrXY(x1,380,'(2) zufllig verteilt');
      WriteStrXY(x3,360,'oder');
      WriteStrXY(x1,340,'(3) bereits absteigend');
      WriteStrXY(x3,320,'sortiert.');
      WriteStrXY(x3,290,'Art (1..3) =');
      WriteStrXY(x3,270,'<ENTER> fr (2)');
      C.SetForeColor(120,160,160);
      REPEAT
        C.Bar(696,290,704,274,C.ERASE);
        C.GotoXY(696,290); C.ReadStr(zahlStr,1,resCode);
        IF zahlStr = '' THEN
          zahlStr:= '2'
        END
      UNTIL (zahlStr > '0') & (zahlStr < '4');
      art.eigen:= SHORT(S.Val(zahlStr));
      C.Bar(544,445,752,254,C.ERASE);
      WriteStrXY(x1,445,'Eigenschaft:');
      C.SetForeColor(120,208,160);
      C.GotoXY(x2,420); 
      CASE art.eigen OF
        1: C.WriteStr('bereits aufsteigend');
           WriteStrXY(x2,400,'sortierte Zahlen')
       |2: C.WriteStr('zufllig verteilte');
           WriteStrXY(x2,400,'Zahlen')
       |3: C.WriteStr('bereits absteigend');
           WriteStrXY(x2,400,'sortierte Zahlen')
      END
    END Eigenschaften;


  PROCEDURE Trennelement;

    CONST x1 = 560;  x2 = 576;  x3 = 592;

    VAR   resCode : CHAR;
          zahlStr : tZahlStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(x1,365,'Das Trennelement ist');
      WriteStrXY(x1,340,'(1) das erste Element');
      WriteStrXY(x1,320,'(2) das mittlere Element');
      WriteStrXY(x1,300,'(3) das letzte Element');
      WriteStrXY(x1,280,'(4) ein zuflliges Element');
      WriteStrXY(x3,260,'der Teilfolge.');
      WriteStrXY(x3,230,'Art (1..4) =');
      WriteStrXY(x3,210,'<ENTER> fr (4)');
      C.SetForeColor(120,160,160);
      REPEAT
        C.Bar(696,230,704,214,C.ERASE);
        C.GotoXY(696,230); C.ReadStr(zahlStr,1,resCode);
        IF zahlStr = '' THEN
          zahlStr:= '4'
        END
      UNTIL (zahlStr > '0') & (zahlStr < '5');
      art.trenn:= SHORT(S.Val(zahlStr));
      C.Bar(544,365,766,154,C.ERASE);
      WriteStrXY(x1,365,'Trennelement:');
      C.SetForeColor(120,208,160);
      C.GotoXY(x2,340);
      CASE art.trenn OF
        1: C.WriteStr('das erste Element');
       |2: C.WriteStr('das mittlere Element');
       |3: C.WriteStr('das letzte Element');
       |4: C.WriteStr('ein zuflliges Element');
           WriteStrXY(x2,320,'der Teilfolge')
      END
    END Trennelement;


  PROCEDURE Verzoegerungszeit;

    CONST x1 = 560;  x2 = 576;

    VAR   resCode : CHAR;
          zahlStr : tZahlStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(x1,285,'Verzgerungs-');
      WriteStrXY(x1,265,'zeitfaktor =');
      WriteStrXY(x1+8,245,'<ENTER> fr 25');
      C.GotoXY(664,265); C.ReadStr(zahlStr,6,resCode);
      IF zahlStr = '' THEN
        art.dTfaktor:= default;
        zahlStr:= '25'
      END;
      art.dTfaktor:= SHORT(S.Val(zahlStr));
      C.Bar(x1,285,766,234,C.ERASE);
      WriteStrXY(x1,285,'Verzgerung:');
      C.SetForeColor(120,208,160);
      WriteStrXY(x2,260,'Verzgerungsfaktor ');
      C.WriteStr(zahlStr);
      Erlaeuterung
    END Verzoegerungszeit;


  PROCEDURE Art;
    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(533,445,766,50,C.ERASE);
      FlushKeyBuffer;
      Eigenschaften;
      Trennelement;
      Verzoegerungszeit;
      Delay_ms(delayTime*5)
    END Art;

                  
  PROCEDURE Start;

    PROCEDURE Sort ( l, r : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Sortieren der Werte der Komponenten der Reihung nach auf-   |
    |           steigender Gre mittels Quicksort (Teile und Herrsche).    |
    | Hinweis:  Beschreibung des Algorithmus siehe:                         |
    |             ...\SORT2\TXT\QUICKSOR.TXT und ...\QUICKS1.TXT            |
    |            oder bei Nutzung von MS-WORD (o..) unter                  |
    |             ...\SORT2\DOC\QUICKSOR.DOC und ...\QUICKS1.DOC            |
    |           Die Originalwerte der Komponenten der Reihung werden als    |
    |           violette Pixel angezeigt. Die beiden Pixel, die an einer    |
    |           Tauschaktion beteiligt sind werden gelscht (mit Hinter-    |
    |           grundfarbe berschrieben) und an ihrer jeweils neuen Posi-  |
    |           tion als gelbes Pixel ausgegeben.                           |
    *-----------------------------------------------------------------------*)

      VAR h, i, j, x, w : INTEGER;
          zahlStr : tZahlStr;

      BEGIN
        i:= l;
        j:= r;
        CASE art.trenn OF 
           1: x := a[l]
         | 2: x := a[(l + r) DIV 2]
         | 3: x := a[r]
         | 4: x := a[l + SHORT(ENTIER(R.Uniform() * (r - l + 1)))]
        END;
        WritePivoteelementXY(xOffset+((x-1) MOD chrAnz)*strBreite,
                            yOffset-(((x-1) DIV chrAnz)+1)*zeiHoehe,a[x]);
        Delay_ms(art.dTfaktor);

        P.Yield;                                     (* keine Multitasking  *)

        REPEAT
          WHILE a[i] < x DO INC(i) END;
          WHILE x < a[j] DO DEC(j) END;
          IF i <= j THEN
            WriteNumberInversXY(xOffset+((i-1) MOD chrAnz)*strBreite,
                                yOffset-(((i-1) DIV chrAnz)+1)*zeiHoehe,a[i]);
            WriteNumberInversXY(xOffset+((j-1) MOD chrAnz)*strBreite,
                                yOffset-(((j-1) DIV chrAnz)+1)*zeiHoehe,a[j]);
            Delay_ms(art.dTfaktor);
            IF C.KeyPressed() THEN AendernVerzoegerungsfaktor END;
            w:= a[i];
            a[i]:= a[j];
            a[j]:= w;
            WriteNumberXY(xOffset+((i-1) MOD chrAnz)*strBreite,
                          yOffset-(((i-1) DIV chrAnz)+1)*zeiHoehe,a[i],TRUE);
            WriteNumberXY(xOffset+((j-1) MOD chrAnz)*strBreite,
                          yOffset-(((j-1) DIV chrAnz)+1)*zeiHoehe,a[j],TRUE);
            Delay_ms(art.dTfaktor);
            INC(i);
            DEC(j)
          END
        UNTIL i > j;
        IF l < j THEN Sort(l,j) END;
        IF i < r THEN Sort(i,r) END
      END Sort;

    BEGIN  (* Start *)
      ReihungAnlegenAusgeben;
      Delay_ms(delayTime*25);
      Sort(1,anzahl);
    END Start;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m     : ARRAY 91 OF CHAR;

    BEGIN
      dummy:= W.ShowWindow(W.GetActiveWindow(),3); (* Groes Fenster ffnen *)
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBoxA(W.GetActiveWindow(),Sy.ADR(m),Sy.ADR(m3),40H);
      Delay_ms(delayTime*50)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR a   : tReihung;
        i   : INTEGER;

    BEGIN
      Ueberschrift;
      i:= 0;
      REPEAT
        start:= TRUE;
        INC(i);
        IF i > 1 THEN Ausgabefenster END;
        Art;
        R.Randomize;(* Zufallszahlengenerator mit neuem Wert initialisieren *)
        Start;
        art.dTfaktor:= 25;
        Delay_ms(delayTime*25);
        C.SetForeColor(223,223,127);
        AusgabeReihung
      UNTIL ~Wiederholung();
      Information
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   C.SetBackColor(bc_red,bc_green,bc_blue);
   C.Bar(5,0,850,650,C.ERASE);
   C.SetForeColor(200,200,200);
   C.Bar(1,0,3,650,C.DRAW);
   C.SetForeColor(63,63,63);
   C.Line(4,0,4,650,C.DRAW);
   C.SetForeColor(96,220,220)
   (* Da die Vordergrundfarbe Schwarz automatisch eingestellt ist,  m u s s *)
   (* die Textfarbe bereits hier initialisiert werden. Sonst sieht man die  *)
   (* Eingabe bei der Anweisung 'C.ReadStr()' nicht!                        *)
 END QuiDemoOs.

 (*-------------------------------------------------------------------------*)

