
 (*--------------------------------------------------------------------------*
  |  Programmieren in Oberon-2 POW! (32 Bit)         Dateiname: SCANDIR.MOD  |
  |--------------------------------------------------------------------------|
  |  Autor:    Manfred Kmmerer, Wllnitzer Str. 40, 07749 JENA              |
  |            Erstellungsdatum: 03.02.2001 Version: V 1.0                   |
  |--------------------------------------------------------------------------|
  |  Thema:    S E R V I C E - M O D U L  in POW! (OBERON-2)                 |
  |            Umgang mit Zeigern (DT: Zeiger, Verbund und Reihung)          |
  |--------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA               |
  *--------------------------------------------------------------------------*)

 MODULE ScanDir;

  IMPORT S:= Strings, V:= Volume;

  TYPE   tNumbStr  = ARRAY  6 OF CHAR;
         tShStr*   = ARRAY 36 OF CHAR;

         pNode     = POINTER TO tListnode;          (* Die GROSSE Ausnahme! *)
         tListnode = RECORD
                       key  : tShStr;               (* Dateiname.Extension  *)
                       next : pNode                 (* Zeiger zum Nchsten  *)
                     END;

         tList*    = RECORD
                       first,
                       current : pNode
                     END;


  PROCEDURE MakeEmptyList ( VAR list : tList );
   (*-----------------------------------------------------------------------*
    | Referenz:  Verbund (zwei Zeiger; DT: pListe)                          |
    | Leistung:  Anlegen einer leeren Liste (list.first = NIL).             |
    *-----------------------------------------------------------------------*)

    BEGIN
      list.first:= NIL;                  (* Am Anfang ist die Liste leer  *)
    END MakeEmptyList;


  PROCEDURE ListEmpty* ( list : tList ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Verbund (zwei Zeiger; DT: pListe)                          |
    | Referenz:  Funktionswert (DT: BOOLEAN)                                |
    | Leistung:  Diese Operation liefert den Wert TRUE, wenn die Liste leer |
    |            ist (list.first = NIL), sonst FALSE.                       |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN list.first = NIL
    END ListEmpty;


  PROCEDURE GotoFirstEntry* ( VAR list : tList );
   (*-----------------------------------------------------------------------*
    | bernahme: Verbund (zwei Zeiger; DT: tListe)                          |
    | Referenz:  aktualisierte Zeiger (eigentlich nur: list.current)        |
    | Leistung:  Falls die Liste nicht leer ist, wird der Zeiger            |
    |            liste.current auf den Listenanfang gesetzt.                |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF ~ListEmpty(list) THEN
        list.current:= list.first
      END
    END GotoFirstEntry;


  PROCEDURE EndOfList* ( list : tList ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Verbund (zwei Zeiger; DT: tList)                           |
    | Referenz:  Funktionswert (DT: BOOLEAN)                                |
    | Leistung:  Diese Operation liefert den Wert TRUE, wenn die Liste      |
    |            leer ist (list.first = NIL) oder der bewegliche Zeiger     |
    |            (list.current) auf das Listenende (NIL) zeigt, sonst FALSE.|
    *-----------------------------------------------------------------------*)

    BEGIN
      IF ListEmpty(list) THEN
        RETURN TRUE
      ELSE
        RETURN list.current = NIL
      END
    END EndOfList;


  PROCEDURE SearchInList ( newKey : tShStr; VAR list : tList );
   (*-----------------------------------------------------------------------*
    | bernahme: Wert der Variable 'newKey' (DT: tShStr) und den Verbund,   |
    |            der u. a. die Zeiger-Variable 'current' enthlt            |
    | Referenz:  aktualisierte Zeiger (eigentlich nur: list.current)        |
    | Leistung:  Suche nach der Stelle in der Liste, an der der 'grere'   |
    |            Wert eingetragen werden kann.                              |
    *-----------------------------------------------------------------------*)

    BEGIN
      WHILE (list.current.next # NIL) &      (* solange Bedingungen(...)    *)
        (newKey > list.current.next.key) DO  (* 'wahr' sind, setze den      *)
        list.current:= list.current.next     (* Zeiger 'current' um einen   *)
      END                                    (* ListenEintrag weiter        *)
    END SearchInList;


  PROCEDURE InsertInList ( newKey : tShStr; VAR list : tList );
   (*-----------------------------------------------------------------------*
    | bernahme: Inhalt der Variable 'newKey' (Wertparameter) und den Ver-  |
    |            Verbund, der u. a. die Zeiger-Variable 'current' enthlt   |
    | Referenz:  aktualisierte Zeiger (eigentlich nur: list.current)        |
    | Leistung:  Einen neuen Verbund erzeugen und den auf den Verbund wei-  |
    |            senden Zeiger (new) bereitstellen, den im Verbund existie- |
    |            renden Zeiger sicherheitshalber 'auf NIL setzen', Zuweisen |
    |            der bernommenen Zeichenkette (neuer Schlssel) und 'Um-   |
    |            hngen' der Zeiger.                                        |
    *-----------------------------------------------------------------------*)

    VAR new : pNode;

    BEGIN
      NEW(new);                              (* neuen Verbund erzeugen      *)
      new.next:= NIL;                        (* neue Zeiger immer auf NIL   *)
      new.key:= newKey;                      (* 'newKey' den Wert zuweisen  *)
      list.current:= list.first;             (* Stets am Listenanfang mit   *)
                                             (* dem Suchen beginnen!        *)
      IF list.current # NIL THEN             (* ist die Liste nicht leer,   *)
        SearchInList(newKey,list)            (* dann Suche in Richtung Ende *)
      END;
      IF (list.current = NIL) OR             (* Liste ist leer oder  v o r  *)
           (newKey <= list.current.key) THEN (* dem ersten Eintrag einfgen *)
        new.next:= list.current;
        list.first:= new                     (* neuer Listenanfang          *)
      ELSE
        new.next:= list.current.next;        (* ... ein-  o d e r  anfgen  *)
        list.current.next:= new
      END (* if .. then .. else *)
    END InsertInList;


  PROCEDURE FetchEntry* ( list : tList; VAR errorMessage, key : tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme:  Verbund (zwei Zeiger; DT: tList)                          |
    | Referenzen: Fehlermeldung und Dateiname (beide DT: tShStr)            |
    | Leistung:   Ist die Liste leer, wird eine Fehlermeldung ausgegeben    |
    |             und die Abarbeitung des Programms abgebrochen, sonst wird |
    |             der Inhalt der Verbund-Variable list.key dem Referenzpa-  |
    |             rameter key zugewiesen.                                   |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF ListEmpty(list) THEN
        errorMessage:= 'Liste ist leer!';
        HALT(101)
      ELSE key:= list.current.key
      END
    END FetchEntry;


  PROCEDURE GotoNextEntry* ( VAR list : tList; VAR errorMessage : tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme:  Verbund (zwei Zeiger; DT: tList)                          |
    | Referenzen: aktualisierte Zeiger (eigentlich nur: list.current) und   |
    |             Fehlermeldung (errorMessage DT: tShStr)                   |
    | Leistung:   Wenn die Liste nicht leer ist und der Zeiger list.current |
    |             nicht auf das letzte Listenelement zeigt, wird der dieser |
    |             Zeiger um eine Position in Richtung Listenende gesetzt.   |
    |             Falls der Zeiger list.current bereits auf das letze Lis-  |
    |             tenelement zeigt, wird eine Fehlermeldung ausgegeben und  |
    |             die Abarbeitung des Programms abgebrochen.                |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF ~ListEmpty(list) THEN
        IF EndOfList(list) THEN
          errorMessage:= 'Zugriff nach dem letzten Element!';
          HALT(100)
        ELSE
          list.current:= list.current.next
        END
      END
    END GotoNextEntry;


  PROCEDURE ScanDirectoryEntrys* ( VAR counter : INTEGER; VAR list : tList;
                                 extend : tShStr; VAR withoutFile : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme:  Dateierweiterung (extend, DT: tShStr)                     |
    | Referenzen: Anzahl der gefundenen Eintrge (counter, DT: INTEGER),    |
    |             aktualisierte Zeiger und ein Wahrheitswert (withoutFile,  |
    |             DT: BOOLEAN)                                              |
    | Leistung:   Im aktuellen Pfad werden alle Dateinamen, die die berge- |
    |             bene Dateierweiterung besitzen, samt dieser alphabetisch  |
    |             geordnet in eine zeigerverkettete Liste eingetragen.      |
    |             Werden keine entsprechenden Dateinamen gefunden wird der  |
    |             Variablen withoutFile der Wert TRUE zugewiesen.           |
    *-----------------------------------------------------------------------*)

    VAR   resCode  : CHAR;
          result   : INTEGER;
          dirPfad,
          errorMsg,
          message  : tShStr;
          scan     : V.Scan;
  
      BEGIN
        dirPfad:= '*.';
        S.Append(dirPfad,extend);
        MakeEmptyList(list);
        scan.First(dirPfad,result);
        IF result = V.NOERROR THEN
          withoutFile:= FALSE;
          WHILE result = V.NOERROR DO
            scan.GetName(dirPfad);
            InsertInList(dirPfad,list);
            scan.Next(result)
          END
        ELSE
          withoutFile:= TRUE
        END
      END ScanDirectoryEntrys;

 (* ------------------------  OHNE INITIALISIERUNG  ----------------------- *)

 END ScanDir.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

