
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)         Dateiname: QUICKSO3.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    S O R T I E R E N   (Teile und Herrsche)  3. Variante         |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von INTEGER-Zahlen (Quicksort)                      |
  |           Der Programmbenutzer kann die Eigenschaft whlen, die die zu  |
  |           sortierenden Zahlen besitzen sollen, welches Element als Pi-  |
  |           votelement (Trennelement) genommen wird und wie viele Zahlen  |
  |           zu sortieren sind.                                            |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul als lauffhiges Grundgerst finden Sie unter:       |
  |            ...\sort2\good\quickso3.mod                                  |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\sort2\txt\quickso3.txt und                        |
  |                   ...\sort2\doc\quickso3.doc                            |
  *-------------------------------------------------------------------------*)

 MODULE Quickso3;

  IMPORT In, Out, F:= File, P:= Process, R:= RandomNumbers, S:= Strings,
         W:= Windows, Z:= Zeit;


  CONST  max      = 16;     (* maximale Anzahl Messungen                    *)

  TYPE   tArt   = RECORD
                    eigen,             (* Eigenschaften der Zahlen          *)
                    trenn,             (* Eigenschaft des Trennelements     *)
                    dTfaktor : INTEGER (* Verzgerungsfaktor beim Messen    *)
                  END;                 (* der zum Sortieren bentigten Zeit *)

         tPunkt = RECORD
                    x, y : INTEGER
                  END;      (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tDaten = ARRAY max+1 OF tPunkt;

         tShStr = ARRAY 45 OF CHAR;

  VAR    a     : ARRAY 16001 OF INTEGER;
         quali : tArt;
         daten : tDaten;
         fPfad : tShStr;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Delay;
   (*-----------------------------------------------------------------------*
    | Leistung: Eine kleine Zeit warten.                                    |
    *-----------------------------------------------------------------------*)

    VAR i : LONGINT;

    BEGIN
      i:= 0;
      REPEAT
        INC(i);
        Out.String('')
      UNTIL i = 10 * quali.dTfaktor 
    END Delay;


  PROCEDURE Anfrage() : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Anfrage, ob eine Datei geschrieben werden soll.             |
    | Referenz: Funktinswert (boolescher Wert, Wahrheitswert)               |
    *-----------------------------------------------------------------------*)

    VAR ch : CHAR;

    BEGIN
      In.Prompt('Datei (J/N) N');
      In.Char(ch);
      ch:= S.UpCaseChar(ch);
      IF ch = 'J' THEN
        RETURN TRUE
      ELSE
        RETURN FALSE
      END;
    END Anfrage;


  PROCEDURE Hinweis;
    BEGIN
      Out.Ln;
      Out.String('   ACHTUNG!  Falls die Ergebnisse in eine Datei');
      Out.String(' geschrieben werden'); Out.Ln;
      Out.String('             sollen, wiederholen Sie den Sortiervorgang,');
      Out.String(' bis die An-'); Out.Ln;
      Out.String('             zahl der zu sortierenden Zahlen 16000');
      Out.String(' betrgt.'); Out.Ln;
      Out.Ln
    END Hinweis;


  PROCEDURE Arbeit ( n : INTEGER );

    VAR i, laenge : INTEGER;
        message,
        zahlStr   : tShStr;

    BEGIN
      message:= '   Das System sortiert ';
      S.Str(n,zahlStr); S.Append(message,zahlStr);
      S.Append(message,' Zahlen.');
      Out.Ln;
      Out.String(message); Out.Ln;
      laenge:= SHORT(S.Length(message)-4);
      Out.String('   ');
      FOR i:= 1 TO laenge DO Out.Char('') END
    END Arbeit;


  PROCEDURE Warnung;
    BEGIN
      Out.Ln;
      Out.String('   Wenn bei der 1. Option die (1) gewhlt wurde, darf bei');
      Out.Ln;
      Out.String('   der 2. Option die (1)  n i c h t  gewhlt werden!');
      Out.Ln;
      Out.String('   (Heapgre)            '); Out.Ln;
    END Warnung;


  PROCEDURE Art;
    BEGIN
      Out.Ln;
      Out.String('   Die Zahlen sind '); Out.Ln;
      Out.String('     (1) bereits aufsteigend sortiert,'); Out.Ln;
      Out.String('     (2) zufllig verteilt oder'); Out.Ln;
      Out.String('     (3) bereits absteigend sortiert.'); Out.Ln; Out.Ln;
      REPEAT
        In.Prompt('Art  =  1 .. 3');
        In.Int(quali.eigen)
      UNTIL quali.eigen IN {1..3};
      Out.String('   Das Trennelement ist'); Out.Ln;
      Out.String('     (1) das erste Element der Teilfolge,'); Out.Ln;
      Out.String('     (2) das mittlere Element der Teilfolge,'); Out.Ln;
      Out.String('     (3) das letzte Element der Teilfolge oder'); Out.Ln;
      Out.String('     (4) ein zuflliges Element der Teilfolge.'); Out.Ln;
      IF quali.eigen = 1 THEN Warnung END;
      REPEAT
        In.Prompt('Art  =  1 .. 4');
        In.Int(quali.trenn)
      UNTIL (quali.eigen # 1) & (quali.trenn IN {1..4})
            OR (quali.eigen = 1) & (quali.trenn IN {2..4});
      Out.Ln;
      Out.String('   Verzgerungsfaktor (1..9)'); Out.Ln;
      REPEAT
        In.Prompt('Verzgerung');
        In.Int(quali.dTfaktor)
      UNTIL quali.dTfaktor IN {1..9}
    END Art;


  PROCEDURE Sort ( l, r : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Sortieren durch Auswhlen.                                  |
    |           Die ausfhrliche Beschreibung des Algorithmus' findet man   |
    |           unter  ...\sort2\doc\quicksor.doc (WORD-Version)            |
    |           oder   ...\sort2\txt\quicksor.txt (Text-Version)            |
    *-----------------------------------------------------------------------*)

    VAR i, j, x, w : INTEGER;

    BEGIN
      i:= l;
      j:= r;
      CASE quali.trenn OF 
         1: x := a[l]
       | 2: x := a[(l + r) DIV 2]
       | 3: x := a[r]
       | 4: x := a[l + SHORT(ENTIER(R.Uniform() * (r - l + 1)))]
      END;
      REPEAT
        WHILE a[i] < x DO INC(i) END;
        WHILE x < a[j] DO DEC(j) END;
        IF i <= j THEN
          Delay;
          w:= a[i];
          a[i]:= a[j];
          a[j]:= w;
          INC(i);
          DEC(j)
        END
      UNTIL i > j;
      IF l < j THEN Sort(l,j) END;
      IF i < r THEN Sort(i,r) END
    END Sort;


  PROCEDURE Sortieren ( anzahl, n : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Inhalt der Verbundvariable quali, die die Eigenschaft der  |
    |            zu sortierenden Zahlen, die Art des Trennelements und den  |
    |            Verzgerungsfaktor enthlt. Die Variable anzahl hlt den   |
    |            Wert (Ordnungszahl) des aktuellen Sortiervorgangs und n    |
    |            die Zahl (Menge) der zu sortierenden Zahlen.               |
    | Leistung:  Erzeugen der zu sortierenden Zahlen mit den geforderten    |
    |            Eigenschaften, Starten der Zeitmessung, Aufruf der Proze-  |
    |            dur Sort, Stoppen der Zeitmessung und Anzeige der Ergeb-   |
    |            nise (Eigenschaften und zum Sortieren bentigte Zeit).     |
    *-----------------------------------------------------------------------*)

    VAR i, k         : INTEGER;
        start, stopp : LONGINT;

    BEGIN
      CASE quali.eigen OF
        1: FOR k:= 1 TO n DO
             a[k]:= k
           END
      | 2: FOR k:= 1 TO n DO
             a[k]:= SHORT(ENTIER(R.Uniform() * 32000))
           END;
      | 3: FOR k:= 1 TO n DO
             a[k]:= -k
           END
      END;
      Z.Start;
      P.Yield;                                       (* keine Multitasking  *)
      Sort(1,n);
      Z.Stopp;
      IF Z.differenz < 0 THEN HALT(200) END;
      daten[anzahl].x:= n;
      daten[anzahl].y:= SHORT(Z.differenz);
      Out.Ln;
      Out.String('     Art:          '); Out.Int(quali.eigen,6); Out.Ln;
      Out.String('     Trennelement: '); Out.Int(quali.trenn,6); Out.Ln;
      Out.String('     Anzahl:       '); Out.Int(n,6); Out.Ln;
      Out.String('     Sortierzeit:  '); Out.Int(Z.differenz,6);
      Out.String(' msec'); Out.Ln
    END Sortieren;


  PROCEDURE PfadStr ( VAR message : tShStr );
    BEGIN
      message:= "   Die Datei '";
      S.Append(message,fPfad);
      S.Append(message,"' wurde gelesen.");
    END PfadStr;


  PROCEDURE ErlaeuterungDateiname;
    BEGIN
      Out.Ln; Out.Ln;
      Out.String('   Der Dateiname darf 8 Zeichen lang sein.'); Out.Ln;
      Out.String("   Die Dateierweiterung '.DAT' wird automatisch ergnzt.");
      Out.Ln; Out.Ln;
      Out.String("   Verwenden Sie 'sprechende' Dateinamen, z. B.:"); Out.Ln;
      Out.String("    'QUICKS24' fr Quicksort Art = 2 (zufllig verteilt)");
      Out.Ln;
      Out.String('                    Trennelement = 4 (zuflliges Element)');   
      Out.Ln; Out.Ln
    END ErlaeuterungDateiname;


  PROCEDURE DateiSchreiben ( anzahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Sicheres einlesen eines Dateinamens und Ergnzen der Datei- |
    |           erweiterung '.DAT'. Falls eine Datei gleichen Namens im ak- |
    |           tuellen Pfad existiert, wird gefragt, ob sie gelscht wer-  |
    |           den soll. Wenn ja, ok, wenn nein, kann ein neuer Dateiname  |
    |           eingelesen werden. Falls alles i. O. ist, wird die Datei    |
    |           geffnet und blockweise geschrieben. Der erste Block ent-   |
    |           hlt den Inhalt der Verbundvariablen, somit die Eigenschaf- |
    |           ten der Zahlen usw. vor dem Sortiervorgang (z. B.: 2 fr    |
    |           Zufallszahlen), die folgenden Blcke enthalten die Daten    |
    |           (Anzahl Zahlen und Sortierzeit in Millisekunden).           |
    *-----------------------------------------------------------------------*)

    VAR ok         : BOOLEAN;
        ch         : CHAR;
        handle     : F.Handle;
        laenge,
        i, resCode : INTEGER;
        n          : LONGINT;
        message    : tShStr;
        punkt      : tPunkt;

    BEGIN
      ErlaeuterungDateiname;
      REPEAT
        In.Prompt('Dateiname');
        In.String(fPfad);
        laenge:= SHORT(S.Length(fPfad)); 
        IF (fPfad > ' ') & (laenge <= 8) THEN
          S.Append(fPfad,'.DAT');
          ok:= TRUE
        END;
        IF ok & F.Exist(fPfad) THEN
          In.Prompt('Lschen ( J/N )? N');
          In.Char(ch); ch:= CAP(ch);
          IF ch = 'J' THEN
            F.Delete(fPfad,resCode);
            ok:= TRUE
          ELSE
            ok:= FALSE;
            RETURN ok
          END
        END
      UNTIL ok;
      F.Open(fPfad,TRUE,F.DENYALL,F.WRITEONLY,handle,resCode);
      Out.Ln;
      PfadStr(message);
      Out.String(message); Out.Ln;
      n:= SIZE(tArt);
      F.WriteBlock(handle,quali,n,resCode);
      n:= SIZE(tPunkt);
      FOR i:= 1 TO anzahl DO
        F.WriteBlock(handle,daten[i],n,resCode)
      END;
      F.Close(handle);
      Out.String('   Anzahl Datenstze = '); Out.Int(anzahl,2); Out.Ln;
      Out.Ln;
      RETURN ok
    END DateiSchreiben;


  PROCEDURE DatenLoeschen ( anzahl : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: Anzahl der geschriebenen Datenstze und die Verbundvariable |
    |           quali, somit die Eigenschaften der Zahlen usw. vor dem Sor- |
    |           tiervorgang (z. B.: 2 fr Zufallszahlen)                    |
    | Leistung: 'Lschen' (mit 0 berschreiben) aller Daten im Arbeitsspei- |
    |           cher des Computers, damit die Daten vom externen Speicher   |
    |           eingelesen und nicht die des Arbeitsspeichers fr die Kon-  |
    |           trollausgabe genutzt werden.                                |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      quali.eigen:= 0;
      quali.trenn:= 0;
      FOR i:= 1 TO anzahl DO
        daten[i].x:= 0;
        daten[i].y:= 0
      END;
      anzahl:= 0
    END DatenLoeschen;


  PROCEDURE DateiLesen ( VAR anzahl : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Falls die Datei im aktuellen Pfad existiert, wird sie geff-|
    |           net und die Daten blockweise gelesen. Der erste Block ent-  |
    |           hlt die die Verbundvariable quali, die die Eigenschaften   |
    |           der Zahlen usw. vor dem Sortiervorgang (z. B.: 1 fr be-    |
    |           reits aufsteigend sortierte Zahlen) enthlt und die folgen- |
    |           den Blcke enthalten die Daten (Anzahl Zahlen und Sortier-  |
    |           zeit in Millisekunden).                                     |
    *-----------------------------------------------------------------------*)

    VAR handle  : F.Handle;
        resCode : INTEGER;
        n, read : LONGINT;
        message : tShStr;
        punkt   : tPunkt;

    BEGIN
      IF F.Exist(fPfad) THEN
        F.Open(fPfad,FALSE,F.DENYALL,F.READONLY,handle,resCode);
        n:= SIZE(tArt);
        F.ReadBlock(handle,quali,n,read,resCode);
        anzahl:= 0;
        n:= SIZE(tPunkt);
        WHILE ~F.AtEnd(handle) DO
          INC(anzahl);
          F.ReadBlock(handle,daten[anzahl],n,read,resCode);
        END;
        Out.Ln;
        PfadStr(message);
        Out.String(message); Out.Ln;
        F.Close(handle);
        Out.String('   Anzahl Datenstze = '); Out.Int(anzahl,2); Out.Ln;
        Out.Ln
      ELSE
        HALT(100)
      END 
    END DateiLesen;


  PROCEDURE Kontrollausgabe ( anzahl : INTEGER );

    VAR i : INTEGER;

    BEGIN
      Out.Ln;
      Out.String('   Art:          '); Out.Int(quali.eigen,6); Out.Ln;
      Out.String('   Trennelement: '); Out.Int(quali.trenn,6); Out.Ln;
      FOR i:= 1 TO anzahl DO
        Out.Ln;
        Out.String('   Anzahl:       '); Out.Int(daten[i].x,6); Out.Ln;
        Out.String('   Sortierzeit:  '); Out.Int(daten[i].y,6);
        Out.String(' msec'); Out.Ln
      END;
    END Kontrollausgabe;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR ok     : BOOLEAN;
        ch     : CHAR;
        n,
        anzahl : INTEGER;

    BEGIN
      anzahl:= 0;
      n:= 0;
      ok:= FALSE;
      Art;
      Hinweis;
      REPEAT
        INC(n,1000);
        INC(anzahl);
        Arbeit(n);
        R.Randomize;(* Zufallszahlengenerator mit neuem Wert initialisieren *)
        Sortieren(anzahl,n);
        IF anzahl < max THEN
          In.Prompt('Wiederhlg. (J/N) J');
          In.Char(ch)
        END                      
      UNTIL (ch = 'n') OR (anzahl = max);
      IF (anzahl > 10) & Anfrage() THEN
        ok:= DateiSchreiben(anzahl)
      END;
      IF ok THEN
        DatenLoeschen(anzahl);
        DateiLesen(anzahl);
        Kontrollausgabe(anzahl)
      END
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END Quickso3.

 (* ----------------------------------------------------------------------- *)

