
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)         File-Name: QUICKSO3.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    S O R T I E R E N   (Teile und Herrsche)  3. Variante         |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von INTEGER-Zahlen (Quicksort)                      |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul ist lauffhig, stellt aber nur ein Grundgerst dar. |
  |           Es sollte der Dateiname der '.DAT'-Datei eingebbar und nicht  |
  |           fest vorgegeben sein.                                         |
  |           Siehe ...\sort2\fine\quickso3.mod                             |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\sort2\txt\quickso3.txt und                        |
  |                   ...\sort2\doc\quickso3.doc                            |
  *-------------------------------------------------------------------------*)

 MODULE Quickso3;

  IMPORT In, Out, F:= File, P:= Process, Ran:= RandomNumbers, S:= Strings,
         Zeit;


  CONST  max    = 25;       (* maximale Anzahl der Messungen                *)

         fPfad  = 'QUICKSOR.DAT';

  TYPE   tPunkt = RECORD
                    x, y : LONGINT
                  END;      (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tWerte = RECORD
                    art,
                    trenn : INTEGER;
                    punkt : tPunkt
                  END;

         tDaten = ARRAY max+1 OF tWerte;

  VAR    a      : ARRAY 16001 OF REAL;
         anzahl,
         art, 
         trenn  : INTEGER;
         n      : LONGINT;
         daten  : tDaten;


  PROCEDURE Anfrage() : BOOLEAN;

    VAR ch : CHAR;

    BEGIN
      In.Prompt('Datei (J/N) N');
      In.Char(ch);
      ch:= S.UpCaseChar(ch);
      IF ch = 'J' THEN
        RETURN TRUE
      ELSE
        RETURN FALSE
      END;
    END Anfrage;


  PROCEDURE Art;
    BEGIN
      Out.Ln;
      Out.String('   Die Zahlen sind '); Out.Ln;
      Out.String('     (1) bereits aufsteigend sortiert,'); Out.Ln;
      Out.String('     (2) zufllig verteilt oder'); Out.Ln;
      Out.String('     (3) bereits absteigend sortiert.'); Out.Ln; Out.Ln;
      REPEAT
        In.Prompt('Art  =  1 .. 3');
        In.Int(art)
      UNTIL art IN {1..3};
      daten[anzahl].art:= art;
      Out.String('   Das Trennelement ist'); Out.Ln;
      Out.String('     (1) das erste Element der Teilfolge,'); Out.Ln;
      Out.String('     (2) das mittlere Element der Teilfolge,'); Out.Ln;
      Out.String('     (3) das letzte Element der Teilfolge oder'); Out.Ln;
      Out.String('     (4) ein zuflliges Element der Teilfolge.'); Out.Ln;
      Out.Ln;
      REPEAT
        In.Prompt('Art  =  1 .. 4');
        In.Int(trenn)
      UNTIL trenn IN {1..4};
      daten[anzahl].trenn:= trenn
    END Art;


  PROCEDURE Sort ( l, r : LONGINT );

    VAR i, j : LONGINT;
        x, w : REAL;

    BEGIN
      i:= l;
      j:= r;
      CASE trenn OF 
        1: x:= a[l]
      | 2: x:= a[(l+r) DIV 2]
      | 3: x:= a[r]
      | 4: x:= a[l+ENTIER(Ran.Uniform()*(r-l+1))]
             (* Der Index ist ein zuflliger Wert aus dem Intervall [l, r]. *)
      END;
      REPEAT
        WHILE a[i] < x DO INC(i) END;
        WHILE x < a[j] DO DEC(j) END;
        IF i <= j THEN
          w:= a[i];
          a[i]:= a[j];
          a[j]:= w;
          INC(i);
          DEC(j)
        END
      UNTIL i > j;
      IF l < j THEN Sort(l,j) END;
      IF i < r THEN Sort(i,r) END
    END Sort;


  PROCEDURE Sortieren;

    VAR k : LONGINT;

    BEGIN
      IF (trenn = 2) OR ( trenn = 4) THEN
        REPEAT
          In.Prompt('Anzahl <= 16000');
          In.LongInt(n)
        UNTIL n < 163001
      ELSE
        REPEAT
          In.Prompt('Anzahl <= 450');
          In.LongInt(n)
        UNTIL n < 451
      END;
      CASE art OF
        1: FOR k:= 1 TO n DO
             a[k]:= k
           END
      | 2: FOR k:= 1 TO n DO
             a[k]:= Ran.Uniform() * 100000
           END;
      | 3: FOR k:= 1 TO n DO
             a[k]:= -k
           END
      END;
      Zeit.Start;
      Sort(1,n);
      Zeit.Stopp;
      daten[anzahl].punkt.x:= n;
      daten[anzahl].punkt.y:= Zeit.differenz;
      Out.Ln;
      Out.String('   Art:          '); Out.Int(art,6); Out.Ln;
      Out.String('   Trennelement: '); Out.Int(trenn,6); Out.Ln;
      Out.String('   Anzahl:       '); Out.Int(n,6); Out.Ln;
      Out.String('   Sortierzeit:  '); Out.Int(Zeit.differenz,6);
      Out.String(' msec'); Out.Ln
    END Sortieren;


  PROCEDURE DateiSchreiben;

    VAR handle     : F.Handle;
        i, resCode : INTEGER;
        n          : LONGINT;
        werte      : tWerte;

    BEGIN
      IF F.Exist(fPfad) THEN
        F.Delete(fPfad,resCode)
      END;
      F.Open(fPfad,TRUE,F.DENYNONE,F.WRITEONLY,handle,resCode);
      Out.Ln;
      Out.String("   Die Datei 'QUICKSOR.DAT' wurde angelegt."); Out.Ln;
      n:= SIZE(tWerte);
      FOR i:= 1 TO anzahl DO
        F.WriteBlock(handle,daten[i],n,resCode)
      END;
      F.Close(handle);
      Out.String('   Anzahl Datenstze = '); Out.Int(anzahl,2); Out.Ln; Out.Ln 
    END DateiSchreiben;


  PROCEDURE ProgMain*();

    VAR ch      : CHAR;
        resCode : INTEGER;

    BEGIN
      anzahl:= 0; 
      REPEAT
        INC(anzahl);
        Art;
        (* Hier sollten Anweisungen eingefgt werden *)
        Sortieren;
        In.Prompt('Wiederhlg. (J/N) J');
        In.Char(ch)
      UNTIL (ch = 'n') OR (anzahl = max);
       (* Hier sollten Anweisungen eingefgt werden *)
      IF Anfrage() THEN DateiSchreiben END
    END ProgMain;


 BEGIN
   In.Open;
   Out.Open
 END Quickso3.
