
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (32 Bit)         File-Name: TEXTSTAT.MOD |
  |-------------------------------------------------------------------------|
  |  Autor:   Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: 06-07.05.2001 Version: V 1.2                |
  |-------------------------------------------------------------------------|
  | Thema:    Textstatistik (Wort- und Buchstabenanzahl eines Textes),      |
  |           demonstrativer Einsatz eines Sortierverfahrens                |
  |-------------------------------------------------------------------------|
  | Leistung: Ermitteln, aus wie vielen Abstzen und Wrtern ein Text be-   |
  |           steht und wie hufig die 26 Buchstaben des deutschen Alpha-   |
  |           bets in dem Text vorkommen (Statistik).                       |
  |-------------------------------------------------------------------------|
  | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.    |
  |           GRAFIKQ.MOD (Oberon-2 V4)                                     |
  |-------------------------------------------------------------------------|
  | Hinweise: Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |           siehe:                                                        |
  |                ...\GRUNDLAGEN\ALLGEMEIN\DOC\GRAFIK.DOC  oder            |
  |                ...\GRUNDLAGEN\ALLGEMEIN\TXT\GRAFIK.TXT                  |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE TextStat;

  IMPORT C:= ColorPlane, F:= File, Fl:= Float, P:= Process, SD:= ScanDir,
         S:= Strings, Sy:= SYSTEM, W:= Windows;

  CONST  delayTime =   20;
         maxAnzahl = 1441;
         bc_red    =    8;
         bc_green  =   16;
         bc_blue   =   48;
         br        =  480;
         chrAnz    =   60;               (* Anzahl Zeichen pro Ausgabezeile *)
         chrBreite =    8;
         xOffset   =   36;
         yOffset   =  530;
         zeiHoehe  =   20;

  TYPE   tZahlStr  = ARRAY  6 OF CHAR;
         tAlphabet = ARRAY 27 OF INTEGER;
         tStr30    = ARRAY 31 OF CHAR;
         tReihung  = ARRAY maxAnzahl+1 OF INTEGER;

         tArt      = RECORD
                       eigen, trenn, dTfaktor : INTEGER
                     END;

  VAR    lg,
         maxStellen : INTEGER;
         dummy      : LONGINT;
         zeichen    : SD.tShStr;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE FlushKeybuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;
 
    BEGIN
      WHILE C.KeyPressed() DO Dummy:= C.ReadKey() END
    END FlushKeybuffer;


  PROCEDURE Delay ( dTfaktor : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR blue, green, red : INTEGER;
        i, endwert       : LONGINT;

   BEGIN
     endwert:= delayTime * dTfaktor;
     FOR i:= 1 TO endwert DO C.GetDot(0,0,red,green,blue) END 
   END Delay;


  PROCEDURE Delay_ms ( delayTime : LONGINT );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine definierte Zeit warten.                                |
    *-----------------------------------------------------------------------*)

    VAR endwert, tick : LONGINT;

   BEGIN
     tick:= W.GetTickCount();
     endwert:= tick + delayTime;
     REPEAT UNTIL endwert < W.GetTickCount()
   END Delay_ms;


  PROCEDURE StringLaengenkorrektur ( n : INTEGER; VAR zahlStr : tZahlStr );
   (*-----------------------------------------------------------------------*
    | bernahme: Geforderte Zeichenzahl n fr die bernommene Zeichenkette  |
    | Referenz:  lngenkorrigierte Zeichenkette                             |
    | Leistung:  In die bernommene Zeichenkette werden linksbndig so      |
    |            viele Leerzeichen eingefgt, bis die geforderte Gesamt-    |
    |            lnge erreicht ist.                                        |
    *-----------------------------------------------------------------------*)

    BEGIN
      WHILE S.Length(zahlStr) < n DO S.Insert(' ',zahlStr,1) END
    END StringLaengenkorrektur;


  PROCEDURE WriteShortStrXY ( x, y : INTEGER; text : SD.tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate und der auszugebender Text (string)    |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette im Grafikmodus.   |
    |            x = Pixelspalte; x = 0 entspricht der linken Begrenzung    |
    |                                   des aktuellen Grafikfensters        |
    |            y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung   |
    |                                   des aktuellen Grafikfensters        |
    *-----------------------------------------------------------------------*)

   BEGIN
     C.GotoXY(x,y); C.WriteStr(text)
   END WriteShortStrXY;


  PROCEDURE WriteShortColStrXY ( r, g, b, x, y : INTEGER; Text : SD.tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: RGB-Werte der Textfarbe, x- und y-Koordinate sowie der     |
    |            auszugebender Text (string)                                |
    | Leistung:  Positionierte farbige Ausgabe einer Zeichenkette.          |
    *-----------------------------------------------------------------------*)

   BEGIN
     C.SetForeColor(r,g,b);
     C.GotoXY(x,y); C.WriteStr(Text)
   END WriteShortColStrXY;


  PROCEDURE MaxAnzahlZeichen ( alphabet : tAlphabet );
   (*-----------------------------------------------------------------------*
    | bernahme: Reihung, deren Komponenten die Hufigkeit des Vorhanden-   |
    |            seins der Buchstaben des deutschen Alphabets im analysier- |
    |            ten Text in alphabetisch aufsteigender Ordnung enthlt     |
    | Leistung:  Ermitteln der grten Anzahl und Bestimmen der Stellenzahl |
    |            (maxStellen) fr die formatierte Ausgabe.                  |
    *-----------------------------------------------------------------------*)

    VAR i, max  : INTEGER;
        zahlStr : tZahlStr;

    BEGIN
      max:= 0;
      FOR i:= 1 TO 26 DO
        IF alphabet[i] > max THEN max:= alphabet[i] END
      END;
      S.Str(max,zahlStr);
      maxStellen:= SHORT(S.Length(zahlStr))
    END MaxAnzahlZeichen;


  PROCEDURE Enter ( x, y : INTEGER );
   (*-----------------------------------------------------------------------|
    | bernahme: x- und y-Koordinate fr die Position des Zeichenkette,     |
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    |            Die Wartezeit sorgt fr einen ruhigeren Programmlauf.      |
    |-----------------------------------------------------------------------*)

   VAR  ChTaste : CHAR;
     
   BEGIN
     Delay_ms(delayTime*5);
     C.SetBackColor(bc_red,bc_green,bc_blue);
     WriteShortColStrXY(32,160,160,x,y,'< ENTER >');
     FlushKeybuffer;
     REPEAT
        ChTaste:= C.ReadKey()
     UNTIL (ChTaste = CHR(13)) OR (ChTaste = CHR(27));
     C.Bar(x,y-14,x+72,y,C.erase)
   END Enter;


  PROCEDURE Wiederholung() : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information, Warten auf eine Tastenbettigung |
    |           (Tastatur), Wandeln des Zeichens in einen Grobuchstaben,   |
    |           falls ein Kleinbuchstabe eingegeben wurde und Auswerten der |
    |           Eingabe (boolescher Wert).                                  |
    | Referenz: Funktionswert TRUE, falls eine Wiederholung gewnscht wird, |
    |           d.h. es wurde eine beliebige Taste auer <N> bettigt,      |
    |           sonst FALSE.                                                |
    *-----------------------------------------------------------------------*)

    VAR ch : CHAR;
      
    BEGIN
      Delay_ms(delayTime*5);
      C.SetBackColor(bc_red,bc_green,bc_blue);
      WriteShortColStrXY(32,160,160,575,35,'Wiederholung (J/N) J');
      FlushKeybuffer;
      ch:= C.ReadKey();
      ch:= S.UpCaseChar(ch);
      C.Bar(575,35,735,19,C.ERASE);
      RETURN ~(ch = 'N') 
    END Wiederholung;


  PROCEDURE AusgabefensterLoeschen;
    BEGIN
      C.SetBackColor(0,0,0);
      C.Bar(xOffset-2,yOffset-br-18,xOffset+br+2,yOffset-15,C.ERASE)
    END AusgabefensterLoeschen;


  PROCEDURE Ausgabefenster;
    BEGIN
      AusgabefensterLoeschen;
      C.SetForeColor(0,192,192);
      C.Box(xOffset-5,yOffset-br-21,xOffset+br+5,yOffset-12,C.DRAW)
    END Ausgabefenster;


  PROCEDURE Ueberschrift;
    BEGIN
      Ausgabefenster;
      C.SetBackColor(bc_red,bc_green,bc_blue);
      WriteShortColStrXY(32,208,208,574,495,'T E X T A N A L Y S E');
      C.Line(564,476,752,476,C.DRAW);
      C.Line(564,475,752,475,C.DRAW);
      Delay_ms(delayTime*2)
    END Ueberschrift;


  PROCEDURE AuswaehlenTextDatei ( VAR fPfad : SD.tShStr;
                                    VAR exitFlag, ohneTXTdatei : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | Referenz: Dateipfad (fPfad, DT: SD.tShStr, aus Service-Modul ScanDir) |
    |           Wahrheitswert der Variable ohneTXTdatei (DT: BOOLEAN)       |
    | Leistung: Aufnehmen aller Dateinamen des aktuellen Pfads, die die Da- |
    |           teierweiterung 'TXT' besitzen, in eine einfach zeigerver-   |
    |           kettete Liste. Die Dateinamen werden alphabetisch aufstei-  |
    |           gend in die Liste eingefgt. Sind alle Dateinamen erfasst,  |
    |           werden sie mit Kennziffern versehen am Bildschirm ausgege-  |
    |           ben. Der Programmanwender kann eine Kennzahl eingeben, die  |
    |           den Programmablauf steuert. (Wahl des Textes, der angezeigt |
    |           und ausgewertet wird.)                                      |
    |           Befinden sich im aktuellen keine TXT-Dateien wird der Vari- |
    |           able 'ohneTXTdatei' der Wert TRUE zugewiesen und nach Rck- |
    |           kehr zum rufenden UP eine Fehlermeldung angezeigt.          |
    *-----------------------------------------------------------------------*)

    CONST x = 560;  y = 430;

    VAR   resCode    : CHAR;
          kennZahl,
          txtRed,
          txtGreen,
          txtBlue,
          zaehler    : INTEGER;
          list       : SD.tList;
          dateiErw,
          dateiname,
          fehlerMldg,
          message    : SD.tShStr;
          zahlStr,
          kZahlStr   : tZahlStr;


    PROCEDURE AnzeigenTXTdateien;
     (**
      | Leistung: Beweglichen Zeiger (Verbund list, DT: SD.tList) auf den   |
      |           Listenanfang setzten.                                     |
      |           Einen Eintrag (Dateinamen) aus der Liste hohlen, Zhler   |
      |           inkrementieren, Wert des Zhlers in eine Zeichenkette     |
      |           wandeln, in runde Klammern einschlieen, Leerzeichen und  |
      |           Dateiname anfgen und am Bildschirm als Menpunkt ausge-  |
      |           ben. Vorgang wiederholen, bis das Listenende erreicht ist.|
      **)

      VAR x1 : INTEGER;

      BEGIN
        txtRed:=     0;
        txtGreen:= 160;
        txtBlue:=  160;
        x1:= x + 40;
        zaehler:= 0;
        SD.GotoFirstEntry(list);               (* beweglichen Zeiger auf    *)
                                               (* den Listenanfang setzen   *)
        WHILE ~SD.EndOfList(list) DO           (* solange kein Listenende   *)
          SD.FetchEntry(list,fehlerMldg,dateiname);
          INC(zaehler);
          message:= '(';
          S.Str(zaehler,zahlStr); S.Append(message,zahlStr);
          S.Append(message,') '); S.Append(message,dateiname);
                                               (*  ... Eintrag entnehmen    *)
          WriteShortColStrXY(txtRed,txtGreen,txtBlue,x1,y-zaehler*16,message);
                                               (*  ... Eintrag ausgeben     *)
          SD.GotoNextEntry(list,fehlerMldg);   (* nchsten Verbund anpeilen *)
        END
      END AnzeigenTXTdateien;


    PROCEDURE AuswaehlenTXTdatei ( VAR zaehler : INTEGER );
     (**
      | Referenz: Anzahl der gltigen Dateinamen (zaehler, DT: INTEGER)     |
      | Leistung: Positionierte Ausgabe einer Zeichenkette, sicheres Ein-   |
      |           lesen einer Kennzahl aus dem zugelassenen Bereich, d. h.  |
      |           es werden nur Eingaben zwischen 1 und dem Wert der Vari-  |
      |           able zaehler zugelassen, fehlerhafte Eingabe werden abge- |
      |           wiesen und zur Wiederholung der Eingabe aufgefordert,     |
      |           statt einen Laufzeitfehler zu verursachen.                |
      |           Wird nur die Enter-Taste bettigt, erhlt die Variable    |
      |           kZahlStr den Wert '1' (---> kennZahl:= 1).                |
      |           Das Bettigen der Esc-Taste oder Eingabe der Kennzahl '0' |
      |           beendet die Programmfortfhrung.                          |
      **)

      BEGIN
        WriteShortColStrXY(0,112,112,x+40,100,'Kennzahl = 0 oder');
        WriteShortStrXY(x+40,80,'< Esc > = E N D E');
        kennZahl:= 0;
        WriteShortColStrXY(120,208,160,x+56,y-(zaehler+2)*16,'Kennzahl =');
        REPEAT
          C.GotoXY(x+144,y-32-zaehler*16); C.WriteStr(' ');
          C.GotoXY(x+144,y-32-zaehler*16); C.ReadStr(kZahlStr,1,resCode)
        UNTIL (resCode = C.ESC)
            OR ((kZahlStr >= '0') OR (kZahlStr = '')) & (kZahlStr <= zahlStr);
        IF (resCode # C.ESC) & (kZahlStr = '') THEN
          kennZahl:= 1;
          WriteShortStrXY(x+144,y-32-zaehler*16,'1')
        ELSIF kZahlStr >= '0' THEN
          kennZahl:= SHORT(S.Val(kZahlStr))
        END;
        IF kennZahl > 0 THEN
          SD.GotoFirstEntry(list);
          FOR zaehler:= 1 TO kennZahl-1 DO
            SD.GotoNextEntry(list,fehlerMldg)  (* nchsten Verbund anpeilen *)
          END;
          SD.FetchEntry(list,fehlerMldg,fPfad) (*  ... Eintrag zuweisen     *)
        ELSE
          exitFlag:= TRUE
        END  
      END AuswaehlenTXTdatei;


    BEGIN  (* AuswaehlenTextDatei *)
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(x,y,x+200,y-366,C.ERASE);
      dateiErw:= 'TXT';
      SD.ScanDirectoryEntrys(zaehler,list,dateiErw,ohneTXTdatei);
      IF ~ohneTXTdatei THEN
        AnzeigenTXTdateien;
        AuswaehlenTXTdatei(zaehler)
      ELSE
        WriteShortColStrXY(255,127,0,x+30,y,'Der aktuellen Pfad');
        WriteShortStrXY(x+6,y-20,'enthlt keine TXT-Datei!');
        HALT(101)
      END
    END AuswaehlenTextDatei;


  PROCEDURE Textausgabe ( a : tReihung; blueFlag, firstFlag : BOOLEAN;
                               anzahl, xTextoffset, yTextoffset : INTEGER;
                                                            VAR y : INTEGER );
   (*-----------------------------------------------------------------------|
    | bernahme: Text, der zeichenweise der Reihung a zugewiesen wurde,     |
    |            zwei boolesche Variablen, die fr die Textfarbe (blueFlag) |
    |            bzw. fr die Anzeige der Anzahl Abstze (firstFlag) stehen,|
    |            im Wert der Variablen anzahl steht die Gesamtzahl der Zei- |
    |            chen des Textes und zwei Offsets fr die Formatierung des  |
    |            Textes im Ausgabefenster                                   |
    | Referenz:  Wert der y-Koordinate des Grafikcursors, der fr die fol-  |
    |            gende Ausgabe im kleinen Fenster bentigt wird.            |
    | Leistung:  Positionierte, farbige Ausgabe aller druckbaren Zeichen    |
    |            des bernommenen Textes unter Bercksichtigung der Zeilen- |
    |            umbrche. Wird der Text zum ersten Mal ausgegeben, werden  |
    |            die Zeilenumbrche unter 'Anzahl Abstze' ausgegeben.      |
    |-----------------------------------------------------------------------*)

    VAR  absAnzahl, i, k,
         spalte, zeile   : INTEGER; 
         zahlStr         : tZahlStr;

    BEGIN
      spalte:= xOffset;
      zeile:=  yOffset;
      C.SetBackColor(0,0,0);
      IF blueFlag THEN
        WriteShortColStrXY(120,208,160,680,400,'kompr.');
        C.SetForeColor(160,96,255)
      ELSE
        C.SetForeColor(255,255,128)
      END;
      absAnzahl:= 0;
      i:= 0; k:= 0;
      WHILE i < anzahl-1 DO
        INC(i); INC(k);
        IF k MOD chrAnz = 1 THEN
          DEC(zeile,zeiHoehe);
          spalte:= xOffset
        END;
        IF (a[i] = 13) & (a[i+1] = 10) THEN    (* Zeilenumbruch ausfhrenen *)
          DEC(zeile,zeiHoehe);
          spalte:= xOffset;
          INC(absAnzahl);
          INC(i);
          k:= 1
        ELSE
          zeichen:= '';
          S.AppendChar(zeichen,CHR(a[i]));
          WriteShortStrXY(spalte+xTextoffset,zeile-yTextoffset,zeichen);
          INC(spalte,chrBreite)
        END
      END; (* while *)
      y:= zeile - yTextoffset;
      IF firstFlag THEN
        C.SetBackColor(bc_red,bc_green,bc_blue);
        WriteShortColStrXY(120,208,160,576,330,'Anzahl Abstze   = ');
        S.Str(absAnzahl,zahlStr);
        StringLaengenkorrektur(lg,zahlStr);
        C.WriteStr(zahlStr)
      END
    END Textausgabe;


  PROCEDURE Wortanzahl ( a : tReihung; anzahl : INTEGER; 
                                                   VAR wortAnzahl : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Text, der zeichenweise der Reihung a zugewiesen wurde, Ge- |
    |            samtzahl der Zeichen des Textes (anzahl)                   |
    | Referenz:  Anzahl erkannter Wrter (wortAnzahl, DT: INTEGER)          |
    | Leistung:  Die Komponenten der Reihung werden zeichenweise getestet.  |
    |            Ist ein Zeichen der Menge der Klein- oder Grobuchstaben   |
    |            zuordenbar, wird es gezhlt (chrAnzahl).                   |
    |            Die boolesche Variable imWort wird mit dem Wert FALSE ini- |
    |            tialisiert. Im Schleifenkrper der Zhlschleife wird durch |
    |            geschickte Abfrage (IF ... THEN ... ELSIF ... END) ermit-  |
    |            telt, ob ein Zeichen zu einem Wort gehrt oder ein Trenn-  |
    |            zeichen ist. Die Wrter werden gezhlt.                    |
    |            Die Anzahl der gefundenen Wrter und die Gesamtzahl der    |
    |            Zeichen werden am Bildschirm angezeigt.                    |
    *-----------------------------------------------------------------------*)

    CONST minChar = 'A';
          maxChar = 'Z';
          x       = 576;

    VAR imWort,
        buchstabe : BOOLEAN;
        zeichen   : CHAR;
        i,
        chrAnzahl : INTEGER;
        zahlStr   : tZahlStr;

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.SetForeColor(120,208,160);
      imWort:= FALSE;
      chrAnzahl:= 0; wortAnzahl:= 0;
      FOR i:= 1 TO anzahl-1 DO
        zeichen:= CHR(a[i]);
        buchstabe:= (minChar <= CAP(zeichen)) & (CAP(zeichen) <= maxChar);
        IF buchstabe THEN INC(chrAnzahl) END;
        IF imWort & ~buchstabe THEN
          imWort:= FALSE
        ELSIF ~imWort & buchstabe THEN
          imWort:= TRUE;
          INC(wortAnzahl)
        END
      END;
      WriteShortColStrXY(120,208,160,x,310,'Anzahl Wrter    = ');
      S.Str(wortAnzahl,zahlStr);
      StringLaengenkorrektur(lg,zahlStr);
      C.WriteStr(zahlStr);
      WriteShortColStrXY(120,208,160,x,290,'Anzahl Zeichen   = ');
      S.Str(anzahl,zahlStr);
      StringLaengenkorrektur(lg,zahlStr);
      C.WriteStr(zahlStr);
      WriteShortColStrXY(120,208,160,x,270,'davon Buchstaben = ');
      S.Str(chrAnzahl,zahlStr);
      StringLaengenkorrektur(lg,zahlStr);
      C.WriteStr(zahlStr);
      WriteShortColStrXY(56,144,96,x,240,'Ein Absatzende, auch');
      WriteShortStrXY(x,220,'harter Zeilenumbruch');
      WriteShortStrXY(x,200,'genannt, enthlt zwei');
      WriteShortStrXY(x,180,'Zeichen (0A 0D Hex)');
      Enter(680,35)
    END Wortanzahl;


  PROCEDURE TextEinlesenAusgeben ( fPfad : SD.tShStr;
                                   VAR a : tReihung; VAR anzahl : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Dateiname der zu lesenden Textdatei im aktuellen Pfad      |
    | Referenz:  Text, der zeichenweise der Reihung a zugewiesen wurde, Ge- |
    |            samtzahl der Zeichen des Textes (anzahl)                   |
    | Leistung:  Lesen der Inhalte der ersten beiden Zeilen der Textdatei   |
    |            von einem externen Datentrger und Ausgabe des Inhalts am  |
    |            Bildschirm. Dann wird der Rest der Textdatei zeichenweise  |
    |            gelesen und die Ordinalzahl des Zeichens (ASCII) je einer  |
    |            Komponenten der Reihung a zugewiesen.                      |
    *-----------------------------------------------------------------------*)

    CONST x1 = 560;  x2 = 576;  y1 = 430;

    VAR   key      : CHAR;
          fHandle  : F.Handle;
          resCode,
          i, y     : INTEGER;
          row      : SD.tShStr;
          zahlStr  : tZahlStr;

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(x1,y1,x1+200,y1-366,C.ERASE);
      WriteShortColStrXY(0,160,160,x1,y1,'Ausgabe:');
      IF F.Exist(fPfad) THEN
        F.Open(fPfad,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode);
        F.ReadLn(fHandle,row,resCode);
        WriteShortColStrXY(120,208,160,x2,y1-30,row);
        F.ReadLn(fHandle,row,resCode);
        WriteShortColStrXY(56,144,96,x2,y1-56,row);
        i:= 1;
        WHILE (~F.AtEnd(fHandle)) & (i < maxAnzahl) DO
          F.ReadChar(fHandle,key,resCode);
          a[i]:= ORD(key);
          INC(i)
        END;
        anzahl:= i;
        F.Close(fHandle);
        S.Str(anzahl,zahlStr);
        lg:= SHORT(S.Length(zahlStr));
        Textausgabe(a,FALSE,TRUE,anzahl,10,14,y)
      ELSE
        WriteShortColStrXY(255,127,0,x2+16,y1-30,'Fehler beim Lesen');
        WriteShortStrXY(x2+32,y1-50,'der Textatei!');
        HALT(102)
      END
    END TextEinlesenAusgeben;


  PROCEDURE EntfernenZeilenumbrueche ( VAR a : tReihung;
                                                 VAR anzahl, leer : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: Text ohne Zeilenumbrche, Reihung a, Gesamtzahl der Zeichen |
    |           des Textes, anzahl, und Anzahl der im Text enthaltenen Leer-|
    |           zeichen                                                     |
    | Leistung: Entfernen aller harten Zeilenumbrche (Absatzende) und      |
    |           Zhlen der im Text enthaltenen Leerzeichen.                 |
    *-----------------------------------------------------------------------*)

    VAR i, k : INTEGER;
        b    : tReihung;

    BEGIN
      k:= 1; leer:= 0;
      FOR i:= 1 TO anzahl DO
        IF (a[i] = 13) OR (a[i] = 10) THEN
          b[k]:= 0
        ELSIF
          a[i] = 32 THEN
          b[k]:= 183;
          INC(leer);
          INC(k)
        ELSE
          b[k]:= a[i];
          INC(k)
        END  
      END;
      a:= b;
      anzahl:= k-1
    END EntfernenZeilenumbrueche;


  PROCEDURE AnzeigeLeerzeichen ( leer, y : INTEGER; blueFlag : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | bernahme: Anzahl Leerzeichen, leer, aktuelle y-Koordinate des Gra-   |
    |            fikcursors, y, und eine boolesche Variable, blueFlag       |
    | Leistung:  In Abhngigkeit des Werts der boolesche Variablen, wird    |
    |            entweder eine komplette Information ber die Anzahl der    |
    |            Leerzeichen am Bildschirm unterhalb des in komprimierter   |
    |            Form angezeigten Textes gegeben oder die Farbe des Leer-   |
    |            zeichensymbols ('') innerhalb der Information gendert.   |
    *-----------------------------------------------------------------------*)

    VAR zahlStr : tZahlStr;

    BEGIN
      y:= y - 36;
      IF blueFlag THEN
        WriteShortColStrXY(120,208,160,70,y,'( ) = Leerzeichen (');
        S.Str(leer,zahlStr); C.WriteStr(zahlStr);
        C.WriteStr(' Stck)');
        WriteShortColStrXY(160,96,255,78,y,'')
      ELSE
        WriteShortColStrXY(255,255,128,78,y,'')
      END
    END AnzeigeLeerzeichen;


  PROCEDURE Sortieren ( VAR a : tReihung; anzahl, leer : INTEGER );

    VAR  x, y : INTEGER;

    PROCEDURE Quicksort ( l, r : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Sortieren der Werte der Komponenten der Reihung nach auf-   |
    |           steigender Gre mittels Quicksort (Teile und Herrsche).    |
    | Hinweise: Beschreibung des Algorithmus siehe:                         |
    |              ...\SORT2\TXT\QUICKSOR.TXT und ...\QUICKS1.TXT           |
    |             oder bei Nutzung von MS-WORD (o..) unter                 |
    |              ...\SORT2\DOC\QUICKSOR.DOC und ...\QUICKS1.DOC           |
    |           Die Zeichen werden in der originalen Anordnung der Kompo-   |
    |           nenten der Reihung blau angezeigt. Die beiden Zeichen, die  |
    |           an einer Tauschaktion beteiligt sind werden gelscht und    |
    |           an ihrer jeweils neuen Position als gelbes Zeichen ausge-   |
    |           geben.                                                      |
    |           Zum Sortieren kann selbstverstndlich auch ein anderer Sor- |
    |           tieralgorithmus eingesetzt werden!                          |
    |           Da Quicksort sehr schnell ist, wurde es fr die Animation   |
    |           stark gebremst (Delay(delayTime);). Sollte die Geschwindig- |
    |           keit ungnstig sein, sollte die globale Konstante delayTime |
    |           einen der Rechnerleistung angepassten Wert erhalten.        |
    *-----------------------------------------------------------------------*)

      VAR i, j, n, w : INTEGER;

      BEGIN
        i:= l;
        j:= r;
        n:= a[(l+r) DIV 2];
        P.Yield;                                     (* keine Multitasking  *)
        REPEAT
          WHILE a[i] < n DO INC(i) END;
          WHILE n < a[j] DO DEC(j) END;
          IF i <= j THEN
            Delay(delayTime);
            w:= a[i];
            a[i]:= a[j];
            a[j]:= w;
            Delay(delayTime);
            zeichen:= '';
            S.AppendChar(zeichen,CHR(a[i]));
            x:= xOffset+((i-1) MOD chrAnz)*chrBreite;
            y:= yOffset-(((i-1) DIV chrAnz)+2)*zeiHoehe+6;
            WriteShortStrXY(x,y,' '); WriteShortStrXY(x,y,zeichen);
            zeichen:= '';
            S.AppendChar(zeichen,CHR(a[j]));
            WriteShortStrXY(xOffset+((j-1) MOD chrAnz)*chrBreite,
                       yOffset-(((j-1) DIV chrAnz)+2)*zeiHoehe+6,zeichen);
            INC(i);
            DEC(j)
          END
        UNTIL i > j;
        IF l < j THEN Quicksort(l,j) END;
        IF i < r THEN Quicksort(i,r) END
      END Quicksort;

    BEGIN  (* Start *)
      C.SetBackColor(bc_red,bc_green,bc_blue);
      Textausgabe(a,TRUE,FALSE,anzahl,0,14,y);
      AnzeigeLeerzeichen(leer,y,TRUE);
      Enter(680,35);
      WriteShortColStrXY(255,127,0,576,400,'Sortiervorgang luft!');
      AnzeigeLeerzeichen(leer,y,FALSE);
      C.SetBackColor(0,0,0);
      C.SetForeColor(255,255,127);
      Quicksort(1,anzahl);
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(576,400,744,384,C.ERASE);
      WriteShortColStrXY(120,208,160,576,400,'Sortierte Zeichen');
      Enter(680,35)
    END Sortieren;


  PROCEDURE Strichgruppen ( alphabet : tAlphabet; i, x : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Reihung alphabet, deren Komponenten die Hufigkeit des     |
    |            Vorhandenseins der Buchstaben des deutschen Alphabets im   |
    |            analysierten Text in alphabetisch aufsteigender Ordnung    |
    |            enthlt. Die Variable i hlt den Index der aktuellen Kom-  |
    |            ponente und x die x-Koordinate der Bildschirmposition an   |
    |            der der erste Strich bzw. die erste Fnfer-Strichgruppe    |
    |            ausgegeben werden soll.                                    |
    | Leistung:  Die Hufigkeit des Vorhandenseins eines Buchstabens wird   |
    |            durch Anzeige von Fnfer-Strichgruppen und Einzelstrichen  |
    |            symbolisiert. Jedes Fnfer-Strichgruppenpaar wird farbig   |
    |            abgesetzt, so dass der Eindruck von Zehner-Strichgruppen   |
    |            entsteht.                                                  |
    *-----------------------------------------------------------------------*)

    CONST y = 400;

    VAR colorFlag : BOOLEAN;
        einer,
        fuenfer,
        x1, y1    : INTEGER;

    PROCEDURE DarstellenFuenfergruppe;
     (**
      | Leistung: Darstellen farbig abgesetzter Fnfer-Strichgruppen.       |
      **)

      VAR i, k : INTEGER;

      BEGIN
        x1:= x;
        FOR i:= 1 TO fuenfer DO
          INC(x1,28);
          IF ODD(i) THEN
            C.SetForeColor(255,128,96);
            colorFlag:= TRUE
          ELSE
            C.SetForeColor(127,223,223);
            colorFlag:= FALSE
          END;
          FOR k:= 1 TO 4 DO
            C.Line(x1+k*4,y1,x1+k*4-3,y1-13,C.DRAW)
          END;
          C.Line(x1-1,y1-6,x1+18,y1-6,C.DRAW)
        END;
      END DarstellenFuenfergruppe;

    PROCEDURE DarstellenEiner;
     (**
      | Leistung: Darstellen einzelner Striche (1..4) unter Bercksichti-   |
      |           gung der gewnchten Farbfolge.                            |
      **)

      VAR i : INTEGER;

      BEGIN
        INC(x1,24);
        IF colorFlag THEN
          C.SetForeColor(127,223,223)
        ELSE
          C.SetForeColor(255,128,96)
        END;
        FOR i:= 1 TO einer DO
          INC(x1,2);
          C.Line(x1+i*2,y1,x1+i*2-3,y1-13,C.DRAW)
        END
      END DarstellenEiner;

    BEGIN  (* Strichgruppen *)
      y1:= 522 - i * 18;
      IF alphabet[i] # 0 THEN
        fuenfer:= alphabet[i] DIV 5;
        einer:= alphabet[i] MOD 5;
        DarstellenFuenfergruppe;
        DarstellenEiner
      END
    END Strichgruppen;


  PROCEDURE InitialisierenAlphabet ( VAR alphabet : tAlphabet );
   (*-----------------------------------------------------------------------*
    | Referenz: Reihung alphabet, deren Komponenten die Hufigkeit des Vor- |
    |           handenseins der Buchstaben des deutschen Alphabets im zu    |
    |           analysierenden Text in alphabetisch aufsteigender Ordnung   |
    |           enthalten werden.                                           |
    | Leistung: Initialisieren aller Komponenten mit dem Wert 0.            |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      FOR i:= 1 TO 26 DO alphabet[i]:= 0 END;    
    END InitialisierenAlphabet;


  PROCEDURE AusgabeAnzahlBuchstaben ( a : tReihung; VAR alphabet : tAlphabet;
                                         anzahl : INTEGER; VAR x1 : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Text, der zeichenweise der Reihung a zugewiesen wurde, Ge- |
    |            samtzahl der Zeichen des Textes (anzahl)                   |
    | Referenz:  Reihung alphabet, deren Komponenten die Hufigkeit des     |
    |            Vorhandenseins der Buchstaben des deutschen Alphabets im   |
    |            analysierten Text in alphabetisch aufsteigender Ordnung    |
    |            enthlt und x1, die x-Koordinate, an der Grafikcursor in   |
    |            Abhngigkeit der grten auszugebenden Zahl positioniert   |
    |            wurde.                                                     |
    | Leistung:  Ermitteln der Buchstabenhufigkeit innerhalb des zu ana-   |
    |            lysierenden Textes. Einzelheiten, siehe Erluterungen im   |
    |            Quelltext (remarks).                                       |
    *-----------------------------------------------------------------------*)

    VAR key      : ARRAY 5 OF CHAR;
        zeichen  : CHAR;
        einzel,
        fuenfer,
        i, x,
        y, y1    : INTEGER;
        zahlStr  : tZahlStr;

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      WriteShortColStrXY(120,208,160,576,400,'S t a t i s t i k');
      C.SetBackColor(0,0,0);
      x:= 60; y:= 522;
      C.SetForeColor(255,255,127);
      InitialisierenAlphabet(alphabet);
      FOR i:= 1 TO anzahl DO
        IF a[i] > 96 THEN a[i]:= a[i] - 32 END;
                               (* Kleinbuchstaben in Grobuchstaben wandeln *)
        IF (65 <= a[i]) & (a[i] <= 91) THEN
                               (* ... in der Menge der Grobuchstaben       *)
          INC(alphabet[a[i]-64])
        END                    (* a[i] = Ordinalwert eines Grobuchstabens  *)
      END;                     (* [a[i]-64] = Index im deutschen Alphabet   *)
      MaxAnzahlZeichen(alphabet);
                               (* Ermitteln der grten Stellenzahl der An- *)
                               (* zahl Zeichen pro Buchstabe (Formatierung) *)
      FOR i:= 1 TO 26 DO
        key:= '';
        S.AppendChar(key,CHR(i+64));
                               (* einen Buchstaben in die leere Zeichenket- *)
                               (* te einfgen, da im Modul ColorPlane die   *)
                               (* Mglichkeit, einzelne Zeichen (DT: CHAR)  *)
                               (* am Bildschirm auszugeben, nicht implemen- *)
                               (* tiert wurde ( --> C.WriteStr(string); )   *)
        y1:= y-i*18;
        C.GotoXY(x,y1); C.WriteStr(key);            (* Grobuchstabe        *)
        C.GotoXY(x+12,y1); C.WriteStr('/');
        key:= '';
        S.AppendChar(key,CHR(i+96));
        C.GotoXY(x+24,y1); C.WriteStr(key);         (* Kleinbuchstabe       *)
        C.WriteStr('  = '); 
        S.Str(alphabet[i],zahlStr);
        StringLaengenkorrektur(maxStellen+1,zahlStr);
                                                    (* fhrende Leerzeichen *)
                                                    (* in die Zk einfgen   *)
        C.WriteStr(zahlStr);
        x1:= 128 + maxStellen * 8;                  (* Position fr die     *)
                                                    (* Strichgruppen        *)
        Strichgruppen(alphabet,i,x1);
        C.SetForeColor(255,255,127);
        x1:= x1 + 24                                (* Position fr statis- *)
                                                    (* tische Angaben       *)
      END;
      Enter(680,35)
    END AusgabeAnzahlBuchstaben;


  PROCEDURE AusgabeRelativeHaeufigkeit ( alphabet : tAlphabet;
                                                      anzahl, x : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Reihung alphabet, deren Komponenten die Hufigkeit des     |
    |            Vorhandenseins der Buchstaben des deutschen Alphabets im   |
    |            analysierten Text in alphabetisch aufsteigender Ordnung    |
    |            enthlt, Gesamtzahl der Zeichen des Textes (anzahl) und    |
    |            x, die x-Koordinate, an der Grafikcursor in Abhngigkeit   |
    |            der grten auszugebenden Zahl positioniert wird.          |
    | Leistung:  Berechnen der relativen Hufigkeiten, mit der die Buchsta- |
    |            ben des deutschen Alphabets im analysierten Text vertreten |
    |            sind. Ausgabe der Summe der relativen Hufigkeiten mit de- |
    |            nen Gro- und Kleinbuchstaben im analysierten Text enthal- |
    |            ten sind. (Den Rest bilden die Trennzeichen ohne die Ab-   |
    |            satzendezeichen)                                           |
    *-----------------------------------------------------------------------*)

    CONST x1 = 280;  yMax = 522;

    VAR   i, y     : INTEGER;
          h, summe : REAL;
          zahlStr  : SD.tShStr;

    BEGIN
      C.SetBackColor(0,0,0);
      C.Bar(x,yOffset-br-18,xOffset+br+2,yOffset-15,C.ERASE);
      WriteShortColStrXY(120,208,160,x1+7,yMax-36,'Relative Hufigkeiten');
      summe:= 0.0;
      FOR i:= 1 TO 26 DO
        h:= alphabet[i] / anzahl * 100;
        y:= yMax - i * 18;
        summe:= summe + h;
        Fl.StrF(h,2,2,zahlStr);
        WriteShortStrXY(x,y,zahlStr); C.WriteStr(' %')
      END;
      C.SetForeColor(0,160,160);
      WriteShortStrXY(x1,350,'Die Summe der relativen');
      WriteShortStrXY(x1+4,330,'Hufigkeiten der Gro-');
      WriteShortStrXY(x1+16,310,'und Kleinbuchstaben');
      WriteShortStrXY(x1+32,290,'betrgt im Text');
      Fl.StrF(summe,3,2,zahlStr);
      C.SetForeColor(120,208,160);
      WriteShortStrXY(x1+48,265,zahlStr); C.WriteStr(' %')
    END AusgabeRelativeHaeufigkeit;


  PROCEDURE Analyse ( a : tReihung; anzahl : INTEGER );

    VAR leer, x,
        wortAnzahl : INTEGER;
        alphabet   : tAlphabet;

    BEGIN
      Wortanzahl(a,anzahl,wortAnzahl);
      AusgabefensterLoeschen;
      EntfernenZeilenumbrueche(a,anzahl,leer);
      Sortieren(a,anzahl,leer);
      C.SetBackColor(0,0,0);
      AusgabefensterLoeschen;
      IF wortAnzahl > 0 THEN
        AusgabeAnzahlBuchstaben(a,alphabet,anzahl,x);
        AusgabeRelativeHaeufigkeit(alphabet,anzahl,x)
      ELSE
        C.SetBackColor(bc_red,bc_green,bc_blue);
        C.Bar(560,430,760,80,C.ERASE)
      END
    END Analyse;


  PROCEDURE Initialisieren ( VAR a : tReihung );
   (*-----------------------------------------------------------------------*
    | Leistung: Initialisieren der Komponenten der Reihung a, die den zu    |
    |           analysierenden Text aufnehmen wird.                         |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      FOR i:= 1 TO maxAnzahl DO a[i]:= 0 END
    END Initialisieren;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m : ARRAY 91 OF CHAR;

    BEGIN
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBoxA(W.GetActiveWindow(),Sy.ADR(m),Sy.ADR(m3),40H);
      Delay(delayTime)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR exitFlag,
        ohneTXTdatei : BOOLEAN;
        anzahl, i    : INTEGER;
        fPfad        : SD.tShStr;
        a            : tReihung;

    BEGIN
      Ueberschrift;
      i:= 0;
      exitFlag:= FALSE;
      REPEAT
        INC(i);
        IF i > 1 THEN Ausgabefenster END;
        AuswaehlenTextDatei(fPfad,exitFlag,ohneTXTdatei);
        IF ~ exitFlag THEN
          Initialisieren(a);
          TextEinlesenAusgeben(fPfad,a,anzahl);
          Analyse(a,anzahl)
        END
      UNTIL exitFlag OR ~Wiederholung();
      Information
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   C.SetBackColor(bc_red,bc_green,bc_blue);
   C.Bar(5,0,850,650,C.ERASE);
   C.SetForeColor(200,200,200);
   C.Bar(1,0,3,650,C.DRAW);
   C.SetForeColor(63,63,63);
   C.Line(4,0,4,650,C.DRAW)
 END TextStat.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

