 
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16-Bit)         Dateiname: COLORVAL.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: Dezember 2000 Version: V 1.2                |
  |-------------------------------------------------------------------------|
  | Thema:    Grafikprogrammierung in POW! (OBERON-2)                       |
  |           T e s t p r o g r a m m  zum MODUL VGAColor                   |
  |-------------------------------------------------------------------------|
  | Leistung: Das Programm zeigt die Belegung der Farbtabelle und die       |
  |           Farbauflsung des eingestellten Grafikmodus an. Farbmodi      |
  |           unter 32768 Farben (Highcolor, 15 Bit) sind fr POW! nicht    |
  |           geeignet!                                                     |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Programm enthlt eine Arbeitsanweisung, die einen Eintrag |
  |           im Programm VGAColor, Dateiname VGACOLOR.MOD, betrifft.       |
  |           Dieses Programm ist fr den Grafikmodus 800 x 600 Pixel      |
  |           konzipiert (Ausnutzung der gesamten Bildschirmflche).        |
  |-------------------------------------------------------------------------|
  |            2000/01  Manfred Kmmerer, Chamberlain-Soft JENA            |
  *-------------------------------------------------------------------------*)

 
 MODULE ColorVal;

  IMPORT C:= ColorPlane, S:= Strings, V:= VGAColor, W:= Window;

  TYPE   tStrg4 = ARRAY 4 OF CHAR;

  CONST  dTime  = 125;

  VAR    dummy  : BOOLEAN;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE SetForeColorNb ( colorNb : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Setzen einer Vordergrundfarbe unter Verwendung einer VGA-   |
    |           Standardfarbtabellennummer.                                 |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetForeColor(V.vgaColors[colorNb].r,V.vgaColors[colorNb].g,
                                                      V.vgaColors[colorNb].b)
    END SetForeColorNb;


  PROCEDURE Ueberschrift;
    BEGIN
      SetForeColorNb(V.White);
      Delay_ms(dTime);
      C.GotoXY(180,480);
      C.WriteStr('Ermitteln der RGB-Werte der z. Z. aktiven VGA-Palette');
      C.Line(170,462,614,462,C.DRAW);
      Delay_ms(dTime)
    END Ueberschrift;


  PROCEDURE Enter;
    BEGIN
      Delay_ms(dTime*5);
      SetForeColorNb(V.Cyan);
      C.GotoXY(650,32); C.WriteStr('< ENTER >');
      REPEAT UNTIL C.KeyPressed()
    END Enter;


 PROCEDURE Farbbalken;
 (*-------------------------------------------------------------------------*
  | Leistung: 16 farbige Balken (je 8 Punkte hoch, Standard-VGA-Farben) am  |
  |           Bildschirm untereinander ausgeben. Dabei wird geprft, ob 'i' |
  |           die maximale Anzahl vorhandener Farben berschreitet.         |
  *-------------------------------------------------------------------------*)

   VAR i, k : INTEGER;
     
   BEGIN
     FOR i:= 0 TO 15 DO
       IF i IN V.colorNumbers THEN
         SetForeColorNb(i)     (* Zur Sicherheit bei anderen Anwendungen: *)
       ELSE                      (* wenn 'i' grer als 15 ist, was dann?   *)
         C.SetForeColor(0,96,96) (*  --->  V.ColorNumbers, Bereich {0..15 } *)
       END; (* If *)
       C.Bar(302,396-i*16,418,407-i*16,C.DRAW);
     END; (* For i .. *)  
       SetForeColorNb(V.LightGray);
       C.Box(302,395,418,407,C.DRAW)
   END Farbbalken;


 PROCEDURE Formatieren ( Wert : INTEGER; VAR Strg : tStrg4 );
 (*-------------------------------------------------------------------------*
  | bernahme: Zu formatierender Wert und zu belegender Referenzparameter   |
  | Referenz:  Wert als formatierte Zeichenkette (DT: ARRAY 4 OF CHAR)      |
  | Leistung:  Der bergebene Wert (positive ganze Zahl) wird in eine Zei-  |
  |            chenkette gewandelt. Danach werden falls erforderlich Leer-  |
  |            zeichen vorangestellt (Dezimalformatierung).                 |
  *-------------------------------------------------------------------------*)

   BEGIN
     S.Str(Wert,Strg);
     IF Wert < 10 THEN 
       S.Insert('  ',Strg,0) 
     ELSIF Wert < 100 THEN 
       S.Insert(' ',Strg,0) 
     END
   END Formatieren;


 PROCEDURE Ermitteln_und_Beschriften ( VAR Gruen : INTEGER );
 (*-------------------------------------------------------------------------*
  | Referenz: Der G-Wert (R-G-B) der Farbe Grn. Er wird zur Bestimmung der |
  |           Farbauflsung (Wortbreite) genutzt.                           |
  | Leistung: Positionierte farbige Ausgabe von drei Zeichenketten, Ausgabe |
  |           der Farbnummer und der RGB-Werte vor bzw. hinter dem entspre- |
  |           chenden Farbbalken (VGA-Standard-Farben).                     |
  *-------------------------------------------------------------------------*)

   CONST y = 410;
     
   VAR   Strg       : tStrg4;
         i, B, G, R : INTEGER;

   BEGIN
     SetForeColorNb(V.LightRed);  C.GotoXY(448,y+20); C.WriteStr('R(ed)');
     SetForeColorNb(V.LightGreen);C.GotoXY(507,y+20); C.WriteStr('G(reen)');
     SetForeColorNb(V.LightBlue); C.GotoXY(581,y+20); C.WriteStr('B(lue)');
     SetForeColorNb(V.White);
     FOR i:= 0 TO 15 DO
       S.Str(i,Strg);
       Formatieren(i,Strg);
       C.GotoXY(147,y-i*16); C.WriteStr(Strg);
       C.GotoXY(184,y-i*16); C.WriteStr(V.colorName[i]);
       C.GetDot(310,y-10-i*16,R,G,B);
       IF i = 8 THEN Gruen:= G END;
       Formatieren(R,Strg);
       C.GotoXY(450,y-i*16); C.WriteStr(Strg);
       Formatieren(G,Strg);
       C.GotoXY(520,y-i*16); C.WriteStr(Strg);
       Formatieren(B,Strg);
       C.GotoXY(590,y-i*16); C.WriteStr(Strg)
     END
   END Ermitteln_und_Beschriften;


 PROCEDURE Aufloesung ( Gruen : INTEGER; BkColor : V.tVGACol );
 (*-------------------------------------------------------------------------*
  | bernahme: G-Wert der Farbe Grn, RGB-Werte der Hintergrundfarbe        |
  | Leistung:  Vergleich des G-Wertes mit dem von MS-WINDOWS ermittelten   |
  |            Wert der naheliegendsten Farbe (nearest color) der internen  |
  |            Farbtabelle und Ausgabe der entsprechenden Meldung und An-   |
  |            weisung am Bildschirm.                                       |
  |            Sind weniger als 15 Bit Farbtiefe eingestellt, wird eine     |
  |            Warnung ausgegeben.                                          |
  | Hinweise:  Geditterte Farben sehen schlimm aus. Der Hintergrund von     |
  |            Zeichenketten wird ungedittert in einer den Standard-VGA-    |
  |            Farben entsprechenden Farbe ausgegeben!                      |
  |            Natrlich kann die Abfrage  v i e l  professioneller erfol-  |
  |            gen. Dafr mssten 'WINDOWS-Interna' abgefragt werden.       |
  |            Wer kennt sich da aus? Lohnt sich der ntige Aufwand?        |
  *-------------------------------------------------------------------------*) 

   PROCEDURE Eintrag ( ziffer : ARRAY 2 OF CHAR );
     BEGIN
       SetForeColorNb(V.White);
       C.GotoXY(478,62); C.WriteStr(ziffer)
     END Eintrag;


   BEGIN
     SetForeColorNb(V.Cyan);
     IF (Gruen = 80) OR (Gruen = 84) OR (Gruen = 85) THEN
       C.GotoXY(144,80);
       C.WriteStr('Tragen Sie bitte im Programm VGAColor, ');
       C.WriteStr('Dateiname VGACOLOR.MOD,');
       C.GotoXY(262,62);
       C.WriteStr('unter CONST modus den Wert   ein!');
     ELSE
       C.GotoXY(148,100);
       C.WriteStr("Dieser Farbmodus ist fr den 'POW!-Graphikmodus' ungeeignet")
     END;
     SetForeColorNb(V.Yellow);
     IF Gruen = 80 THEN
       C.GotoXY(222,110);
       C.WriteStr('H I G H C O L O R  (32768 Farben, 15 Bit)');
       Eintrag('3')
     ELSIF Gruen = 84 THEN
       C.GotoXY(222,110);
       C.WriteStr('H I G H C O L O R  (65536 Farben, 16 Bit)');
       Eintrag('2')
     ELSIF Gruen = 85 THEN
       C.GotoXY(158,110);
       C.WriteStr('T R U E C O L O R  (16777216 Farben, 24 Bit, ggf. 32 Bit)');
       Eintrag('1')
     END;
     Enter
   END Aufloesung;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)
 
 PROCEDURE ProgMain*();

   VAR BackColor : V.tVGACol;
       Green     : INTEGER;
     
   BEGIN             
     Ueberschrift;
     Farbbalken;
     Ermitteln_und_Beschriften(Green);
     Aufloesung(Green,BackColor);
     C.Close
   END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   C.SetBackColor(200,200,200);
   C.Bar(0,0,3,650,C.ERASE);
   SetForeColorNb(V.DarkGray);
   C.Line(4,0,4,650,C.DRAW);
   C.SetBackColor(8,0,32);       (* weil diese Hintergrundfarbe keine Farbe *)
                                 (*  aus der VGA-Standardfarbe ist ...      *)
   C.Bar(5,0,800,650,C.ERASE)
 END ColorVal.

 (* --------   2000/01  Manfred Kmmerer, Chamberlain-Soft JENA  --------- *)

