
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)            Dateiname: DATEN.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    D a t u m  (vorhergehendes und nachfolgendes Datum eines      |
  |                       Tages des gregorianischen Kalenders)              |
  |-------------------------------------------------------------------------|
  | Leistung: Einlesen eines korrekten Datums und Ermitteln des Datums des  |
  |           nchsten und des vorhergehenden Tages.                        |
  |           Zu beachten ist die Reihenfolge: Jahr Monat Tag!              |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst, das eine 'Lcke' im    |
  |           Quelltext enthlt, befindet sich in:                          |
  |            ...\datum1\good\daten.mod                                    |
  |-------------------------------------------------------------------------|
  | Aufgabenstellung: ...\datum1\txt\datum.txt u. ...\datum1\doc\datum.doc  |
  | Erluterungstext: ...\datum1\txt\daten.txt u. ...\datum1\doc\daten.doc  |
  *-------------------------------------------------------------------------*)

 MODULE Daten;

  IMPORT In, Out;

  TYPE   tShortStr = ARRAY 52 OF CHAR;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Ueberschrift;

    VAR i : INTEGER;

    BEGIN
      Out.Ln; Out.Ln;
      Out.String('   Vorhergehendes und nachfolgendes Datum'); Out.Ln;
      Out.String('  '); FOR i:= 1 TO 40 DO Out.Char('') END;
      Out.Ln
    END Ueberschrift;


  PROCEDURE Beispieldaten;
    BEGIN
      Out.Ln;
      Out.String('   Beispieldaten:  ');
      Out.String('2005 10 31   2000 2 29   2003 12 31   1583 1  1'); Out.Ln;
      Out.String('                   ');
      Out.String('2005 11  1   2000 3  1   2004  1  1   2002 7 16'); Out.Ln
    END Beispieldaten;


  PROCEDURE Textausgabe;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe eines Textes, dessen Zeilen den Elementen einer     |
    |           Reihung zugewiesen wurden, unter Verwendung einer Zhl-     |
    |           schleife. (Verbesserung der Programmstruktur)               |
    |           Fr sieben Zeilen eine separate Textdatei anzulegen, ist    |
    |           nicht angebracht.                                           |
    *-----------------------------------------------------------------------*)

    VAR i    : INTEGER;
        text : ARRAY 8 OF tShortStr;

    BEGIN
      text[1]:= 'Das Datum des vorherigen Tages fllt in das Jahr';
      text[2]:= '  der Einfhrung des gregorianischen Kalenders.';
      text[3]:= 'Im Jahr 1582 folgte dem 4. Oktober der 15. Oktober.';
      text[4]:= '     Kalenderreform durch Papst Gregor XIII.';
      text[5]:= ' Bulle Intergravissimas, vom 24. Februar 1582.';
      text[6]:= '     Diese Reform wurde damals nur von den';
      text[7]:= '        katholischen Lnder angenommen.';
      Out.Ln;
      FOR i:= 1 TO 7 DO
        Out.String('   '); Out.String(text[i]); Out.Ln
      END
    END Textausgabe;


  PROCEDURE Schaltjahr ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres (DT: INTEGER)           |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            Schaltjahr handelt.                                        |
    |            TRUE = Schaltjahr, FALSE = kein Schaltjahr                 |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    VAR c, j : INTEGER;

    BEGIN
      c:= jahreszahl DIV 100; 
      j:= jahreszahl MOD 100;
      IF j = 0 THEN 
        RETURN c MOD 4 = 0
      ELSE
        RETURN j MOD 4 = 0 
      END;
    END Schaltjahr;


  PROCEDURE JahrGueltig ( jahreszahl : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres                         |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert)             |
    |            TRUE = korrekt, FALSE = nicht korrekt (siehe Hinweis)      |
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            nach dem gregorianischen Kalender gltiges Jahr handelt.   |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF (jahreszahl < 1583) OR (jahreszahl > 3000) THEN
        Out.Ln;
        Out.String('   Nur Jahreszahlen von 1583 bis 3000 eingeben!'); Out.Ln;
        Out.String('           (gregorianischer Kalender)'); Out.Ln;
        RETURN FALSE
      ELSE
        RETURN TRUE
      END;
    END JahrGueltig;


  PROCEDURE AnzahlTage ( jahreszahl, monatsnummer : INTEGER;
                                                       VAR anzahl: INTEGER);
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl und Monatsnummer des zu testenden Datums        |
    | Referenz:  Anzahl der Tage des zu testenden Monats (anzahl)           |
    | Leistung:  Ermitteln der Anzahl Tage des Monats. Falls es sich um     |
    |            den Monat Februar handelt, wird die Jahreszahl auf die     |
    |            Mglichkeit, dass es sich um ein Schaltjahr handelt, ge-   |
    |            testet.                                                    |
    *-----------------------------------------------------------------------*)

    BEGIN
      CASE monatsnummer OF
         1, 3, 5, 7, 8, 10, 12: anzahl:= 31 
       | 4, 6, 9, 11:           anzahl:= 30
       | 2:                     IF Schaltjahr(jahreszahl) THEN
                                  anzahl:= 29
                                ELSE
                                  anzahl:= 28
                                END
      END
    END AnzahlTage;


  PROCEDURE EinlesenKalenderdatum ( VAR anzahl, jahreszahl,
                                      monatsnummer, tagesnummer : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: Anzahl der Tage des zu testenden Monats (anzahl) und die    |
    |           eingelesenen Daten des zu testenden Kalenderdatums          |
    | Leistung: Eingelesen eines nach dem gregorianischen Kalender gltigen |
    |           Kalenderdatums. Kontrollausgabe dieses Kalenderdatums am    |
    |           Bildschirm, da das importierte In-Moduls, die Eingabe, den  |
    |           sogenannten Eingabestraem (denglisch = deutsch englisch),   |
    |           beim Einlesen aus dem Eingabefenster entfernt.              |
    *-----------------------------------------------------------------------*)

    BEGIN
      REPEAT
        In.Prompt('Jahreszahl ( jjjj )');
        In.Int(jahreszahl);
      UNTIL JahrGueltig(jahreszahl);
      REPEAT
        In.Prompt('Monat  ( mm )');
        In.Int(monatsnummer);
      UNTIL (monatsnummer > 0) & (monatsnummer < 13);
      REPEAT
        In.Prompt('Tag  ( tt )');
        In.Int(tagesnummer);
        AnzahlTage(jahreszahl,monatsnummer,anzahl)
      UNTIL anzahl >= tagesnummer;
      Out.Ln;
      Out.String('   Das eingegebene Datum lautet:');
      Out.Int(jahreszahl,8); Out.Int(monatsnummer,3); Out.Int(tagesnummer, 3);
      Out.Ln
    END EinlesenKalenderdatum;


  PROCEDURE Naechster ( anzahl, jahreszahl,
                                      monatsnummer, tagesnummer : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Anzahl der Tage des zu testenden Monats (anzahl) und Jah-  |
    |            reszahl, Monats- und Tagesnummer des Kalenderdatums, von   |
    |            dem der Nachfolger bestimmt werden soll.                   |
    | Leistung:  Ermitteln des kompletten Kalenderdatums des folgenden Ta-  |
    |            ges trotz mglichem Monats- und Jahreswechsel.             |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF tagesnummer < anzahl THEN
        INC(tagesnummer)
      ELSIF monatsnummer < 12 THEN
        tagesnummer:= 1;
        INC(monatsnummer)
      ELSE
        tagesnummer:= 1;
        monatsnummer:= 1;
        INC(jahreszahl)
      END;
      Out.Ln;
      Out.String('   Das Datum des nchsten Tages:');
      Out.Int(jahreszahl,8); Out.Int(monatsnummer,3); Out.Int(tagesnummer,3);
      Out.Ln
    END Naechster;


  PROCEDURE Vorheriger ( anzahl, jahreszahl,
                                      monatsnummer, tagesnummer : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Anzahl der Tage des zu testenden Monats (anzahl) und Jah-  |
    |            reszahl, Monats- und Tagesnummer des Kalenderdatums, von   |
    |            dem der Vorgnger bestimmt werden soll.                    |
    | Leistung:  Ermitteln des kompletten Kalenderdatums des vorherigen Ta- |
    |            ges trotz mglichem Monats- und Jahreswechsel. Sollte als  |
    |            Datum der 31. Dezember des Jahr 1582 ergeben, wird ein     |
    |            kurzer geschichtlicher Exkurs angezeigt.                   |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF tagesnummer > 1 THEN
        DEC(tagesnummer)
      ELSIF monatsnummer > 1 THEN
        DEC(monatsnummer)
      ELSE
        tagesnummer:= 31;
        monatsnummer:= 12;
        DEC(jahreszahl)
      END;
      Out.Ln;
      Out.String('   Das Datum des vorherigen Tages:');
      Out.Int(jahreszahl,6); Out.Int(monatsnummer,3); Out.Int(tagesnummer,3);
      Out.Ln;
      IF jahreszahl = 1582 THEN
        Textausgabe
      END
    END Vorheriger;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR anzahl,
        jahreszahl,
        monatsnummer,
        tagesnummer  : INTEGER;

    BEGIN
      Ueberschrift;
      Beispieldaten;
      EinlesenKalenderdatum(anzahl,jahreszahl,monatsnummer,tagesnummer);
      Naechster(anzahl,jahreszahl,monatsnummer,tagesnummer);
      Vorheriger(anzahl,jahreszahl,monatsnummer,tagesnummer)
      
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END Daten.

 (* ----------------------------------------------------------------------- *)

