
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: KALENDER.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    D a t u m   (Import des Servicemoduls DatumOperationen)       |
  |-------------------------------------------------------------------------|
  | Leistung: Erstellen des Jahreskalenders fr das eingegebene Kalender-   |
  |           jahr (gregorianischer Kalender, 1583 bis 3000)                |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul als lauffhiges Grundgerst, befindet sich in:      |
  |            ...\datum2\good\kalender.mod                                 |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\datum2\txt\datumtoo.txt und                       |
  |                   ...\datum2\doc\datumtoo.doc                           |
  *-------------------------------------------------------------------------*)


 MODULE Kalender;

  IMPORT In, Out, Op := DatumOpe, S:= Strings, W:= Window;

  VAR    dummy : BOOLEAN;


 (* ---------------------  U N T E R P R O G R A M M  --------------------- *)


  PROCEDURE Ueberschrift ( jahreszahl : INTEGER );

    VAR i : INTEGER;

    BEGIN
      Out.Ln;
      Out.String('  K A L E N D E R   fr das Jahr'); Out.Int(jahreszahl,5);
      Out.Ln; Out.Char(' ');
      FOR i:= 1 TO 37 DO Out.Char('') END;
            (* '' = <Alt> + <0><1><6><8> auf dem Nummernblock der Tastatur *)
      Out.Ln
    END Ueberschrift;


  PROCEDURE AnzeigeSchaltjahr ( jahreszahl : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer des eingelesenen und   |
    |            geprften Kalenderdatums (Verbund datum, DT: tDatum)       |
    | Leistung:  Ausgabe der Information, ob es sich beim bernommenen Ka-  |
    |            lenderjahr um ein Schaltjahr handelt oder nicht.           |
    |            Der Wahrheitswert der Schaltjahrinformation, der als boole-|
    |            scher Ausdruck der zweiseitigen Auswahl dient, wird durch  |
    |            den Aufruf der Funktionsprozedur Schaltjahr() aus dem Ser- |
    |            vicemoduls DatumOperationen geliefert.                     |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF Op.Schaltjahr(jahreszahl) THEN
        Out.String('   Das Jahr ist ein Schaltjahr.')
      ELSE
        Out.String('   Das Jahr ist kein Schaltjahr.')
      END;
      Out.Ln
    END AnzeigeSchaltjahr;


  PROCEDURE Einlesen ( VAR datum : Op.tDatum );
   (*-----------------------------------------------------------------------*
    | Referenz: Jahreszahl, Monats- und Tagesnummer des eingelesenen bzw.   |
    |           zugewiesenen Kalenderdatums (Verbund datum, DT: tDatum).    |
    |           Der Verbundvariable datum.monat und datum.tag wird jeweils  |
    |           eine eins zugewiesen, da fr die Funktionsprozedur Existenz |
    |           ein volles Kalenderdatum bentigt wird.                     |
    | Leistung: Ausgabe von Eingabehinweisen (Prompt), Einlesen eines Ka-   |
    |           lenderdatums, Prfen der Gltigkeit (gregorianisher Kalen-  |
    |           der) und formatierte Kontrollausgabe des Datums oder Aus-   |
    |           gabe einer Fehlermeldung am Bildschirm.                     |
    |           Falls das eingegebene Datum die Monatsnummer 2 enthlt,     |
    |           wird die Funktionsprozedur Schaltjahr() aus dem Servicemo-  |
    |           dul DatumOperationen einbezogen und ggf. eine entsprechende |
    |           Mitteilung am Bildschirm ausgegeben.                        |
    *-----------------------------------------------------------------------*)

    BEGIN
      REPEAT
        In.Prompt('Jahreszahl');
        In.Int(datum.jahr);
        datum.monat:= 1; datum.tag:= 1; 
      UNTIL Op.Existenz(datum)
    END Einlesen;


  PROCEDURE Erstellen ( datum : Op.tDatum );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer des eingelesenen Ka-   |
    |            lenderdatums (Verbund datum, DT: tDatum).                  |
    | Leistung:  Ausgabe eines kompletten Jahreskalendariums fr das ber-  |
    |            nommene Jahr. Die Monatsnamen werden aus dem Servicemodul  |
    |            DatumOperationen importiert und unterstrichen am Bild-     |
    |            schirm ausgegeben. Darunter erscheinen die Krzel der Wo-  |
    |            chentagsnamen und das Kalendarium im engeren Sinne. Damit  |
    |            die formatierte Ausgabe korrekt funktioniert, mssen die   |
    |            Wochentagsnummer und die Anzahl Tage eines jeden Monats    |
    |            errechnet und in die Kontrollstrukturen einbezogen werden. |
    *-----------------------------------------------------------------------*)

    VAR i, m, t    : INTEGER;
        monatsname : Op.tName;

    BEGIN
      FOR m:= 1 TO 12 DO
        Op.Monatsname(m,monatsname);
        Out.Ln;
        Out.String('  '); Out.String(monatsname); Out.Ln;
        Out.String('  ');
        FOR i:= 1 TO S.Length(monatsname) DO Out.Char('') END;
            (* '' = <Alt> + <0><1><7><5> auf dem Nummernblock der Tastatur *)
        Out.Ln;
        Out.String('   So   Mo   Di   Mi   Do   Fr   Sa'); Out.Ln;
        datum.monat:= m;
        datum.tag:= 1;
        FOR i:= 1 TO Op.Wochentagsnummer(datum) DO Out.String('     ') END;
        FOR t:= 1 TO Op.TageImMonat(datum.jahr,datum.monat) DO
          Out.Int(t,5);
          datum.tag:= t;
          IF Op.Wochentagsnummer(datum) = 6 THEN Out.Ln END
        END;
        IF Op.Wochentagsnummer(datum) # 6 THEN Out.Ln END;
        Out.Ln
      END
    END Erstellen;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR datum : Op.tDatum;

    BEGIN
      Einlesen(datum);
      Ueberschrift(datum.jahr);
      AnzeigeSchaltjahr(datum.jahr);
      Erstellen(datum)
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   In.Open;
   Out.Open
 END Kalender.

 (* ----------------------------------------------------------------------- *)

