
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)        Datei-Name: LABYDEMO.MOD |
  |-------------------------------------------------------------------------|
  | Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: 28.-29.3.2001 Version: V 1.1                |
  |-------------------------------------------------------------------------|
  | Thema:    S u c h e   i m   L a b y r i n t h   (Backtracking)          |
  |-------------------------------------------------------------------------|
  | Leistung: Das Programm liest nach dem Start die Textdatei bild01.txt    |
  |           automatisch ein, zeigt das Labyrinth am Bildschirm an und     |
  |           sucht alle mglichen Wege durch dieses Labyrinth.             |
  |           Die geforderten Eingaben, < ENTER> oder die Wahl der Farbe    |
  |           der Mauern des Labyrinths, knnen mit einem Mausklick (linke  |
  |           Maustaste) oder einer fast beliebigen Tastenbettigung der    |
  |           Tastatur bewirkt werden.                                      |
  |-------------------------------------------------------------------------|
  | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.    |
  |           GRAFIKQ.MOD (Oberon-2 V4)                                     |
  |-------------------------------------------------------------------------|
  | Hinweis:  Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |           Siehe:                                                        |
  |            ...\GRDLAGEN\ALLGEM\DOC\GRAFIK.DOC  o d e r                  |
  |            ...\GRDLAGEN\ALLGEM\TXT\GRAFIK.TXT                           |
  |                                                                         |
  |           Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |            ...\laby\good\labyrint.mod                                   |
  |-------------------------------------------------------------------------|
  | Erluterungstext:  ...\laby\txt\labyrint.txt                            |
  |                    ...\laby\doc\labyrint.doc                            |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE LabyDemo;

  IMPORT C:= ColorPlane, F:= File, S:= Strings, SY:= SYSTEM, W:= Window;

  CONST dTime     =  25;            (* 25 ist ein gut geeigneter Mittelwert *)
        faktor    =   9;
        RGBmax    = 255;
        xOffset   =  50;
        xText     = 410;
        yOffset   =  30;

        TxtDat1   = 'bild01.txt';
        TxtDat2   = 'bildkorr.txt';

  TYPE  tBild     = ARRAY 80, 80 OF CHAR;
        tKoordStr = ARRAY  3 OF CHAR;
        tZeile    = ARRAY 76 OF CHAR;

  VAR   bc_Red,
        bc_Green,
        bc_Blue,                           (* HG-Farben des groen Fenster  *)
        xMax, yMax,
        xStart, yStart,
        xZiel, yZiel  : INTEGER;
        laby           : tBild; 


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE WriteStrXY ( pixelSpalte,pixelZeile : INTEGER; 
                                                     mitteilung : tZeile );
    BEGIN
      C.GotoXY(pixelSpalte,pixelZeile);
      C.WriteStr(mitteilung)
    END WriteStrXY;


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;

    BEGIN
      WHILE C.KeyPressed() DO Dummy:= C.ReadKey() END
    END FlushKeyBuffer;


  PROCEDURE FlushMouseBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Mauspuffers                                     |
    *-----------------------------------------------------------------------*)

    VAR i, xMouse, yMouse : INTEGER;
        buttons           : SET;

    BEGIN
      FOR i:= 1 TO 100 DO
        C.GetMouse(buttons,xMouse,yMouse);
      END;
    END FlushMouseBuffer;


  PROCEDURE EnterEvaluation ( x, y : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Referenz: Rckgabewert (Funktionswert) TRUE, falls das Wort < Enter > |
    |           mit der linken Maustaste angeklickt wurde                   |
    | Leistung: Mauspuffer lschen, solange Maus und Tastatur abfragen, bis |
    |           entweder eine Tastatur- oder Maustastenbettigung regis-    |
    |           triert wurde. Auswerten der Mauszeigerkoordinaten (Klick)   |
    |           und der Nummer der bettigten Maustaste, ggf. Wiederholung  |
    |           des Vorgangs bis zum erfolgreichen Abschlu. Wert der Maus- |
    |           tastenvariable berschreiben und Tastaturpuffer lschen.    |                                          
    *-----------------------------------------------------------------------*)

    VAR Ok      : BOOLEAN;
        x1, y1,
        xMouse,
        yMouse  : INTEGER;
        buttons : SET;

    BEGIN
      Ok:= FALSE;
      x1:= x + 83; y1:= y - 21;
      FlushMouseBuffer;
      REPEAT 
        C.GetMouse(buttons,xMouse,yMouse);
        IF C.KeyPressed() THEN Ok:= TRUE END
      UNTIL Ok OR (xMouse > x-3) & (xMouse < x1+4) &
                  (yMouse < y+2) & (yMouse > y1-4) &
                  (buttons = {0});
      buttons:= {1};
      FlushKeyBuffer;
      RETURN TRUE
    END EnterEvaluation;


  PROCEDURE Enter ( dT, x, y : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    *-----------------------------------------------------------------------*)

    BEGIN
      Delay_ms(dT);
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(0,127,127);
      WriteStrXY(x,y,'< ENTER >');
      FlushKeyBuffer;
      REPEAT UNTIL EnterEvaluation(x,y);
      C.SetForeColor(0,0,0)
    END Enter;


  PROCEDURE Fehlermeldung ( flag : BOOLEAN );
    BEGIN
      C.SetForeColor(207,48,0);
      IF flag THEN  
        WriteStrXY(134,350,'Die Textdatei konnte nicht geffnet ');
        C.WriteStr('werden!')
      ELSE
        WriteStrXY(70,350,'Die Bilddatei des Labyrinths konnte nicht ');
        C.WriteStr('geffnet werden!')
      END;
      WriteStrXY(120,325,'Wahrscheinlich ist der Datei-Pfad nicht korrekt.');
      HALT(1)
    END Fehlermeldung;


  PROCEDURE BereichLoeschen ( gross : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | Leistung: Einen Bildschirmbereich lschen, dessen Gre vom Wert der  |
    |           booleschen Variable abhngt.                                |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      IF gross THEN
        C.Bar(0,0,C.RESOLUTIONX-1,C.RESOLUTIONY-1,C.ERASE)
      ELSE
        C.Bar(xText,260,586,170,C.ERASE)
      END
    END BereichLoeschen;


  PROCEDURE Bildlagekorrektur;
   (*-----------------------------------------------------------------------*
    | Leistung: Prfen, ob die Textdatei existiert, wenn ja, ffnen, sonst  |
    |           Fehlermeldung anzeigen. Textdatei ffnen, zeilenweise ein-  |
    |           lesen und den Inhalt am Bildschirm ausgeben. Aufruf der     |
    |           Prozedur Enter und Textdatei schlieen.                     |
    *-----------------------------------------------------------------------*)

    VAR textZeile   : tZeile;
        fHandle     : F.Handle;
        resCode,
        z, pZeile   : INTEGER;

    BEGIN
      BereichLoeschen(TRUE);
      IF F.Exist(TxtDat2) THEN
        F.Open(TxtDat2,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode);
      ELSE
        Fehlermeldung(TRUE)
      END;
      z:= 1;
      pZeile:= 350;
      C.SetForeColor(0,103,127);
      WHILE ~F.AtEnd(fHandle) DO
        F.ReadLn(fHandle,textZeile,resCode);
        IF z = 7 THEN C.SetForeColor(127,127,15)
          ELSIF z = 9 THEN C.SetForeColor(0,175,127)
        END;
        WriteStrXY(150,pZeile,textZeile);
        INC(z); DEC(pZeile,18)
      END;
      Enter(dTime*25,500,35);
      BereichLoeschen(TRUE);
      C.SetForeColor(0,0,0)
    END Bildlagekorrektur;


  PROCEDURE StartZielSetzen ( x, y : INTEGER; flag : BOOLEAN);
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate des auszugebenden Symbols sowie das    |
    |            Start/Zielflag fr das Darstellen des entsprechenden Sym-  |
    |            bols (TRUE fr Startsymbol, FALSE fr Zielsymbol)          |
    | Leistung:  Anzeigen der Symbole fr den Start- und den Zielpunkt.     |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF flag THEN
        C.SetForeColor(207,79,0)
      ELSE
        C.SetForeColor(79,207,0)
      END;
      C.Bar(faktor*x+2+xOffset,faktor*y+2+yOffset,
              faktor*x+6+xOffset,faktor*y+6+yOffset,C.DRAW);
      C.SetForeColor(0,0,0);
      C.Box(faktor*x+1+xOffset,faktor*y+1+yOffset,
              faktor*x+7+xOffset,faktor*y+7+yOffset,C.DRAW)
    END StartZielSetzen;


  PROCEDURE WahlEvaluation ( x, y : INTEGER ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate der unteren linken Ecke des Gesamtbe-  |
    |            reichs der farbigen Auswahlflchen                         |
    | Leistung:  Auswerten eines Mausklicks, der im Bereich der farbigen    |
    |            Flchen stattfand oder einer Tastenbettigung, ggf. Er-    |
    |            rechnen der Farbnummer aus den Koordinaten, die durch die  |
    |            Mausbettigung geliefert wurden und Rckgabe der Farbnum-  |
    |            mer als Funktionswert.                                     |
    *-----------------------------------------------------------------------*)

    CONST yM1o = 281; yM1u = 263;
          yM2o = 256; yM2u = 238;
          yM3o = 231; yM3u = 213;

    VAR   zeichen : ARRAY 2 OF CHAR;
          resCode : CHAR;
          Ok      : BOOLEAN;
          color,
          x1, y1,
          xMouse,
          yMouse  : INTEGER;
          buttons : SET;

    BEGIN
      x1:= x + 50; y1:= y - 16;
      C.SetForeColor(0,0,0);
      REPEAT 
        FlushMouseBuffer;
        C.GetMouse(buttons,xMouse,yMouse);
        IF C.KeyPressed() THEN
          REPEAT
            C.GotoXY(360,200);
            zeichen:= '';
            C.ReadStr(zeichen,1,resCode);
            IF (zeichen > '0') & (zeichen < '4') THEN
              color:= SHORT(S.Val(zeichen))
            ELSIF zeichen = '' THEN 
              color:= 3
            END
          UNTIL (resCode = C.ESC) OR (color > 0) & (color < 4);
          Ok:= TRUE;
        ELSIF (buttons = {0}) & (xMouse > x)
          & (xMouse < x1) & (yMouse < y) & (yMouse > yM3u) THEN
          Ok:= TRUE
        ELSIF (yMouse < yM1o) & (yMouse > yM1u) THEN color:= 1
          ELSIF (yMouse < yM2o) & (yMouse > yM2u) THEN color:= 2
            ELSIF (yMouse < yM3o) & (yMouse > yM3u) THEN color:= 3
              ELSE Ok:= FALSE
        END
      UNTIL Ok;
      buttons:= {1};
      FlushKeyBuffer;
      RETURN color
    END WahlEvaluation;


  PROCEDURE Wahl() : INTEGER;
   (*-----------------------------------------------------------------------*
    | Referenz: Rckgabewert (Funktionswert) Farbnummer fr die Mauern des  |
    |           Labyrinths                                                  |
    | Leistung: Anzeige der Farbbalken und eines Textes und Ermitteln der   |
    |           Farbnummer fr die Mauern des Labyrinths.                   |
    *-----------------------------------------------------------------------*)

    CONST x1 = 147;  x2 = 180;  x3 = 230;  x4 = 240;

    VAR   farbKennzahl, maxLen : INTEGER;

    BEGIN
      WriteStrXY(190,315,'FARBE DER LABYRINTHMAUERN');
      C.Line(180,297,400,297,C.DRAW);
      WriteStrXY(x1,280,'(1)');
      WriteStrXY(x1,255,'(2)');
      WriteStrXY(x1,230,'(3)');
      C.SetForeColor(87,191,103);  C.Bar(x2,280,x3,264,C.DRAW);
      C.SetForeColor(207,191,103); C.Bar(x2,255,x3,239,C.DRAW);
      C.SetForeColor(0,0,0);       C.Bar(x2,230,x3,214,C.DRAW);
      WriteStrXY(x4,280,'Labyrinth aus grnen Hecken');
      WriteStrXY(x4,255,'Labyrinth aus braunen Mauern');
      WriteStrXY(x4,230,'Labyrinth aus schwarzen Mauern');
      WriteStrXY(x4,200,'Farbkennzahl = ');
      C.SetForeColor(0,128,128);
      WriteStrXY(x2+12,170,'... oder mittels Mausklick der');
      WriteStrXY(x2-28,150,'linken Maustaste in eine der Farbflchen');
      farbKennzahl:= WahlEvaluation(x2,280);
      IF (farbKennzahl = 1) OR (farbKennzahl = 2) THEN
        RETURN farbKennzahl
      ELSE
        RETURN 0
      END
    END Wahl;


  PROCEDURE Beschriften;
    BEGIN
      StartZielSetzen(40,30,TRUE);
      StartZielSetzen(40,28,FALSE);
      WriteStrXY(426,311,'= Start');
      WriteStrXY(426,293,'= Ziel')
    END Beschriften;


  PROCEDURE WegsucheText ( i : INTEGER );
    BEGIN
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(0,119,119);
      WriteStrXY(xText,260,'Suche nach der');
      IF i = 0 THEN
        WriteStrXY(xText,242,'ersten Lsung')
      ELSE
        WriteStrXY(xText+88,260,'einer');
        WriteStrXY(xText,242,'weiteren Lsung')
      END
    END WegsucheText;


  PROCEDURE AusgabeLoesungsnummer ( VAR i : INTEGER );

    VAR zahlStr : ARRAY 3 OF CHAR;

    BEGIN
      BereichLoeschen(FALSE);
      INC(i);
      S.Str(i,zahlStr);
      C.GotoXY(xText,260); C.WriteStr(zahlStr); C.WriteStr('. Lsung');
      Enter(dTime*25,500,35);
      WegsucheText(i);
      C.SetForeColor(0,119,119);
      C.GotoXY(xText,210);
      IF i = 1 THEN
        C.WriteStr('Bisher wurde eine');
        WriteStrXY(xText,192,'Lsung gefunden.');
      ELSE
        C.WriteStr('Bisher wurden');
        C.GotoXY(xText,192); C.WriteStr(zahlStr); C.WriteStr(' Lsungen gefunden.')
      END;
      C.SetBackColor(RGBmax,RGBmax,RGBmax)
    END AusgabeLoesungsnummer;


  PROCEDURE Einlesen;
   (*-----------------------------------------------------------------------*
    | Leistung: Prfen, ob die Textdatei existiert, wenn ja, ffnen, sonst  |
    |           Fehlermeldung anzeigen. Textdatei ffnen, die ersten beiden |
    |           Blcke lesen, die eingelesenen Informationen (KoordStr) in  |
    |           INTEGER-Werte wandeln, Rest der Datei zeilenweise einlesen, |
    |           wobei Zeilenendezeichen ignoriert werden, und den Inhalt    |
    |           den Elementen einer zweidimensionalen Reihung zuweisen.     |
    |           Textdatei schlieen.                                        |
    *-----------------------------------------------------------------------*)

    VAR KoordStr : tKoordStr;
        CR       : BOOLEAN;
        zeichen  : CHAR;
        fHandle  : F.Handle;
        anzahl,
        resCode,
        x, y     : INTEGER;
        read     : LONGINT;

    BEGIN
      IF F.Exist(TxtDat1) THEN
        F.Open(TxtDat1,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode);
      ELSE
        Fehlermeldung(FALSE)
      END;
      F.ReadBlock(fHandle,KoordStr,SIZE(tKoordStr),read,resCode);
      xMax:= SHORT(S.Val(KoordStr));        (* xMax = Breite des Labyrinths *)
      F.ReadBlock(fHandle,KoordStr,SIZE(tKoordStr),read,resCode);
      yMax:= SHORT(S.Val(KoordStr));        (* yMax = Hhe des Labyrinths   *)
      FOR y:= yMax TO 0 BY -1 DO
        FOR x:= 0 TO xMax DO
          F.ReadChar(fHandle,zeichen,resCode);
          laby[x,y]:= zeichen;
          IF zeichen = '3' THEN xStart:= x; yStart:= y END;
          IF zeichen = '4' THEN xZiel:= x;  yZiel:= y END;
        END;
        F.ReadChar(fHandle,zeichen,resCode) (* berlesen des Zeilenwechsels *)
      END;
      F.Close(fHandle)
    END Einlesen;


  PROCEDURE BauenUndBeschriften;
   (*-----------------------------------------------------------------------*
    | Leistung: Vorbereitungen zum Bau des Labyrinths treffen und die Mau-  |
    |           ern in der gewnschten Farbe darstellen.                    |
    *-----------------------------------------------------------------------*)

    VAR zeichen   : CHAR;
        zei       : ARRAY 2 OF CHAR;
        farbKennz,
        x, y      : INTEGER;

    PROCEDURE MauerSetzen ( x, y : INTEGER );
      BEGIN
        C.Bar(faktor*x+xOffset,faktor*y+yOffset,
                faktor*x+8+xOffset,faktor*y+8+yOffset,C.DRAW)
      END MauerSetzen;

    BEGIN
      farbKennz:= Wahl();
      IF farbKennz = 0 THEN               (* Labyrinth aus schwarzen Mauern *)
        C.SetForeColor(0,0,0)
      ELSIF farbKennz = 1 THEN            (* Labyrinth aus grnen Hecken    *)
        C.SetForeColor(87,191,103);
      ELSE                                (* Labyrinth aus braunen Mauern   *)
        C.SetForeColor(207,191,103);       
      END;
      BereichLoeschen(TRUE);
      C.SetBackColor(RGBmax,RGBmax,RGBmax);
      C.Bar(xOffset,yOffset+2*faktor,
              xMax*faktor+xOffset,yMax*faktor+yOffset,C.ERASE);
      FOR y:= 1 TO yMax DO
        FOR x:= 0 TO xMax-1 DO
          zeichen:= laby[x,y];
          IF (zeichen = '1') OR (zeichen = '2') THEN MauerSetzen(x,y) END
        END
      END;
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      StartZielSetzen(xStart,yStart,TRUE);
      StartZielSetzen(xZiel,yZiel,FALSE);
      Beschriften;
      Enter(dTime*25,500,35)
    END BauenUndBeschriften;


  PROCEDURE Suchen ( VAR i : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: Anzahl gefundener Wege durch das Labyrinth                  |
    | Leistung: Schritt fr Schritt (Rekursion) wird ein Weg durch das La-  |
    |           byrinth gesucht. Dabei wird die 'Linke-Hufregel', deren     |
    |           Entwickler Minotaurus gewesen sein soll, dauerhaft angewen- |
    |           det. Der bereits zurckgelegte Weg einer Wegsuche wird mit  |
    |           kleinen Farbflchen gekennzeichnet, Prozedur SteinSetzen(). |
    |           Von Wegen die rckwrts beschritten werden mssen, Irrwege, |
    |           werden die Farbflchen wieder entfernt, Prozedur SteinEnt-  |
    |           nen().                                                      |
    *-----------------------------------------------------------------------*)

    PROCEDURE SteinSetzen ( x, y : INTEGER );
      BEGIN
        Delay_ms(dTime);
        C.SetForeColor(0,79,207);
        C.Bar(faktor*x+2+xOffset,faktor*y+2+yOffset,
              faktor*x+6+xOffset,faktor*y+6+yOffset,C.DRAW);
      END SteinSetzen;
      
    PROCEDURE SteinEntfernen ( x, y : INTEGER );
      BEGIN
        Delay_ms(dTime DIV 2);
        C.SetBackColor(RGBmax,RGBmax,RGBmax);
        C.Bar(faktor*x+2+xOffset,faktor*y+2+yOffset,
              faktor*x+6+xOffset,faktor*y+6+yOffset,C.ERASE);
      END SteinEntfernen;
      
    PROCEDURE Schritt ( xAlt, yAlt : INTEGER );

      VAR richtung, xNeu, yNeu : INTEGER;

      BEGIN
        FOR richtung:= 1 TO 4 DO
          CASE richtung OF
              1: xNeu:= xAlt + 1; yNeu:= yAlt
            | 2: xNeu:= xAlt;     yNeu:= yAlt + 1
            | 3: xNeu:= xAlt - 1; yNeu:= yAlt
            | 4: xNeu:= xAlt;     yNeu:= yAlt - 1
           END;
           IF laby[xNeu,yNeu] = '4' THEN AusgabeLoesungsnummer(i) END;
           IF laby[xNeu,yNeu] = '0' THEN
             SteinSetzen(xNeu,yNeu);  
             laby[xNeu,yNeu]:= '5';  
             Schritt(xNeu,yNeu);                       (* rekursiver Aufruf *)
             laby[xNeu,yNeu]:= '0';  
             SteinEntfernen(xNeu,yNeu)
           END
        END
      END Schritt;
      
    BEGIN
      WegsucheText(i);
      Schritt(xStart,yStart)                           (* erster Aufruf     *)
    END Suchen;


  PROCEDURE AnzahlLoesungen ( i : INTEGER );

    VAR zahlStr : ARRAY 3 OF CHAR;

    BEGIN
      BereichLoeschen(FALSE);
      S.Str(i,zahlStr);
      C.SetForeColor(207,79,0);
      WriteStrXY(xText,260,'Es existieren ');
      C.GotoXY(xText,242); C.WriteStr(zahlStr); C.WriteStr(' Lsungen.');
      Enter(dTime*25,500,35)
    END AnzahlLoesungen;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m     : ARRAY 91 OF CHAR;
          dum   : BOOLEAN;
          dummy : LONGINT;

    BEGIN
      dum:= W.ShowWindow(W.GetActiveWindow(),3); (* Groes Fenster ffnen *)
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBox(W.GetActiveWindow(),SY.ADR(m),SY.ADR(m3),40H);
      Delay_ms(dTime)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR i : INTEGER;

    BEGIN
      Bildlagekorrektur;
      Einlesen;
      BauenUndBeschriften;
      i:= 0;
      Suchen(i);
      AnzahlLoesungen(i);
      Information
    END ProgMain;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   bc_Red:= 240; bc_Green:= 240; bc_Blue:= 220
 END LabyDemo.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

