
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 BIT)             Dateiname: MONO.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    Monoalpabetische Verschlsselung                              |
  |-------------------------------------------------------------------------|
  | Leistung: Verschlsseln und Entschlsseln eines Textes mittels          |
  |           monoalphabetischer Substitution.                              |
  | Hinweise: Statt den Service-Modulen In und Out wird das Service-Modul   |
  |           Display eingesetzt.                                           |
  |           Dieses Modul initialisiert beim Import einen Textbildschirm,  |
  |           der in 25 Zeilen mit je 80 Zeichen eingeteilt ist. Es steht   |
  |           kein Rollbalken zur Verfgung. Der Bildschirm kann komplett   |
  |           oder bereichsweise gelscht werden, der Cursor ist positio-   |
  |           nierbar, es lassen sich Vorder- und Hintergrundfarben defi-   |
  |           nieren u.v.m.                                                 |
  |           Siehe in der Hilfe (<Strg>+<F1>) unter dem Schlsselwort      |
  |           Display. Es gengt aber auch ein Mausklick mit der rechten    |
  |           Maustaste auf das Wort Display in diesem Satz.                |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul als lauffhiges Grundgerst befindet sich in:       |
  |            ...\mono\good\mono.mod                                       |
  |-------------------------------------------------------------------------|
  | Erluterungstexte: ...\mono\txt\mono.txt                                |
  |                    ...\mono\txt\monotool.txt                            |
  |                    ...\mono\doc\mono.doc                                |
  |                    ...\mono\doc\monotool.doc                            |
  *-------------------------------------------------------------------------*)

 MODULE Mono;

  IMPORT D:= Display, F:= File, S:= Strings, W:= Window;


  CONST  delayTime   = 25;
         
         fPfad       = 'ALPHABET.TXT';

  TYPE   tAlphabet   = ARRAY 26 OF CHAR;
         tGeheimtext = ARRAY 61 OF CHAR;


 (* --------------------  U N T E R P R O G R A M M E --------------------- *)


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE Enter ( spalte, zeile : INTEGER; dTime : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung:  Ausgabe einer Zeichenkette, Lschen des Tastaturpuffers,   |
    |            Warten auf eine Tastatureingabe (Enter- oder Escape-Taste) |
    |            und Lschen der Zeichenkette.                              |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;

    BEGIN
      Delay_ms(dTime * 4);
      D.WriteStrXY(spalte,zeile,'< ENTER >');
      D.FlushKeyBuffer;
      Dummy:= D.ReadKey();
      D.WriteStrXY(spalte,zeile,'         ');
    END Enter;


  PROCEDURE ErstesAlphabet ( wahl : INTEGER; VAR alphabet1 : tAlphabet );
   (*-----------------------------------------------------------------------*
    | Leistung: Einlesen der Elemente der Reihung alphabet1 aus der Text-   |
    |           datei ...\ALPHABET.TXT.                                     |
    *-----------------------------------------------------------------------*)

    VAR name       : ARRAY 65 OF CHAR;
        zeichen    : CHAR;
        fHandle    : F.Handle;
        anzahl, i,
        resCode    : INTEGER;

    BEGIN
      IF F.Exist(fPfad) THEN
        F.Open(fPfad,FALSE,F.DENYNONE,F.READONLY,fHandle,resCode)
      ELSE
        D.WriteStrXY(29,23,'Datei nicht gefunden!');
        HALT(1)
      END;
      FOR i:= 0 TO 82 DO
        F.ReadChar(fHandle,zeichen,resCode);
        IF ~(zeichen = CHR(13)) & ~(zeichen = CHR(10)) THEN
                    (* CHR(13) = carage return  und  CHR(10) = line feed *)
          CASE wahl OF
            1: IF i < 26 THEN alphabet1[i]:= zeichen END
           |2: IF (i > 27) & (i < 54) THEN alphabet1[i-28]:= zeichen END
           |3: IF (i > 55) THEN alphabet1[i-56]:= zeichen END
          ELSE
          END
        END
      END;
      F.Close(fHandle);
      D.WriteStrXY(5,3,'1. Alphabet eingelesen')
    END ErstesAlphabet;


  PROCEDURE ZweitesAlphabet ( alphabet1 : tAlphabet; VAR alphabet2 : tAlphabet );
   (*-----------------------------------------------------------------------*
    | Leistung: Ermitteln der Elemente der Reihung alphabet2.               |
    *-----------------------------------------------------------------------*)

    VAR gefunden : BOOLEAN;
        zeichen  : CHAR;
        i, k     : INTEGER;

    BEGIN
      FOR i:= 0 TO 25 DO
        zeichen:= CHR(65+i);
        gefunden:= FALSE;
        k:= 0;
        WHILE ~gefunden & (k <= 25) DO
          IF alphabet1[k] = zeichen THEN
            gefunden:= TRUE 
          ELSE
            INC(k)
          END
        END;
        IF ~gefunden THEN
          HALT(200)
        END;                 
        alphabet2[i]:= CHR(97+k)
      END;
      D.WriteStrXY(5,5,'2. Alphabet ermittelt')
    END ZweitesAlphabet;
    

  PROCEDURE HoleBuchstabe ( spalte : INTEGER ) : CHAR;
   (*-----------------------------------------------------------------------*
    | Leistung: Einen Kleinbuchstaben von der Tastatur einlesen. Falls ver- |
    |           sehentlich ein Grobuchstabe eingegeben wurde, wird er in   |
    |           einen Kleinbuchstaben gewandelt.                            |
    *-----------------------------------------------------------------------*)

    CONST zeile = 12;

    VAR buchstabe : CHAR;

    BEGIN
      D.WriteCharXY(spalte,zeile,'_');
      REPEAT
        buchstabe:= D.ReadKey();
        IF (buchstabe >= 'A') & (buchstabe <= 'Z') THEN
          buchstabe:= CHR(ORD(buchstabe)+32)  (* in Kleinbuchstaben wandeln *)
        END;
      UNTIL (ORD(buchstabe) = 13) OR (buchstabe >= 'a') & (buchstabe <= 'z');
      IF ORD(buchstabe) # 13 THEN
        D.WriteCharXY(spalte,zeile,buchstabe)
      ELSE
        D.WriteCharXY(spalte,zeile,' ')
      END;
      RETURN buchstabe
    END HoleBuchstabe;


  PROCEDURE Verschluesseln ( alphabet1 : tAlphabet;
                                               VAR geheimtext : tGeheimtext );
   (*-----------------------------------------------------------------------*
    | Leistung: Verschlsseln des Klartextes (zeichenweise), das Zeilenab-  |
    |           schlusszeichen wird ignoriert.                              |
    *-----------------------------------------------------------------------*)

    VAR geheimbuchstabe,
        klarbuchstabe    : CHAR;
        spalte1, spalte2 : INTEGER;

    BEGIN
      D.WriteStrXY(5,9,'Bitte Klartextzeile eingeben und mit < ENTER > abschlieen!');
      D.WriteStrXY(5,10,'ACHTUNG! Der Klartext sollte nur aus Kleinbuchstaben bestehen.');
      D.WriteStrXY(7,12,'Klartext:');
      D.WriteStrXY(7,14,'Geheimtext:');
      spalte1:= 20; spalte2:= spalte1;
      klarbuchstabe:= HoleBuchstabe(spalte1);
      geheimtext:= '';
      WHILE ORD(klarbuchstabe) # 13 DO
        geheimbuchstabe:= alphabet1[ORD(klarbuchstabe)-97];
        D.WriteCharXY(spalte2,14,geheimbuchstabe);
        S.AppendChar(geheimtext,geheimbuchstabe);
        INC(spalte1); INC(spalte2);
        klarbuchstabe:= HoleBuchstabe(spalte1)
      END
    END Verschluesseln;


  PROCEDURE Entschluesseln ( alphabet2 : tAlphabet; geheimtext : tGeheimtext);
   (*-----------------------------------------------------------------------*
    | Leistung:  Entschlsseln des Geheimtextes (zeichenweise).             |
    *-----------------------------------------------------------------------*)

    VAR geheimbuchstabe,
        klarbuchstabe    : CHAR;
        i                : INTEGER;

    BEGIN
      D.WriteStrXY(5,18,'Der Geheimtext wird entschlsselt.');
      D.WriteStrXY(7,20,'Klartext:');
      FOR i:= 0 TO SHORT(S.Length(geheimtext)-1) DO
        klarbuchstabe:= alphabet2[ORD(geheimtext[i])-65];
        D.WriteCharXY(20+i,20,klarbuchstabe);
        Delay_ms(delayTime);
      END
    END Entschluesseln;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)


  PROCEDURE ProgMain*();

    VAR alphabet1,
        alphabet2  : tAlphabet;
        geheimtext : tGeheimtext;

    BEGIN
      D.SetWindowTitle('Monoalphabetische Substitution');
      ErstesAlphabet(1,alphabet1);
      ZweitesAlphabet(alphabet1,alphabet2);
      Verschluesseln(alphabet1,geheimtext);
      Enter(65,23,delayTime);
      Entschluesseln(alphabet2,geheimtext);
      Enter(65,23,delayTime)
    END ProgMain;

 END Mono.

 (* ----------------------------------------------------------------------- *)
