
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: GRAFAUSW.MOD |
  |-------------------------------------------------------------------------|
  |  Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                  |
  |            Erstellungsdatum: 16-18.02.2001 Version: V 1.0               |
  |-------------------------------------------------------------------------|
  | Thema:    SORTIEREN  (selection sort); grafische Darstellung des        |
  |                      Sortiervorgangs nach Auswahl von Parametern        |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren einer unstrukturierten Punktwolke.                  |
  |           Der Sortiervorgang wird durch wiederholtes Setzen und Lschen |
  |           der Punkte im Grafikfenster (ColorPlane) veranschaulicht.     |
  |           Whrend der Laufzeit knnen verschiedene Parameter durch den  |
  |           Anwender des Programms eingestellt werden.                    |
  |-------------------------------------------------------------------------|
  | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.    |
  |           GRAFIKQ.MOD (Oberon-2 V4)                                     |
  |-------------------------------------------------------------------------|
  | Hinweise: Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |           siehe:                                                        |
  |                ...\ALL\DOC\GRAFIK.DOC  oder  ...\ALL\TXT\GRAFIK.TXT     |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE GrAuswOS;

  IMPORT C:= ColorPlane, F:=  Float,   P:=  Process, R:= RandomNumbers,
         S:= Strings,    Sy:= SYSTEM,  W:= Window;

  CONST  delayTime =   2;      (* 2 ist der Wert fr eine CPU i80486 66 MHz *)
                               (* Bitte an Ihren Prozessor anpassen!        *)
         anzahl    = 480;
         xOffset   =  32;
         yOffset   =  32;

  TYPE   tStr30    = ARRAY 31 OF CHAR;
         tReihung  = ARRAY anzahl + 1 OF INTEGER;
    
         tArt      = RECORD
                       eigen, dTfaktor : INTEGER
                     END;

  VAR    bc_Red, bc_Green, bc_Blue,            (* HG-Farben groes Fenster  *)
         bc_red, bc_green, bc_blue : INTEGER;  (* HG-Farben kleines Fenster *)
         dummy                     : BOOLEAN;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;
 
    BEGIN
      WHILE C.KeyPressed() DO Dummy:= C.ReadKey() END
    END FlushKeyBuffer;


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE WriteStrXY ( x, y : INTEGER; string : tStr30 );
   (*-----------------------------------------------------------------------*
    | Leistung: Positionierte Ausgebe einer Zeichenkette im Grafikmodus,    |
    |           x = Pixelspalte; x = 0 entspricht der linken Begrenzung des |
    |                                  aktuellen Grafikfensters             |
    |           y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung    |
    |                                  des aktuellen Grafikfensters         |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.GotoXY(x,y); C.WriteStr(string)
    END WriteStrXY;


  PROCEDURE Wiederholung() : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information, Warten auf eine Tastenbettigung |
    |           (Tastatur), Wandeln des Zeichens in einen Grobuchstaben,   |
    |           falls ein Kleinbuchstabe eingegeben wurde und Asuwerten der |
    |           Eingabe (boolescher Wert).                                  |
    | Referenz: Funktionswert TRUE, falls eine Wiederholung gewnscht wird, |
    |           d.h. es wurde eine beliebige Taste auer <N> bettigt,      |
    |           sonst FALSE.                                                |
    *-----------------------------------------------------------------------*)

    VAR ch : CHAR;
      
    BEGIN
      Delay_ms(delayTime*5);
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(32,160,160);
      WriteStrXY(575,65,'Wiederholung (J/N) J');
      FlushKeyBuffer;
      ch:= C.ReadKey();
      ch:= S.UpCaseChar(ch);
      RETURN ~(ch = 'N') 
    END Wiederholung;


  PROCEDURE Ausgabefenster;
    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(xOffset-3,yOffset-3,xOffset+anzahl+3,yOffset+anzahl+3,C.ERASE);
      C.SetForeColor(0,192,192);
      C.Box(xOffset-6,yOffset-6,xOffset+anzahl+6,yOffset+anzahl+6,C.DRAW)
    END Ausgabefenster;


  PROCEDURE Ueberschrift;
    BEGIN
      Ausgabefenster;
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(32,208,208);
      WriteStrXY(590,495,'S O R T I E R E N ');
      WriteStrXY(622,477,'d u r c h');
      WriteStrXY(590,455,'A U S W  H L E N');
      Delay_ms(delayTime*15)
    END Ueberschrift;


  PROCEDURE ReihungAnlegenAusgeben ( VAR a : tReihung; art : tArt );
   (*-----------------------------------------------------------------------*
    | Leistung: In Abhngigkeit des Wertes der Variablen art, werden die    |
    |           Komponenten der Reihung mit aufsteigenden, zuflligen oder  |
    |           absteigenden Werten der Menge {1..n} belegt und im Ausgabe- |
    |           fenster als violetter Punkt (C.Dot(x,y,C.DRAW) dargestellt. |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      C.SetForeColor(255,127,255);
      IF art.eigen = 1 THEN
        FOR i:= 1 TO anzahl DO
          a[i]:= i;
          C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
        END
      ELSIF art.eigen = 2 THEN
        FOR i:= 1 TO anzahl DO
          a[i]:= SHORT(ENTIER(R.Uniform()*anzahl));
          C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
        END
      ELSE
        FOR i:= 1 TO anzahl DO
          a[i]:= anzahl - i;
          C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
        END
      END;
      Delay_ms(art.dTfaktor)
    END ReihungAnlegenAusgeben;


  PROCEDURE Eigenschaften ( VAR art : tArt );

    CONST x1 = 560;  x2 = 576;  x3 = 592;

    VAR   zahlStr : tStr30;
          resCode : CHAR;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(544,385,'Die Zahlen sind ');
      WriteStrXY(x1,360,'(1) bereits aufsteigend');
      WriteStrXY(x3,340,'sortiert,');
      WriteStrXY(x1,320,'(2) zufllig verteilt');
      WriteStrXY(x3,300,'oder');
      WriteStrXY(x1,280,'(3) bereits absteigend');
      WriteStrXY(x3,260,'sortiert.');
      WriteStrXY(x3,230,'Art (1..3) = ');
      WriteStrXY(x3,210,'<ENTER> fr (2)');
      C.SetForeColor(120,160,160);
      REPEAT
        C.Bar(696,230,704,214,C.ERASE);
        C.GotoXY(696,230); C.ReadStr(zahlStr,1,resCode);
        IF zahlStr = '' THEN
          art.eigen:= 2
        ELSE  
          art.eigen:= SHORT(S.Val(zahlStr))
        END
      UNTIL art.eigen IN {1..3};
      C.Bar(544,385,752,196,C.ERASE);
      C.SetForeColor(120,160,160);
      C.GotoXY(x1,385); C.WriteStr('Eigenschaft:');
      C.SetForeColor(120,208,160);
      C.GotoXY(x2,360); 
      CASE art.eigen OF
        1: C.WriteStr('bereits aufsteigend');
           WriteStrXY(x2,340,'sortierte Zahlen')
       |2: C.WriteStr('zufllig verteilte');
           WriteStrXY(x2,340,'Zahlen')
       |3: C.WriteStr('bereits absteigend');
           WriteStrXY(x2,340,'sortierte Zahlen')
      END
    END Eigenschaften;


  PROCEDURE Verzoegerungszeit ( VAR art : tArt );

    CONST x1 = 560;  x2 = 576;

    VAR   zahlStr : tStr30;
          resCode : CHAR;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(x1,295,'Verzgerungs-');
      WriteStrXY(x1,275,'zeitfaktor =  ');
      WriteStrXY(x1+8,255,'<ENTER> fr 25');
      C.GotoXY(664,275); C.ReadStr(zahlStr,6,resCode);
      IF zahlStr = '' THEN
        art.dTfaktor:= 25;
        zahlStr:= '25'
      ELSE  
        art.dTfaktor:= SHORT(S.Val(zahlStr))
      END;
      C.Bar(560,295,766,244,C.ERASE);
      C.SetForeColor(120,160,160);
      WriteStrXY(x1,295,'Verzgerung:');
      C.SetForeColor(120,208,160);
      WriteStrXY(x2,270,'Verzgerungsfaktor ');
      C.WriteStr(zahlStr)
    END Verzoegerungszeit;


  PROCEDURE Art ( VAR art : tArt );
    BEGIN
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.Bar(533,385,766,50,C.ERASE);
      Eigenschaften(art);
      Verzoegerungszeit(art);
      Delay_ms(delayTime*5)
    END Art;


  PROCEDURE Sortieren ( VAR a : tReihung; art : tArt );
   (*-----------------------------------------------------------------------*
    | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.  |
    *-----------------------------------------------------------------------*
    | Leistung: Sortieren der Werte der Komponenten der Reihung nach auf-   |
    |           steigender Gre durch Auswhlen (selection sort).          |
    | Hinweise: Beschreibung des Algorithmus siehe:                         |
    |             ...\SORT1\TXT\AUSWAEH1.TXT                                |
    |            oder bei Nutzung von MS-WORD (o..) unter                  |
    |             ...\SORT1\DOC\AUSWAEH1.DOC                                |
    |           Die Originalwerte der Komponenten der Reihung werden als    |
    |           violette Pixel angezeigt. Die beiden Pixel, die an einer    |
    |           Tauschaktion beteiligt sind werden gelscht (mit Hinter-    |
    |           grundfarbe berschrieben) und an ihrer jeweils neuen Posi-  |
    |           tion als gelbes Pixel ausgegeben.                           |
    *-----------------------------------------------------------------------*)

    VAR r, s, t, x : INTEGER;

    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      P.Yield;                                  (* keine Multitasking       *)
      FOR r:= 1 TO anzahl-1 DO
        t:= r;
        x:= a[r];
        C.Dot(r+xOffset,a[r]+yOffset,C.ERASE);  (* Pixel lschen            *)
        Delay_ms(art.dTfaktor);
        FOR s:= r+1 TO anzahl DO
          IF a[s] < x THEN
            t:= s;
            x:= a[s]
          END
        END;
        C.Dot(t+xOffset,a[t]+yOffset,C.ERASE);  (* Pixel lschen            *)
        a[t]:= a[r];
        C.SetForeColor(255,255,100);
        C.Dot(t+xOffset,a[t]+yOffset,C.DRAW);   (* Getauschtes Pixel am ... *)
        a[r]:= x;                               (* neuen Platz setzen,      *)
        C.Dot(r+xOffset,a[r]+yOffset,C.DRAW)    (* 'sortiertes' Pixsel am   *)
      END;                                      (* entgltigen Platz setzen *)
    END Sortieren;


  PROCEDURE AusgabeZahlenfeld ( a : tReihung );
   (*-----------------------------------------------------------------------*
    | Leistung: Die sortierten Werte der Komponenten der Reihung werden auf-|
    |           steigend geordnet tabellarisch im kleinen Ausgabefenster an-|
    |           gezeigt. Weil dabei die gelbe 'Diagonale' zum Teil ber-    |
    |           schrieben wird, muss sie reproduziert werden.               |
    *-----------------------------------------------------------------------*)

    CONST cZeile   = 508;

    VAR   i,spalte,
          zeile    : INTEGER;
          zahlStr  : tStr30;

    BEGIN
      spalte:= -2; zeile:= cZeile;
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.SetForeColor(0,96,96);
      FOR i:= 1 TO 360 DO
        IF i MOD 24 = 1 THEN
          INC(spalte,32);
          zeile:= cZeile
        END;
        F.StrF(a[i],3,0,zahlStr); S.Delete(zahlStr,S.Length(zahlStr),1);
        WriteStrXY(spalte,zeile,zahlStr);
        DEC(zeile,20)
      END;
      C.SetForeColor(255,255,100);
      FOR i:= 1 TO anzahl DO
        C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
      END;
    END AusgabeZahlenfeld;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m     : ARRAY 91 OF CHAR;
          dummy : INTEGER;

    BEGIN
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBox(W.GetActiveWindow(),Sy.ADR(m),Sy.ADR(m3),40H);
      Delay_ms(delayTime)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR i   : INTEGER;
        art : tArt;
        a   : tReihung;

    BEGIN
      Ueberschrift;
      i:= 0;
      REPEAT
        INC(i);
        IF i > 1 THEN Ausgabefenster END;
        Art(art);
        R.Randomize;(* Zufallszahlengenerator mit neuem Wert initialisieren *)
        ReihungAnlegenAusgeben(a,art);
        Sortieren(a,art);
        AusgabeZahlenfeld(a)
      UNTIL ~Wiederholung();
      Information
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   bc_Red:=  24; bc_Green:=  16; bc_Blue:=  48; (* HG-Farbe groes Fenster  *)
   bc_red:=   8; bc_green:=  12; bc_blue:=  16; (* HG-Farbe kleines Fenster *)
   C.SetBackColor(bc_Red,bc_Green,bc_Blue);
   C.Bar(5,0,850,650,C.ERASE);
   C.SetForeColor(200,200,200);
   C.Bar(1,0,3,650,C.DRAW);
   C.SetForeColor(96,96,96);
   C.Line(4,0,4,650,C.DRAW);
   C.SetForeColor(96,220,220);
   (* Da die Vordergrundfarbe Schwarz automatisch eingestellt ist,  m u s s *)
   (* die Textfarbe bereits hier initialisiert werden. Sonst sieht man die  *)
   (* Eingabe bei der Anweisung 'C.ReadStr()' nicht!                        *)
 END GrAuswOS.

 (* ----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ---------- *)

