
 (*--------------------------------------------------------------------------*
  |  Programmieren in Oberon (16 Bit)                File-Name: RANDOMNU.MOD |
  |--------------------------------------------------------------------------|
  |  Autor:       Niklaus WIRTH, Martin ReISER 1992                          |
  |  Algorithmus: D. H. LEHMER (1930), lineare Kongruenzmethode              |
  |--------------------------------------------------------------------------|
  |  Thema:       Erzeugen von gleichverteilten (uniformen) Zufallszahlen    |
  |--------------------------------------------------------------------------|
  |  Leistung:    Mehr als 2 Milliarden Zufallszahlen, die strenge statisti- |                                       |
  |               sche Tests bestehen.                                       |
  |               Intervall: 0 < rn < 1                                      |
  |--------------------------------------------------------------------------|
  |  Literatur:   Martin ReISER, Niklaus WIRTH, Programmieren in Oberon: Das |
  |               neue Pascal, Addison-Wesley, 1994, Seite 11 ff.            |
  |--------------------------------------------------------------------------|
  |  Abschrift u. Kommentare: Manfred Kmmerer, Chamberlain-Soft JENA, 1998  |
  *--------------------------------------------------------------------------*)

 MODULE RandomNu;

   IMPORT W:= Window;

   VAR z : LONGINT;
 
 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)

  PROCEDURE Uniform*() : REAL;
   (*-----------------------------------------------------------------------*
    | Referenz: Zufallszahl als Funktionswert                               |
    | Leistung: Erzeugen von gleichverteilten (uniformen) Zufallszahlen.    |
    *-----------------------------------------------------------------------*)

   CONST a =      16807;
         m = 2147483647;
         q = m DIV a;
         r = m MOD a;
 
   VAR   gamma : LONGINT;
     
   BEGIN
     gamma:= a * (z MOD q) - r * (z DIV q);
     IF gamma > 0 THEN 
       z:= gamma 
     ELSE 
       z:= gamma + m
     END;
     RETURN z * (1.0 / m);
   END Uniform;


  PROCEDURE InitSeed* ( seed : LONGINT );
   (*-----------------------------------------------------------------------*
    | bernahme: Samen (engl. seed) zur Erzeugung von Zufallszahlen         |
    | Leistung: Initialisierung (ffentliches Unterprogramm)                |
    *-----------------------------------------------------------------------*)

   BEGIN 
     z:= seed;
   END InitSeed;


 PROCEDURE Randomize*;
 (*-------------------------------------------------------------------------*
  | Leistung: Initialisierung mit dem Wert des WINDOWS-Tickers, der die Be- |
  |           triebsdauer der aktuellen Computersitzung seit dem Einschal-  |
  |           ten in Millisekunden zhlt. Windows.GetTickCount() liefert    |
  |           einen Wert vom DT: LONGINT (2 hoch 32), da knnte der Compu-  |
  |           ter etwas mehr als 52 Tage arbeiten bis ein Zahlenberlauf    |
  |           zu erwarten wre. (ffentliches Unterprogramm)                |
  |-------------------------------------------------------------------------|
  | Hinweis:  Die Prozedur Randomize* ist im Originalquelltext nicht ent-   |
  |           halten, sie wurde nach der Abschrift ergnzt.                 |
  *-------------------------------------------------------------------------*)

   VAR z : LONGINT;

   BEGIN
     z:= W.GetTickCount();
     IF z < 0 THEN z := -z END;
     InitSeed(z)
   END Randomize;


 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   z:=314159; 
 END RandomNu.

 (*-------------------------------------------------------------------------*)

