
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: AUSWAEHZ.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    S O R T I E R E N   durch Auswhlen - 5. Variante             |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von n Zahlen durch Auswhlen.                       |
  |           Zum Ermitteln des Zeitverhaltens des Algorithmus' in Abhn-   |
  |           gigkeit von der Anordnung der zu sortierenden Zahlen wird die |
  |           zum Sortieren bentigte Zeit ermittelt.                       |
  |           Die jeweils bentigte Zahlen-Reihung wird vom Programm gene-  |
  |           riert nach dem der Anwender eine Kennziffer eingegeben hat.   |
  |           Die Messdaten werden auf einem externen Speichermedium abge-  |
  |           legt.                                                         |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul als lauffhiges Grundgerst finden Sie unter:       |
  |            ...\sort1\good\auswaehz.mod                                  |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\sort1\txt\auswaeh5.txt und                        |
  |                   ...\sort1\doc\auswaeh5.doc                            |
  *-------------------------------------------------------------------------*)

 MODULE AuswaehZ;

  IMPORT In, Out, F:= File, R:= RandomNu, S:= Strings, Z:= Zeit;

  CONST  max    = 16;

  TYPE   tShStr = ARRAY 51 OF CHAR;

         tPunkt = RECORD
                    x, y : INTEGER
                  END;      (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tDaten = ARRAY max+1 OF tPunkt;

  VAR    a     : ARRAY 16001 OF INTEGER;
         daten : tDaten;
         fPfad : tShStr;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Ueberschrift;
    BEGIN
      Out.Ln; Out.Ln;
      Out.String('    S O R T I E R E N   durch Auswhlen'); Out.Ln;
      Out.String('   '); Out.Ln; Out.Ln
             (* '' = <Alt> + < 0 1 6 8 > auf dem Nummernblock der Tastatur *)
    END Ueberschrift;


  PROCEDURE Anfrage() : BOOLEAN;

    VAR ch : CHAR;

    BEGIN
      In.Prompt('Datei schreib. (J/N)');
      In.Char(ch);
      ch:= S.UpCaseChar(ch);
      IF ch = 'J' THEN
        RETURN TRUE
      ELSE
        RETURN FALSE
      END;
    END Anfrage;


  PROCEDURE Arbeit ( n : INTEGER );

    VAR i, laenge : INTEGER;
        message,
        zahlStr   : tShStr;

    BEGIN
      message:= '    Das System sortiert ';
      S.Str(n,zahlStr); S.Append(message,zahlStr);
      S.Append(message,' Zahlen.');
      Out.Ln;
      Out.String(message); Out.Ln;
      laenge:= SHORT(S.Length(message)-4);
      Out.String('    ');
      FOR i:= 1 TO laenge DO Out.Char('') END
    END Arbeit;


  PROCEDURE AusgabeArtShStrg;

    VAR artShStr : ARRAY 4 OF tShStr;
        i      : INTEGER;

    BEGIN
      artShStr[0]:= 'aufsteigend sortierte Zahlen';
      artShStr[1]:= 'Zufallszahlen';
      artShStr[2]:= 'absteigend, also "falsch herum", sortierte Zahlen';
      artShStr[3]:= 'alle drei Arten nacheinander';
      FOR i:= 0 TO 3 DO
        Out.Int(i+1,5); Out.String('. Art: '); Out.String(artShStr[i]); Out.Ln
      END;
      Out.Ln
    END AusgabeArtShStrg;


  PROCEDURE Art ( VAR art : INTEGER );
    BEGIN
      REPEAT
        In.Prompt('Art ( 1..4 )');
        In.Int(art)
      UNTIL (art > 0) & (art < 5)
    END Art;


  PROCEDURE Anzahl ( VAR n : INTEGER );
    BEGIN
      REPEAT
        In.Prompt(' Anzahl ( DT: INT )');
        In.Int(n)
        UNTIL (n > 0) & (n <= max)
    END Anzahl;


  PROCEDURE Zufallszahl() : INTEGER;
    BEGIN
      RETURN SHORT(ENTIER(R.Uniform() * 10000))
    END Zufallszahl;


  PROCEDURE Sortieren ( art, anzahl, n : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Sortieren durch Auswhlen.                                  |
    |           Die ausfhrliche Beschreibung des Algorithmus' findet man   |
    |           unter  ...\sort1\doc\auswaeh1.doc (WORD-Version)            |
    |           oder   ...\sort1\txt\auswaeh1.txt (Text-Version)            |
    *-----------------------------------------------------------------------*)
    
    VAR k, msec, r, s, sec, t, x : INTEGER;
    
    BEGIN
      CASE art OF
         1: FOR k:= 1 TO n DO
               a[k]:= k
             END
      | 2: FOR k:= 1 TO n DO
               a[k]:= Zufallszahl()
             END;
      | 3: FOR k:= 1 TO n DO
              a[k]:= -k
            END
      END;
      Z.Start;
      FOR r:= 1 TO n-1 DO
         t:= r;
         x:= a[r];
         FOR s:= r+1 TO n DO
            IF a[s] < x THEN
               t:= s;
               x:= a[s]
            END
         END;
         a[t]:= a[r];
         a[r]:= x
      END;
      Z.Stopp;
      IF Z.differenz < 0 THEN HALT(200) END;
      daten[anzahl].x:= n;
      daten[anzahl].y:= SHORT(Z.differenz);
      Out.Ln;
      Out.String('    Art:          '); Out.Int(art,6); Out.Ln;
      Out.String('    Anzahl:       '); Out.Int(n,6); Out.Ln;
      Out.String('    Sortierzeit:  ');
      IF Z.differenz < 1000 THEN
        Out.Int(Z.differenz,6); Out.String(' msec.'); Out.Ln
      ELSE
        sec:= SHORT(Z.differenz DIV 1000);
        msec:= SHORT(Z.differenz MOD 1000);
        Out.Int(sec,6); Out.String(','); Out.Int(msec,1);
        Out.String(' sec.'); Out.Ln
      END
    END Sortieren;


  PROCEDURE PfadStr ( VAR message : tShStr );
    BEGIN
      message:= "   Die Datei '";
      S.Append(message,fPfad);
      S.Append(message,"' wurde gelesen.");
    END PfadStr;


  PROCEDURE ErlaeuterungDateiname;
    BEGIN
      Out.Ln; Out.Ln;
      Out.String('    Der Dateiname darf 8 Zeichen lang sein.'); Out.Ln;
      Out.String("    Die Dateierweiterung '.DAT' wird automatisch ergnzt.");
      Out.Ln; Out.Ln;
      Out.String("    Verwenden Sie 'sprechende' Dateinamen, z. B.:"); Out.Ln;
      Out.String("    'AUSWAE02' fr Art = 2 (zufllig verteilt)");
      Out.Ln; Out.Ln
    END ErlaeuterungDateiname;


  PROCEDURE DateiSchreiben ( anzahl, art : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Sicheres einlesen eines Dateinamens und Ergnzen der Datei- |
    |           erweiterung '.DAT'. Falls eine Datei gleichen Namens im ak- |
    |           tuellen Pfad existiert, wird gefragt, ob sie gelscht wer-  |
    |           den soll. Wenn ja, ok, wenn nein, kann ein neuer Dateiname  |
    |           eingelesen werden. Falls alles i. O. ist, wird die Datei    |
    |           geffnet und blockweise geschrieben. Der erste Block ent-   |
    |           hlt die Kennzahl fr die Eigenschaft der Zahlen vor dem    |
    |           Sortiervorgang (z. B.: 1 fr Zufallszahlen), die folgenden  |
    |           Blcke enthalten die Daten (Anzahl Zahlen und Sortierzeit   |
    |           in Millisekunden).                                          |
    *-----------------------------------------------------------------------*)

    VAR ok         : BOOLEAN;
        ch         : CHAR;
        handle     : F.Handle;
        laenge,
        i, resCode : INTEGER;
        n          : LONGINT;
        message    : tShStr;
        punkt      : tPunkt;

    BEGIN
      ErlaeuterungDateiname;
      REPEAT
        In.Prompt('Dateiname');
        In.String(fPfad);
        laenge:= SHORT(S.Length(fPfad)); 
        IF (fPfad > ' ') & (laenge <= 8) THEN
          S.Append(fPfad,'.DAT');
          ok:= TRUE
        END;
        IF ok & F.Exist(fPfad) THEN
          In.Prompt('Lschen ( J/N )? N');
          In.Char(ch); ch:= CAP(ch);
          IF ch = 'J' THEN
            F.Delete(fPfad,resCode);
            ok:= TRUE
          ELSE
            ok:= FALSE;
            RETURN ok
          END
        END
      UNTIL ok;
      F.Open(fPfad,TRUE,F.DENYALL,F.WRITEONLY,handle,resCode);
      Out.Ln;
      PfadStr(message);
      Out.String(message); Out.Ln;
      n:= SIZE(INTEGER);
      F.WriteBlock(handle,art,n,resCode);
      n:= SIZE(tPunkt);
      FOR i:= 1 TO anzahl DO
        F.WriteBlock(handle,daten[i],n,resCode)
      END;
      F.Close(handle);
      Out.String('   Anzahl Datenstze = '); Out.Int(anzahl,2); Out.Ln;
      Out.Ln;
      RETURN ok
    END DateiSchreiben;


  PROCEDURE DatenLoeschen ( VAR anzahl, art : INTEGER );
   (*-----------------------------------------------------------------------*
    | Referenz: Anzahl der geschriebenen Datenstze und die Kennzahl der    |
    |           Eigenschaften der Zahlen.                                   |
    | Leistung: 'Lschen' (mit 0 berschreiben) aller Daten im Arbeitsspei- |
    |           cher des Computers, damit die Daten vom externen Speicher   |
    |           eingelesen und nicht die des Arbeitsspeichers fr die Kon-  |
    |           trollausgabe genutzt werden.                                |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      art:= 0;
      FOR i:= 1 TO anzahl DO
        daten[i].x:= 0;
        daten[i].y:= 0
      END;
      anzahl:= 0
    END DatenLoeschen;


  PROCEDURE DateiLesen ( VAR anzahl, art : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Falls die Datei im aktuellen Pfad existiert, wird sie geff-|
    |           net und die Daten blockweise gelesen. Der erste Block ent-  |
    |           hlt die Kennzahl fr die Eigenschaft der Zahlen vor dem    |
    |           Sortiervorgang (z. B.: 0 fr aufsteigend sortierte Zahlen), |
    |           die folgenden Blcke enthalten die Daten (Anzahl Zahlen und |
    |           Sortierzeit in Millisekunden).                              |
    *-----------------------------------------------------------------------*)

    VAR handle  : F.Handle;
        resCode : INTEGER;
        n, read : LONGINT;
        message : tShStr;
        punkt   : tPunkt;

    BEGIN
      IF F.Exist(fPfad) THEN
        F.Open(fPfad,FALSE,F.DENYALL,F.READONLY,handle,resCode);
        n:= SIZE(INTEGER);
        F.ReadBlock(handle,art,n,read,resCode);
        anzahl:= 0;
        n:= SIZE(tPunkt);
        WHILE ~F.AtEnd(handle) DO
          INC(anzahl);
          F.ReadBlock(handle,daten[anzahl],n,read,resCode);
        END;
        Out.Ln;
        PfadStr(message);
        Out.String(message); Out.Ln;
        F.Close(handle);
        Out.String('   Anzahl Datenstze = '); Out.Int(anzahl,2); Out.Ln;
        Out.Ln
      ELSE
        HALT(100)
      END 
    END DateiLesen;


  PROCEDURE Kontrollausgabe ( anzahl, art : INTEGER );

    VAR i : INTEGER;

    BEGIN
      Out.Ln;
      Out.String('   Art:          '); Out.Int(art,6); Out.Ln;
      FOR i:= 1 TO anzahl DO
        Out.Ln;
        Out.String('   Anzahl:       '); Out.Int(daten[i].x,6); Out.Ln;
        Out.String('   Sortierzeit:  '); Out.Int(daten[i].y,6);
        Out.String(' msec'); Out.Ln
      END;
    END Kontrollausgabe;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR ok        : BOOLEAN;
        ch        : CHAR;
        anzahl,
        art, i, n : INTEGER;

    BEGIN
      anzahl:= 0;
      n:= 0;
      ok:= FALSE;
      Ueberschrift;
      AusgabeArtShStrg;
      Art(art);
      REPEAT
        INC(n,1000);
        INC(anzahl);
        R.Randomize;(* Zufallszahlengenerator mit neuem Wert initialisieren *)
        IF art = 4 THEN
          Arbeit(n);
          FOR i:= 1 TO 3 DO
            Sortieren(i,anzahl,n)
          END
        ELSE
          Arbeit(n);
          Sortieren(art,anzahl,n)
        END;
        IF anzahl < max THEN
          In.Prompt('Wiederhlg. (J/N) J');
          In.Char(ch)
        END                      
      UNTIL (ch = 'n') OR (anzahl = max);
      IF (art < 4) & (anzahl > 10) & Anfrage() THEN
        REPEAT
          ok:= DateiSchreiben(anzahl,art)
        UNTIL ok
      END;
      IF ok THEN
        DatenLoeschen(anzahl,art);
        DateiLesen(anzahl,art);
        Kontrollausgabe(anzahl,art)
      END
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   In.Open;
   Out.Open
 END AuswaehZ.

 (*-------------------------------------------------------------------------*)

