
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)          Dateiname: GRAFIKA.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    SORTIEREN durch Auswhlen  -  Ergnzung zur 5. Variante       |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von INTEGER-Zahlen, die vom Anwender eingegeben     |
  |           werden.                                                       |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul als lauffhiges Grundgerst finden Sie unter:       |
  |            ...\sort1\good\grafika.mod                                   |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\sort1\txt\auswaeh1.txt und                        |
  |                   ...\sort1\doc\auswaeh1.doc                            |
  *-------------------------------------------------------------------------*)

 MODULE GrafikA;
  
  IMPORT In, Out, C:= ColorPlane, F:= File, Fl:= Float, S:= Strings,
         Sy:= SYSTEM, W:= Window;

  CONST  max     = 16;       (* maximale Anzahl Messungen                    *)
         xOffset = 20;
         yOffset = 30;
         xSkal   = 290;
         ySkal   = 290;


  TYPE   tPunkt  = RECORD
                     x, y : INTEGER
                   END;     (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tDaten  = ARRAY max+1 OF tPunkt;

         tPfad   = ARRAY 14 OF CHAR;
         tShStr  = ARRAY 45 OF CHAR;

  VAR    dummy   : BOOLEAN;
         daten   : tDaten;
         fPfad   : tPfad;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Anleitung1;
    BEGIN
      Out.Ln;
      Out.String('   Geben Sie den kompletten Dateinamen der auszuwerten');
      Out.String('den Messdatendatei an!'); Out.Ln; Out.Ln;
      Out.String('   Zum Beispiel: AUSWAE02.DAT'); Out.Ln
    END Anleitung1;


  PROCEDURE Mitteilung;
    BEGIN
      Out.Ln;  Out.Ln;
      Out.String('   max. x-Ausdehnung ='); Out.Int(xSkal,3); Out.Ln;
      Out.String('   max. y-Ausdehnung ='); Out.Int(ySkal,3); Out.Ln;
      Out.Ln;  Out.Ln; Out.Ln
    END Mitteilung;


  PROCEDURE Skalieren ( anzahl  : INTEGER; VAR xS, yS : REAL );

    VAR xMax, yMax : LONGINT;
        art, i     : INTEGER;

    BEGIN
      xMax:= 0;  yMax:= 0;
      FOR i:= 1 TO anzahl DO
        IF daten[i].x > xMax THEN xMax:= daten[i].x END;
        IF daten[i].y > yMax THEN yMax:= daten[i].y END;
      END;
      xS:= xSkal/xMax;
      yS:= ySkal/yMax
    END Skalieren;


  PROCEDURE RoundReal ( x : REAL ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme:  zu rundender Wert (DT: REAL)                              |
    | Referenz:   gerundeter Wert (Funktionswert, DT: INTEGER)              |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN Sy.VAL(INTEGER,x+0.5)
    END RoundReal;


  PROCEDURE WriteShortStrXY ( x, y : INTEGER; text : tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate und der auszugebender Text (string)    |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette im Grafikmodus.   |
    |            x = Pixelspalte; x = 0 entspricht der linken Begrenzung    |
    |                                   des aktuellen Grafikfensters        |
    |            y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung   |
    |                                   des aktuellen Grafikfensters        |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.GotoXY(x,y); C.WriteStr(text)
    END WriteShortStrXY;


  PROCEDURE Beschriften;

    CONST x   =  324;
          y   =  240;

    VAR   zeichen : ARRAY 2 OF CHAR;
          i       : INTEGER;
          str     : tShStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteShortStrXY(144,22,'Anzahl Zahlen      >');
      C.Line(256,15,300,15,C.DRAW);
      C.Line(x+3,y-2,x+3,y+42,C.DRAW);
      C.Line(x-1,y+38,x+3,y+42,C.DRAW); C.Line(x+3,y+42,x+7,y+38,C.DRAW);
      str:= 'Sortierzeit';
      FOR i:= 1 TO S.Length(str) DO
        S.Copy(str,zeichen,i,1);
        C.GotoXY(x,y-i*15); C.WriteStr(zeichen);
      END;
    END Beschriften;


  PROCEDURE Zeichnen ( anzahl  : INTEGER; xS, yS : REAL );

    VAR   art, i, x, x1, y, y1 : INTEGER;

    BEGIN
      Mitteilung;
      Beschriften;
      C.SetBackColor(255,255,220);
      C.Bar(xOffset,yOffset,xOffset+xSkal+5,yOffset+ySkal+5,C.ERASE);
      C.SetForeColor(0,0,0);
      C.Box(xOffset-1,yOffset-1,xOffset+xSkal+4,yOffset+ySkal+4,C.DRAW);
      x:= 0; y:= 0;
      FOR i:= 1 TO anzahl DO
        x1:= RoundReal(daten[i].x * xS);
        y1:= RoundReal(daten[i].y * yS);
        C.SetForeColor(0,200,220);
        C.Line(x+xOffset,y+yOffset,x1+xOffset,y1+yOffset,C.DRAW);
        x:= x1; y:= y1;
        C.Box(x+xOffset-3,y+yOffset-3,x+xOffset+3,y+yOffset+3,C.DRAW);
        C.SetForeColor(255,128,0);
        C.Bar(x+xOffset-2,y+yOffset-2,x+xOffset+2,y+yOffset+2,C.DRAW)
      END;
      C.SetBackColor(255,255,255)
    END Zeichnen;


  PROCEDURE DateiLesen ( VAR anzahl, art : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Falls die Datei im aktuellen Pfad existiert, wird sie geff-|
    |           net und die Daten blockweise gelesen. Der erste Block ent-  |
    |           hlt die Kennzahl fr die Eigenschaft der Zahlen vor dem    |
    |           Sortiervorgang (z. B.: 0 fr aufsteigend sortierte Zahlen), |
    |           die folgenden Blcke enthalten die Daten (Anzahl Zahlen und |
    |           Sortierzeit in Millisekunden).                              |
    *-----------------------------------------------------------------------*)

    VAR handle  : F.Handle;
        resCode : INTEGER;
        n, read : LONGINT;
        message : tShStr;
        punkt   : tPunkt;

    BEGIN
      IF F.Exist(fPfad) THEN
        F.Open(fPfad,FALSE,F.DENYALL,F.READONLY,handle,resCode);
        n:= SIZE(INTEGER);
        F.ReadBlock(handle,art,n,read,resCode);
        anzahl:= 0;
        n:= SIZE(tPunkt);
        WHILE ~F.AtEnd(handle) DO
          INC(anzahl);
          F.ReadBlock(handle,daten[anzahl],n,read,resCode);
        END;
        F.Close(handle);
      ELSE
        HALT(100)
      END 
    END DateiLesen;


  PROCEDURE Kontrollausgabe ( anzahl, art : INTEGER );

    CONST x1_Offset = 378;
          x2_Offset = 530;
          x3_Offset = 648;
          yOffset   = 318;

    VAR   artStr : ARRAY 5 OF tShStr;
          i      : INTEGER;
          datStr : tShStr;

    BEGIN
      artStr[0]:= 'aufsteigend';
      artStr[1]:= 'sortierte Zahlen';
      artStr[2]:= 'Zufallszahlen';
      artStr[3]:= 'absteigend, also';
      artStr[4]:= ' "falsch herum",';
      C.SetForeColor(127,127,0);
      WriteShortStrXY(x1_Offset-16,yOffset-36,'Eigenschaften:');
      C.Bar(x1_Offset,yOffset-72,x1_Offset+136,yOffset-124,C.ERASE);
      C.SetForeColor(0,128,128);
      CASE art OF
        1: WriteShortStrXY(x1_Offset,yOffset-72,artStr[0]);
           WriteShortStrXY(x1_Offset,yOffset-90,artStr[1])
       |2: WriteShortStrXY(x1_Offset,yOffset-72,artStr[2])
       |3: WriteShortStrXY(x1_Offset,yOffset-72,artStr[3]);
           WriteShortStrXY(x1_Offset,yOffset-90,artStr[4]);
           WriteShortStrXY(x1_Offset,yOffset-108,artStr[1])
      END;
      C.SetForeColor(0,0,0);
      WriteShortStrXY(x2_Offset+16,yOffset,'Anzahl');
      WriteShortStrXY(x3_Offset-24,yOffset,'Sortierzeit (msec)');
      FOR i:= 1 TO anzahl DO
        Fl.StrF(daten[i].x,6,0,datStr);       (* Zahl in Zeichenkette wand. *)
        S.Delete(datStr,S.Length(datStr)-1,2);(* Nachkommastellen entfernen *)
        WriteShortStrXY(x2_Offset,yOffset-i*18,datStr);
        Fl.StrF(daten[i].y,6,0,datStr);
        S.Delete(datStr,S.Length(datStr)-1,2);
        WriteShortStrXY(x3_Offset,yOffset-i*18,datStr);
      END;
    END Kontrollausgabe;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR ok     : BOOLEAN;
        ch     : CHAR;
        art,
        anzahl : INTEGER;
        xS, yS : REAL;

    BEGIN
      Anleitung1;
      REPEAT
        REPEAT  
          In.Prompt('Dateiname');
          In.String(fPfad);
          IF F.Exist(fPfad) THEN ok:= TRUE ELSE ok:= FALSE END
        UNTIL ok;
        DateiLesen(anzahl,art);
        Skalieren(anzahl,xS,yS);
        Zeichnen(anzahl,xS,yS);
        Kontrollausgabe(anzahl,art);
        In.Prompt('Wiederhlg. ( J/N ) J');
        In.Char(ch); ch:= CAP(ch);
      UNTIL ch = 'N'
    END ProgMain;
  
 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3)
 END GrafikA.

 (*-------------------------------------------------------------------------*)

