
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: DATUMOPE.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    SERVICEMODUL fr   D a t u m s o p e r a t i o n e n          |
  |-------------------------------------------------------------------------|
  | Leistung: Bereitstellen verschiedenster Operationen, die bei Datums-    |
  |           berechnungen bentigt werden.                                 |
  |-------------------------------------------------------------------------|
  | Hinweise: Das Modul ist lauffhig, stellt aber nur ein Grundgerst dar. |
  |           Es sollten Kommentare eingefgt werden.                       |
  |           Siehe ...\datum2\fine\datumope.mod                            |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\datum2\txt\datumtoo.txt und                       |
  |                   ...\datum2\doc\datumtoo.doc                           |
  *-------------------------------------------------------------------------*)

 MODULE DatumOpe;

  IMPORT Out;

  TYPE   tDatum* = RECORD            (* Ein dem Bezeichner nachgestelltes   *)
                     jahr*,          (* Sternchen '*' bedeutet, dieser Typ  *)
                     monat*,         (* diese Variable, Konstante oder die- *)
                     tag* : INTEGER  (* ser Bezeichner einer Prozedur bzw.  *)
                   END;              (* Funktionsprozedur ist ffentlich    *)
                                     (* deklariert und somit von anderen    *)
                                     (* Modulen importierbar.               *)

       tName*    = ARRAY  11 OF CHAR;


  PROCEDURE Schaltjahr* ( jahr : INTEGER ) : BOOLEAN;

    VAR c, j : INTEGER;

    BEGIN
      c:= jahr DIV 100;
      j:= jahr MOD 100;
      IF j = 0 THEN 
        RETURN c MOD 4 = 0
      ELSE
        RETURN j MOD 4 = 0 
      END;
    END Schaltjahr;


  PROCEDURE TageImMonat* ( jahr, monat : INTEGER ) : INTEGER;

    VAR anzahl: INTEGER;

    BEGIN
      CASE monat OF
         1, 3, 5, 7, 8, 10, 12: anzahl:= 31 
       | 4, 6, 9, 11:           anzahl:= 30
       | 2:                     IF Schaltjahr(jahr) THEN
                                  anzahl:= 29
                                ELSE
                                  anzahl:= 28
                                END
      END;
      RETURN anzahl
    END TageImMonat;


  PROCEDURE TageImJahr* ( jahr : INTEGER ) : INTEGER;
    BEGIN
      IF Schaltjahr(jahr) THEN
        RETURN 366
      ELSE
        RETURN 365
      END
    END TageImJahr;


  PROCEDURE Existenz* ( d : tDatum ) : BOOLEAN;
    BEGIN
      RETURN (d.jahr >= 1583) & (d.jahr <= 3000)
               & (d.monat >= 1) & (d.monat <= 12)
                & (d.tag >= 1) & (d.tag <= TageImMonat(d.jahr, d.monat))
    END Existenz;

   
  PROCEDURE Wochentag* ( d : tDatum) : INTEGER;

    (* 0 steht fr Sonntag, 1 steht fr Montag usw. *)

    VAR k, c, j, w: INTEGER;

    BEGIN
      CASE d.monat OF
         1: IF Schaltjahr(d.jahr) THEN k := 5
              ELSE k := 6
            END
       | 2: IF Schaltjahr(d.jahr) THEN k := 1
              ELSE k := 2
            END
       | 3: k := 2
       | 4: k := 5
       | 5: k := 0
       | 6: k := 3
       | 7: k := 5
       | 8: k := 1
       | 9: k := 4
       |10: k := 6
       |11: k := 2
       |12: k := 4
      END;
      c:= d.jahr DIV 100;
      j:= d.jahr MOD 100;
      w:= d.tag + k + j + j DIV 4 - 2 * (c MOD 4);
      RETURN w MOD 7
    END Wochentag;


  PROCEDURE Wochentagsname* ( tagesnummer : INTEGER;
                                               VAR wochentagsname : tName );
    BEGIN
      CASE tagesnummer OF
         0: wochentagsname:= 'Sonntag'
       | 1: wochentagsname:= 'Montag'
       | 2: wochentagsname:= 'Dienstag'
       | 3: wochentagsname:= 'Mittwoch'
       | 4: wochentagsname:= 'Donnerstag'
       | 5: wochentagsname:= 'Freitag'
       | 6: wochentagsname:= 'Samstag'
      ELSE
        Out.String(' Fehlerhafte Wochentagsnummer!'); Out.Ln
      END
    END Wochentagsname;


  PROCEDURE Monatsname* ( monatsnummer: INTEGER; VAR monatsname : tName );
    BEGIN
      CASE monatsnummer OF
         1: monatsname:= 'Januar'
       | 2: monatsname:= 'Februar'
       | 3: monatsname:= 'Mrz'
       | 4: monatsname:= 'April'
       | 5: monatsname:= 'Mai'
       | 6: monatsname:= 'Juni'
       | 7: monatsname:= 'Juli'
       | 8: monatsname:= 'August'
       | 9: monatsname:= 'September'
       |10: monatsname:= 'Oktober'
       |11: monatsname:= 'November'
       |12: monatsname:= 'Dezember'
      ELSE
        Out.String(' Fehlerhafte Monatsnummer!'); Out.Ln
      END
    END Monatsname;


  PROCEDURE Ostersonntag* ( jahr : INTEGER; VAR d : tDatum );

    (* MNU 51/3 (15.4.1998) Seiten 141-144 *)

    VAR k, a, e, n, w, s: INTEGER;

    BEGIN
      k:= jahr DIV 100;
      a:= jahr MOD 19;
      e:= (11 * a + 8 + (8 * k + 13) DIV 25 - (3 * k + 3) DIV 4) MOD 30;
      IF e < 24 THEN
        n:= 44 - e
      ELSIF e = 24 THEN
        n:= 49
      ELSIF e = 25 THEN
        n:= 49 - a DIV 11
      ELSE
        n:= 74 - e
      END;
      w:= (n + 5 * jahr DIV 4 + 2 - (3 * k + 3) DIV 4) MOD 7;
      s:= n + 7 - w;
      IF s <= 31 THEN 
        d.jahr := jahr;
        d.monat:= 3;
        d.tag  := s
      ELSE
        d.jahr := jahr;
        d.monat:= 4;
        d.tag  := s - 31
      END
    END Ostersonntag;

    
  PROCEDURE IstGleich* ( a, b : tDatum ) : BOOLEAN;
    BEGIN
      RETURN (a.jahr = b.jahr) & (a.monat = b.monat) & (a.tag = b.tag)
    END IstGleich;


  PROCEDURE LiegtVor* ( a, b : tDatum) : BOOLEAN;
    BEGIN
      IF a.jahr < b.jahr THEN
        RETURN TRUE
      ELSIF a.jahr > b.jahr THEN
        RETURN FALSE
      ELSIF a.monat < b.monat THEN
        RETURN TRUE
      ELSIF a.monat > b.monat THEN
        RETURN FALSE
      ELSE
        RETURN a.tag < b.tag
      END
    END LiegtVor;


  PROCEDURE Nachfolger* ( VAR d : tDatum );

    VAR r : tDatum; 

    BEGIN
      IF d.tag < TageImMonat(d.jahr,d.monat) THEN 
        r.jahr := d.jahr;
        r.monat:= d.monat;
        r.tag  := d.tag + 1
      ELSIF d.monat < 12 THEN
        r.jahr := d.jahr;
        r.monat:= d.monat + 1;
        r.tag  := 1
      ELSE
        r.jahr := d.jahr + 1;
        r.monat:= 1;
        r.tag  := 1
      END;
      d:= r
    END Nachfolger;


  PROCEDURE Vorgaenger* ( VAR d : tDatum );

    VAR r : tDatum;

    BEGIN
      IF d.tag > 1 THEN
        r.jahr := d.jahr;
        r.monat:= d.monat;
        r.tag  := d.tag - 1
      ELSIF d.monat > 1 THEN
        r.jahr := d.jahr;
        r.monat:= d.monat - 1;
        r.tag  := TageImMonat(r.jahr, r.monat)
      ELSE
        r.jahr := d.jahr - 1;
        r.monat:= 12;
        r.tag  := 31
      END;
      d:= r
    END Vorgaenger;


  PROCEDURE Summe* ( a: tDatum; anzahl : LONGINT; VAR b : tDatum);

    VAR z: LONGINT;

    BEGIN
      IF anzahl > 0 THEN
        FOR z:= 1 TO anzahl DO Nachfolger(a) END   
      ELSIF anzahl < 0 THEN
        FOR z:= 1 TO - anzahl DO Vorgaenger(a) END
      END;
      b:= a
    END Summe;


  PROCEDURE TauschenDaten* ( VAR a, b : tDatum );

    VAR  temp : tDatum;

    BEGIN
      temp.jahr:= a.jahr;      temp.monat:= a.monat;      temp.tag:= a.tag;
      a.jahr:=    b.jahr;      a.monat:=    b.monat;      a.tag:=    b.tag;
      b.jahr:=    temp.jahr;   b.monat:=    temp.monat;   b.tag:=    temp.tag
    END TauschenDaten;


  PROCEDURE Abstand* ( a, b : tDatum) : LONGINT;

    (* Voraussetzung: a liegt vor b *)

    VAR i      : INTEGER;
        anzahl : LONGINT;

    BEGIN
      IF LiegtVor(b,a) THEN TauschenDaten(a,b) END;
      IF a.jahr # b.jahr THEN
        anzahl:= TageImMonat(a.jahr,a.monat) - a.tag;
        FOR i:= a.monat + 1 TO 12 DO
          anzahl:= anzahl + TageImMonat(a.jahr,i)
        END;
        FOR i:= a.jahr + 1 TO b.jahr - 1 DO
          anzahl:= anzahl + TageImJahr(i)
        END;
        FOR i:= 1 TO b.monat - 1 DO
          anzahl:= anzahl + TageImMonat(b.jahr,i)
        END;
        anzahl:= anzahl + b.tag
      ELSIF a.monat # b.monat THEN
        anzahl:= TageImMonat(a.jahr, a.monat) - a.tag;
        FOR i:= a.monat + 1 TO b.monat - 1 DO
            anzahl:= anzahl + TageImMonat(b.jahr,i)
        END;
        anzahl:= anzahl + b.tag
      ELSE
        anzahl:= b.tag - a.tag
      END;
      RETURN anzahl
    END Abstand;

 END DatumOpe.
