
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: GRAFIKQS.MOD |
  |-------------------------------------------------------------------------|
  |  Autor:   Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                   |
  |           Erstellungsdatum: 03-04.01.2001 Version: V 1.0                |
  |-------------------------------------------------------------------------|
  | Thema:    SORTIEREN  (Teile und Herrsche), grafische Auswertung einer   |
  |           Messwertreihe, die mit dem Programm ...\FINE\QUICKSO3.MOD     |
  |           nach Auswahl optionaler Parameter aufgenommen und anschlie-   |
  |           end unter dem Dateinamen QUICKxxx.DAT gespeichert wurde.     |
  |-------------------------------------------------------------------------|
  | Leistung: Ausgabe der Daten einer Messreihe einerseits als Tabelle und  |
  |           andererseits als grafische Darstellung, die die Daten der     |
  |           Messreihe neben den Daten fr den besten (best case) und den  |
  |           schlechtesten Fall (worst case) zeigt.                        |
  |-------------------------------------------------------------------------|
  | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.    |
  |           GRAFIKQ.MOD (Oberon-2 V4)                                     |
  |-------------------------------------------------------------------------|
  | Hinweis:  Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |           Siehe:                                                        |
  |            ...\GRDLAGEN\ALLGEM\DOC\GRAFIK.DOC  o d e r                  |
  |            ...\GRDLAGEN\ALLGEM\TXT\GRAFIK.TXT                           |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE GrafikQs;

  IMPORT C:= ColorPlane, F:= File, Fl:= Float, SD:= ScanDir, S:= Strings, 
         Sy:= SYSTEM,    W:= Window;


  CONST  delayTime =   2;      (* 2 ist der Wert fr eine CPU i80486 66 MHz *)
                               (* Bitte an Ihren Prozessor anpassen!        *)

         max       =  25;                       (* Anzahl der Messwertpaare *)
         aY        = 400;                       (* y-Auflsung              *)
         xText     = 440;
         yText     = 420;

  TYPE   tArt      = RECORD
                       eigen, trenn, dTfaktor : INTEGER
                     END;

         tPunkt    = RECORD
                       x, y : INTEGER
                     END;   (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tDaten    =  ARRAY max+1 OF tPunkt;
         
         tInputStr = ARRAY 7 OF CHAR;
         tZahlStr  = ARRAY 6 OF CHAR;


  VAR    dummy                      : BOOLEAN;
         bc_Red, bc_Green, bc_Blue,            (* HG-Farben groes Fenster  *)
         bc_red, bc_green, bc_blue,            (* HG-Farben kleines Fenster *)
         anzahl                     : INTEGER;
         quali                      : tArt;
         daten                      : tDaten;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE FlushKeybuffer;
   (*-----------------------------------------------------------------------|
    | Leistung: Lschen des Tastaturpuffers                                 |
    |-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;
 
    BEGIN
      WHILE C.KeyPressed() DO Dummy:= C.ReadKey() END
    END FlushKeybuffer;


  PROCEDURE Delay ( dTfaktor : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine definierte Zeit warten                                 |
    *-----------------------------------------------------------------------*)

    VAR   i : LONGINT;

   BEGIN
     FOR i:= 1 TO delayTime * dTfaktor DO C.WriteStr('') END 
   END Delay;


  PROCEDURE RoundReal ( x : REAL ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: zu rundender Wert (DT: REAL)                               |
    | Referenz:  gerundeter Wert (Funktionswert, DT INTEGER)                |
    | Leistung:  Runden eines bernommenen REAL-Wertes unter Verwendung der |
    |            im Modul SYSTEM enthaltenen Funktionsprozedur VAL(T,x).    |
    |            T und x sind von beliebigem Typ. Ergebnistyp T (muss als   |
    |            DT eingetragen werden), x wird als Typ von T ausgelegt.    |
    *-----------------------------------------------------------------------*)

   BEGIN
     RETURN Sy.VAL(INTEGER,x+0.5)
   END RoundReal;


  PROCEDURE MessPunktSetzen ( x, y : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate des Mittelpunkts (DT: INTEGER)         |
    | Leistung:  Positionierte Ausgabe eines kleinen quadratischen Symbols. |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.Box(x-2,y-2,x+2,y+2,C.DRAW)
    END MessPunktSetzen;


  PROCEDURE WriteShortStrXY ( x, y : INTEGER; text : SD.tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate und der auszugebender Text (string)    |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette im Grafikmodus.   |
    |            x = Pixelspalte; x = 0 entspricht der linken Begrenzung    |
    |                                   des aktuellen Grafikfensters        |
    |            y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung   |
    |                                   des aktuellen Grafikfensters        |
    *-----------------------------------------------------------------------*)

   BEGIN
     C.GotoXY(x,y); C.WriteStr(text)
   END WriteShortStrXY;


  PROCEDURE WriteShortColStrXY ( r, g, b, x, y : INTEGER; Text : SD.tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: RGB-Werte der Textfarbe, x- und y-Koordinate sowie der     |
    |            auszugebender Text (string)                                |
    | Leistung:  Positionierte farbige Ausgabe einer Zeichenkette.          |
    *-----------------------------------------------------------------------*)

   BEGIN
     C.SetForeColor(r,g,b);
     C.GotoXY(x,y); C.WriteStr(Text)
   END WriteShortColStrXY;


  PROCEDURE ReadStrXY ( x, y, laenge : INTEGER; VAR kennzahl : tInputStr;
                                                        VAR resCode : CHAR );
   (*-----------------------------------------------------------------------*
    | bernahme: Position (x = Pixelspalte, y = Pixelzeile) des blinkenden  |
    |            Cursors, Anzahl der einlesbaren Zeichen (laenge) und ein   |
    |            Flag (boolesche Variable)                                  |
    | Referenz:  zeichenkette, DT: ARRAY 7 OF CHAR,damit die Prozedur       |
    |                          ReadStrXY() fr alle Eingaben einsetzbar ist,|
    |            resCode,      DT: CHAR, zum Auswerten der vom System be-   |
    |                          reitgestellten Konstanten C.ENTER und C.ESC  |
    | Leistung:  Einlesen einer Zeichenkette im Grafikmodus an einer durch  |
    |            die Werte der Variablen x (Pixelspalte) und y (Pixelzeile) |
    |            bestimmten Stelle des Bildschirms.                         |
    |            Besitzt die boolesche Variable flag den Wert TRUE, wird    |
    |            ein Hinweis zur Programmsteuerung am Bildschirm angezeigt. |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.SetForeColor(96,220,220);
      C.GotoXY(x,y); C.ReadStr(kennzahl,laenge,resCode)
    END ReadStrXY;


  PROCEDURE Wiederholung () : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Referenz: Funktionswert vom Datentyp BOOLEAN (TRUE fr Wiederholung,  |
    |           FALSE fr ohne Wiederholung)                                |
    | Leistung: Ausgabe der Frage mit unterstrichenem 'J' als Hinweis, dass |
    |           eine beliebige Tastenbettigung fr 'JA' (Wiederholung ge-  |
    |           wnscht) und < N > (N-Taste) fr 'NEIN' (Wiederholung nicht |
    |           gewnscht) steht.                                           |
    *-----------------------------------------------------------------------*)

    VAR resCode : CHAR;
        zeichen : tInputStr;
        x, y    : INTEGER;

    BEGIN
      x:= 320; y:= 32;
      WriteShortColStrXY(127,160,160,x,y,'Wiederholung (J/N)?');
      C.Line(x+112,y-14,x+120,y-14,C.DRAW);
      C.Line(x+112,y-15,x+120,y-15,C.DRAW);
      ReadStrXY(x+164,y,1,zeichen,resCode);
      C.SetForeColor(96,220,220);
      IF CAP(zeichen[0]) = 'N' THEN
        RETURN FALSE
      ELSE
        C.Bar(5,10,800,600,C.ERASE);
        RETURN TRUE
      END
    END Wiederholung;


  PROCEDURE Enter;
   (*-----------------------------------------------------------------------*
    | Leistung: Zeitlich verzgerte Ausgabe einer Zeichenkette, Lschen     |
    |           des Tastaturpuffers, Warten auf eine Tastenbettigung auf   |
    |           der Tastatur, Lschen der Zeichenkette, zeitlich verzgerte |
    |           Fortfhrung des Programms. Durch die Verzgerungen wird die |
    |           Abarbeitung des Programms gleitend. Eine 'hektische' Pro-   |
    |           grammausfhrung wird weistestgehend verhindert.             |
    *-----------------------------------------------------------------------*)

    BEGIN
      Delay(delayTime);
      WriteShortColStrXY(128,128,142,675,40,'< ENTER >');
      FlushKeybuffer;
      REPEAT UNTIL C.KeyPressed();
      FlushKeybuffer;
      C.Bar(675,40,747,24,C.ERASE);
      Delay(delayTime DIV 2)
    END Enter;


  PROCEDURE Ueberschrift ( x, y : INTEGER );
   (*-----------------------------------------------------------------------*
    | bernahme: x- und y-Koordinate (DT: INTEGER)                          |
    | Leistung:  Positionierte farbige Ausgabe der berschrift samt Unter-  |
    |            streichung. ...                                            |
    *-----------------------------------------------------------------------*)

    BEGIN
      WriteShortColStrXY(32,208,188,x,y,'Q  U  I  C  K  S  O  R  T');
      C.Line(x-10,y-19,x+208,y-19,C.DRAW);
      WriteShortColStrXY(0,136,120,x+32,y-32,'(Zeitkomplexitt)')
    END Ueberschrift;


  PROCEDURE Beschriften;
   (*-----------------------------------------------------------------------*
    | Leistung:  Komplette Beschriftung der dargestellten Zeitkomplexitt.  |
    *-----------------------------------------------------------------------*)

    CONST x1 = 284;
          x2 = 412;
          y  = yText-20;

    VAR zahlStr : ARRAY 4 OF CHAR;

    BEGIN
      Ueberschrift(330,504);
      WriteShortColStrXY(127,127,127,x1,y,'Eigenschaft:');
      WriteShortStrXY(x1,y-20,'Trennelement:');
      WriteShortStrXY(x1,y-40,'Verzgerung:');
      WriteShortStrXY(75,512,'Anzahl');
      WriteShortStrXY(79,496,'Werte');
      WriteShortStrXY(149,512,'Sortierzeit');
      WriteShortStrXY(165,496,'in msec');
      WriteShortStrXY(695,277,'Vergl.-');
      WriteShortStrXY(695,259,'Punkt');
      WriteShortStrXY(x1,68,'n * ld(n)');
      WriteShortStrXY(564,68,'n');
      WriteShortStrXY(416,68,'reale Serie');
      WriteShortColStrXY(0,160,160,x2,y-40,'Faktor ');
      S.Str(quali.dTfaktor,zahlStr); C.WriteStr(zahlStr);
      C.GotoXY(x2,y); 
      CASE quali.eigen OF
        1: C.WriteStr('bereits aufsteigend sortierte Zahlen')
       |2: C.WriteStr('zufllig verteilte Zahlen')
       |3: C.WriteStr('bereits absteigend sortierte Zahlen')
      END;
      C.GotoXY(x2,y-20);
      CASE quali.trenn OF
        1: C.WriteStr('das erste Element der Teilfolge')
       |2: C.WriteStr('das mittlere Element der Teilfolge')
       |3: C.WriteStr('das letzte Element der Teilfolge')
       |4: C.WriteStr('ein zuflliges Element der Teilfolge')
      END;
      C.SetForeColor(96,128,128);
      C.Line(670,260,690,260,C.DRAW); C.Line(680,270,680,250,C.DRAW);
      C.SetForeColor(196,96,56); MessPunktSetzen(270,60);
      C.SetForeColor(56,196,96); MessPunktSetzen(550,60);
      C.SetForeColor(0,184,200); MessPunktSetzen(402,60)
    END Beschriften;


  PROCEDURE LesenMessDat ( fPfad : SD.tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: Dateiname einer Messdatendatei (DT: SD.tShStr, der aus dem |
    |            Service-Modul ScanDir importiert wurde)                    |
    | Leistung:  ffen der Messdatendatei und bergabe gewillkrter Parame- |
    |            ter an das Betriebssystem.                                 |
    |            Referenzen: fHandle (DT: F.fhandle, aus dem importierten   |
    |            Standardmodul File) und resCode (DT: INTEGER), der eine    |
    |            auswertbare Fehlermeldung enthlt (siehe GetErrorMessage). |
    |            Bestimmen der Blockgre des ersten Blocks, der Angaben zu |
    |            den Eigenschaften der Messdatendatei enthlt und Lesen die-|
    |            ses Blocks. Ermitteln der Blockgre der Messdaten, Lesen  |
    |            aller Messdaten und Zhlen der Datenstze.                 |
    |            Datei schlieen.                                           |
    |            Ausgabe des erfolgreichen Lesens und der Anzahl der gele-  |
    |            senen Messdatenstze.                                      |
    *-----------------------------------------------------------------------*)

    VAR zahlStr : ARRAY 4 OF CHAR;
        fHandle : F.Handle;
        resCode : INTEGER;
        n, read : LONGINT;
        punkt   : tPunkt;

    BEGIN
      F.Open(fPfad,FALSE,F.DENYALL,F.READONLY,fHandle,resCode);
      n:= SIZE(tArt);                         (* Datentyp (!) als Argument  *)
      F.ReadBlock(fHandle,quali,n,read,resCode);
      anzahl:= 0;
      n:= SIZE(tPunkt);                       (* Datentyp (!) als Argument  *)
      WHILE ~F.AtEnd(fHandle) DO              (* solange kein Dateiende ... *)
        INC(anzahl);
        F.ReadBlock(fHandle,punkt,n,read,resCode);
        daten[anzahl]:= punkt
      END;
      C.SetForeColor(64,192,160);
      C.GotoXY(250,100);
      C.WriteStr('Die Datei '); C.WriteStr(fPfad);
      C.WriteStr(' wurde gelesen.');
      F.Close(fHandle);
      S.Str(anzahl,zahlStr);
      C.GotoXY(310,80);
      C.WriteStr('Anzahl Datenstze = '); C.WriteStr(zahlStr)
    END LesenMessDat;


  PROCEDURE AuswaehlenMessdatenDatei ( VAR fPfad : SD.tShStr;
                                                VAR ohneDATdatei : BOOLEAN );
   (*-----------------------------------------------------------------------*
    | Referenz: Dateipfad (fPfad, DT: SD.tShStr, aus Service-Modul ScanDir) |
    |           Wahrheitswert der Variable ohneDATdatei (DT: BOOLEAN)       |
    | Leistung: Aufnehmen aller Dateinamen des aktuellen Pfads, die die Da- |
    |           teierweiterung 'DAT' besitzen, in eine einfach zeigerver-   |
    |           kettete Liste. Die Dateinamen werden alpahbetisch aufstei-  |
    |           gend in die Liste eingefgt. Sind alle Dateinamen erfasst,  |
    |           werden sie mit Kennziffern versehen am Bildschirm ausgege-  |
    |           ben. Der Programmanwender kann eine Kennzahl eingeben, die  |
    |           den Programmablauf steuert. (Wahl der Messdaten, die ange-  |
    |           zeigt und ausgewertet werden.)                              |
    |           Befinden sich im aktuellen keine DAT-Dateien wird der Vari- |
    |           able 'ohneDATdatei' der Wert TRUE zugewiesen und nach Rck- |
    |           kehr zur ProgMain eine Fehlermeldung angezeigt.             |
    *-----------------------------------------------------------------------*)

    CONST x = 284;
          y = 400;

    VAR   resCode    : CHAR;
          kennZahl,
          txtRed,
          txtGreen,
          txtBlue,
          zaehler    : INTEGER;
          list       : SD.tList;
          dateiErw,
          dateiname,
          fehlerMldg,
          message    : SD.tShStr;
          zahlStr,
          kZahlStr   : tZahlStr;


    PROCEDURE AnzeigenDATdateien;
     (**
      | Leistung: Beweglichen Zeiger (Verbund list, DT: SD.tList) auf den   |
      |           Listenanfang setzten.                                     |
      |           Einen Eintrag (Dateinamen) aus der Liste hohlen, Zhler   |
      |           inkrementieren, Wert des Zhlers in eine Zeichenkette     |
      |           wandeln, in runde Klammern einschlieen, Leerzeichen und  |
      |           Dateiname anfgen und am Bildschirm als Menpunkt ausge-  |
      |           ben. Vorgang wiederholen, bis das Listenende erreicht ist.|
      **)

      VAR x1 : INTEGER;

      BEGIN
        txtRed:=     0;
        txtGreen:= 160;
        txtBlue:=  160;
        x1:= x + 40;
        zaehler:= 0;
        SD.GotoFirstEntry(list);               (* beweglichen Zeiger auf    *)
                                               (* den Listenanfang setzen   *)
        WHILE ~SD.EndOfList(list) DO           (* solange kein Listenende   *)
          SD.FetchEntry(list,fehlerMldg,dateiname);
          INC(zaehler);
          message:= '(';
          S.Str(zaehler,zahlStr); S.Append(message,zahlStr);
          S.Append(message,') '); S.Append(message,dateiname);
                                               (*  ... Eintrag entnehmen    *)
          WriteShortColStrXY(txtRed,txtGreen,txtBlue,x1,y-zaehler*16,message);
                                               (*  ... Eintrag ausgeben     *)
          SD.GotoNextEntry(list,fehlerMldg);   (* nchsten Verbund anpeilen *)
        END
      END AnzeigenDATdateien;


    PROCEDURE AuswaehlenDATdatei ( VAR zaehler : INTEGER );
     (**
      | Referenz: Anzahl der gltigen Dateinamen (zaehler, DT: INTEGER)     |
      | Leistung: Positionierte Ausgabe einer Zeichenkette, sicheres Ein-   |
      |           lesen einer Kennzahl aus dem zugelassenen Bereich, d. h.  |
      |           es werden nur Eingaben zwischen 1 und dem Wert der Vari-  |
      |           able zaehler zugelassen, fehlerhafte Eingabe werden abge- |
      |           wiesen und zur Wiederholung der Eingabe aufgefordert,     |
      |           statt einen Laufzeitfehler zu verursachen.                |
      |           Wird nur die Enter-Taste bettigt, erhlt die Variable    |
      |           kZahlStr den Wert '1' (---> kennZahl:= 1).                |
      **)
      BEGIN
        WriteShortColStrXY(120,208,160,x+56,y-(zaehler+2)*16,'Kennzahl =');
        REPEAT
          C.GotoXY(x+144,y-32-zaehler*16); C.WriteStr(' ');
          C.GotoXY(x+144,y-32-zaehler*16); C.ReadStr(kZahlStr,1,resCode)
        UNTIL ((kZahlStr >= '1') OR (kZahlStr = '')) & (kZahlStr <= zahlStr);
        IF kZahlStr = '' THEN
          kennZahl:= 1;
          WriteShortStrXY(x+144,y-32-zaehler*16,'1')
        ELSE  
          kennZahl:= SHORT(S.Val(kZahlStr))
        END;
        SD.GotoFirstEntry(list);
        FOR zaehler:= 1 TO kennZahl-1 DO
          SD.GotoNextEntry(list,fehlerMldg)    (* nchsten Verbund anpeilen *)
        END;
        SD.FetchEntry(list,fehlerMldg,fPfad)   (*  ... Eintrag zuweisen     *)
      END AuswaehlenDATdatei;


    BEGIN  (* AuswaehlenMessdatenDatei *)
      dateiErw:= 'DAT';
      SD.ScanDirectoryEntrys(zaehler,list,dateiErw,ohneDATdatei);
      IF ~ohneDATdatei THEN
        AnzeigenDATdateien;
        AuswaehlenDATdatei(zaehler)
      ELSE
        C.SetForeColor(255,127,0);
        WriteShortColStrXY(255,64,0,x+30,y,'Der aktuellen Pfad');
        WriteShortStrXY(x+6,y-20,'enthlt keine DAT-Datei!');
        Enter;
        C.Bar(x-16,y,x+176,y-36,C.ERASE)
      END
    END AuswaehlenMessdatenDatei;


  PROCEDURE AusgabeDATdatei ( fPfad : SD.tShStr );
   (*-----------------------------------------------------------------------*
    | bernahme: Dateipfad (fPfad : SD.tShStr aus Service-Modul ScanDir)    |
    | Leistung:  Positionierte farbige Ausgabe einer Zeichenkette.          |
    *-----------------------------------------------------------------------*)

    VAR message : SD.tShStr;

    BEGIN
      WriteShortColStrXY(127,127,127,284,425,'Messdatendatei:');
      WriteShortColStrXY(120,192,160,+412,425,fPfad)
    END AusgabeDATdatei;


  PROCEDURE Skalieren() : REAL;
   (*-----------------------------------------------------------------------*
    | Referenz: Funktionswert, Skalierungsfaktor, vom Datentyp REAL         |
    | Leistung: Berechnen des Skalierungsfaktors aus den Sortierzeiten.     |
    *-----------------------------------------------------------------------*)

    VAR i, y, yMax : INTEGER;

    BEGIN
      yMax:= 0;
      FOR i:= 1 TO anzahl DO
        IF daten[i].y > yMax THEN yMax:= daten[i].y END
      END; 
      RETURN aY/(yMax*2)
    END Skalieren;


  PROCEDURE BerechnenAbweichung () : REAL;
   (*-----------------------------------------------------------------------*
    | Referenz: Funktionswert, mittlere Abweichung, vom Datentyp REAL       |
    | Leistung: Berechnen der mittleren Abweichung aus den Sortierzeiten.   |
    |           (---> Zeichnen einer Geraden (linearen Regressions) als Ver-|
    |                 gleich, auch wenn es mathematisch nicht relevant ist, |
    |                 da es sich nicht um lineare Regression handeln kann)  |
    *-----------------------------------------------------------------------*)

    VAR i           : INTEGER;
        abweichung,
        aMittel,
        summe       : REAL;

    BEGIN
      summe:= 0;
      FOR i:= 1 TO anzahl DO
        summe:= summe + daten[i].y/aY
      END;
      aMittel:= summe/anzahl;
      abweichung:= (summe - aMittel) * (summe - aMittel);
      abweichung:= Fl.Sqrt(Fl.Sqrt(abweichung/anzahl));
      RETURN abweichung
    END BerechnenAbweichung;


  PROCEDURE AusgebenMessdaten;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe der Messdaten, Anzahl zu sortiernder Zahlen und zum |
    |           Sortieren bentigte Zeit, in tabellarischer und grafischer  |
    |           Form. Bei der grafischen Darstellung wird auf ein Koordina- |
    |           tensystem verzichtet, weil nur die relativen Abstnde der   |
    |           Kurvenverlufe von Interesse sein drften (halbquantitative |
    |           Betrachtungsweise).                                         |
    *-----------------------------------------------------------------------*)

    CONST faktor  =  25;
          xOffset =  10;
          yOffset =  60;
          yKoord  = 490;

    VAR   i, x1, y1,
          x2, y2     : INTEGER;
          a, s       : REAL;
          zahlStr    : ARRAY 20 OF CHAR;

    PROCEDURE ZeichnenVerbindungslinien;
     (**
      | Leistung: Zeichnen der Verbindungslinien fr die 'Messpunkte' des   |
      |           besten (best) und des schlechtesten Falls (worst case).   |
      **)
      BEGIN
        C.SetForeColor(104,64,0);
        x1:= 40+xOffset;
        y1:= RoundReal(faktor*Fl.Log2(1))+yOffset;
        FOR i:= 2 TO anzahl DO                                (* best case  *)
          x2:= i*40+xOffset;
          y2:= RoundReal(faktor*Fl.Log2(i))+yOffset;
          C.Line(x1,y1,x2,y2,C.DRAW);
          x1:= x2; y1:= y2
        END;
        C.SetForeColor(0,104,64);
        x1:= 40+xOffset;  y1:= yOffset;
        FOR i:= 2 TO anzahl DO                                (* worst case *)
          x2:= i*40+xOffset;  y2:= i*i+yOffset;
          C.Line(x1,y1,x2,y2,C.DRAW);
          x1:= x2; y1:= y2
        END
      END ZeichnenVerbindungslinien;

    BEGIN  (* AusgebenMessdaten *)
      s:= Skalieren();
      a:= BerechnenAbweichung();
      ZeichnenVerbindungslinien;
      C.SetForeColor(96,96,104);
      C.Line(xOffset+40,yOffset,daten[anzahl].x DIV faktor+xOffset,
                           RoundReal(daten[anzahl].y*s-a)+yOffset,C.DRAW);
                                              (* eine Art Regressionsgerade *)
      x1:= daten[1].x DIV faktor+xOffset;
      y1:= RoundReal(daten[1].y*s+yOffset);
      FOR i:= 1 TO anzahl DO        
        C.SetForeColor(0,160,160);
        Fl.StrF(daten[i].x,8,0,zahlStr);
            (* daten[i].x (DT: INTEGER) = Anzahl sortierter Zahlen in eine  *)
            (*                            formatierte Zeichenkette wandeln  *)
        S.Delete(zahlStr,S.Length(zahlStr),1);
            (* Float.StrF() wandelt eine Zahl (DT: LONGREAL) in eine forma- *)
            (* tierte Zeichenkette. Auch bei der Option null Nachkommastel- *)
            (* len bleibt der Dezimalpunkt erhalten. Daher wird die Proze-  *)
            (* Strings.Delete() genutzt, um diesen Punkt zu entfernen.      *)
        C.GotoXY(xOffset+31,yKoord-i*20); C.WriteStr(zahlStr);   (* Tabelle *)
        Fl.StrF(daten[i].y,8,0,zahlStr);
            (* daten[i].y (DT: INTEGER) = bentigte Sortierzeit in eine     *)
            (*                            formatierte Zeichenkette wandeln  *)
        S.Delete(zahlStr,S.Length(zahlStr),1);
        C.GotoXY(xOffset+127,yKoord-i*20); C.WriteStr(zahlStr);  (* Tabelle *)
        C.SetForeColor(196,96,56);
        MessPunktSetzen(i*40+xOffset,RoundReal(faktor*Fl.Log2(i))+yOffset);
                                                     (* Wert fr best case  *)
        C.SetForeColor(56,196,96);
        MessPunktSetzen(i*40+xOffset,i*i+yOffset);   (* Wert fr worst case *)
        x2:= daten[i].x DIV faktor +xOffset;
        y2:= RoundReal(daten[i].y*s+yOffset);
        C.SetForeColor(0,184,200);
        MessPunktSetzen(x2,y2);                      (* realer Messwert     *)
        C.Line(x1,y1,x2,y2,C.DRAW);
        x1:= x2; y1:= y2
      END;
      Beschriften
    END AusgebenMessdaten;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m     : ARRAY 91 OF CHAR;
          dummy : INTEGER;

    BEGIN
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBox(W.GetActiveWindow(),Sy.ADR(m),Sy.ADR(m3),40H);
      Delay(delayTime)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR errorFlag : BOOLEAN;
        fPfad     : SD.tShStr;

    BEGIN
      REPEAT
        Ueberschrift(290,480);
        AuswaehlenMessdatenDatei(fPfad,errorFlag);
        IF ~errorFlag THEN
          LesenMessDat(fPfad);
          Enter;
          C.Bar(245,480,545,10,C.ERASE);
          AusgabeDATdatei(fPfad);
          AusgebenMessdaten;
          Enter
        END
      UNTIL errorFlag OR ~Wiederholung();
        (* 'errorFlag'  m u s s  unbedingt vor dem Aufruf der Funktionspro- *)
        (* zedur Wiederholung() stehen, da diese sonst vor der Prfung des  *)
        (* Wertes der Variable 'errorFlag' aufgerufen wrde, was selbstver- *)
        (* stndlich bei gesetztem 'errorFlag' nicht das erwartete Ende der *)
        (* Programmabarbeitung ergbe.                                      *)
      IF ~errorFlag THEN Information END
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   bc_Red:= 24; bc_Green:= 16; bc_Blue:= 48;    (* HG-Farbe groes Fenster  *)
   C.SetBackColor(bc_Red,bc_Green,bc_Blue);
   C.Bar(5,0,850,650,C.ERASE);
   C.SetForeColor(200,200,200);
   C.Bar(1,0,3,650,C.DRAW);
   C.SetForeColor(96,96,96);
   C.Line(4,0,4,650,C.DRAW);
   C.SetForeColor(96,220,220);
   (* Da die Vordergrundfarbe Schwarz automatisch eingestellt ist,  m u s s *)
   (* die Textfarbe bereits hier initialisiert werden. Sonst sieht man die  *)
   (* Eingabe bei der Anweisung 'C.ReadStr()' nicht!                        *)
 END GrafikQs.

 (* -----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ----------*)

