
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: QUICK2MS.MOD |
  |-------------------------------------------------------------------------|
  |  Autor:    Manfred Kmmerer, Carl-Zeiss-Gymnasium JENA                  |
  |            Erstellungsdatum: 16-18.02.2001 Version: V 1.0               |
  |-------------------------------------------------------------------------|
  | Thema:    SORTIEREN  (Quicksort); grafische Darstellung des             |
  |                      Sortiervorgangs nach Auswahl von Parametern        |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von INTEGER-Zahlen (Quicksort, Teile und Herrsche). |
  |           Der Sortiervorgang wird durch wiederholtes Setzen und Lschen |
  |           der Punkte im Grafikfenster (ColorPlane) veranschaulicht.     |
  |           Whrend der Laufzeit knnen verschiedene Parameter durch den  |
  |           Anwender des Programms eingestellt werden.                    |
  |-------------------------------------------------------------------------|
  | Hinweis:  Grafikmodus 800 x 600 Pixel, Farbauflsung mindestens 15 Bit |
  |           Siehe:                                                        |
  |            ...\GRDLAGEN\ALLGEM\DOC\GRAFIK.DOC  o d e r                  |
  |            ...\GRDLAGEN\ALLGEM\TXT\GRAFIK.TXT                           |
  |                                                                         |
  | ACHTUNG!  Dieses Programm ist nur lauffhig, wenn das Speaker-Modul     |
  |           installiert wurde.                                            |
  |                                                                         |
  |           Zum Einsatz des Speaker-Moduls (Installation und Import)      |
  |           siehe:                                                        |
  |                ...\GRDLAGEN\ALLGEM\DOC\SPEAKER.DOC  oder                |
  |                ...\GRDLAGEN\ALLGEM\TXT\SPEAKER.TXT                      |
  |-------------------------------------------------------------------------|
  |             2001  Manfred Kmmerer, Chamberlain-Soft JENA              |
  *-------------------------------------------------------------------------*)

 MODULE Quick2ms;

  IMPORT C:= ColorPlane, F:=  Float,   P:=  Process, R:= RandomNu,
         S:= Strings,    Sp:= Speaker, Sy:= SYSTEM,  W:= Window;

  CONST  delayTime =   2;      (* 2 ist der Wert fr eine CPU i80486 66 MHz *)
                               (* Bitte an Ihren Prozessor anpassen!        *)
         anzahl    = 480;
         default   =   25;     (* Standardverzgerungsfaktor                *)
         trenn     =   4;
         xOffset   =  32;
         yOffset   =  32;

  TYPE   tZahlStr  = ARRAY  6 OF CHAR;
         tStr30    = ARRAY 31 OF CHAR;
         tReihung  = ARRAY anzahl + 1 OF INTEGER;

         tArt      = RECORD
                       eigen, trenn, dTfaktor : INTEGER
                     END;

  VAR    dummy                      : BOOLEAN;
         bc_Red, bc_Green, bc_Blue,            (* HG-Farben groes Fenster  *)
         bc_red, bc_green, bc_blue : INTEGER;  (* HG-Farben kleines Fenster *)


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Beep ( frequenz, tonlaenge : INTEGER );
   (*-----------------------------------------------------------------------*
   
    | Leistung: Ausgabe eines Tones (PC-Lautsprecher) der bergebenen Fre-  |
    |           quenz und Tondauer.                                         |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      Sp.Sound(frequenz);
      Sp.Delay(tonlaenge);
      Sp.NoSound
    END Beep;


  PROCEDURE FlushKeyBuffer;
   (*-----------------------------------------------------------------------*
    | Leistung: Lschen des Tastaturpuffers                                 |
    *-----------------------------------------------------------------------*)

    VAR Dummy : CHAR;
 
    BEGIN
      WHILE C.KeyPressed() DO Dummy:= C.ReadKey() END
    END FlushKeyBuffer;


  PROCEDURE Delay_ms ( msec : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Eine bestimmte Zeit warten.                                 |
    *-----------------------------------------------------------------------*)

    VAR startzeit, zielzeit : LONGINT;

    BEGIN
      startzeit:= W.GetTickCount(); 
      REPEAT
        zielzeit:= W.GetTickCount()
      UNTIL zielzeit > startzeit + msec 
    END Delay_ms;


  PROCEDURE WriteStrXY ( x, y : INTEGER; string : tStr30 );
   (*-----------------------------------------------------------------------*
    | bernahme: x = Pixelspalte; x = 0 entspricht der linken Begrenzung    |
    |                                   des aktuellen Grafikfensters        |
    |            y = Pixelzeile;  y = 0 entspricht der unteren Begrenzung   |
    |                                   des aktuellen Grafikfensters        |
    |            string = Zeichenkette, maximale Lnge: 30 Zeichen          |
    | Leistung:  Positionierte Ausgabe einer Zeichenkette im Grafikmodus,   |
    *-----------------------------------------------------------------------*)

    BEGIN
      C.GotoXY(x,y); C.WriteStr(string)
    END WriteStrXY;


  PROCEDURE Wiederholung() : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information, Warten auf eine Tastenbettigung |
    |           (Tastatur), Wandeln des Zeichens in einen Grobuchstaben,   |
    |           falls ein Kleinbuchstabe eingegeben wurde und Auswerten der |
    |           Eingabe (boolescher Wert).                                  |
    | Referenz: Funktionswert TRUE, falls eine Wiederholung gewnscht wird, |
    |           d.h. es wurde eine beliebige Taste auer <N> bettigt,      |
    |           sonst FALSE.                                                |
    *-----------------------------------------------------------------------*)

    VAR ch : CHAR;
      
    BEGIN
      Delay_ms(delayTime*5);
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(32,160,160);
      WriteStrXY(575,65,'Wiederholung (J/N) J');
      FlushKeyBuffer;
      ch:= C.ReadKey();
      ch:= S.UpCaseChar(ch);
      RETURN ~(ch = 'N') 
    END Wiederholung;


  PROCEDURE Ausgabefenster;
    BEGIN
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.Bar(xOffset-3,yOffset-3,xOffset+anzahl+3,yOffset+anzahl+3,C.ERASE);
      C.SetForeColor(0,192,192);
      C.Box(xOffset-6,yOffset-6,xOffset+anzahl+6,yOffset+anzahl+6,C.DRAW)
    END Ausgabefenster;


  PROCEDURE Ueberschrift;
    BEGIN
      Ausgabefenster;
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.SetForeColor(32,208,208);
      WriteStrXY(590,495,'Q U I C K S O R T');
      C.Line(580,476,734,476,C.DRAW);
      C.Line(580,475,734,475,C.DRAW);
      Delay_ms(delayTime*25)
    END Ueberschrift;


  PROCEDURE ReihungAnlegenAusgeben ( VAR a : tReihung; art : tArt );
   (*-----------------------------------------------------------------------*
    | Leistung: In Abhngigkeit des Wertes der Variablen art, werden die    |
    |           Komponenten der Reihung mit aufsteigenden, zuflligen oder  |
    |           absteigenden Werten der Menge {1..n} belegt und im Ausgabe- |
    |           fenster als violetter Punkt (C.Dot(x,y,C.DRAW) dargestellt. |
    *-----------------------------------------------------------------------*)

    VAR i : INTEGER;

    BEGIN
      C.SetForeColor(255,127,255);
      IF art.eigen = 1 THEN
        FOR i:= 1 TO anzahl DO
          a[i]:= i;
          C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
        END
      ELSIF art.eigen = 2 THEN
        FOR i:= 1 TO anzahl DO
          a[i]:= 1 + SHORT(ENTIER(R.Uniform()*anzahl));
          C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
        END
      ELSE
        FOR i:= 1 TO anzahl DO
          a[i]:= 1 + anzahl - i;
          C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
        END
      END;
      Delay_ms(art.dTfaktor)
    END ReihungAnlegenAusgeben;


  PROCEDURE Eigenschaften ( VAR art : tArt );

    CONST x1 = 560;  x2 = 576;  x3 = 592;

    VAR   resCode : CHAR;
          zahlStr : tZahlStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(544,445,'Die Zahlen sind ');
      WriteStrXY(x1,420,'(1) bereits aufsteigend');
      WriteStrXY(x3,400,'sortiert,');
      WriteStrXY(x1,380,'(2) zufllig verteilt');
      WriteStrXY(x3,360,'oder');
      WriteStrXY(x1,340,'(3) bereits absteigend');
      WriteStrXY(x3,320,'sortiert.');
      WriteStrXY(x3,290,'Art (1..3) =');
      WriteStrXY(x3,270,'<ENTER> fr (2)');
      C.SetForeColor(120,160,160);
      REPEAT
        C.Bar(696,290,704,274,C.ERASE);
        C.GotoXY(696,290); C.ReadStr(zahlStr,1,resCode);
        IF zahlStr = '' THEN
          zahlStr:= '2'
        END
      UNTIL (zahlStr > '0') & (zahlStr < '4');
      art.eigen:= SHORT(S.Val(zahlStr));
      C.Bar(544,445,752,254,C.ERASE);
      C.SetForeColor(120,160,160);
      C.GotoXY(x1,445); C.WriteStr('Eigenschaft:');
      C.SetForeColor(120,208,160);
      C.GotoXY(x2,420); 
      CASE art.eigen OF
        1: C.WriteStr('bereits aufsteigend');
           WriteStrXY(x2,400,'sortierte Zahlen')
       |2: C.WriteStr('zufllig verteilte');
           WriteStrXY(x2,400,'Zahlen')
       |3: C.WriteStr('bereits absteigend');
           WriteStrXY(x2,400,'sortierte Zahlen')
      END
    END Eigenschaften;


  PROCEDURE Trennelement ( VAR art : tArt );

    CONST x1 = 560;  x2 = 576;  x3 = 592;

    VAR   resCode : CHAR;
          zahlStr : tZahlStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(544,365,'Das Trennelement ist');
      WriteStrXY(x1,340,'(1) das erste Element');
      WriteStrXY(x1,320,'(2) das mittlere Element');
      WriteStrXY(x1,300,'(3) das letzte Element');
      WriteStrXY(x1,280,'(4) ein zuflliges Element');
      WriteStrXY(x3,260,'der Teilfolge.');
      WriteStrXY(x3,230,'Art (1..4) =');
      WriteStrXY(x3,210,'<ENTER> fr (4)');
      C.SetForeColor(120,160,160);
      REPEAT
        C.Bar(696,230,704,214,C.ERASE);
        C.GotoXY(696,230); C.ReadStr(zahlStr,1,resCode);
        IF zahlStr = '' THEN
          zahlStr:= '4'
        END
      UNTIL (zahlStr > '0') & (zahlStr < '5');
      art.trenn:= SHORT(S.Val(zahlStr));
      C.Bar(544,365,766,154,C.ERASE);
      C.SetForeColor(120,160,160);
      WriteStrXY(x1,365,'Trennelement:');
      C.SetForeColor(120,208,160);
      C.GotoXY(x2,340);
      CASE art.trenn OF
        1: C.WriteStr('das erste Element');
       |2: C.WriteStr('das mittlere Element');
       |3: C.WriteStr('das letzte Element');
       |4: C.WriteStr('ein zuflliges Element');
           WriteStrXY(x2,320,'der Teilfolge')
      END
    END Trennelement;


  PROCEDURE Verzoegerungszeit ( VAR art : tArt );

    CONST x1 = 560;  x2 = 576;

    VAR   resCode : CHAR;
          zahlStr : tZahlStr;

    BEGIN
      C.SetForeColor(0,160,160);
      WriteStrXY(x1,285,'Verzgerungs-');
      WriteStrXY(x1,265,'zeitfaktor =');
      WriteStrXY(x1+8,245,'<ENTER> fr 25');
      C.GotoXY(664,265); C.ReadStr(zahlStr,6,resCode);
      IF zahlStr = '' THEN
        art.dTfaktor:= default;
        zahlStr:= '25'
      END;
      art.dTfaktor:= SHORT(S.Val(zahlStr));
      C.Bar(x1,285,766,234,C.ERASE);
      C.SetForeColor(120,160,160);
      WriteStrXY(x1,285,'Verzgerung:');
      C.SetForeColor(120,208,160);
      WriteStrXY(x2,260,'Verzgerungsfaktor ');
      C.WriteStr(zahlStr)
    END Verzoegerungszeit;


  PROCEDURE Art ( VAR art : tArt );
    BEGIN
      C.SetBackColor(bc_Red,bc_Green,bc_Blue);
      C.Bar(533,445,766,50,C.ERASE);
      Eigenschaften(art);
      Trennelement(art);
      Verzoegerungszeit(art);
      Delay_ms(delayTime*5)
    END Art;

                  
  PROCEDURE Sortieren ( VAR a : tReihung; art : tArt );

    PROCEDURE Sort ( l, r : INTEGER );
   (*-----------------------------------------------------------------------*
    | Quelle:   ThILLM-Materialien 38, SERVICE-CD, Dr. Michael Fothe u. a.  |
    *-----------------------------------------------------------------------*
    | Leistung: Sortieren der Werte der Komponenten der Reihung nach auf-   |
    |           steigender Gre mittels Quicksort (Teile und Herrsche).    |
    | Hinweis:  Beschreibung des Algorithmus siehe:                         |
    |             ...\SORT2\TXT\QUICKSOR.TXT und ...\QUICKS1.TXT            |
    |            oder bei Nutzung von MS-WORD (o..) unter                  |
    |             ...\SORT2\DOC\QUICKSOR.DOC und ...\QUICKS1.DOC            |
    |           Die Originalwerte der Komponenten der Reihung werden als    |
    |           violette Pixel angezeigt. Die beiden Pixel, die an einer    |
    |           Tauschaktion beteiligt sind werden gelscht (mit Hinter-    |
    |           grundfarbe berschrieben) und an ihrer jeweils neuen Posi-  |
    |           tion als gelbes Pixel ausgegeben.                           |
    *-----------------------------------------------------------------------*)

      VAR i, j, x, w : INTEGER;

      BEGIN
        i:= l;
        j:= r;
        CASE art.trenn OF 
           1: x := a[l]
         | 2: x := a[(l + r) DIV 2]
         | 3: x := a[r]
         | 4: x := a[l + SHORT(ENTIER(R.Uniform() * (r - l + 1)))]
        END;
        REPEAT
          WHILE a[i] < x DO INC(i) END;
          WHILE x < a[j] DO DEC(j) END;
          IF i <= j THEN
            Delay_ms(art.dTfaktor);
            C.Dot(i+xOffset,a[i]+yOffset,C.ERASE);   (* Punkt lschen       *)
            C.Dot(j+xOffset,a[j]+yOffset,C.ERASE);   (* Punkt lschen       *)
            w:= a[i];
            a[i]:= a[j];
            a[j]:= w;
            C.Dot(i+xOffset,a[i]+yOffset,C.DRAW);    (* Punkt setzen        *)
            C.Dot(j+xOffset,a[j]+yOffset,C.DRAW);    (* Punkt setzen        *)
            INC(i);
            DEC(j)
          END
        UNTIL i > j;
        IF l < j THEN Sort(l,j) END;
        IF i < r THEN Sort(i,r) END
      END Sort;

    BEGIN
      ReihungAnlegenAusgeben(a,art);
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.SetForeColor(255,255,100);
      Beep(440,250);
      P.Yield;                                       (* keine Multitasking  *)
      Sort(1,anzahl);
      Beep(880,250)
    END Sortieren;


  PROCEDURE AusgabeZahlenfeld ( a : tReihung );
   (*-----------------------------------------------------------------------*
    | Leistung: Die sortierten Werte der Komponenten der Reihung werden auf-|
    |           steigend geordnet tabellarisch im kleinen Ausgabefenster an-|
    |           gezeigt. Weil dabei die gelbe 'Diagonale' zum Teil ber-    |
    |           schrieben wird, muss sie reproduziert werden.               |
    *-----------------------------------------------------------------------*)

    CONST cZeile   = 508;

    VAR   i,spalte,
          zeile    : INTEGER;
          zahlStr  : tStr30;

    BEGIN
      spalte:= -2; zeile:= cZeile;
      C.SetBackColor(bc_red,bc_green,bc_blue);
      C.SetForeColor(0,96,96);
      FOR i:= 1 TO 360 DO
        IF i MOD 24 = 1 THEN
          INC(spalte,32);
          zeile:= cZeile
        END;
        F.StrF(a[i],3,0,zahlStr); S.Delete(zahlStr,S.Length(zahlStr),1);
        WriteStrXY(spalte,zeile,zahlStr);
        DEC(zeile,20)
      END;
      C.SetForeColor(255,255,100);
      FOR i:= 1 TO anzahl DO
        C.Dot(i+xOffset,a[i]+yOffset,C.DRAW)
      END;
    END AusgabeZahlenfeld;


  PROCEDURE Information;
   (*-----------------------------------------------------------------------*
    | Leistung: Ausgabe einer Information nach Abschluss des Programms.     |
    *-----------------------------------------------------------------------*)

    CONST m1 = 'Besuchen Sie die POW-Seiten des CZ-Gymnasiums!';
          m2 = ' http://www.cz.j.th.schule.de/informatik';
          m3 = '     2001 Chamberlain-Soft';

    VAR   m     : ARRAY 91 OF CHAR;
          dummy : INTEGER;

    BEGIN
      C.Bar(5,10,850,600,C.ERASE);
      m:= m1; S.Append(m,m2);
      dummy:= W.MessageBox(W.GetActiveWindow(),Sy.ADR(m),Sy.ADR(m3),40H);
      Delay_ms(delayTime*50)
    END Information;


 (* ---------------------  H A U P T P R O G R A M M  --------------------- *)

  PROCEDURE ProgMain*();

    VAR a   : tReihung;
        i   : INTEGER;
        art : tArt;

    BEGIN
      Ueberschrift;
      i:= 0;
      REPEAT
        INC(i);
        IF i > 1 THEN Ausgabefenster END;
        Art(art);
        R.Randomize;(* Zufallszahlengenerator mit neuem Wert initialisieren *)
        Sortieren(a, art);
        AusgabeZahlenfeld(a)
      UNTIL ~Wiederholung();
      Information
    END ProgMain;

 (* -------------------  I N I T I A L I S I E R U N G  ------------------- *)

 BEGIN
   C.Open;
   dummy:= W.ShowWindow(W.GetActiveWindow(),3);
   bc_Red:=  24; bc_Green:=  16; bc_Blue:=  48; (* HG-Farbe groes Fenster  *)
   bc_red:=   8; bc_green:=  12; bc_blue:=  16; (* HG-Farbe kleines Fenster *)
   C.SetBackColor(bc_Red,bc_Green,bc_Blue);
   C.Bar(5,0,850,650,C.ERASE);
   C.SetForeColor(200,200,200);
   C.Bar(1,0,3,650,C.DRAW);
   C.SetForeColor(96,96,96);
   C.Line(4,0,4,650,C.DRAW);
   C.SetForeColor(96,220,220);
   (* Da die Vordergrundfarbe Schwarz automatisch eingestellt ist,  m u s s *)
   (* die Textfarbe bereits hier initialisiert werden. Sonst sieht man die  *)
   (* Eingabe bei der Anweisung 'C.ReadStr()' nicht!                        *)
 END Quick2ms.

 (* ----------   2001  Manfred Kmmerer, Chamberlain-Soft JENA  ---------- *)

