
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon (POW!-16)               Dateiname: ADTLISTE.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    Abstrakter Datentyp Liste (ADT Liste)                         |
  |-------------------------------------------------------------------------|
  | Leistung: S e r v i c e - M o d u l                                     |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul ist bis auf wenige Ausnahmen unkommentiert.         |
  |-------------------------------------------------------------------------|
  | Erluterungstext: Die Spezifikation des abstrakten Datentyps Liste      |
  |                   (ADT Liste) befindet sich unter:                      |
  |                   ...\adt\txt\spezifik.txt und                          |
  |                   ...\adt\doc\spezifik.doc                              |
  *-------------------------------------------------------------------------*)

 MODULE Adtliste;

  IMPORT Out;

  TYPE pliste = POINTER TO tliste;

       tliste = RECORD
                  daten     : INTEGER;
                  naechstes : pliste
                END;

       Liste* = RECORD
                  anker,
                  aktuelles, 
                  letztes    : pliste
                END;


  (** 1. Operation *)

  PROCEDURE Erzeugen* ( VAR liste : Liste );
    BEGIN
      liste.anker:= NIL
    END Erzeugen;


  (** 10. Operation *)

  PROCEDURE ListeLeer* ( liste : Liste ) : BOOLEAN;
    BEGIN
      RETURN liste.anker = NIL
    END ListeLeer;


  PROCEDURE ListenAnfang ( liste : Liste ) : BOOLEAN;

   (* Diese Operation liefert den Wert TRUE, wenn das erste Element aktuel- *)
   (* les Element ist. TRUE wird auch geliefert, wenn die Liste leer ist.   *)
   (* Ansonsten liefert die Operation den Wert FALSE.                       *)

    BEGIN
      IF ListeLeer(liste) THEN RETURN TRUE
      ELSE RETURN liste.aktuelles = liste.anker
      END
    END ListenAnfang;


  (** 11. Operation *)

  PROCEDURE ListenEnde* ( liste : Liste ) : BOOLEAN;
    BEGIN
      IF ListeLeer(liste) THEN RETURN TRUE
      ELSE RETURN liste.aktuelles = liste.letztes
      END
    END ListenEnde;


  PROCEDURE Davor ( liste : Liste ) : pliste;

   (* Diese Operation liefert einen Zeiger auf das Element vor dem aktuel-  *)
   (* len Element. NIL wird geliefert, wenn die Liste leer ist oder wenn    *)
   (* das erste Element aktuelles Element ist.                              *)

    VAR zeiger : pliste;

    BEGIN
      IF ListenAnfang(liste) THEN RETURN NIL
      ELSE      (* Die Liste enthlt mindestens ZWEI Elemente.              *)
                (* Das erste Element der Liste ist nicht aktuelles Element. *)
        zeiger:= liste.anker;
        WHILE zeiger^.naechstes # liste.aktuelles DO
          zeiger:= zeiger^.naechstes
        END;
        RETURN zeiger
      END
    END Davor;


  (** 2. Operation *)

  PROCEDURE EinfuegenElement* ( VAR liste : Liste; inhalt : INTEGER );

    VAR neues, voraktuelles : pliste;

    BEGIN
      NEW(neues);   
      neues^.daten:= inhalt;
      IF ListeLeer(liste) THEN                       (* Die Liste ist leer. *)
        neues^.naechstes:= NIL;
        liste.anker:= neues;
        liste.aktuelles:= neues;
        liste.letztes:= neues
      ELSIF ListenAnfang(liste) THEN 
                                (* Das erste Element ist aktuelles Element. *)
        neues^.naechstes:= liste.aktuelles;
        liste.anker:= neues;
        liste.aktuelles:= neues
      ELSE                (* Das erste Element ist nicht aktuelles Element. *)
        voraktuelles:= Davor(liste);
        voraktuelles^.naechstes:= neues;
        neues^.naechstes:= liste.aktuelles;
        liste.aktuelles:= neues
      END
    END EinfuegenElement;


  (** 3. Operation *)

  PROCEDURE AnhaengenElement* ( VAR liste : Liste; inhalt : INTEGER );


    VAR neues : pliste;

    BEGIN
      NEW(neues);
      neues^.daten:= inhalt;
      IF ListeLeer(liste) THEN                       (* Die Liste ist leer. *)
        neues^.naechstes:= NIL;
        liste.anker:= neues;
        liste.aktuelles:= neues;
        liste.letztes:= neues
      ELSE                     (* Die Liste enthlt mindestens ein Element. *)
        liste.letztes^.naechstes:= neues;
        neues^.naechstes:= NIL;
        liste.aktuelles:= neues;
        liste.letztes:= neues
      END
    END AnhaengenElement;


  (** 4. Operation *)

  PROCEDURE LoeschenElement* ( VAR liste : Liste );

    VAR voraktuelles : pliste;

    BEGIN
      IF ~ListeLeer(liste) THEN 
                               (* Die Liste enthlt mindestens ein Element. *)
        IF liste.anker = liste.letztes THEN 
                                    (* Die Liste enthlt genau ein Element. *)
          liste.anker:= NIL
        ELSIF ListenAnfang(liste) THEN 
                             (* Die Liste enthlt mindestens zwei Elemente. *)
                             (* Das erste Element ist aktuelles Element.    *)
          liste.anker:= liste.anker^.naechstes;
          liste.aktuelles:= liste.anker
        ELSIF ListenEnde(liste) THEN 
                               (* Das letzte Element ist aktuelles Element. *)
          voraktuelles:= Davor(liste);
          voraktuelles^.naechstes:= NIL;
          liste.letztes:= voraktuelles;
          liste.aktuelles:= liste.anker
        ELSE                     (* Weder das erste noch das letzte Element *)
                                 (*                  ist aktuelles Element. *)
          voraktuelles:= Davor(liste);
          voraktuelles^.naechstes:= liste.aktuelles^.naechstes;
          liste.aktuelles:= liste.anker
        END
      END
    END LoeschenElement;


  (** 5. Operation *)

  PROCEDURE GeheErstes* ( VAR liste : Liste );
    BEGIN
      IF ~ListeLeer(liste) THEN
        liste.aktuelles:= liste.anker
      END
    END GeheErstes;


  (** 6. Operation *)

  PROCEDURE GeheLetztes* ( VAR liste : Liste );
    BEGIN
      IF ~ListeLeer(liste) THEN
        liste.aktuelles:= liste.letztes
      END
    END GeheLetztes;


  (** 7. Operation *)

  PROCEDURE GeheNaechstes* ( VAR liste : Liste );
    BEGIN
      IF ~ListeLeer(liste) THEN
        IF ListenEnde(liste) THEN
          Out.String(' Laufzeitfehler: Zugriff nach dem letzten Element!');
          HALT(100)
        ELSE liste.aktuelles:= liste.aktuelles^.naechstes
        END
      END
    END GeheNaechstes;


  (** 8. Operation *)

  PROCEDURE HoleEintrag* ( liste : Liste; VAR inhalt : INTEGER );
    BEGIN
      IF ListeLeer(liste) THEN
        Out.String(' Laufzeitfehler: Liste ist leer!');
        HALT(101)
      ELSE inhalt:= liste.aktuelles^.daten
      END
    END HoleEintrag;


  (** 9. Operation *)

  PROCEDURE SchreibeEintrag* ( liste : Liste; inhalt : INTEGER );
    BEGIN
      IF ListeLeer(liste) THEN
        Out.String(' Laufzeitfehler: Liste ist leer!');
        HALT(102)
      ELSE liste.aktuelles^.daten:= inhalt
      END
    END SchreibeEintrag;


 END Adtliste.

