
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: AUSWAEHZ.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    S O R T I E R E N   durch Auswhlen - 5. Variante             |
  |-------------------------------------------------------------------------|
  | Leistung: Sortieren von INTEGER-Zahlen mit bestimmten Eigenschaften:    |
  |            - die Zahlen sind bereits aufsteigend sortiert               |
  |            - es handelt sich um Zufallszahlen                           |
  |            - die Zahlen sind absteigend (also "falsch herum") sortiert  |
  |            - Messen der zum Sortieren bentigten Zeit                   |
  |            - Speichern der Messdaten auf einem externen Speichermedium  |
  |-------------------------------------------------------------------------|
  | Hinweis:  Das Modul ist lauffhig, stellt aber nur ein Grundgerst dar. |
  |-------------------------------------------------------------------------|
  | Erluterungstext: ...\sort1\txt\auswaeh5.txt ...\sort1\doc\auswaeh5.doc |
  *-------------------------------------------------------------------------*)

 MODULE AuswaehZ;

  IMPORT In, Out, F:= File, R:= RandomNu, S:= Strings, Zeit;

  CONST  max = 16;

  TYPE   tStr   = ARRAY 51 OF CHAR;

         tPunkt = RECORD
                    x, y : INTEGER
                  END;      (* Abszissenachse (x): Anzahl sortierter Zahlen *)
                            (* Ordinatenachse (y): bentigte Sortierzeit    *)

         tDaten = ARRAY max+1 OF tPunkt;

  VAR    a     : ARRAY 16001 OF INTEGER;
         daten : tDaten;
         fPfad : tStr;


  PROCEDURE Anfrage() : BOOLEAN;

    VAR ch : CHAR;

    BEGIN
      In.Prompt('Datei schreib. (J/N)');
      In.Char(ch);
      ch:= S.UpCaseChar(ch);
      IF ch = 'J' THEN
        RETURN TRUE
      ELSE
        RETURN FALSE
      END;
    END Anfrage;


  PROCEDURE Arbeit ( n : INTEGER );
    BEGIN
      Out.Ln;
      Out.String('  Das System sortiert '); Out.Int(n,1);
      Out.String(' Zahlen.'); Out.Ln; Out.Ln
    END Arbeit;


  PROCEDURE AusgabeArtStrg;

    VAR artStr : ARRAY 4 OF tStr;
        i      : INTEGER;

    BEGIN
      artStr[0]:= 'aufsteigend sortierte Zahlen';
      artStr[1]:= 'Zufallszahlen';
      artStr[2]:= 'absteigend, also "falsch herum", sortierte Zahlen';
      artStr[3]:= 'alle drei Arten nacheinander';
      Out.Ln;
      FOR i:= 0 TO 3 DO
        Out.Int(i+1,5); Out.String('. Art: '); Out.String(artStr[i]); Out.Ln
      END;
      Out.Ln
    END AusgabeArtStrg;


  PROCEDURE Art ( VAR art : INTEGER );
    BEGIN
      REPEAT
        In.Prompt('Art ( 1..4 )');
        In.Int(art)
      UNTIL art IN {1..4}
    END Art;


  PROCEDURE Anzahl ( VAR n : INTEGER );
    BEGIN
      In.Prompt(' Anzahl ( DT: INT )');
      In.Int(n)
    END Anzahl;


  PROCEDURE Zufallszahl() : INTEGER;
    BEGIN
      RETURN SHORT(ENTIER(R.Uniform() * 10000))
    END Zufallszahl;


  PROCEDURE Sortieren ( art, n : INTEGER );
   (*-----------------------------------------------------------------------*
    | Leistung: Sortieren durch Auswhlen.                                  |
    |           Die ausfhrliche Beschreibung des Algorithmus' findet man   |
    |           unter  ...\sort1\doc\auswaeh1.doc (WORD-Version)            |
    |           oder   ...\sort1\txt\auswaeh1.txt (Text-Version)            |
    *-----------------------------------------------------------------------*)

    VAR k, r, s, t, x : INTEGER;
    
    BEGIN
      CASE art OF
         1: FOR k:= 1 TO n DO
               a[k]:= k
             END
      | 2: FOR k:= 1 TO n DO
               a[k]:= Zufallszahl()
             END;
      | 3: FOR k:= 1 TO n DO
              a[k]:= -k
            END
      END;
      Zeit.Start;
      FOR r:= 1 TO n-1 DO
         t:= r;
         x:= a[r];
         FOR s:= r+1 TO n DO
            IF a[s] < x THEN
               t:= s;
               x:= a[s]
            END
         END;
         a[t]:= a[r];
         a[r]:= x
      END;
      Zeit.Stopp;
      Out.Int(art,4);
      Out.Int(n,8);
      Out.Int(Zeit.differenz,8); Out.Ln   (* Zeitdifferenz in Millisekunden *)
    END Sortieren;


  PROCEDURE PfadStr ( VAR message : tStr );
    BEGIN
      message:= "   Die Datei '";
      S.Append(message,fPfad);
      S.Append(message,"' wurde geschrieben.");
    END PfadStr;


  PROCEDURE ErlaeuterungDateiname;
    BEGIN
      Out.Ln; Out.Ln;
      Out.String('    Der Dateiname darf 8 Zeichen lang sein.'); Out.Ln;
      Out.String("    Die Dateierweiterung '.DAT' wird automatisch ergnzt.");
      Out.Ln; Out.Ln;
      Out.String("    Verwenden Sie 'sprechende' Dateinamen, z. B.:"); Out.Ln;
      Out.String("    'AUSWAE02' fr Art = 2 (zufllig verteilt)");
      Out.Ln; Out.Ln
    END ErlaeuterungDateiname;


  PROCEDURE DateiSchreiben ( anzahl, art : INTEGER ) : BOOLEAN;

    VAR ok         : BOOLEAN;
        ch         : CHAR;
        handle     : F.Handle;
        laenge,
        i, resCode : INTEGER;
        n          : LONGINT;
        message    : tStr;
        punkt      : tPunkt;

    BEGIN
      ErlaeuterungDateiname;
      REPEAT
        In.Prompt('Dateiname');
        In.String(fPfad);
        laenge:= SHORT(S.Length(fPfad)); 
        IF (fPfad > ' ') & (laenge <= 8) THEN
          S.Append(fPfad,'.DAT');
          ok:= TRUE
        END;
        IF ok & F.Exist(fPfad) THEN
          In.Prompt('Lschen ( J/N )? N');
          In.Char(ch); ch:= CAP(ch);
          IF ch = 'J' THEN
            F.Delete(fPfad,resCode);
            ok:= TRUE
          ELSE
            ok:= FALSE;
            RETURN ok
          END
        END
      UNTIL ok;
      F.Open(fPfad,TRUE,F.DENYNONE,F.WRITEONLY,handle,resCode);
      PfadStr(message);
      n:= SIZE(tPunkt);
      FOR i:= 1 TO anzahl DO
        F.WriteBlock(handle,daten[i],n,resCode)
      END;
      Out.Int(resCode,3); Out.Ln; Out.Ln;
      F.Close(handle);
      Out.String(message); Out.Ln; Out.Ln;
      Out.String('   Anzahl Datenstze = '); Out.Int(anzahl,2); Out.Ln;
      Out.Ln;
      RETURN ok
    END DateiSchreiben;


  PROCEDURE ProgMain*();

    VAR ok        : BOOLEAN;
        ch        : CHAR;
        anzahl,
        art, i, n : INTEGER;


    BEGIN
      anzahl:= 0;
      n:= 0;
      ok:= FALSE;
      AusgabeArtStrg;
      Art(art);
      REPEAT
        INC(n,1000);
        INC(anzahl);
        R.Randomize;(* Zufallszahlengenerator mit neuem Wert initialisieren *)
        IF art = 4 THEN
          Arbeit(n);
          FOR i:= 1 TO 3 DO            (* alle drei Ausgangskonfigurationen *)
            Sortieren(i,n)
          END
        ELSE
          Arbeit(n);
          Sortieren(art,n)
        END;
        IF anzahl < max THEN
          In.Prompt('Wiederhlg. (J/N) J');
          In.Char(ch)
        END                      
      UNTIL (ch = 'n') OR (anzahl = max);
      IF (art < 4) & (anzahl > 10) & Anfrage() THEN
        REPEAT
          ok:= DateiSchreiben(anzahl,art)
        UNTIL ok
      END
    END ProgMain;


 BEGIN
   In.Open;
   Out.Open
 END AuswaehZ.
