
 (*-------------------------------------------------------------------------*
  | Programmieren in Oberon-2 POW! (16 Bit)         Dateiname: DATUMOPE.MOD |
  |-------------------------------------------------------------------------|
  | Thema:    SERVICEMODUL fr   D a t u m s o p e r a t i o n e n          |
  |-------------------------------------------------------------------------|
  | Leistung: Bereitstellen verschiedenster Operationen, die bei Datums-    |
  |           berechnungen bentigt werden.                                 |
  *-------------------------------------------------------------------------*)

 MODULE DatumOpe;

  IMPORT Out;

  TYPE   tDatum* = RECORD
                     jahr*,
                     monat*,
                     tag* : INTEGER
                   END;

       tName*    = ARRAY 11 OF CHAR;


 (* --------------------  U N T E R P R O G R A M M E  -------------------- *)


  PROCEDURE Schaltjahr* ( jahr : INTEGER ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Jahres                         |
    | Referenz:  Wahrheitswert als Rckgabewert (Funktionswert, DT: BOOLEAN)|
    | Leistung:  Ermitteln, ob es sich bei dem zu testenden Jahr um ein     |
    |            Schaltjahr handelt.                                        |
    |            TRUE = Schaltjahr, FALSE = kein Schaltjahr                 |
    | Hinweis:   Ist eine Jahreszahl des gregorianischen Kalenders durch    |
    |            4 oder (einschlieend) 400 teilbar, so handelt es sich bei |
    |            diesem Jahr um ein Schaltjahr, der 29.02. existiert.       |
    *-----------------------------------------------------------------------*)

    VAR c, j : INTEGER;

    BEGIN
      c:= jahr DIV 100; 
      j:= jahr MOD 100;
      IF j = 0 THEN 
        RETURN c MOD 4 = 0
      ELSE
        RETURN j MOD 4 = 0 
      END;
    END Schaltjahr;



  PROCEDURE TageImMonat* ( jahr, monat : INTEGER ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl und Monatsnummer des zu testenden Kalendermo-   |
    |            nats                                                       |
    | Referenz:  Tagesanzahl dieses Monats                                  |
    | Leistung:  Innerhalb der Anweisungsfolge einer mehrseitigen Auswahl   |
    |            wird, unter Bercksichtigung eines mglichen Schaltjahrs,  |
    |            der Variablen anzahl ein von der Monatsnummer abhngiger   |
    |            Wert zugewiesen.                                           |
    *-----------------------------------------------------------------------*)

    VAR anzahl: INTEGER;

    BEGIN
      CASE monat OF
         1, 3, 5, 7, 8, 10, 12: anzahl:= 31 
       | 4, 6, 9, 11:           anzahl:= 30
       | 2:                     IF Schaltjahr(jahr) THEN
                                  anzahl:= 29
                                ELSE
                                  anzahl:= 28
                                END
      END;
      RETURN anzahl
    END TageImMonat;


  PROCEDURE TageImJahr* ( jahr : INTEGER ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des zu testenden Kalenderjahres                 |
    | Referenz:  Tagesanzahl (Rckgabewert, Funktionswert)                  |
    | Leistung:  Ermitteln der Tagesanzahl des als Wertparameter bergebe-  |
    |            nen Kalenderjahrs unter Bercksichtigung des Sonderfalls   |
    |            Schaltjahr.                                                |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF Schaltjahr(jahr) THEN
        RETURN 366
      ELSE
        RETURN 365
      END
    END TageImJahr;


  PROCEDURE Existenz* ( datum : tDatum ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer des zu testenden Ka-   |
    |            lenderdatums (Verbund datum, DT: tDatum)                   |
    | Referenz:  Wahrheitswert (Rckgabewert, Funktionswert)                |
    | Leistung:  Bewerten eines vom Anwender eingegebenen Kalenderdatums    |
    |            (gregorianischer Kalender) auf Korrektheit.                |
    |            Funktionswert TRUE  = Tag existiert                        |
    |                          FALSE = Tag existiert nicht                  |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN (datum.jahr > 1582) & (datum.jahr < 3001)
        & (datum.monat > 0) & (datum.monat < 13)
        & (datum.tag > 0) & (datum.tag <= TageImMonat(datum.jahr,datum.monat))
    END Existenz;

   
  PROCEDURE Wochentagsnummer* ( d : tDatum ) : INTEGER;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer des zu testenden Ka-   |
    |            lenderdatums (Verbund datum, DT: tDatum)                   |
    | Referenz:  Wochentagsnummer (Rckgabewert, Funktionswert)             |
    | Leistung:  Ermitteln der Wochentagsnummer eines nach dem  gregoriani- |
    |            schen Kalender gltigen Datums mit Hilfe der Kalenderfor-  |
    |            mel nach JACOBSTHAL.                                       |
    |            Es steht 0 fr Sonntag, 1 fr Montag 2 fr Dienstag usw.   |
    *-----------------------------------------------------------------------*)

    VAR k, c, j, w: INTEGER;

    BEGIN
      CASE d.monat OF
         1: IF Schaltjahr(d.jahr) THEN k := 5
              ELSE k := 6
            END
       | 2: IF Schaltjahr(d.jahr) THEN k := 1
              ELSE k := 2
            END
       | 3: k := 2
       | 4: k := 5
       | 5: k := 0
       | 6: k := 3
       | 7: k := 5
       | 8: k := 1
       | 9: k := 4
       |10: k := 6
       |11: k := 2
       |12: k := 4
      END;
      c:= d.jahr DIV 100;
      j:= d.jahr MOD 100;
      w:= d.tag + k + j + j DIV 4 - 2 * (c MOD 4);
      RETURN w MOD 7
    END Wochentagsnummer;


  PROCEDURE Wochentagsname* ( tagesnummer : INTEGER;
                                               VAR wochentagsname : tName );
   (*-----------------------------------------------------------------------*
    | bername: Wochentagsnummer (tagesnummer {0..6})                       |
    | Referenz: Wochentagsname (DT: tName)                                  |
    | Leistung: Ermitteln des Wochentagsnamens (mehrseitige Auswahl). Da    |
    |           die Gltigkeit der Wochentagsnummer nicht garantiert werden |
    |           kann,  m u s s  der Fallunterscheidung ein ELSE-Zweig nach- |
    |           gestellt werden. Vermeiden der Fehlermeldung: CASE trap (no |
    |           ELSE includet) at line ..., nach bernahme einer ungltigen |
    |           Wochentagsnummer.                                           |
    *-----------------------------------------------------------------------*)

    BEGIN
      CASE tagesnummer OF
         0: wochentagsname:= 'Sonntag'
       | 1: wochentagsname:= 'Montag'
       | 2: wochentagsname:= 'Dienstag'
       | 3: wochentagsname:= 'Mittwoch'
       | 4: wochentagsname:= 'Donnerstag'
       | 5: wochentagsname:= 'Freitag'
       | 6: wochentagsname:= 'Samstag'
      ELSE
        Out.String(' Fehlerhafte Wochentagsnummer!'); Out.Ln
      END
    END Wochentagsname;


  PROCEDURE Monatsname* ( monatsnummer: INTEGER; VAR monatsname : tName );
   (*-----------------------------------------------------------------------*
    | bername: Nummer eines Kalendermonats (monatsnummer {1..12})          |
    | Referenz: Monatsname (DT: tName)                                      |
    | Leistung: Ermitteln des Monatsnamens (mehrseitige Auswahl). Da die    |
    |           Gltigkeit der Monatsnummer nicht garantiert werden kann,   |
    |           m u s s  der Fallunterscheidung ein ELSE-Zweig nachgestellt |
    |           Vermeiden der Fehlermeldung: CASE trap (no ELSE includet)   |
    |           at line ..., nach bernahme einer ungltigen Monatsnummer.  |
    *-----------------------------------------------------------------------*)

    BEGIN
      CASE monatsnummer OF
         1: monatsname:= 'Januar'
       | 2: monatsname:= 'Februar'
       | 3: monatsname:= 'Mrz'
       | 4: monatsname:= 'April'
       | 5: monatsname:= 'Mai'
       | 6: monatsname:= 'Juni'
       | 7: monatsname:= 'Juli'
       | 8: monatsname:= 'August'
       | 9: monatsname:= 'September'
       |10: monatsname:= 'Oktober'
       |11: monatsname:= 'November'
       |12: monatsname:= 'Dezember'
      ELSE
        Out.String(' Fehlerhafte Monatsnummer!'); Out.Ln
      END
    END Monatsname;


  PROCEDURE Ostersonntag* ( jahr : INTEGER; VAR d : tDatum );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl des Kalenderjahres, fr das das Osterdatum, das |
    |            Kalenderdatum des Ostersonntags, berechnet werden soll.    |
    | Referenz:  Jahreszahl, Monatsnummer und Tagesnummer des Osterdatums   |
    |            des gewnschten Kalenderjahrs (Verbund datum, DT: tDatum)  |
    | Leistung:  Ermitteln des Osterdatums eines nach dem  gregorianischen  |
    |            Kalender gltigen Jahres mit Hilfe des in der Quelle ange- |
    |            gebenen Algorithmus'.                                      |
    |-----------------------------------------------------------------------|
    | Literatur: MNU 51/3 (15.4.1998) Seiten 141-144                        |
    *-----------------------------------------------------------------------*)

    VAR k, a, e, n, w, s: INTEGER;

    BEGIN
      k:= jahr DIV 100;
      a:= jahr MOD 19;
      e:= (11 * a + 8 + (8 * k + 13) DIV 25 - (3 * k + 3) DIV 4) MOD 30;
      IF e < 24 THEN
        n:= 44 - e
      ELSIF e = 24 THEN
        n:= 49
      ELSIF e = 25 THEN
        n:= 49 - a DIV 11
      ELSE
        n:= 74 - e
      END;
      w:= (n + 5 * jahr DIV 4 + 2 - (3 * k + 3) DIV 4) MOD 7;
      s:= n + 7 - w;
      IF s <= 31 THEN 
        d.jahr := jahr;
        d.monat:= 3;
        d.tag  := s
      ELSE
        d.jahr := jahr;
        d.monat:= 4;
        d.tag  := s - 31
      END
    END Ostersonntag;

    
  PROCEDURE IstGleich* ( a, b : tDatum ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahlen, Monats- und Tagesnummern zweier zu testender |
    |            Kalenderdaten (a, b; Verbund datum, DT: tDatum)            |
    | Referenz:  Wahrheitswert (Rckgabewert, Funktionswert)                |
    | Leistung:  Bewerten zweier vom Anwender eingegebener Kalenderdaten    |
    |            (gregorianischer Kalender) auf Gleichheit.                 |
    |            Funktionswert TRUE  = Kalenderdaten sind gleich            |
    |                          FALSE = Kalenderdaten sind ungleich          |
    *-----------------------------------------------------------------------*)

    BEGIN
      RETURN (a.jahr = b.jahr) & (a.monat = b.monat) & (a.tag = b.tag)
    END IstGleich;


  PROCEDURE LiegtVor* ( a, b : tDatum ) : BOOLEAN;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahlen, Monats- und Tagesnummern zweier zu testender |
    |            Kalenderdaten (a, b; Verbund datum, DT: tDatum)            |
    | Referenz:  Wahrheitswert (Rckgabewert, Funktionswert)                |
    | Leistung:  Ermitteln, ob das zu bewertende Kalenderdatum a vor dem    |
    |            Kalenderdatum b (gregorianischer Kalender) liegt.          |
    |            Funktionswert TRUE  = Kalenderdaten a liegt vor b          |
    |                          FALSE = Kalenderdaten a liegt nicht vor b    |
    *-----------------------------------------------------------------------*)

    BEGIN
      IF a.jahr < b.jahr THEN
        RETURN TRUE
      ELSIF a.jahr > b.jahr THEN
        RETURN FALSE
      ELSIF a.monat < b.monat THEN
        RETURN TRUE
      ELSIF a.monat > b.monat THEN
        RETURN FALSE
      ELSE
        RETURN a.tag < b.tag
      END
    END LiegtVor;


  PROCEDURE Nachfolger* ( VAR datum : tDatum );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer des zu testenden Ka-   |
    |            lenderdatums (Verbund datum, DT: tDatum) von dem der       |
    |            Nachfolger bestimmt werden soll.                           |
    | Referenz:  komplettes Kalenderdatum des 'Nachfolgers'                 |
    | Leistung:  Ermitteln des kompletten Kalenderdatums des folgenden Ta-  |
    |            ges trotz mglichem Monats- und Jahreswechsel unter Beach- |
    |            tung, dass es sich um ein Schaltjahr handeln knnte.       |
    *-----------------------------------------------------------------------*)

    VAR naechstes : tDatum; 

    BEGIN
      IF datum.tag < TageImMonat(datum.jahr,datum.monat) THEN 
        naechstes.jahr := datum.jahr;
        naechstes.monat:= datum.monat;
        naechstes.tag  := datum.tag + 1
      ELSIF datum.monat < 12 THEN
        naechstes.jahr := datum.jahr;
        naechstes.monat:= datum.monat + 1;
        naechstes.tag  := 1
      ELSE
        naechstes.jahr := datum.jahr + 1;
        naechstes.monat:= 1;
        naechstes.tag  := 1
      END;
      datum:= naechstes
    END Nachfolger;


  PROCEDURE Vorgaenger* ( VAR datum : tDatum );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer des zu testenden Ka-   |
    |            lenderdatums (Verbund datum, DT: tDatum) von dem der       |
    |            Vorgnger bestimmt werden soll.                            |
    | Referenz:  komplettes Kalenderdatum des 'Vorgngers'                  |
    | Leistung:  Ermitteln des kompletten Kalenderdatums des vorangegange-  |
    |            nen Tages trotz mglichem Monats- und Jahreswechsel unter  |
    |            Beachtung, dass es sich um ein Schaltjahr handeln knnte.  |
    *-----------------------------------------------------------------------*)

    VAR vorheriges : tDatum;

    BEGIN
      IF datum.tag > 1 THEN
        vorheriges.jahr := datum.jahr;
        vorheriges.monat:= datum.monat;
        vorheriges.tag  := datum.tag - 1
      ELSIF datum.monat > 1 THEN
        vorheriges.jahr := datum.jahr;
        vorheriges.monat:= datum.monat - 1;
        vorheriges.tag  := TageImMonat(vorheriges.jahr, vorheriges.monat)
      ELSE
        vorheriges.jahr := datum.jahr - 1;
        vorheriges.monat:= 12;
        vorheriges.tag  := 31
      END;
      datum:= vorheriges
    END Vorgaenger;


  PROCEDURE DavorDanach* ( a: tDatum; anzahl : LONGINT; VAR b : tDatum );
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer eines Kalenderdatums   |
    |            (Verbund datum, DT: tDatum) und die Tagesanzahl (anzahl)   |
    |            die den Abstand zum Nachfolgedatum (anzahl positiv) oder   |
    |            zum vorherigen Kalenderdatum (anzahl negativ) angibt.      |
    | Referenz:  komplettes Kalenderdatum                                   |
    | Leistung:  Ermitteln des geforderten Kalenderdatums unter Nutzung der |
    |            Prozedur Nachfolger(), falls der Wert der Variablen anzahl |
    |            positiv ist, oder Vorgaenger(), falls dieser Wert negativ  |
    |            sein sollte.                                               |
    *-----------------------------------------------------------------------*)

    VAR z: LONGINT;

    BEGIN
      IF anzahl > 0 THEN
        FOR z:= 1 TO anzahl DO Nachfolger(a) END   
      ELSIF anzahl < 0 THEN
        FOR z:= 1 TO - anzahl DO Vorgaenger(a) END
      END;
      b:= a
    END DavorDanach;


  PROCEDURE TauschenDaten* ( VAR a, b : tDatum );
   (*-----------------------------------------------------------------------*
    | Referenz: Jahreszahlen, Monats- und Tagesnummern der beiden zu tau-   |
    |           schenden Kalenderdaten (Verbund datum, DT: tDatum)          |
    | Leistung: Tauschvorgang                                               |
    *-----------------------------------------------------------------------*)

    VAR  temp : tDatum;

    BEGIN
      temp.jahr:= a.jahr;      temp.monat:= a.monat;      temp.tag:= a.tag;
      a.jahr:=    b.jahr;      a.monat:=    b.monat;      a.tag:=    b.tag;
      b.jahr:=    temp.jahr;   b.monat:=    temp.monat;   b.tag:=    temp.tag
    END TauschenDaten;


  PROCEDURE Abstand* ( a, b : tDatum ) : LONGINT;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahlen, Monats- und Tagesnummern zweier Kalenderda-  |
    |            ten (a, b; Verbund datum, DT: tDatum), deren Abstand in    |
    |            Tage ermittelt werden soll.                                |
    | Referenz:  Abstand in Tagen als Rckgabewert (Funktionswert) der      |
    |            Funktionsprozedur.                                         |
    | Leistung:  Prfen, ob das erste Kalenderdatum (a)  v o r  dem zweiten |
    |            (b) liegt. Ist das  n i c h t  der Fall, so werden die     |
    |            beiden Kalenderdaten (Verbunde a und b) getauscht.         |
    |            Anschlieend wird der Abstand (anzahl) Schritt fr Schritt |
    |            errechnet, indem bei unterschiedlichen Kalenderjahren oder |
    |            unterschiedlichen Kalendermonaten die Tagesanzahl bis zum  |
    |            Monatsende des ersten Datums bestimmt und der Variablen    |
    |            anzahl zugewiesen wird, danach die Anzahl der Tage fr die |
    |            Monate bis zum Jahresende des ersten Kalenderjahrs ...     |
    *-----------------------------------------------------------------------*)

    VAR i      : INTEGER;
        anzahl : LONGINT;

    BEGIN
      IF LiegtVor(b,a) THEN TauschenDaten(a,b) END;
      IF a.jahr # b.jahr THEN
        anzahl:= TageImMonat(a.jahr,a.monat) - a.tag;
        FOR i:= a.monat + 1 TO 12 DO
          anzahl:= anzahl + TageImMonat(a.jahr,i)
        END;
        FOR i:= a.jahr + 1 TO b.jahr - 1 DO
          anzahl:= anzahl + TageImJahr(i)
        END;
        FOR i:= 1 TO b.monat - 1 DO
          anzahl:= anzahl + TageImMonat(b.jahr,i)
        END;
        anzahl:= anzahl + b.tag
      ELSIF a.monat # b.monat THEN
        anzahl:= TageImMonat(a.jahr, a.monat) - a.tag;
        FOR i:= a.monat + 1 TO b.monat - 1 DO
            anzahl:= anzahl + TageImMonat(b.jahr,i)
        END;
        anzahl:= anzahl + b.tag
      ELSE
        anzahl:= b.tag - a.tag
      END;
      RETURN anzahl
    END Abstand;


  PROCEDURE JulianischeTagesnummer* ( datum : tDatum ) : LONGINT;
   (*-----------------------------------------------------------------------*
    | bernahme: Jahreszahl, Monats- und Tagesnummer eines Kalenderdatums   |
    |            (Verbund datum, DT: tDatum)                                |
    | Referenz:  Tagesnummer des sogenannten 'julianischen Datums'          |
    | Leistung:  Berechnen der Tagesnummer des julianischen Tages, nach     |
    |            einem leicht modifizierten Algorithmus von R. G. Tantzen   |
    | Hinweise:  'julianisch' hat nichts mit dem julianischen Kalender oder |
    |            Julius Csar zu tun. Tatzens Vater hie Julius. Deshalb    |
    |            hat er diese Konstante (julianisches Datum) so genannt.    |
    |            Das 'julianische Datum' bezeichnet den Abstand eines Ka-   |
    |            lenderdatums zum Datum 4713 01 01 v. Chr. in Tagen.        |
    *-----------------------------------------------------------------------*)

    VAR c, y : LONGINT;

    BEGIN
      IF datum.monat > 2 THEN
        DEC(datum.monat,3)
      ELSE
        INC(datum.monat,9);
        DEC(datum.jahr)
      END;
      datum.tag:= datum.tag + (153 * datum.monat + 2) DIV 5;
      c:= 146097 * (datum.jahr DIV 100) DIV 4;
      y:= 1461 * (datum.jahr MOD 100) DIV 4;
      RETURN c + y + datum.tag + 1721119
    END JulianischeTagesnummer;


  PROCEDURE GibDatum* ( julTagesnr : LONGINT; VAR datum : tDatum );
   (*-----------------------------------------------------------------------*
    | bernahme: 'julianisches Datum', Tage seit 4713 01 01 v. Chr.         |
    | Referenz:  Kalenderdatum nach dem nach dem gregorianischen Kalender   |
    | Leistung:  Berechnen eines Kalenderdatums mittels des 'julianischen   |
    |            Datums' nach einem Algorithmus von R. G. Tantzen.          |
    |            Zum 'julianisches Datum', siehe Hinweise zur Prozedur:     |
    |                                             JulianischeTagesnummer()  |
    *-----------------------------------------------------------------------*)

    CONST teiler1 = 146097;
          teiler2 = teiler1 DIV 100 + 1;
          teiler3 = teiler1 DIV 955 + 1;

    VAR   jj, jm, jt, temp1, temp2, temp3 : LONGINT;

    BEGIN
      julTagesnr:= julTagesnr - 1721119;
      temp1:= 4 * julTagesnr - 1;
      jj:= temp1 DIV teiler1;
      julTagesnr:= temp1 MOD teiler1;
      jt:= julTagesnr DIV 4;
      temp2:= jt * 4 + 3;
      julTagesnr:= temp2 DIV teiler2;
      jt:= temp2 MOD teiler2;
      jt:= (jt + 4) DIV 4;
      temp3:= 5 * jt - 3;
      jm:= temp3 DIV teiler3;
      jt:= temp3 MOD teiler3;
      jt:= (jt + 5) DIV 5;
      jj:= 100 * jj + julTagesnr;
      IF (jm < 10) THEN
        INC(jm,3)
      ELSE
        DEC(jm,9);
        INC(jj)
      END;
      datum.jahr:=  SHORT(jj);
      datum.monat:= SHORT(jm);
      datum.tag:=   SHORT(jt)
    END GibDatum;



 (* ------------------------  OHNE INITIALISIERUNG  ----------------------- *)

 END DatumOpe.

 (* ----------------------------------------------------------------------- *)
